package ali

import (
	"encoding/json"
	"errors"
	openapi "github.com/alibabacloud-go/darabonba-openapi/v2/client"
	ocr_api20210707 "github.com/alibabacloud-go/ocr-api-20210707/client"
	util "github.com/alibabacloud-go/tea-utils/v2/service"
	"github.com/alibabacloud-go/tea/tea"
	uuid "github.com/satori/go.uuid"
	"log"
	"net/url"
)

var Ocr = &ocr{}

type ocr struct {
	url             string
	idCardUrl       string
	bankCardUrl     string
	appcode         string
	accessKeyId     string
	accessKeySecret string
	endpoint        string
	client          *ocr_api20210707.Client
}

func InitOcr(url, IdCardUrl, bankCardUrl, appcode, accessKeyId, accessKeySecret, endpoint string) (err error) {
	Ocr.url = url
	Ocr.idCardUrl = IdCardUrl
	Ocr.bankCardUrl = bankCardUrl
	Ocr.appcode = appcode
	Ocr.accessKeyId = accessKeyId
	Ocr.accessKeySecret = accessKeySecret
	Ocr.endpoint = endpoint

	ocrConfig := &openapi.Config{
		// 您的 AccessKey ID
		AccessKeyId: &accessKeyId,
		// 您的 AccessKey Secret
		AccessKeySecret: &accessKeySecret,
	}
	// 访问的域名
	ocrConfig.Endpoint = tea.String(endpoint)
	Ocr.client, err = ocr_api20210707.NewClient(ocrConfig)
	if err != nil {
		return
	}
	return
}

type ocrFaceRes struct {
	AlgoVersion string         `json:"algo_version"`
	Data        ocrFaceResData `json:"data"`
}

type ocrFaceResData struct {
	Face faceData `json:"face"`
}

type faceData struct {
	AlgoVersion string      `json:"algo_version"`
	Data        interface{} `json:"data"`
}

type vehicleInfo struct {
	Address            string `json:"address"`
	EngineNumber       string `json:"engineNumber"`
	IssueAuthority     string `json:"issueAuthority"`
	IssueDate          string `json:"issueDate"`
	LicensePlateNumber string `json:"licensePlateNumber"`
	Model              string `json:"model"`
	Owner              string `json:"owner"`
	RegistrationDate   string `json:"registrationDate"`
	UseNature          string `json:"useNature"`
	VehicleType        string `json:"vehicleType"`
	VinCode            string `json:"vinCode"`
}

type idCardInfo struct {
	Name      string `json:"name"`      //   姓名
	Sex       string `json:"sex"`       //  性别
	Ethnicity string `json:"ethnicity"` //   民族
	BirthDate string `json:"birthDate"` // 出生日期
	Address   string `json:"address"`   // 住址
	IdNumber  string `json:"idNumber"`  //  身份证号码
}
type ocrBankCardRes struct {
	AlgoVersion string   `json:"algo_version"`
	Data        bankInfo `json:"data"`
}

type bankInfo struct {
	BankName    string `json:"bankName"`    // 银行
	CardNumber  string `json:"cardNumber"`  //  银行卡号
	ValidToDate string `json:"validToDate"` // 有效期至
	CardType    string `json:"cardType"`    // 卡种
}

// AliOcrVehicle @Title 读光OCR行驶证识别
func (o *ocr) AliOcrVehicle(pic string) (result vehicleInfo, err error) {
	recognizeVehicleLicenseRequest := &ocr_api20210707.RecognizeVehicleLicenseRequest{
		Url: tea.String(pic),
	}
	runtime := &util.RuntimeOptions{}

	// 复制代码运行请自行打印 API 的返回值
	res, err := o.client.RecognizeVehicleLicenseWithOptions(recognizeVehicleLicenseRequest, runtime)
	if err != nil {
		return
	}
	if *res.StatusCode != 200 {
		log.Println("AliOcr行驶证识别失败：" + *res.Body.Message)
		return result, errors.New("AliOcr行驶证识别失败")
	}
	//log.Println(res.String())
	var resBody ocrFaceRes
	err = json.Unmarshal([]byte(*res.Body.Data), &resBody)
	if err != nil {
		return result, errors.New("aliOcr返回值解析出错")
	}
	marshal, err := json.Marshal(resBody.Data.Face.Data)
	if err != nil {
		return result, errors.New("aliOcrVehicle返回值解析出错")
	}
	var info vehicleInfo
	err = json.Unmarshal(marshal, &info)
	if err != nil {
		return result, errors.New("aliOcrVehicle返回值解析出错")
	}
	return info, nil
}

// AliOcrIdCard @Title 读光OCR身份证识别
func (o *ocr) AliOcrIdCard(pic string) (result idCardInfo, err error) {
	recognizeIdCardRequest := &ocr_api20210707.RecognizeIdcardRequest{
		Url: tea.String(pic),
	}
	runtime := &util.RuntimeOptions{}
	// 复制代码运行请自行打印 API 的返回值
	res, err := o.client.RecognizeIdcardWithOptions(recognizeIdCardRequest, runtime)
	if err != nil {
		return
	}
	if *res.StatusCode != 200 {
		log.Println("AliOcr身份证识别失败：" + *res.Body.Message)
		return result, errors.New("AliOcr身份证识别失败")
	}
	//log.Println(res.String())
	var resBody ocrFaceRes
	err = json.Unmarshal([]byte(*res.Body.Data), &resBody)
	if err != nil {
		return result, errors.New("AliOcr返回值解析出错")
	}
	marshal, err := json.Marshal(resBody.Data.Face.Data)
	if err != nil {
		return result, errors.New("aliOcrVehicle返回值解析出错")
	}
	var info idCardInfo
	err = json.Unmarshal(marshal, &info)
	if err != nil {
		return result, errors.New("aliOcrVehicle返回值解析出错")
	}
	return info, nil
}

// AliOcrBankCard @Title 读光OCR银行卡识别
func (o *ocr) AliOcrBankCard(pic string) (result bankInfo, err error) {
	recognizeBankCardRequest := &ocr_api20210707.RecognizeBankCardRequest{
		Url: tea.String(pic),
	}
	runtime := &util.RuntimeOptions{}
	// 复制代码运行请自行打印 API 的返回值
	res, err := o.client.RecognizeBankCardWithOptions(recognizeBankCardRequest, runtime)
	if err != nil {
		return
	}
	if *res.StatusCode != 200 {
		log.Println("AliOcr银行卡识别失败：" + *res.Body.Message)
		return result, errors.New("AliOcr银行卡识别失败")
	}
	//log.Println(res.String())
	var resBody ocrBankCardRes
	err = json.Unmarshal([]byte(*res.Body.Data), &resBody)
	if err != nil {
		return result, errors.New("返回值解析出错")
	}
	return resBody.Data, nil
}

type ret struct {
	Ret  int         `json:"ret"`
	Msg  string      `json:"msg"`
	Data VehicleInfo `json:"data"`
}

type VehicleInfo struct {
	Owner     string `json:"owner"`
	Engineno  string `json:"engineno"`
	Cartype   string `json:"cartype"`
	Address   string `json:"address"`
	Lstype    string `json:"lstype"`
	Lsnum     string `json:"lsnum"`
	Regdate   string `json:"regdate"`
	Vin       string `json:"vin"`
	Issuedate string `json:"issuedate"`
	Usetype   string `json:"usetype"`
}

// Vehicle @Title 行驶证识别
func (o *ocr) Vehicle(pic string) (result VehicleInfo, err error) {
	bytes, err := request("POST", o.url, url.Values{
		"pic":  {pic},
		"type": {"1"},
	}.Encode(), map[string]string{
		"Content-Type":  "application/x-www-form-urlencoded",
		"Authorization": "APPCODE " + o.appcode,
	})
	if err != nil {
		return
	}
	var res ret
	if err = json.Unmarshal(bytes, &res); err != nil {
		return
	}
	if res.Ret != 200 {
		return result, errors.New(res.Msg)
	}
	result = res.Data
	return
}

type retIdCard struct {
	Ret  int        `json:"ret"`
	Msg  string     `json:"msg"`
	Data IdCardInfo `json:"data"`
}

type IdCardInfo struct {
	Number  string `json:"number"`
	Address string `json:"address"`
	Nation  string `json:"nation"`
	Sex     string `json:"sex"`
	Name    string `json:"name"`
	Birth   string `json:"birth"`
}

// IdCard @Title 身份证识别
func (o *ocr) IdCard(pic string) (result IdCardInfo, err error) {
	bytes, err := request("POST", o.idCardUrl, url.Values{
		"pic":  {pic},
		"type": {"1"},
	}.Encode(), map[string]string{
		"Content-Type":  "application/x-www-form-urlencoded",
		"Authorization": "APPCODE " + o.appcode,
	})
	if err != nil {
		return
	}
	if len(bytes) == 0 {
		return result, errors.New("身份证识别接口异常")
	}
	var res retIdCard
	if err = json.Unmarshal(bytes, &res); err != nil {
		return
	}
	if res.Ret != 200 {
		return result, errors.New(res.Msg)
	}
	result = res.Data
	return
}

type retBankCard struct {
	Code    string `json:"code"`
	Message string `json:"message"`
	ReqNo   string `json:"reqNo"`
	BankCardInfo
}

type BankCardInfo struct {
	BankCardNo   string `json:"bankCardNo"`
	BankCardType string `json:"bankCardType"`
	BankName     string `json:"bankName"`
}

// BankCard @Title 银行卡识别
func (o *ocr) BankCard(pic string) (result BankCardInfo, err error) {
	bytes, err := request("POST", o.bankCardUrl, url.Values{
		"img": {pic},
	}.Encode(), map[string]string{
		"Content-Type":  "application/x-www-form-urlencoded",
		"Authorization": "APPCODE " + o.appcode,
		"X-Ca-Nonce":    uuid.NewV4().String(),
	})
	if err != nil {
		return
	}
	if len(bytes) == 0 {
		return result, errors.New("身份证识别接口异常")
	}
	var res retBankCard
	if err = json.Unmarshal(bytes, &res); err != nil {
		return
	}
	if res.Code != "FP00000" {
		return result, errors.New(res.Message)
	}
	result = res.BankCardInfo
	return
}
