package apple

import (
	"base/app/config"
	"crypto/ecdsa"
	"crypto/x509"
	"encoding/base64"
	"encoding/json"
	"encoding/pem"
	"errors"
	"github.com/dgrijalva/jwt-go"
	"io/ioutil"
	"net/http"
	"net/url"
	"strings"
	"time"
)

var Login = &login{}

type login struct {
	keyId           string
	appleSignSecret string
	teamId          string
	appId           string
	aud             string
	authTokenUrl    string
}

const (
	GRANTTYPEAUTHORIZATION = "authorization_code"
	GRANTTYPEREFRESHTOKEN  = "refresh_token"
)

type AuthTokenRes struct {
	Error            string `json:"error"`
	ErrorDescription string `json:"error_description"`
	ExpiresIn        int    `json:"expires_in"`
	TokenType        string `json:"token_type"`
	AccessToken      string `json:"access_token"`
	IdToken          string `json:"id_token"`
	RefreshToken     string `json:"refresh_token"`
}

type IdTokenHeaderData struct {
	Kid string
	Alg string
}

type IdTokenPreloadData struct {
	Iss            string
	Aud            string
	Exp            uint
	Iat            uint
	Sub            string
	AtHash         string
	Email          string
	EmailVerified  string
	IsPrivateEmail string
	AuthTime       uint
}

// Init @Title 初始化苹果登录
func (l login) Init(data config.AppleLogin) {
	Login.keyId = data.KeyId
	Login.appleSignSecret = data.AppleSignSecret
	Login.teamId = data.TeamId
	Login.appId = data.AppId
	Login.aud = data.Aud
	Login.authTokenUrl = data.AuthTokenUrl
}

// getAppleSecret @Title 生成jwt签名密钥
func (l *login) getAppleSecret() (string, error) {
	token := &jwt.Token{
		Header: map[string]interface{}{
			"alg": "ES256",
			"kid": l.keyId,
		},
		Claims: jwt.MapClaims{
			"iss": l.teamId,
			"iat": time.Now().Unix(),
			// constraint: exp - iat <= 180 days
			"exp": time.Now().Add(24 * time.Hour).Unix(),
			"aud": l.aud,
			"sub": l.appId,
		},
		Method: jwt.SigningMethodES256,
	}
	secret := "-----BEGIN PRIVATE KEY-----\n" + l.appleSignSecret + "\n-----END PRIVATE KEY-----"
	ecdsaKey, err := l.authKeyFromBytes([]byte(secret))
	if err != nil {
		return "", errors.New("密钥转换失败")
	}
	ss, signErr := token.SignedString(ecdsaKey)
	if signErr != nil {
		return "", errors.New("签名失败")
	}
	return ss, nil
}

// AuthKeyFromBytes @Title 转换签名密钥
func (l *login) authKeyFromBytes(key []byte) (*ecdsa.PrivateKey, error) {
	var block *pem.Block
	if block, _ = pem.Decode(key); block == nil {
		return nil, errors.New("读取p8密钥失败")
	}
	parsedKey, keyErr := x509.ParsePKCS8PrivateKey(block.Bytes)
	if keyErr != nil {
		return nil, keyErr
	}
	var pkey *ecdsa.PrivateKey
	var ok bool
	if pkey, ok = parsedKey.(*ecdsa.PrivateKey); !ok {
		return nil, errors.New("转换ecdsa密钥失败")
	}
	return pkey, nil
}

// AppleAuthToken @Title 获取苹果登录token
func (l *login) AppleAuthToken(grantType, code string) (*AuthTokenRes, error) {
	secret, secretErr := l.getAppleSecret()
	if secretErr != nil {
		return nil, secretErr
	}
	form := url.Values{}
	form.Set("client_id", l.appId)
	form.Set("client_secret", secret)
	form.Set("code", code)
	form.Set("grant_type", grantType)
	//form.Set("redirect_uri", '') //重定向url 用于web登录appleId 原生可不传
	request, err := http.NewRequest("POST", l.authTokenUrl, strings.NewReader(form.Encode()))
	if err != nil {
		return nil, err
	}
	request.Header.Set("Content-Type", "application/x-www-form-urlencoded")
	var response *http.Response
	if response, err = http.DefaultClient.Do(request); nil != err {
		return nil, err
	}
	defer response.Body.Close()
	data, readErr := ioutil.ReadAll(response.Body)
	if readErr != nil {
		return nil, err
	}
	res := &AuthTokenRes{}
	err = json.Unmarshal(data, res)
	if err != nil {
		return nil, err
	}
	if res.Error != "" {
		return nil, errors.New(res.ErrorDescription)
	}
	return res, nil
}

//// CheckTokenJwt 验证Token jwt签名
//func (l login) CheckTokenJwt(tokenString, kid string) (idTokenData IdTokenPreloadData, err error) {
//
//	secret, _ := l.GetPublicKey(kid)
//	token, err2 := jwt.Parse(tokenString, func(token *jwt.Token) (interface{}, error) {
//		if _, ok := token.Method.(*jwt.SigningMethodRSA); !ok {
//			return nil, errors.New("错误的签名方式")
//		}
//		return []byte(secret), nil
//	})
//	if err2 != nil {
//		log.Println(err2.Error())
//		err = errors.New("验签失败")
//		return
//	}
//	if claims, ok := token.Claims.(jwt.MapClaims); ok && token.Valid {
//		log.Println(claims["iss"])
//		return
//	} else {
//		err = errors.New("验签失败")
//		return
//	}
//}

//// GetPublicKey @Title 获取公钥
//func (l login) GetPublicKey(kid string) (string, error) {
//	return "", nil
//}

// ParseToken @Title 解析Token
func (l *login) ParseToken(token string) (header IdTokenHeaderData, preload IdTokenPreloadData, err error) {
	str := strings.Split(token, ".")
	headerByte, _ := base64.RawStdEncoding.DecodeString(str[0])
	preloadByte, _ := base64.RawStdEncoding.DecodeString(str[1])
	headerErr := json.Unmarshal(headerByte, &header)
	preloadErr := json.Unmarshal(preloadByte, &preload)
	if headerErr != nil || preloadErr != nil {
		err = errors.New("解析失败")
		return
	}
	return
}

func (l *login) CheckIdentify(data IdTokenPreloadData) error {
	if data.Iss != "https://appleid.apple.com" {
		return errors.New("校验不通过")
	}
	if data.Aud != l.appId {
		return errors.New("校验不通过")
	}
	iatTime := time.Unix(int64(data.Iat), 0)
	expTime := time.Unix(int64(data.Exp), 0)
	if iatTime.After(expTime) {
		return errors.New("token已过期")
	}
	return nil
}
