package che300

import (
	"encoding/json"
	"errors"
	"fmt"
	"github.com/shopspring/decimal"
	"github.com/tidwall/gjson"
	"net/url"
)

const (
	PriceConditionExcellent = "excellent" // 车况优秀
	PriceConditionGood      = "good"      // 车况良好
	PriceConditionNormal    = "normal"    // 车况一般
)

var Api = &api{}

type api struct {
	baseUrl string
	token   string
}

// InitChe300 @Title 初始化车300接口
func InitChe300(baseUrl, token string) {
	Api.baseUrl = baseUrl
	Api.token = token
}

type resStatus struct {
	Status   decimal.Decimal `json:"status"`
	ErrorMsg string          `json:"error_msg"`
}

type export struct {
	resStatus
	Data ExportInfo `json:"data"`
	Url  string     `json:"url"`
}
type ExportInfo struct {
	Brand       []exportBrand       `json:"brand"`
	Series      []exportSeries      `json:"series"`
	SeriesGroup []exportSeriesGroup `json:"series_group"`
	Model       []exportModel       `json:"model"`
	Area        []exportArea        `json:"area"`
	Version     []exportVersion     `json:"version"`
}
type exportBrand struct {
	BrandId      string `json:"brand_id"`
	BrandName    string `json:"brand_name"`
	BrandInitial string `json:"brand_initial"`
	OperType     string `json:"oper_type"`
}
type exportSeries struct {
	BrandId         string `json:"brand_id"`
	BrandName       string `json:"brand_name"`
	SeriesId        string `json:"series_id"`
	SeriesName      string `json:"series_name"`
	SeriesGroupName string `json:"series_group_name"`
	LevelName       string `json:"level_name"`
	MakerType       string `json:"maker_type"`
	SeriesGroupId   string `json:"series_group_id"`
	OperType        string `json:"oper_type"`
}
type exportSeriesGroup struct {
	SeriesGroupId   string `json:"series_group_id"`
	BrandId         string `json:"brand_id"`
	BrandName       string `json:"brand_name"`
	SeriesGroupName string `json:"series_group_name"`
	OperType        string `json:"oper_type"`
}
type exportModel struct {
	BrandId           string `json:"brand_id"`
	BrandName         string `json:"brand_name"`
	SeriesId          string `json:"series_id"`
	SeriesName        string `json:"series_name"`
	SeriesGroupName   string `json:"series_group_name"`
	ModelId           string `json:"model_id"`
	ModelName         string `json:"model_name"`
	Price             string `json:"price"`
	Liter             string `json:"liter"`
	GearType          string `json:"gear_type"`
	ModelYear         string `json:"model_year"`
	MakerType         string `json:"maker_type"`
	DischargeStandard string `json:"discharge_standard"`
	SeatNumber        string `json:"seat_number"`
	MinRegYear        string `json:"min_reg_year"`
	MaxRegYear        string `json:"max_reg_year"`
	SeriesGroupId     string `json:"series_group_id"`
	OperType          string `json:"oper_type"`
}
type exportArea struct {
	ProvId   string `json:"prov_id"`
	ProvName string `json:"prov_name"`
	CityId   string `json:"city_id"`
	CityName string `json:"city_name"`
}
type exportVersion struct {
	BrandTimestamp  string `json:"BrandTimestamp"`
	SeriesTimestamp string `json:"SeriesTimestamp"`
	ModelTimestamp  string `json:"ModelTimestamp"`
	CurrentVersion  string `json:"CurrentVersion"`
	InitialVersion  string `json:"InitialVersion"`
}

// Export @Title 导出车型数据
func (a *api) Export(version string) (res ExportInfo, err error) {
	var resExport export
	result, err := a.exec("/service/common/eval", url.Values{
		"oper":        {"exportModel"},
		"fromVersion": {version},
	})
	if err != nil {
		return res, err
	}
	if err := json.Unmarshal(result, &resExport); err != nil {
		return res, err
	}
	if err := a.verRes(resExport.resStatus); err != nil {
		return res, err
	}
	return resExport.Data, nil
}

type brand struct {
	resStatus
	BrandList []BrandItem `json:"brand_list"`
	Url       string      `json:"url"`
}

type BrandItem struct {
	BrandId    string `json:"brand_id"`
	BrandName  string `json:"brand_name"`
	Initial    string `json:"initial"`
	UpdateTime string `json:"update_time"`
}

// Brand @Title 获取品牌
func (a *api) Brand() ([]BrandItem, error) {
	var resBrand brand
	result, err := a.exec("/service/getCarBrandList", url.Values{})
	if err != nil {
		return nil, err
	}
	if err := json.Unmarshal(result, &resBrand); err != nil {
		return nil, err
	}
	if err := a.verRes(resBrand.resStatus); err != nil {
		return nil, err
	}
	return resBrand.BrandList, nil
}

type series struct {
	resStatus
	SeriesList []SeriesItem `json:"series_list"`
	Url        string       `json:"url"`
}

type SeriesItem struct {
	SeriesId        string `json:"series_id"`
	SeriesName      string `json:"series_name"`
	MakerType       string `json:"maker_type"`
	SeriesGroupName string `json:"series_group_name"`
	UpdateTime      string `json:"update_time"`
}

// Series @Title 获取车系
func (a *api) Series(brandId uint) ([]SeriesItem, error) {
	var resSeries series
	result, err := a.exec("/service/getCarSeriesList", url.Values{"brandId": {fmt.Sprintf("%d", brandId)}})
	if err != nil {
		return nil, err
	}
	if err := json.Unmarshal(result, &resSeries); err != nil {
		return nil, err
	}
	if err := a.verRes(resSeries.resStatus); err != nil {
		return nil, err
	}
	return resSeries.SeriesList, nil
}

type model struct {
	resStatus
	Model []Model `json:"model"`
}

type Model struct {
	Id         string          `json:"id"`
	Name       string          `json:"name"`
	RetainStar decimal.Decimal `json:"retain_star"`
	Liter      string          `json:"liter"`
	Gear       string          `json:"gear"`
	Price      string          `json:"price"`
	Discharge  string          `json:"discharge"`
	Year       string          `json:"year"`
	MinRegYear string          `json:"min_reg_year"`
	MaxRegYear string          `json:"max_reg_year"`
	UpdateTime string          `json:"update_time"`
}

// Model @Title 获取车型
func (a *api) Model(modelId uint) ([]Model, error) {
	var resModel model
	result, err := a.exec("/service/getCarModelInfo", url.Values{"modelId": {fmt.Sprintf("%d", modelId)}})
	if err != nil {
		return nil, err
	}
	if err := json.Unmarshal(result, &resModel); err != nil {
		return nil, err
	}
	if err := a.verRes(resModel.resStatus); err != nil {
		return nil, err
	}
	return resModel.Model, nil
}

// ModelInfo @Title 获取车型详情
func (a *api) ModelInfo(modelId uint) (resGjson gjson.Result, err error) {
	var res resStatus
	result, err := a.exec("/service/getModelParameters", url.Values{"modelId": {fmt.Sprintf("%d", modelId)}})
	if err != nil {
		return resGjson, err
	}
	if err = json.Unmarshal(result, &res); err != nil {
		return resGjson, err
	}
	if err = a.verRes(res); err != nil {
		return resGjson, err
	}
	resGjson = gjson.ParseBytes(result).Get("paramters")
	return resGjson, nil
}

type modelInfo struct {
	resStatus
	ModelInfo []ModelInfoItem `json:"modelInfo"`
}
type ModelInfoItem struct {
	SeriesGroupName       string  `json:"series_group_name"`
	Color                 string  `json:"color"`
	ModelLiter            string  `json:"model_liter"`
	ModelYear             int     `json:"model_year"`
	BrandName             string  `json:"brand_name"`
	ModelId               int     `json:"model_id"`
	BrandId               int     `json:"brand_id"`
	SeriesId              int     `json:"series_id"`
	ModelName             string  `json:"model_name"`
	ModelPrice            float64 `json:"model_price"`
	ModelEmissionStandard string  `json:"model_emission_standard"`
	ModelGear             string  `json:"model_gear"`
	SeriesName            string  `json:"series_name"`
	MinRegYear            int     `json:"min_reg_year"`
	MaxRegYear            int     `json:"max_reg_year"`
	ExtModelId            int     `json:"ext_model_id"`
}

// VinModel @Title 根据vin获取车型
func (a *api) VinModel(vin string) ([]ModelInfoItem, error) {
	var resModelInfo modelInfo
	result, err := a.exec("/service/identifyModelByVIN", url.Values{"vin": {vin}})
	if err != nil {
		return nil, err
	}
	if err = json.Unmarshal(result, &resModelInfo); err != nil {
		return nil, err
	}
	if err = a.verRes(resModelInfo.resStatus); err != nil {
		return nil, err
	}
	return resModelInfo.ModelInfo, nil
}

type resPriceData struct {
	resStatus
	PriceInfo
}

type Price struct {
	Condition              string  `json:"condition"`
	DealerLowBuyPrice      float64 `json:"dealer_low_buy_price"`
	DealerBuyPrice         float64 `json:"dealer_buy_price"`
	IndividualLowSoldPrice float64 `json:"individual_low_sold_price"`
	IndividualPrice        float64 `json:"individual_price"`
	DealerLowSoldPrice     float64 `json:"dealer_low_sold_price"`
	DealerPrice            float64 `json:"dealer_price"`
	DealerHighSoldPrice    float64 `json:"dealer_high_sold_price"`
}
type PriceInfo struct {
	DefaultCarCondition string  `json:"default_car_condition"`
	EvalPrices          []Price `json:"eval_prices"`
	DischargeStandard   string  `json:"discharge_standard"`
	ModelPrice          string  `json:"model_price"`
	DetailReportUrl     string  `json:"detail_report_url"`
}

// Price @Title 车辆估价
func (a *api) Price(modelId, cityId uint, regDate string, mile decimal.Decimal) (PriceInfo, error) {
	var resPrice resPriceData
	result, err := a.exec("/service/getUsedCarPrice", url.Values{
		"modelId":  {fmt.Sprintf("%d", modelId)},
		"regDate":  {regDate},
		"zone":     {fmt.Sprintf("%d", cityId)},
		"allLevel": {"1"},
		"mile":     {mile.String()},
	})
	if err != nil {
		return PriceInfo{}, err
	}
	//result = []byte(`{"status":"1","default_car_condition":"good","eval_prices":[{"condition":"excellent","dealer_low_buy_price":11.71,"dealer_buy_price":12.29,"individual_low_sold_price":12.75,"individual_price":13.01,"dealer_low_sold_price":13.12,"dealer_price":13.42,"dealer_high_sold_price":14.11},{"condition":"good","dealer_low_buy_price":11.02,"dealer_buy_price":11.57,"individual_low_sold_price":11.99,"individual_price":12.24,"dealer_low_sold_price":12.34,"dealer_price":12.62,"dealer_high_sold_price":13.28},{"condition":"normal","dealer_low_buy_price":10.34,"dealer_buy_price":10.86,"individual_low_sold_price":11.26,"individual_price":11.49,"dealer_low_sold_price":11.59,"dealer_price":11.85,"dealer_high_sold_price":12.46}],"discharge_standard":"国5","model_price":"31.28","detail_report_url":"https://www.ceshi.che300.com/pinggu/v12c42m24456r2017-01g26?from=yunyunwenche&_s=02a202f5372d3402"}`)
	if err = json.Unmarshal(result, &resPrice); err != nil {
		return PriceInfo{}, err
	}
	if err = a.verRes(resPrice.resStatus); err != nil {
		return PriceInfo{}, err
	}
	return resPrice.PriceInfo, nil
}

// @Title 调用接口
func (a *api) exec(action string, data url.Values) ([]byte, error) {
	data.Add("token", a.token)
	return request("GET", a.baseUrl+action+"?"+data.Encode(), "")
}

// @Title 验证返回数据
func (a *api) verRes(res resStatus) error {
	if res.Status.String() != "1" {
		return errors.New(res.ErrorMsg)
	}
	return nil
}
