package opensearch

import (
	"base/app/lib/mapstructure"
	"bytes"
	"context"
	tls2 "crypto/tls"
	"encoding/json"
	"errors"
	"github.com/opensearch-project/opensearch-go"
	"github.com/opensearch-project/opensearch-go/opensearchapi"
	"io"
	"net/http"
)

var Api = &api{}

type api struct {
	Client *opensearch.Client
}

// InitOpenSearch @Title 初始化openSearch
func InitOpenSearch(addresses []string, tls bool, username string, password string) error {
	client, err := opensearch.NewClient(opensearch.Config{
		Transport: &http.Transport{
			TLSClientConfig: &tls2.Config{InsecureSkipVerify: tls},
		},
		Addresses: addresses,
		Username:  username,
		Password:  password,
	})
	if err != nil {
		return err
	}
	Api.Client = client
	return nil
}

// DeleteIndex @Title 删除索引
func (a *api) DeleteIndex(index []string) error {
	if _, err := a.exec(opensearchapi.IndicesDeleteRequest{
		Index: index,
	}); err != nil {
		return err
	}
	return nil
}

// CreateIndex @Title 创建索引
func (a *api) CreateIndex(index string, body io.Reader) error {
	if _, err := a.exec(opensearchapi.IndicesCreateRequest{
		Index: index,
		Body:  body,
	}); err != nil {
		return err
	}
	return nil
}

// PushDoc @Title 索引文档
func (a *api) PushDoc(index, docId string, body io.Reader) error {
	if _, err := a.exec(opensearchapi.IndexRequest{
		Index:      index,
		DocumentID: docId,
		Body:       body,
	}); err != nil {
		return err
	}
	return nil
}

// DeleteDoc @Title 删除索引文档
func (a *api) DeleteDoc(index, docId string) error {
	if _, err := a.exec(opensearchapi.DeleteRequest{
		Index:      index,
		DocumentID: docId,
	}); err != nil {
		return err
	}
	return nil
}

// Reindex @Title 索引数据
func (a *api) Reindex(sourceIndex, destIndex string) error {
	body, _ := json.Marshal(map[string]map[string]string{
		"source": {
			"index": sourceIndex,
		},
		"dest": {
			"index": destIndex,
		},
	})
	if _, err := a.exec(opensearchapi.ReindexRequest{
		Body: bytes.NewReader(body),
	}); err != nil {
		return err
	}
	return nil
}

// GetAlias @Title 获取别名
func (a *api) GetAlias(Indexes []string) (alias map[string]interface{}, err error) {
	resStr, err := a.exec(opensearchapi.IndicesGetAliasRequest{
		Index: Indexes,
	})
	if err != nil {
		return
	}

	json.Unmarshal([]byte(resStr), &alias)
	return
}

// SetAlias @Title 设置别名
func (a *api) SetAlias(body io.Reader) error {
	_, err := a.exec(opensearchapi.IndicesUpdateAliasesRequest{
		Body: body,
	})
	if err != nil {
		return err
	}
	return nil
}

type resultTask struct {
	Task string `json:"task"`
}

// UpdateIndex @Title 更新索引
func (a *api) UpdateIndex(Indexes []string, conflicts string, waitForCompletion bool) (string, error) {
	result, err := a.exec(opensearchapi.UpdateByQueryRequest{
		Index:             Indexes,
		Conflicts:         conflicts,
		WaitForCompletion: &waitForCompletion,
	})
	if err != nil {
		return "", err
	}
	task := resultTask{}
	json.Unmarshal([]byte(result), &task)
	return task.Task, nil
}

type taskInfo struct {
	Completed bool `json:"completed"`
	Task      struct {
		Node   string `json:"node"`
		Id     int    `json:"id"`
		Type   string `json:"type"`
		Action string `json:"action"`
		Status struct {
			Total            int `json:"total"`
			Updated          int `json:"updated"`
			Created          int `json:"created"`
			Deleted          int `json:"deleted"`
			Batches          int `json:"batches"`
			VersionConflicts int `json:"version_conflicts"`
			Noops            int `json:"noops"`
			Retries          struct {
				Bulk   int `json:"bulk"`
				Search int `json:"search"`
			} `json:"retries"`
			ThrottledMillis      int     `json:"throttled_millis"`
			RequestsPerSecond    float64 `json:"requests_per_second"`
			ThrottledUntilMillis int     `json:"throttled_until_millis"`
		} `json:"status"`
		Description        string `json:"description"`
		StartTimeInMillis  int64  `json:"start_time_in_millis"`
		RunningTimeInNanos int64  `json:"running_time_in_nanos"`
		Cancellable        bool   `json:"cancellable"`
		Cancelled          bool   `json:"cancelled"`
		Headers            struct {
		} `json:"headers"`
	} `json:"task"`
	Response struct {
		Took             int  `json:"took"`
		TimedOut         bool `json:"timed_out"`
		Total            int  `json:"total"`
		Updated          int  `json:"updated"`
		Created          int  `json:"created"`
		Deleted          int  `json:"deleted"`
		Batches          int  `json:"batches"`
		VersionConflicts int  `json:"version_conflicts"`
		Noops            int  `json:"noops"`
		Retries          struct {
			Bulk   int `json:"bulk"`
			Search int `json:"search"`
		} `json:"retries"`
		Throttled            string        `json:"throttled"`
		ThrottledMillis      int           `json:"throttled_millis"`
		RequestsPerSecond    float64       `json:"requests_per_second"`
		ThrottledUntil       string        `json:"throttled_until"`
		ThrottledUntilMillis int           `json:"throttled_until_millis"`
		Failures             []interface{} `json:"failures"`
	} `json:"response"`
}

// GetTask @Title 获取任务
func (a *api) GetTask(taskId string) (info taskInfo, err error) {
	result, err := a.exec(opensearchapi.TasksGetRequest{
		TaskID: taskId,
	})
	if err != nil {
		return
	}
	json.Unmarshal([]byte(result), &info)
	return
}

type refreshItem struct {
	Shards struct {
		Total      int `json:"total"`
		Successful int `json:"successful"`
		Failed     int `json:"failed"`
	} `json:"_shards"`
}

// IndicesRefresh @Title 索引刷新
func (a *api) IndicesRefresh(Indexes []string) (result refreshItem, err error) {
	resStr, err := a.exec(opensearchapi.IndicesRefreshRequest{
		Index: Indexes,
	})
	if err != nil {
		return
	}
	json.Unmarshal([]byte(resStr), &result)
	return
}

type searchRes struct {
	Took     int  `json:"took"`
	TimedOut bool `json:"timed_out"`
	Shards   struct {
		Total      int `json:"total"`
		Successful int `json:"successful"`
		Skipped    int `json:"skipped"`
		Failed     int `json:"failed"`
	} `json:"_shards"`
	Hits struct {
		Total struct {
			Value    int    `json:"value"`
			Relation string `json:"relation"`
		} `json:"total"`
		MaxScore float64                  `json:"max_score"`
		Hits     []map[string]interface{} `json:"hits"`
	} `json:"hits"`
}
type searchResult struct {
	Total    int     `json:"total"`
	MaxScore float64 `json:"max_score"`
}

// Search @Title 搜索
func (a *api) Search(indexes []string, body io.Reader, data interface{}) (result searchResult, err error) {
	resStr, err := a.exec(opensearchapi.SearchRequest{
		Index: indexes,
		Body:  body,
	})
	res := searchRes{}
	json.Unmarshal([]byte(resStr), &res)
	mapstructure.WeakDecode(res.Hits.Hits, &data)
	result = searchResult{
		Total:    res.Hits.Total.Value,
		MaxScore: res.Hits.MaxScore,
	}
	return
}

type analyzerResult struct {
	Tokens []struct {
		Token       string `json:"token"`
		StartOffset int    `json:"start_offset"`
		EndOffset   int    `json:"end_offset"`
		Type        string `json:"type"`
		Position    int    `json:"position"`
	} `json:"tokens"`
}

// Analyzer @Title 分词测试
func (a *api) Analyzer(analyzer, text string) (result analyzerResult, err error) {
	marshal, _ := json.Marshal(map[string]string{
		"analyzer": analyzer,
		"text":     text,
	})
	resStr, err := a.exec(opensearchapi.IndicesAnalyzeRequest{
		Body: bytes.NewReader(marshal),
	})
	json.Unmarshal([]byte(resStr), &result)
	return
}

// CreateIndicesMapping @Title 添加索引字段
func (a *api) CreateIndicesMapping(indices []string, body io.Reader) error {
	//marshal, _ := json.Marshal(&body)
	if _, err := a.exec(opensearchapi.IndicesPutMappingRequest{
		Index: indices,
		Body:  body,
	}); err != nil {
		return err
	}
	return nil
}

// @Title 错误处理
func (a *api) exec(request opensearchapi.Request) (result string, err error) {
	response, err := request.Do(context.Background(), a.Client)
	if err != nil {
		return
	}
	all, err := io.ReadAll(response.Body)
	if response.IsError() {
		if err != nil {
			return
		}
		return result, errors.New(string(all))
	}
	result = string(all)
	return
}
