package business

import (
	"base/app/common"
	"base/app/model"
	"errors"
	"fmt"
	"git.oa00.com/go/mysql"
	"github.com/shopspring/decimal"
	"gorm.io/gorm"
)

var StaffLogic = &staffLogic{}

type staffLogic struct {
}

type staffAllItem struct {
	Id          uint   `json:"id"`
	Name        string `json:"name"`
	Phone       string `json:"phone"`
	AuditStatus uint   `json:"auditStatus"`
	RoleId      uint   `json:"roleId"`
	RoleName    string `json:"roleName"`
	Gender      uint   `json:"gender"`
	BrokerId    uint   `json:"brokerId"`
}

type StaffAllSearch struct {
	Name   string
	Phone  string
	Status uint `binding:"oneof=0 1 2 3" label:"状态"` // 1=审核中 2=审核通过 3=审核拒绝
}

// All @Title 全部员工
func (s *staffLogic) All(businessId uint, searchData StaffAllSearch) (result []staffAllItem) {
	result = []staffAllItem{}
	var staffs []model.BrokerBusinessStoreStaff
	where := mysql.Db.Where("business_id in (?)", businessId)
	if searchData.Name != "" {
		where.Where("name like ?", fmt.Sprintf("%%%s%%", searchData.Name))
	}
	if searchData.Status > 0 {
		where = where.Where("audit_id in (?)", mysql.Db.Model(&model.BrokerBusinessStoreStaffAudit{}).Where("status = ?", searchData.Status).Select("id"))
	}
	mysql.Db.Preload("Role").Preload("Audit").Where(where).Find(&staffs)

	for _, staff := range staffs {
		result = append(result, staffAllItem{
			Id:          staff.Id,
			Name:        staff.Name,
			Phone:       staff.Phone,
			Gender:      staff.Gender,
			AuditStatus: staff.Audit.Status,
			RoleId:      staff.RoleId,
			RoleName:    staff.Role.Name,
			BrokerId:    staff.BrokerId,
		})
	}
	return
}

type StaffAddData struct {
	Name       string          `binding:"required" label:"姓名"`
	Gender     uint            `binding:"required,oneof=1 2" label:"性别"`
	Phone      string          `binding:"required" label:"手机号"`
	StoreId    uint            `label:"门店"`
	RoleId     uint            `binding:"required" label:"权限"`
	Commission decimal.Decimal `binding:"required" label:"提成"`
}

// Add @Title 添加员工
func (s *staffLogic) Add(roleId, storeId, businessId uint, data StaffAddData) error {
	switch roleId {
	case model.BrokerRoleBusinessOwner:
		var businessStoreIds []uint
		mysql.Db.Model(&model.BrokerBusinessStore{}).Where("business_id = ?", businessId).Select("id").Find(&businessStoreIds)
		if !common.InArray(data.StoreId, businessStoreIds) {
			return errors.New("选择门店错误")
		}
	case model.BrokerRoleStoreOwner:
		if storeId != data.StoreId {
			return errors.New("选择门店错误")
		}
	}

	broker := model.Broker{Phone: data.Phone}
	mysql.Db.Where(&broker).First(&broker)
	if broker.Id > 0 && broker.Level != model.BrokerLevelBase {
		return errors.New("手机号已使用")
	}
	if !common.InArray(data.RoleId, model.BrokerAbilityBusinessStaff) {
		return errors.New("员工权限错误")
	}
	storeStaff := model.BrokerBusinessStoreStaff{
		Name:       data.Name,
		Gender:     data.Gender,
		Phone:      data.Phone,
		BusinessId: businessId,
		StoreId:    data.StoreId,
		RoleId:     data.RoleId,
		Commission: data.Commission,
		Audit: model.BrokerBusinessStoreStaffAudit{
			Status: model.BrokerBusinessStoreStaffAuditStatusSubmit,
		},
	}
	if mysql.Db.Where("phone = ?", storeStaff.Phone).First(&model.BrokerBusinessStoreStaff{}).Error == nil {
		return errors.New("请勿重复添加")
	}
	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		if tx.Create(&storeStaff).Error != nil {
			return errors.New("添加失败")
		}
		if tx.Model(&storeStaff.Audit).UpdateColumn("staff_id", storeStaff.Id).Error != nil {
			return errors.New("添加失败")
		}
		return nil
	})
}

type staffInfo struct {
	Id          uint            `json:"id"`
	Name        string          `json:"name"`
	Gender      uint            `json:"gender"`
	Phone       string          `json:"phone"`
	StoreId     uint            `json:"storeId"`
	StoreName   string          `json:"storeName"`
	RoleId      uint            `json:"roleId"`
	RoleName    string          `json:"roleName"`
	Commission  decimal.Decimal `json:"commission"`
	Status      uint            `json:"status"`
	AuditStatus uint            `json:"auditStatus"`
}

// Info @Title 员工详情
func (s *staffLogic) Info(businessId, staffId uint) (result staffInfo, err error) {
	storeStaff := model.BrokerBusinessStoreStaff{Id: staffId, BusinessId: businessId}
	if mysql.Db.Preload("Store").Preload("Role").Preload("Audit").Where(&storeStaff).First(&storeStaff).Error != nil {
		return result, errors.New("员工信息错误")
	}
	result = staffInfo{
		Id:          storeStaff.Id,
		Name:        storeStaff.Name,
		Gender:      storeStaff.Gender,
		Phone:       storeStaff.Phone,
		StoreId:     storeStaff.StoreId,
		StoreName:   storeStaff.Store.Name,
		RoleId:      storeStaff.RoleId,
		RoleName:    storeStaff.Role.Name,
		Commission:  storeStaff.Commission,
		Status:      storeStaff.Status,
		AuditStatus: storeStaff.Audit.Status,
	}
	return
}

type StaffEdit struct {
	StaffId uint   `binding:"required" label:"员工"`
	Name    string `binding:"required" label:"姓名"`
	Gender  uint   `binding:"required,oneof=1 2" label:"性别"`
	//StoreId    uint            `binding:"required" label:"门店"`
	RoleId     uint            `binding:"required" label:"权限"`
	Commission decimal.Decimal `binding:"required" label:"提成"`
}

// Edit @Title 编辑员工
func (s *staffLogic) Edit(businessId uint, data StaffEdit) error {
	storeStaff := model.BrokerBusinessStoreStaff{Id: data.StaffId, BusinessId: businessId}
	if mysql.Db.Where(&storeStaff).First(&storeStaff).Error != nil {
		return errors.New("员工信息错误")
	}
	//if mysql.Db.Where("id = ? and business_id = ?", data.StoreId, storeStaff.BusinessId).First(&model.BrokerBusinessStore{}).Error != nil {
	//	return errors.New("部门信息错误")
	//}
	if mysql.Db.Where("id = ?", data.RoleId).First(&model.BrokerRole{}).Error != nil {
		return errors.New("权限信息错误")
	}
	storeStaff.Name = data.Name
	storeStaff.Gender = data.Gender
	//storeStaff.StoreId = data.StoreId
	storeStaff.RoleId = data.RoleId
	storeStaff.Commission = data.Commission
	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		if tx.Save(&storeStaff).Error != nil {
			return errors.New("编辑失败")
		}
		return nil
	})
}

// Delete @Title 删除员工
func (s *staffLogic) Delete(businessId uint, staffId uint) error {
	staff := model.BrokerBusinessStoreStaff{Id: staffId, BusinessId: businessId}
	if mysql.Db.Preload("Store").First(&staff).Error != nil {
		return errors.New("员工不存在")
	}
	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		if staff.Store.OwnerBrokerId == staff.BrokerId {
			if tx.Model(&model.BrokerBusinessStore{}).Where("id = ? ", staff.StoreId).Updates(map[string]interface{}{
				"owner_broker_id": 0,
			}).Error != nil {
				return errors.New("删除失败")
			}
		}
		if tx.Delete(&staff).Error != nil {
			return errors.New("删除失败")
		}
		if tx.Model(&model.Broker{}).Where("id = ? and business_id = ?", staff.BrokerId, staff.BusinessId).Updates(map[string]interface{}{
			"level":       model.BrokerLevelBase,
			"business_id": 0,
		}).Error != nil {
			return errors.New("删除失败")
		}
		return nil
	})
}
