package car

import (
	"base/app/constant"
	"base/app/lib/bean"
	"base/app/logic/broker/message"
	car2 "base/app/logic/car"
	"base/app/model"
	"database/sql"
	"errors"
	"fmt"
	"git.oa00.com/go/mysql"
	"github.com/shopspring/decimal"
	"gorm.io/gorm"
	"time"
)

var BaseLogic = &baseLogic{}

type baseLogic struct {
}

type carPhoto struct {
	Photo string `binding:"required" label:"图片地址"`
	Text  string `label:"图片描述"`
}

type publicPhotos struct {
	CarPhotos      []carPhoto `binding:"required,min=1" label:"车辆照片"`
	InteriorPhotos []carPhoto `binding:"required,min=1" label:"内饰照片"`
	DefectPhotos   []carPhoto `label:"缺陷照片"`
	DataPhotos     []carPhoto `binding:"required,min=1" label:"资料照片"`
}
type publicBaseInfo struct {
	Source                uint            `binding:"required,oneof=1 2" label:"车辆来源"` // 1=个人直卖 2=车商收购
	SourceId              uint            `label:"挂售车商id"`
	Type                  uint            `binding:"required" label:"车辆类型"`
	ModelId               uint            `binding:"required" label:"车型"`
	Vin                   string          `binding:"required" label:"车架号"`
	EngineNo              string          `binding:"required" label:"发动机号"`
	LicensingDate         string          `binding:"required" label:"上牌日期"`
	Color                 string          `binding:"required" label:"车身颜色"`
	InteriorColor         string          `label:"内饰颜色"`
	TemporaryLicensePlate string          `label:"临时车牌"`
	ParkingNo             string          `label:"车位编号"`
	StockStatus           uint            `binding:"oneof=0 1 2 3" label:"库存状态"`
	UseCharacter          uint            `binding:"required" label:"使用性质"`
	ShamMileage           decimal.Decimal `binding:"required" label:"虚假里程"`
	PurchaseTax           decimal.Decimal `label:"购置税"`
	InstallationCost      decimal.Decimal `label:"加装费用"`
	Location              uint            `binding:"required" label:"车辆所在地"`
	Attribution           uint            `binding:"required" label:"车辆归属地"`
	Condition             string          `binding:"required" label:"车况"`
	Remark                string          `label:"备注"`
}

type publicPriceInfo struct {
	InteriorPrice decimal.Decimal `binding:"required" label:"内部价格"`
	ExteriorPrice decimal.Decimal `binding:"required" label:"外部价格"`
}

type publicPurchaseInfo struct {
	Price   decimal.Decimal `label:"采购价格"`
	Date    string          `label:"采购日期"`
	Liaison string          `label:"采购人"`
}

type publicCertificateInfo struct {
	Transfer                 uint            `label:"过户次数"`
	KeyCount                 uint            `label:"钥匙数量"`
	CompulsoryInsurance      uint            `label:"交强险"`
	CompulsoryInsuranceDate  string          `label:"交强险到期日期"`
	CommercialInsurance      uint            `label:"商业险"`
	CommercialInsuranceDate  string          `label:"商业险到期日期"`
	CommercialInsurancePrice decimal.Decimal `label:"商业险金额"`
}

type PublishData struct {
	Photos          publicPhotos          `binding:"required" label:"照片"`
	BaseInfo        publicBaseInfo        `binding:"required" label:"基本信息"`
	PriceInfo       publicPriceInfo       `binding:"required" label:"价格信息"`
	PurchaseInfo    publicPurchaseInfo    `label:"采购信息"`
	CertificateInfo publicCertificateInfo `label:"牌证信息"`
}

// Publish @Title 申请发布车辆
func (b *baseLogic) Publish(brokerId, roleId, storeId uint, data PublishData) error {
	vinCheck := car2.CarLogic.VinCheck(data.BaseInfo.Vin)
	if vinCheck.IsLegal == 0 {
		return errors.New(vinCheck.Message)
	}
	if data.PriceInfo.ExteriorPrice.LessThan(data.PriceInfo.InteriorPrice) {
		return errors.New("外部价不能小于内部价")
	}

	location := model.Area{Id: data.BaseInfo.Location, Level: model.AreaLevelCity}
	if mysql.Db.First(&location).Error != nil {
		return errors.New("车辆所在地错误")
	}

	attribution := model.Area{Id: data.BaseInfo.Attribution, Level: model.AreaLevelCity}
	if mysql.Db.First(&attribution).Error != nil {
		return errors.New("车辆所在地错误")
	}
	carModel := model.CarModel{ModelId: data.BaseInfo.ModelId}
	if mysql.Db.First(&carModel).Error != nil {
		return errors.New("车型未找到")
	}

	// 入驻商车务可选择门店；其余车商默认本车商
	var dealerId uint
	if roleId == model.BrokerRoleBusinessWorker {
		dealerId = data.BaseInfo.SourceId
	} else {
		dealerId = storeId
	}
	store := model.BrokerBusinessStore{Id: dealerId}
	if mysql.Db.Where(&store).First(&store).Error != nil {
		return errors.New("门店id未找到")
	}

	carBase := model.CarBase{
		BrokerId:                 brokerId,
		DealerId:                 data.BaseInfo.SourceId,
		Source:                   data.BaseInfo.Source,
		CarType:                  data.BaseInfo.Type,
		ModelId:                  data.BaseInfo.ModelId,
		Vin:                      data.BaseInfo.Vin,
		EngineNo:                 data.BaseInfo.EngineNo,
		Color:                    data.BaseInfo.Color,
		InteriorColor:            data.BaseInfo.InteriorColor,
		TemporaryLicensePlate:    data.BaseInfo.TemporaryLicensePlate,
		ParkingNo:                data.BaseInfo.ParkingNo,
		StockStatus:              data.BaseInfo.StockStatus,
		UseCharacter:             data.BaseInfo.UseCharacter,
		ShamMileage:              data.BaseInfo.ShamMileage,
		PurchaseTax:              data.BaseInfo.PurchaseTax,
		InstallationCost:         data.BaseInfo.InstallationCost,
		LocationProvinceId:       location.ParentId,
		LocationCityId:           location.Id,
		AttributionProvinceId:    attribution.ParentId,
		AttributionCityId:        attribution.Id,
		Condition:                data.BaseInfo.Condition,
		MainPhoto:                data.Photos.CarPhotos[0].Photo,
		Remark:                   data.BaseInfo.Remark,
		InteriorPrice:            data.PriceInfo.InteriorPrice,
		ExteriorPrice:            data.PriceInfo.ExteriorPrice,
		PurchasePrice:            data.PurchaseInfo.Price,
		PurchaseLiaison:          data.PurchaseInfo.Liaison,
		Transfer:                 data.CertificateInfo.Transfer,
		KeyCount:                 data.CertificateInfo.KeyCount,
		CompulsoryInsurance:      data.CertificateInfo.CompulsoryInsurance,
		CommercialInsurance:      data.CertificateInfo.CommercialInsurance,
		CommercialInsurancePrice: data.CertificateInfo.CommercialInsurancePrice,
		TheUpper:                 model.CarBaseTheUpperFalse,
	}

	// 上牌日期
	{
		licensingDate, err := time.ParseInLocation("2006-01", data.BaseInfo.LicensingDate, time.Local)
		if err != nil {
			return errors.New("上牌日期时间错误")
		}
		carBase.LicensingDate = sql.NullTime{Time: licensingDate, Valid: true}
	}

	// 交强险到期日期
	if data.CertificateInfo.CompulsoryInsuranceDate != "" {
		compulsoryInsuranceDate, err := time.ParseInLocation("2006-01", data.CertificateInfo.CompulsoryInsuranceDate, time.Local)
		if err != nil {
			return errors.New("交强险到期时间错误")
		}
		carBase.CompulsoryInsuranceDate = sql.NullTime{Time: compulsoryInsuranceDate, Valid: true}
	}

	// 商业险到期日期
	if data.CertificateInfo.CommercialInsuranceDate != "" {
		commercialInsuranceDate, err := time.ParseInLocation("2006-01", data.CertificateInfo.CommercialInsuranceDate, time.Local)
		if err != nil {
			return errors.New("商业险到期日期时间错误")
		}
		carBase.CommercialInsuranceDate = sql.NullTime{Time: commercialInsuranceDate, Valid: true}
	}

	// 采购日期
	if data.PurchaseInfo.Date != "" {
		purchaseDate, err := time.ParseInLocation("2006-01-02", data.PurchaseInfo.Date, time.Local)
		if err != nil {
			return errors.New("采购日期时间错误")
		}
		carBase.PurchaseDate = sql.NullTime{Time: purchaseDate, Valid: true}
	}

	for _, photo := range data.Photos.CarPhotos {
		carBase.Photos = append(carBase.Photos, model.CarPhoto{
			Path: photo.Photo,
			Text: photo.Text,
			Type: model.CarPhotoTypeCar,
		})
	}
	for _, photo := range data.Photos.InteriorPhotos {
		carBase.Photos = append(carBase.Photos, model.CarPhoto{
			Path: photo.Photo,
			Text: photo.Text,
			Type: model.CarPhotoTypeInterior,
		})
	}
	for _, photo := range data.Photos.DefectPhotos {
		carBase.Photos = append(carBase.Photos, model.CarPhoto{
			Path: photo.Photo,
			Text: photo.Text,
			Type: model.CarPhotoTypeDefect,
		})
	}
	for _, photo := range data.Photos.DataPhotos {
		carBase.Photos = append(carBase.Photos, model.CarPhoto{
			Path: photo.Photo,
			Text: photo.Text,
			Type: model.CarPhotoTypeData,
		})
	}
	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		switch data.BaseInfo.Source {
		case model.CarBaseSourceConsignment: // 个人寄售
			consignment := model.OrderConsignment{Vin: data.BaseInfo.Vin, Status: model.OrderConsignmentStatusUnPublish}
			if tx.Where(&consignment).First(&consignment).Error != nil {
				return errors.New("车辆vin未找到")
			}
			carBase.CustomerId = consignment.CustomerId
			carBase.OrderId = consignment.Id
			if tx.Model(&consignment).Where(&consignment).UpdateColumn("status", model.OrderConsignmentStatusPublish).RowsAffected != 1 {
				return errors.New("寄售订单查找失败")
			}
		case model.CarBaseSourceDealerPurchase: // 车商收购
		}

		//  提交至车商审核
		if brokerId != store.OwnerBrokerId {
			carBase.Status = model.CarBaseStatusDealerAuditing
			carBase.DealerAudit = model.DealerAuditing{
				Type:          model.DealerAuditingTypePublish,
				DealerId:      store.Id,
				ApplyBrokerId: brokerId,
				Status:        constant.AuditStatusWaiting,
			}
			if tx.Create(&carBase).Error != nil {
				return errors.New("车辆信息发布失败")
			}
			carBase.DealerAudit.LinkId = carBase.Id
			if tx.Updates(&carBase.DealerAudit).Error != nil {
				return errors.New("车主审核生成失败")
			}
			message.MessageLogic.MessageToBroker(mysql.Db, []uint{carBase.BrokerId}, 0, model.BrokerMessageTypeCarAudit, message.MessageItem{
				Title:    "车辆发布提示",
				Content:  "您有一个新的车辆发布已提交至车商审批",
				LinkedId: carBase.Id,
				Extras: map[string]interface{}{
					"车辆名称": carModel.Name,
					"发布价格": fmt.Sprintf("内部价%s|外部价%s", carBase.InteriorPrice, carBase.ExteriorPrice),
					"发起时间": carBase.DealerAudit.CreatedAt.Format("2006-01-02 15:04:05"),
				},
			})
			message.MessageLogic.MessageToBroker(mysql.Db, []uint{store.OwnerBrokerId}, 0, model.BrokerMessageTypeCarAudit, message.MessageItem{
				Title:    "车辆发布提示",
				Content:  "您有一个新的车辆发布待审批",
				LinkedId: carBase.Id,
				Extras: map[string]interface{}{
					"车辆名称": carModel.Name,
					"发布价格": fmt.Sprintf("内部价%s|外部价%s", carBase.InteriorPrice, carBase.ExteriorPrice),
					"发起时间": carBase.DealerAudit.CreatedAt.Format("2006-01-02 15:04:05"),
				},
			})
			return nil
		}
		carBase.Status = model.CarBaseStatusAuditing
		carBase.Audit = model.CarAudit{
			Type:   model.CarAuditTypePublish,
			Status: constant.AuditStatusWaiting,
		}
		if tx.Create(&carBase).Error != nil {
			return errors.New("车辆信息发布失败")
		}
		carBase.Audit.CarBaseId = carBase.Id
		if tx.Updates(&carBase.Audit).Error != nil {
			return errors.New("车辆审核生成失败")
		}
		message.MessageLogic.MessageToBroker(mysql.Db, []uint{carBase.BrokerId}, 0, model.BrokerMessageTypeCarAudit, message.MessageItem{
			Title:    "车辆发布提示",
			Content:  "您有一个新的车辆发布已提交至后台审批",
			LinkedId: carBase.Id,
			Extras: map[string]interface{}{
				"车辆名称": carModel.Name,
				"发布价格": fmt.Sprintf("内部价%s|外部价%s", carBase.InteriorPrice, carBase.ExteriorPrice),
				"提交时间": carBase.Audit.CreatedAt.Format("2006-01-02 15:04:05"),
			},
		})
		return nil
	})
}

type carBaseItem struct {
	Id            uint            `json:"id"`
	Source        uint            `json:"source"`
	SourceName    string          `json:"sourceName"`
	MainPhoto     string          `json:"mainPhoto"`
	ModelName     string          `json:"modelName"`
	Mileage       decimal.Decimal `json:"mileage"`
	LicensingDate int64           `json:"licensingDate"`
	Status        uint            `json:"status"`
	StatusName    string          `json:"statusName"`
	BrokerId      uint            `json:"brokerId"`
	BrokerName    string          `json:"brokerName"`
	CustomerId    uint            `json:"customerId"`
	CustomerName  string          `json:"customerName"`
	DealerId      uint            `json:"dealerId"`
	DealerName    string          `json:"dealerName"`
	DealerAuditAt int64           `json:"dealerAuditAt"`
	AuditAt       int64           `json:"auditAt"`
	CreatedAt     int64           `json:"createdAt"`
}

type CarBaseSearch struct {
	Source           uint
	Status           uint
	ModelName        string
	CreatedTimeStart string
	CreatedTimeEnd   string
}

type CarBaseBrokerSearch struct {
	CustomerName string
	CustomerTel  string
	DealerName   string
	DealerTel    string
}

// BrokerLists @Title 发布列表
func (b *baseLogic) BrokerLists(brokerId uint, search CarBaseSearch, searchBroker CarBaseBrokerSearch, page bean.Page) (lists []carBaseItem, total int64) {
	return b.Lists([]uint{brokerId}, []uint{}, search, searchBroker, CarBaseDealerSearch{}, page)
}

type CarBaseDealerSearch struct {
	BrokerName string
	BrokerTel  string
}

// DealerLists @Title 车商审批车辆发布列表
func (b *baseLogic) DealerLists(dealerId uint, search CarBaseSearch, searchDealer CarBaseDealerSearch, page bean.Page) (lists []carBaseItem, total int64) {
	return b.Lists([]uint{}, []uint{dealerId}, search, CarBaseBrokerSearch{}, searchDealer, page)
}

// Lists @Title 发布列表
func (b *baseLogic) Lists(brokerIds, dealerIds []uint, search CarBaseSearch, searchBroker CarBaseBrokerSearch, searchDealer CarBaseDealerSearch, page bean.Page) (lists []carBaseItem, total int64) {
	lists = []carBaseItem{}
	var carBases []model.CarBase
	where := mysql.Db
	if len(brokerIds) > 0 {
		where = where.Where("broker_id in ?", brokerIds)
	}
	if len(dealerIds) > 0 {
		where = where.Where("dealer_id in ?", dealerIds)
	}
	if search.Source > 0 {
		where = where.Where("source = ?", search.Source)
	}
	if search.Status > 0 {
		where = where.Where("status = ?", search.Status)
	}
	if search.ModelName != "" {
		where = where.Where("model_id in (?)", mysql.Db.Model(&model.CarModel{}).Where("name like ?", fmt.Sprintf("%%%s%%", search.ModelName)).Select("model_id"))
	}
	if search.CreatedTimeStart != "" {
		where = where.Where("created_at > ?", search.CreatedTimeStart)
	}
	if search.CreatedTimeEnd != "" {
		where = where.Where("created_at < ?", search.CreatedTimeEnd)
	}
	{
		customerSearch := mysql.Db.Model(&model.Customer{}).Select("id")
		if searchBroker.CustomerName != "" {
			customerSearch = customerSearch.Where("nickname like ?", fmt.Sprintf("%%%s%%", searchBroker.CustomerName))
		}
		if searchBroker.CustomerTel != "" {
			customerSearch = customerSearch.Where("phone like ?", fmt.Sprintf("%%%s%%", searchBroker.CustomerTel))
		}
		if _, ok := customerSearch.Statement.Clauses["WHERE"]; ok {
			where = where.Where("customer_id in (?)", customerSearch)
		}
	}
	{
		dealerWhere := mysql.Db.Model(&model.BrokerBusinessStore{}).Select("id")
		if searchBroker.DealerName != "" {
			dealerWhere = dealerWhere.Where("name like ?", fmt.Sprintf("%%%s%%", searchBroker.DealerName))
		}
		if searchBroker.DealerTel != "" {
			dealerWhere = dealerWhere.Where("phone like ?", fmt.Sprintf("%%%s%%", searchBroker.DealerTel))
		}
		if _, ok := dealerWhere.Statement.Clauses["WHERE"]; ok {
			where = where.Where("dealer_id in (?)", dealerWhere)
		}
	}
	{
		brokerWhere := mysql.Db.Model(&model.Broker{}).Select("id")
		if searchDealer.BrokerName != "" {
			brokerWhere = brokerWhere.Where("nickname like ?", fmt.Sprintf("%%%s%%", searchDealer.BrokerName))
		}
		if searchDealer.BrokerTel != "" {
			brokerWhere = brokerWhere.Where("phone like ?", fmt.Sprintf("%%%s%%", searchDealer.BrokerTel))
		}
		if _, ok := brokerWhere.Statement.Clauses["WHERE"]; ok {
			where = where.Where("broker_id in (?)", brokerWhere)
		}
	}
	mysql.Db.Model(&carBases).Where(where).Count(&total)
	if page.HasPage(total) {
		mysql.Db.Preload("Model").Preload("DealerAudit").Preload("Audit").
			Preload("Broker").Preload("Customer").Preload("Dealer").
			Where(where).Order("id desc").Offset(page.GetStart()).Limit(page.GetLimit()).Find(&carBases)
		for _, item := range carBases {
			lists = append(lists, carBaseItem{
				Id:            item.Id,
				MainPhoto:     item.MainPhoto,
				ModelName:     item.Model.Name,
				Source:        item.Source,
				SourceName:    item.GetSourceName(item.Source),
				Mileage:       item.ShamMileage,
				LicensingDate: item.LicensingDate.Time.Unix(),
				Status:        item.Status,
				StatusName:    item.GetStatusName(item.Status),
				BrokerId:      item.BrokerId,
				BrokerName:    item.Broker.Nickname,
				DealerId:      item.DealerId,
				DealerName:    item.Dealer.Name,
				CustomerId:    item.CustomerId,
				CustomerName:  item.Customer.Nickname,
				DealerAuditAt: item.DealerAudit.AuditAt.Time.Unix(),
				AuditAt:       item.Audit.AuditAt.Time.Unix(),
				CreatedAt:     item.Audit.CreatedAt.Unix(),
			})
		}
	}
	return
}

type carBaseInfo struct {
	Id                    uint               `json:"id"`
	ModelId               uint               `json:"modelId"`
	ModelName             string             `json:"modelName"` // 车型名称
	Status                uint               `json:"status"`
	StatusName            string             `json:"statusName"`
	TheUpper              uint               `json:"theUpper"`
	TheUpperName          string             `json:"theUpperName"`
	Vin                   string             `json:"vin"`      // 车架号
	Transfer              uint               `json:"transfer"` // 过户次数
	Price                 decimal.Decimal    `json:"price"`
	LastPrice             decimal.Decimal    `json:"lastPrice"`   // 上次价格
	DownPayment           decimal.Decimal    `json:"downPayment"` // 首付
	Source                uint               `json:"source"`      // 车辆来源    1=个人直卖 2=车商 3=收购
	SourceName            string             `json:"sourceName"`
	Type                  string             `json:"type"`                  // 车辆类型
	EngineNo              string             `json:"engineNo"`              // 发动机号
	LicensingDate         int64              `json:"licensingDate"`         // 上牌日期
	Color                 string             `json:"color"`                 // 车身颜色
	InteriorColor         string             `json:"interiorColor"`         // 内饰颜色
	TemporaryLicensePlate string             `json:"temporaryLicensePlate"` // 临时车牌
	ParkingNo             string             `json:"parkingNo"`             // 车位编号
	StockStatus           uint               `json:"stockStatus"`           // 库存状态
	StockStatusName       string             `json:"stockStatusName"`
	UseCharacter          string             `json:"useCharacter"`     // 使用性质
	Mileage               decimal.Decimal    `json:"mileage"`          // 里程
	PurchaseTax           decimal.Decimal    `json:"purchaseTax"`      // 购置税
	InstallationCost      decimal.Decimal    `json:"installationCost"` // 加装费用
	LocationCityId        uint               `json:"locationCityId"`
	Location              string             `json:"location"` // 车辆所在地
	AttributionCityId     uint               `json:"attributionCityId"`
	Attribution           string             `json:"attribution"`    // 车辆归属地
	Condition             string             `json:"condition"`      // 车况
	Remark                string             `json:"remark"`         // 备注
	CarPhotos             []carPhotoInfo     `json:"carPhotos"`      // 车辆照片
	InteriorPhotos        []carPhotoInfo     `json:"interiorPhotos"` // 内饰照片
	DefectPhotos          []carPhotoInfo     `json:"defectPhotos"`   // 缺陷照片
	DataPhotos            []carPhotoInfo     `json:"dataPhotos"`     // 资料照片
	BrokerInfo            brokerInfo         `json:"brokerInfo"`     // 发布经纪人信息
	DealerInfo            dealerInfo         `json:"dealerInfo"`     // 所属门店信息
	ModelInfo             car2.ModelInfo     `json:"modelInfo"`
	PriceInfo             priceInfo          `json:"priceInfo"`
	PurchaseInfo          purchaseInfo       `json:"purchaseInfo"`
	CertificateInfo       certificateInfo    `json:"certificateInfo"`
	ContractMasterInfo    contractMasterInfo `json:"contractMasterInfo"`
	DealerAuditAt         int64              `json:"dealerAuditAt"`
	DealerRejectReason    string             `json:"dealerRejectReason"`
	AuditAt               int64              `json:"auditAt"`
	RejectReason          string             `json:"rejectReason"`
	CreatedAt             int64              `json:"createdAt"`
}

type carPhotoInfo struct {
	Photo string `json:"photo"`
	Text  string `json:"text"`
}

type brokerInfo struct {
	BrokerId       uint   `json:"brokerId"`
	BrokerNickname string `json:"brokerNickname"`
	BrokerHeadImg  string `json:"brokerHeadImg"`
	BrokerPhone    string `json:"brokerPhone"`
}

type dealerInfo struct {
	Id              uint   `json:"id"`
	Name            string `json:"name"`
	OwnerBrokerId   uint   `json:"ownerBrokerId"`
	OwnerBrokerName string `json:"ownerBrokerName"`
	OwnerBrokerTel  string `json:"ownerBrokerTel"`
}

type priceInfo struct {
	InteriorPrice decimal.Decimal `json:"interiorPrice"` //  内部价格
	ExteriorPrice decimal.Decimal `json:"exteriorPrice"` // 外部价格
}

type purchaseInfo struct {
	Price   decimal.Decimal `json:"price"`   // 采购价格
	Date    int64           `json:"date"`    // 采购日期
	Liaison string          `json:"liaison"` // 采购人
}

type certificateInfo struct {
	Transfer                 uint            `json:"transfer"`                 // 过户次数
	KeyCount                 uint            `json:"keyCount"`                 // 钥匙数量
	CompulsoryInsurance      uint            `json:"compulsoryInsurance"`      // 交强险
	CompulsoryInsuranceDate  int64           `json:"compulsoryInsuranceDate"`  // 交强险到期日期
	CommercialInsurance      uint            `json:"commercialInsurance"`      // 商业险
	CommercialInsuranceDate  int64           `json:"commercialInsuranceDate"`  // 商业险到期日期
	CommercialInsurancePrice decimal.Decimal `json:"commercialInsurancePrice"` // 商业险金额
}

type contractMasterInfo struct {
	Name     string `json:"name"`     // 姓名
	IdCard   string `json:"idCard"`   // 身份证号
	Phone    string `json:"phone"`    // 手机号
	BankCard string `json:"bankCard"` // 银行卡号
	Bank     string `json:"bank"`     // 开户行
}

// Info @Title 详情
func (b *baseLogic) Info(carBaseId uint) (result carBaseInfo, err error) {
	carBase := model.CarBase{Id: carBaseId}
	if mysql.Db.
		Preload("Broker").
		Preload("Model").
		Preload("LocationCity").
		Preload("AttributionCity").
		Preload("Photos").
		Preload("Dealer.OwnerBroker").
		Preload("DealerAudit").
		Preload("Audit").
		First(&carBase).Error != nil {
		return result, errors.New("车辆错误")
	}
	result = carBaseInfo{
		Id:                    carBase.Id,
		Status:                carBase.Status,
		StatusName:            carBase.GetStatusName(carBase.Status),
		TheUpper:              carBase.TheUpper,
		TheUpperName:          carBase.GetTheUpperName(carBase.TheUpper),
		ModelId:               carBase.ModelId,
		ModelName:             carBase.Model.Name,
		Vin:                   carBase.Vin,
		Transfer:              carBase.Transfer,
		Source:                carBase.Source,
		SourceName:            carBase.GetSourceName(carBase.Source),
		Type:                  constant.GetCarTypeName(carBase.CarType),
		DealerAuditAt:         carBase.DealerAudit.AuditAt.Time.Unix(),
		DealerRejectReason:    carBase.DealerAudit.Reason,
		AuditAt:               carBase.Audit.AuditAt.Time.Unix(),
		RejectReason:          carBase.Audit.Reason,
		CreatedAt:             carBase.CreatedAt.Unix(),
		EngineNo:              carBase.EngineNo,
		LicensingDate:         carBase.LicensingDate.Time.Unix(),
		Color:                 carBase.Color,
		InteriorColor:         carBase.InteriorColor,
		TemporaryLicensePlate: carBase.TemporaryLicensePlate,
		ParkingNo:             carBase.ParkingNo,
		StockStatus:           carBase.StockStatus,
		StockStatusName:       carBase.GetStockStatusName(carBase.StockStatus),
		UseCharacter:          constant.GetCarUseCharacterName(carBase.UseCharacter),
		Mileage:               carBase.ShamMileage,
		PurchaseTax:           carBase.PurchaseTax,
		InstallationCost:      carBase.InstallationCost,
		LocationCityId:        carBase.LocationCityId,
		Location:              carBase.LocationCity.Name,
		AttributionCityId:     carBase.AttributionCityId,
		Attribution:           carBase.AttributionCity.Name,
		Condition:             carBase.Condition,
		Remark:                carBase.Remark,
		BrokerInfo: brokerInfo{
			BrokerId:       carBase.BrokerId,
			BrokerNickname: carBase.Broker.Nickname,
			BrokerHeadImg:  carBase.Broker.HeadImg,
			BrokerPhone:    carBase.Broker.Phone,
		},
		DealerInfo: dealerInfo{
			Id:              carBase.Dealer.Id,
			Name:            carBase.Dealer.Name,
			OwnerBrokerId:   carBase.Dealer.OwnerBroker.Id,
			OwnerBrokerName: carBase.Dealer.OwnerBroker.Nickname,
			OwnerBrokerTel:  carBase.Dealer.OwnerBroker.Phone,
		},
		ModelInfo: car2.ModelInfo{
			Year:              carBase.Model.Year,
			Price:             carBase.Model.Price,
			Liter:             carBase.Model.Liter,
			DischargeStandard: carBase.Model.DischargeStandard,
			FuelTypeName:      carBase.Model.FuelTypeName,
			EnginePower:       carBase.Model.EnginePower,
			EnginePowerKw:     carBase.Model.EnginePowerKw,
			IsGreen:           carBase.Model.IsGreen,
			ModelCode:         carBase.Model.ModelCode,
			DriveName:         carBase.Model.DriveName,
			DriveType:         carBase.Model.DriveType,
			ModelStatus:       carBase.Model.ModelStatus,
			GearType:          carBase.Model.GearType,
			MarketDate:        carBase.Model.MarketDate,
			MinRegYear:        carBase.Model.MinRegYear,
			MaxRegYear:        carBase.Model.MaxRegYear,
			StopMakeYear:      carBase.Model.StopMakeYear,
			Intake:            carBase.Model.Intake,
			SeatNumber:        carBase.Model.SeatNumber,
			BodyType:          carBase.Model.BodyType,
			DoorNumber:        carBase.Model.DoorNumber,
			CarStruct:         carBase.Model.CarStruct,
			IsParallel:        carBase.Model.IsParallel,
			PriceAllowance:    carBase.Model.PriceAllowance,
		},
		PriceInfo: priceInfo{
			InteriorPrice: carBase.InteriorPrice,
			ExteriorPrice: carBase.ExteriorPrice,
		},
		PurchaseInfo: purchaseInfo{
			Price:   carBase.PurchasePrice,
			Date:    carBase.PurchaseDate.Time.Unix(),
			Liaison: carBase.PurchaseLiaison,
		},
		CertificateInfo: certificateInfo{
			Transfer:                 carBase.Transfer,
			KeyCount:                 carBase.KeyCount,
			CompulsoryInsurance:      carBase.CompulsoryInsurance,
			CompulsoryInsuranceDate:  carBase.CompulsoryInsuranceDate.Time.Unix(),
			CommercialInsurance:      carBase.CommercialInsurance,
			CommercialInsuranceDate:  carBase.CommercialInsuranceDate.Time.Unix(),
			CommercialInsurancePrice: carBase.CommercialInsurancePrice,
		},
		CarPhotos:      []carPhotoInfo{},
		InteriorPhotos: []carPhotoInfo{},
		DefectPhotos:   []carPhotoInfo{},
		DataPhotos:     []carPhotoInfo{},
	}
	for _, photo := range carBase.Photos {
		switch photo.Type {
		case model.CarPhotoTypeCar: // 车辆照片
			result.CarPhotos = append(result.CarPhotos, carPhotoInfo{
				Photo: photo.Path,
				Text:  photo.Text,
			})
		case model.CarPhotoTypeInterior: // 内饰照片
			result.InteriorPhotos = append(result.InteriorPhotos, carPhotoInfo{
				Photo: photo.Path,
				Text:  photo.Text,
			})
		case model.CarPhotoTypeDefect: // 缺陷照片
			result.DefectPhotos = append(result.DefectPhotos, carPhotoInfo{
				Photo: photo.Path,
				Text:  photo.Text,
			})
		case model.CarPhotoTypeData: // 资料照片
			result.DataPhotos = append(result.DataPhotos, carPhotoInfo{
				Photo: photo.Path,
				Text:  photo.Text,
			})
		}
	}
	switch carBase.Source {
	case model.CarBaseSourceConsignment: // 个人寄售
		orderConsignment := model.OrderConsignment{Id: carBase.OrderId}
		if mysql.Db.Preload("Contract.Master").First(&orderConsignment).Error == nil {
			result.ContractMasterInfo = contractMasterInfo{
				Name:     orderConsignment.Contract.Master.Name,
				IdCard:   orderConsignment.Contract.Master.IdCard,
				Phone:    orderConsignment.Contract.Master.Phone,
				BankCard: orderConsignment.Contract.Master.BankCard,
				Bank:     orderConsignment.Contract.Master.Bank,
			}
		}
	case model.CarBaseSourceDealerPurchase: // 车商收购
	}
	return
}

// Audit @Title 审核通过/驳回
func (b *baseLogic) Audit(dealerId, carBaseId uint, choose uint, reason string) error {
	carBase := model.CarBase{Id: carBaseId}
	if mysql.Db.Preload("DealerAudit").Preload("Audit").First(&carBase).Error != nil {
		return errors.New("发布车辆id错误")
	}
	if carBase.Status != model.CarBaseStatusDealerAuditing {
		return errors.New("不在车商审核状态")
	}
	if carBase.DealerId != dealerId {
		return errors.New("无审核权限")
	}
	carModel := model.CarModel{ModelId: carBase.ModelId}
	if mysql.Db.First(&carModel).Error != nil {
		return errors.New("车型未找到")
	}
	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		switch choose {
		case 1: // 审核通过
			if tx.Model(&carBase.DealerAudit).UpdateColumns(map[string]interface{}{
				"status":   constant.AuditStatusAdopt,
				"audit_at": time.Now(),
			}).RowsAffected != 1 {
				return errors.New("审核失败")
			}
			carBase.Audit = model.CarAudit{
				CarBaseId: carBaseId,
				Type:      model.CarAuditTypePublish,
				Status:    constant.AuditStatusWaiting,
			}
			if tx.Create(&carBase.Audit).Error != nil {
				return errors.New("车辆发布审核创建失败")
			}
			if tx.Model(&model.CarBase{}).Where("id = ?", carBaseId).Updates(map[string]interface{}{
				"audit_id": carBase.Audit.Id,
				"status":   model.CarBaseStatusAuditing,
			}).RowsAffected != 1 {
				return errors.New("车辆发布状态修改失败")
			}
			message.MessageLogic.MessageToBroker(mysql.Db, []uint{carBase.BrokerId}, 0, model.BrokerMessageTypeCarAudit, message.MessageItem{
				Title:    "车辆发布提示",
				Content:  "您的一个车辆发布已被车商审批通过",
				LinkedId: carBase.Id,
				Extras: map[string]interface{}{
					"车辆名称": carModel.Name,
					"发布价格": fmt.Sprintf("内部价%s|外部价%s", carBase.InteriorPrice, carBase.ExteriorPrice),
					"审批时间": carBase.DealerAudit.AuditAt.Time.Format("2006-01-02 15:04:05"),
				},
			})
			return nil
		case 2: // 审核拒绝
			if tx.Model(&carBase.DealerAudit).UpdateColumns(map[string]interface{}{
				"status":   constant.AuditStatusReject,
				"reason":   reason,
				"audit_at": time.Now(),
			}).RowsAffected != 1 {
				return errors.New("审核失败")
			}
			if tx.Model(&model.CarBase{}).Where(&carBase).Updates(map[string]interface{}{
				"status": model.CarBaseStatusDealerReject,
			}).RowsAffected != 1 {
				return errors.New("车辆发布状态修改失败")
			}
			message.MessageLogic.MessageToBroker(mysql.Db, []uint{carBase.BrokerId}, 0, model.BrokerMessageTypeCarAudit, message.MessageItem{
				Title:    "车辆发布提示",
				Content:  "您的一个车辆发布已被车商审批拒绝",
				LinkedId: carBase.Id,
				Extras: map[string]interface{}{
					"车辆名称": carModel.Name,
					"发布价格": fmt.Sprintf("内部价%s|外部价%s", carBase.InteriorPrice, carBase.ExteriorPrice),
					"审批时间": carBase.DealerAudit.AuditAt.Time.Format("2006-01-02 15:04:05"),
				},
			})
			return nil
		}
		return nil
	})
}
