package car

import (
	"base/app/common"
	"base/app/config"
	"base/app/constant"
	"base/app/lib/bean"
	"base/app/logic/car"
	"base/app/model"
	"errors"
	"fmt"
	"git.oa00.com/go/mysql"
	"github.com/shopspring/decimal"
	"gorm.io/gorm"
)

var CarLogic = &carLogic{}

type carLogic struct {
}
type carItem struct {
	Id            uint            `json:"id"`
	ModelName     string          `json:"modelName"`
	Type          uint            `json:"type"`
	MainPhoto     string          `json:"mainPhoto"`
	LicensingDate int64           `json:"licensingDate"`
	Price         decimal.Decimal `json:"price"`
	InteriorPrice decimal.Decimal `json:"interiorPrice"` // 内部价格
	ExteriorPrice decimal.Decimal `json:"exteriorPrice"` // 外部价格
	DownPayment   decimal.Decimal `json:"downPayment"`
	Mileage       decimal.Decimal `json:"mileage"`
	Transfer      uint            `json:"transfer"`
	Collect       uint            `json:"collect"`
	IsSelf        interface{}     `json:"isSelf"`
}

// Lists @Title 车辆列表
func (c *carLogic) Lists(brokerId uint, search CarSelfSearch, order car.CarOrder, page bean.Page) (lists []carItem, total int64) {
	lists = []carItem{}
	search.Status = []uint{model.CarStatusNormal}
	search.TheUpper = model.CarTheUpperTrue
	// 全部车辆仍提供查看不在售车辆
	{
		switch search.SelfCarStatus {
		case 1: // 在售
			search.TheUpper = model.CarTheUpperTrue
			search.Status = []uint{model.CarStatusNormal}
		case 2: // 签订中
			search.TheUpper = model.CarTheUpperTrue
			search.Status = []uint{model.CarStatusSigning, model.CarStatusSigned}
		case 3: // 已预定
			search.TheUpper = model.CarTheUpperTrue
			search.Status = []uint{model.CarStatusDeposit}
		case 4: // 已售
			search.TheUpper = model.CarTheUpperTrue
			search.Status = []uint{model.CarStatusFinal}
		case 5: // 退库
			search.TheUpper = model.CarTheUpperFalse
		}
	}
	var carIds []uint
	var items []car.CarItem
	// 仅有status和keyword筛选参数时，才会走es搜索
	if search.BrokerId != 0 ||
		search.CarType != 0 ||
		search.BrandId != 0 ||
		search.SeriesId != 0 ||
		!search.MinPrice.IsZero() ||
		!search.MaxPrice.IsZero() ||
		search.MinMileage != 0 ||
		search.MaxMileage != 0 ||
		!search.MinLiter.IsZero() ||
		!search.MaxLiter.IsZero() ||
		search.MinTransfer != 0 ||
		search.MaxTransfer != 0 ||
		search.Struct != "" ||
		search.FuelType != 0 ||
		search.GearType != 0 ||
		search.MinSeat != 0 ||
		search.MaxSeat != 0 ||
		search.Year != 0 ||
		search.MinAge != 0 ||
		search.MaxAge != 0 ||
		search.MinLicensingDateYear != 0 ||
		search.LocationProvinceId != 0 ||
		search.LocationCityId != 0 ||
		search.DischargeStandard != "" ||
		search.Color != "" ||
		len(search.CarIds) > 0 ||
		len(search.StoreIds) > 0 ||
		len(config.Config.OpenSearch.Addresses) == 0 ||
		!config.IsProd() {
		items, total = car.CarLogic.Lists(search.CarSearch, order, page)
	} else {
		items, total = car.CarLogic.ListsEs(car.ListsEsSearch{
			Keyword:  search.Keyword,
			Status:   search.Status,
			TheUpper: search.TheUpper,
		}, page)
	}
	for _, item := range items {
		carIds = append(carIds, item.Id)
	}

	mCarCollect := map[uint]uint{}
	if len(carIds) > 0 {
		var collectCars []model.BrokerCollectCar
		mysql.Db.Where("broker_id = ? and car_id in ?", brokerId, carIds).Find(&collectCars)
		for _, collectCar := range collectCars {
			mCarCollect[collectCar.CarId] = 1
		}
	}
	for _, item := range items {
		lists = append(lists, carItem{
			Id:            item.Id,
			ModelName:     item.ModelName,
			Type:          item.Type,
			MainPhoto:     item.MainPhoto,
			LicensingDate: item.LicensingDate,
			Price:         item.Price,
			//InteriorPrice: item.InteriorPrice,
			//ExteriorPrice: item.ExteriorPrice,
			DownPayment: item.DownPayment,
			Mileage:     item.Mileage,
			Transfer:    item.Transfer,
			Collect:     mCarCollect[item.Id],
			IsSelf:      common.Ternary(item.BrokerId == brokerId, 1, 0),
		})
	}
	return
}

type CarSelfSearch struct {
	car.CarSearch
	OwnerType     uint // 1=车商 2=门店 3=个人
	SelfCarStatus uint // 1=在售 2=签订中 3=已预订 4=已售 5=退库
}

// Self @Title 我的车辆列表
func (c *carLogic) Self(brokerId, businessId, storeId uint, search CarSelfSearch, order car.CarOrder, page bean.Page) (lists []carItem, total int64) {
	lists = []carItem{}
	switch search.OwnerType {
	case 1: // 车商
		if businessId <= 0 {
			return lists, 0
		}
		var storeIds []uint
		mysql.Db.Model(&model.BrokerBusinessStore{}).Where("business_id = ?", businessId).Select("id").Find(&storeIds)
		if len(storeIds) <= 0 {
			return lists, 0
		}
		search.StoreIds = storeIds
	case 2: // 门店
		if storeId <= 0 {
			return lists, 0
		}
		search.StoreIds = []uint{storeId}
	case 3: // 个人
		search.BrokerId = brokerId
	default:
		if storeId <= 0 {
			search.BrokerId = brokerId
		} else {
			search.StoreIds = []uint{storeId}
		}
	}
	{
		switch search.SelfCarStatus {
		case 1: // 在售
			search.TheUpper = model.CarTheUpperTrue
			search.Status = []uint{model.CarStatusNormal}
		case 2: // 签订中
			search.TheUpper = model.CarTheUpperTrue
			search.Status = []uint{model.CarStatusSigning, model.CarStatusSigned}
		case 3: // 已预定
			search.TheUpper = model.CarTheUpperTrue
			search.Status = []uint{model.CarStatusDeposit}
		case 4: // 已售
			search.TheUpper = model.CarTheUpperTrue
			search.Status = []uint{model.CarStatusFinal}
		case 5: // 退库
			search.TheUpper = model.CarTheUpperFalse
		}
	}
	items, total := car.CarLogic.Lists(search.CarSearch, order, page)
	var carIds []uint
	for _, item := range items {
		carIds = append(carIds, item.Id)
	}
	mCarCollect := map[uint]uint{}
	if len(carIds) > 0 {
		var collectCars []model.BrokerCollectCar
		mysql.Db.Where("broker_id = ? and car_id in ?", brokerId, carIds).Find(&collectCars)
		for _, collectCar := range collectCars {
			mCarCollect[collectCar.CarId] = 1
		}
	}
	for _, item := range items {
		lists = append(lists, carItem{
			Id:            item.Id,
			ModelName:     item.ModelName,
			Type:          item.Type,
			MainPhoto:     item.MainPhoto,
			LicensingDate: item.LicensingDate,
			Price:         item.Price,
			InteriorPrice: item.InteriorPrice,
			ExteriorPrice: item.ExteriorPrice,
			DownPayment:   item.DownPayment,
			Mileage:       item.Mileage,
			Transfer:      item.Transfer,
			Collect:       mCarCollect[item.Id],
			IsSelf:        common.Ternary(item.BrokerId == brokerId, 1, 0),
		})
	}
	return
}

type carCount struct {
	SellCount    int64 `json:"sellCount"`
	SigningCount int64 `json:"signingCount"`
	//SignedCount  int64 `json:"signedCount"`
	ReserveCount int64 `json:"reserveCount"`
	SoldCount    int64 `json:"soldCount"`
	BackOutCount int64 `json:"backOutCount"`
}

// CarCount @Title 车辆统计
func (c *carLogic) CarCount() (result carCount) {
	mysql.Db.Model(&model.Car{}).
		Select("count(if(the_upper = ? and status = ?,1,null)) sell_count,"+
			"count(if(the_upper = ? and (status = ? or status = ?),1,null)) signing_count,"+
			//"count(if(the_upper = ? and status = ?,1,null)) signed_count,"+
			"count(if(the_upper = ? and status = ?,1,null)) reserve_count,"+
			"count(if(the_upper = ? and status = ?,1,null)) sold_count,"+
			"count(if(the_upper = ?,1,null)) back_out_count",
			model.CarTheUpperTrue, model.CarStatusNormal,
			model.CarTheUpperTrue, model.CarStatusSigning, model.CarStatusSigned,
			//model.CarTheUpperTrue, model.CarStatusSigned,
			model.CarTheUpperTrue, model.CarStatusDeposit,
			model.CarTheUpperTrue, model.CarStatusFinal,
			model.CarTheUpperFalse,
		).
		First(&result)

	return
}

type selfCount struct {
	BusinessCount carCount `json:"businessCount"`
	StoreCount    carCount `json:"storeCount"`
	OwnCount      carCount `json:"ownCount"`
}

// SelfCount @Title 我的车辆统计
func (c *carLogic) SelfCount(brokerId, storeId, businessId uint) (result selfCount) {
	query := fmt.Sprintf("count(if(the_upper = %d and status = %d,1,null)) sell_count,"+
		"count(if(the_upper = %d and (status = %d or status = %d),1,null)) signing_count,"+
		//"count(if(the_upper = %d and status = %d,1,null)) signed_count,"+
		"count(if(the_upper = %d and status = %d,1,null)) reserve_count,"+
		"count(if(the_upper = %d and status = %d,1,null)) sold_count,"+
		"count(if(the_upper = %d,1,null)) back_out_count",
		model.CarTheUpperTrue, model.CarStatusNormal,
		model.CarTheUpperTrue, model.CarStatusSigning, model.CarStatusSigning,
		//model.CarTheUpperTrue, model.CarStatusSigned,
		model.CarTheUpperTrue, model.CarStatusDeposit,
		model.CarTheUpperTrue, model.CarStatusFinal,
		model.CarTheUpperFalse)
	mysql.Db.Model(&model.Car{}).Where("broker_id = ? and customer_type = ?", brokerId, model.CustomerTypeCustomer).Select(query).First(&result.OwnCount)
	if businessId > 0 {
		var storeIds []uint
		mysql.Db.Model(&model.BrokerBusinessStore{}).Where("business_id = ?", businessId).Select("id").Find(&storeIds)
		if len(storeIds) > 0 {
			mysql.Db.Model(&model.Car{}).Where("store_id in ?", storeIds).Select(query).First(&result.BusinessCount)
		}
		if storeId > 0 {
			mysql.Db.Model(&model.Car{}).Where("store_id = ?", storeId).Select(query).First(&result.StoreCount)
		}
	}
	return
}

// countSql
func (c *carLogic) countByBrokerIds(brokerIds []uint) (result carCount) {
	mysql.Db.Model(&model.Car{}).
		Where("broker_id in ?", brokerIds).
		Select("count(if(the_upper = ? and status = ?,1,null)) sell_count,"+
			"count(if(the_upper = ? and status = ?,1,null)) reserve_count,"+
			"count(if(the_upper = ? and status = ?,1,null)) sold_count,"+
			"count(if(the_upper = ?,1,null)) back_out_count",
			model.CarTheUpperTrue, model.CarStatusNormal,
			model.CarTheUpperTrue, model.CarStatusDeposit,
			model.CarTheUpperTrue, model.CarStatusFinal,
			model.CarTheUpperFalse,
		).First(&result)
	return
}

type carInfo struct {
	CarInfo        car.CarInfo `json:"carInfo"`
	IsSelf         uint        `json:"isSelf"`
	IsSelfStore    uint        `json:"isSelfStore"`
	IsSelfBusiness uint        `json:"isSelfBusiness"`
}

// Info @Title 车辆详情
func (c *carLogic) Info(brokerId, level, carId uint) (carInfo, error) {
	// 价格显示为内部价或外部价
	var priceDet bool
	if brokerId > 0 && common.InArray(level, []uint{model.BrokerLevelPartner, model.BrokerLevelBusiness}) {
		priceDet = true
	}
	info, err := car.CarLogic.Info(priceDet, carId)
	carInfo := carInfo{
		CarInfo: info,
		IsSelf:  common.UintTernary(info.BrokerInfo.BrokerId == brokerId, 1, 0),
	}
	if level == model.BrokerLevelBusiness {
		carBrokerStaffInfo := model.BrokerBusinessStoreStaff{BrokerId: info.BrokerInfo.BrokerId, Status: model.BrokerBusinessStoreStaffStatusEnabled}
		if mysql.Db.Where(&carBrokerStaffInfo).First(&carBrokerStaffInfo).Error == nil {
			staffInfo := model.BrokerBusinessStoreStaff{BrokerId: brokerId, Status: model.BrokerBusinessStoreStaffStatusEnabled}
			if mysql.Db.Where(&staffInfo).First(&staffInfo).Error == nil {
				if staffInfo.BusinessId == carBrokerStaffInfo.BusinessId {
					carInfo.IsSelfBusiness = 1
				}
				if staffInfo.StoreId == carBrokerStaffInfo.StoreId {
					carInfo.IsSelfStore = 1
				}
			}
		}
	}
	if carInfo.IsSelfStore != 1 {
		carInfo.CarInfo.Vin = "/"
	}
	return carInfo, err
}

type AdjustPriceData struct {
	CarId         uint            `binding:"required" label:"车辆id"`
	InteriorPrice decimal.Decimal `binding:"required" label:"内部价格"`
	ExteriorPrice decimal.Decimal `binding:"required" label:"外部价格"`
}

// AdjustPrice @Title 调价
func (c *carLogic) AdjustPrice(storeId, brokerId, carId uint, interiorPrice, exteriorPrice decimal.Decimal) error {
	if exteriorPrice.LessThan(interiorPrice) {
		return errors.New("外部价不能小于内部价")
	}
	car := model.Car{Id: carId}
	if mysql.Db.Preload("CarBase").Where("store_id = ?", storeId).First(&car).Error != nil {
		return errors.New("车辆错误")
	}
	if car.Status != model.CarStatusNormal {
		return errors.New("非在售状态")
	}
	if interiorPrice.Equal(car.InteriorPrice) && exteriorPrice.Equal(car.ExteriorPrice) {
		return errors.New("车辆无价格变动")
	}
	carAdjustPriceHistory := model.CarAdjustPriceHistory{
		CarId:            carId,
		OperatorId:       brokerId,
		Type:             model.CarAdjustPriceHistoryTypeDealer,
		InteriorPrice:    interiorPrice,
		ExteriorPrice:    exteriorPrice,
		PreInteriorPrice: car.InteriorPrice,
		PreExteriorPrice: car.ExteriorPrice,
	}
	amount := exteriorPrice.Sub(car.ExteriorPrice)
	if amount.GreaterThan(decimal.Zero) {
		carAdjustPriceHistory.Description = fmt.Sprintf("涨价%s元", amount.String())
	}
	if amount.LessThan(decimal.Zero) {
		carAdjustPriceHistory.Description = fmt.Sprintf("降价%s元", amount.Abs().String())
	}
	if amount.Equal(decimal.Zero) {
		carAdjustPriceHistory.Description = fmt.Sprintf("价格未发生变动")
	}
	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		if tx.Model(&model.Car{}).Where("id = ?", carId).UpdateColumns(map[string]interface{}{
			"interior_price":      interiorPrice,
			"exterior_price":      exteriorPrice,
			"price":               exteriorPrice,
			"price_adjust_amount": amount,
		}).RowsAffected != 1 {
			return errors.New("车辆价格修改失败")
		}
		if tx.Model(&model.CarBase{}).Where("id = ?", car.OrderCarId).UpdateColumns(map[string]interface{}{
			"interior_price": interiorPrice,
			"exterior_price": exteriorPrice,
		}).RowsAffected != 1 {
			return errors.New("车辆价格修改失败")
		}
		if tx.Create(&carAdjustPriceHistory).Error != nil {
			return errors.New("调价记录生成失败")
		}
		return nil
	})
}

type priceHistoryItem struct {
	InteriorPrice    decimal.Decimal `json:"interiorPrice"`
	ExteriorPrice    decimal.Decimal `json:"exteriorPrice"`
	PreInteriorPrice decimal.Decimal `json:"preInteriorPrice"`
	PreExteriorPrice decimal.Decimal `json:"preExteriorPrice"`
	Description      string          `json:"description"`
	CreatedAt        int64           `json:"createdAt"`
}

// PriceHistory @Title 价格历史（至多10次）
func (c *carLogic) PriceHistory(carId uint) (lists []priceHistoryItem, err error) {
	lists = []priceHistoryItem{}
	var history []model.CarAdjustPriceHistory
	mysql.Db.Where("car_id = ?", carId).Order("created_at desc").Limit(10).Find(&history)
	for _, item := range history {
		lists = append(lists, priceHistoryItem{
			InteriorPrice:    item.InteriorPrice,
			ExteriorPrice:    item.ExteriorPrice,
			PreInteriorPrice: item.PreInteriorPrice,
			PreExteriorPrice: item.PreExteriorPrice,
			Description:      item.Description,
			CreatedAt:        item.CreatedAt.Unix(),
		})
	}
	return
}

type UpperDownData struct {
	CarId uint `binding:"required" label:"车辆id"`
}

// Upper @Title 申请上架车辆
func (c *carLogic) Upper(dealerId, brokerId uint, data UpperDownData) error {
	car := model.Car{Id: data.CarId}
	if mysql.Db.Preload("CarBase.Order").Preload("Store").First(&car).Error != nil {
		return errors.New("车辆错误")
	}
	if car.StoreId != dealerId {
		return errors.New("非本门店车辆")
	}
	if car.TheUpper != model.CarTheUpperFalse {
		return errors.New("非下架状态")
	}

	audit := model.CarAudit{
		CarId:  car.Id,
		Type:   model.CarAuditTypeUpper,
		Status: constant.AuditStatusWaiting,
	}
	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		if tx.Create(&audit).Error != nil {
			return errors.New("提交失败")
		}
		if tx.Where("id = ?", car.OrderCarId).Model(&model.CarBase{}).Update("audit_id", audit.Id).RowsAffected != 1 {
			return errors.New("审核失败")
		}
		return nil
	})
}

// Down @Title 申请下架车辆
func (c *carLogic) Down(dealerId, brokerId uint, data UpperDownData) error {
	car := model.Car{Id: data.CarId}
	if mysql.Db.Preload("CarBase.Order").Preload("Store").First(&car).Error != nil {
		return errors.New("车辆错误")
	}
	if car.StoreId != dealerId {
		return errors.New("非本门店车辆")
	}
	if car.TheUpper != model.CarTheUpperTrue {
		return errors.New("非上架状态")
	}

	audit := model.CarAudit{
		CarId:  car.Id,
		Type:   model.CarAuditTypeDown,
		Status: constant.AuditStatusWaiting,
	}
	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		if tx.Create(&audit).Error != nil {
			return errors.New("提交失败")
		}
		if tx.Model(&model.Car{}).Where("id = ?", car.Id).Updates(map[string]interface{}{
			"the_upper": model.CarTheDownAdoptWait,
		}).RowsAffected != 1 {
			return errors.New("审核失败")
		}
		if tx.Model(&model.CarBase{}).Where("id = ?", car.OrderCarId).Update("audit_id", audit.Id).RowsAffected != 1 {
			return errors.New("审核失败")
		}
		return nil
	})
}
