package car

import (
	"base/app/constant"
	"base/app/lib/bean"
	"base/app/logic/broker/message"
	"base/app/logic/car"
	"base/app/model"
	"errors"
	"fmt"
	"git.oa00.com/go/mysql"
	"github.com/shopspring/decimal"
	"gorm.io/gorm"
	"time"
)

var EditLogic = &editLogic{}

type editLogic struct {
}

type CarEditAdd struct {
	CarId    uint           `binding:"required" label:"车辆id"`
	BaseInfo carEditBaseAdd `binding:"required" label:"基本信息"`
	Photos   carEditPhotos  `binding:"required" label:"照片"`
}

type carEditBaseAdd struct {
	Color            string          `binding:"required" label:"车身颜色"`
	InteriorColor    string          `label:"内饰颜色"`
	ParkingNo        string          `label:"车位编号"`
	StockStatus      uint            `binding:"oneof=0 1 2 3" label:"库存状态"`
	UseCharacter     uint            `binding:"required" label:"使用性质"`
	ShamMileage      decimal.Decimal `binding:"required" label:"虚假里程"`
	InstallationCost decimal.Decimal `label:"加装费用"`
	Location         uint            `binding:"required" label:"车辆所在地"`
	Condition        string          `label:"车况"`
}
type carEditPhotos struct {
	CarPhotos      []carEditPhoto `binding:"required,min=1" label:"车辆照片"`
	InteriorPhotos []carEditPhoto `binding:"required,min=1" label:"内饰照片"`
	DefectPhotos   []carEditPhoto `label:"缺陷照片"`
	DataPhotos     []carEditPhoto `binding:"required,min=1" label:"资料照片"`
}

type carEditPhoto struct {
	Photo string `binding:"required" label:"图片地址"`
	Text  string `label:"图片描述"`
}

// Add @Title 申请编辑车辆
func (e *editLogic) Add(brokerId, storeId uint, data CarEditAdd) error {
	carData := model.Car{Id: data.CarId}
	if mysql.Db.First(&carData).Error != nil {
		return errors.New("车辆未找到")
	}
	if carData.Status != model.CarStatusNormal {
		return errors.New("车辆不在出售状态")
	}
	if carData.StoreId != storeId {
		return errors.New("不可操作非本门店车辆")
	}
	location := model.Area{Id: data.BaseInfo.Location, Level: model.AreaLevelCity}
	if mysql.Db.First(&location).Error != nil {
		return errors.New("车辆所在地错误")
	}
	carEdit := model.CarEdit{
		CarId:              data.CarId,
		BrokerId:           brokerId,
		DealerId:           carData.StoreId,
		MainPhoto:          data.Photos.CarPhotos[0].Photo,
		Color:              data.BaseInfo.Color,
		ParkingNo:          data.BaseInfo.ParkingNo,
		StockStatus:        data.BaseInfo.StockStatus,
		InteriorColor:      data.BaseInfo.InteriorColor,
		UseCharacter:       data.BaseInfo.UseCharacter,
		LocationProvinceId: location.ParentId,
		LocationCityId:     location.Id,
		ShamMileage:        data.BaseInfo.ShamMileage,
		InstallationCost:   data.BaseInfo.InstallationCost,
		Condition:          data.BaseInfo.Condition,
	}
	for _, photo := range data.Photos.CarPhotos {
		carEdit.Photos = append(carEdit.Photos, model.CarEditPhoto{
			Path: photo.Photo,
			Text: photo.Text,
			Type: model.CarPhotoTypeCar,
		})
	}
	for _, photo := range data.Photos.InteriorPhotos {
		carEdit.Photos = append(carEdit.Photos, model.CarEditPhoto{
			Path: photo.Photo,
			Text: photo.Text,
			Type: model.CarPhotoTypeInterior,
		})
	}
	for _, photo := range data.Photos.DefectPhotos {
		carEdit.Photos = append(carEdit.Photos, model.CarEditPhoto{
			Path: photo.Photo,
			Text: photo.Text,
			Type: model.CarPhotoTypeDefect,
		})
	}
	for _, photo := range data.Photos.DataPhotos {
		carEdit.Photos = append(carEdit.Photos, model.CarEditPhoto{
			Path: photo.Photo,
			Text: photo.Text,
			Type: model.CarPhotoTypeData,
		})
	}
	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		// 车商车辆编辑非店长发起，须提交店长审核
		store := model.BrokerBusinessStore{Id: carData.StoreId}
		if tx.Where(&store).First(&store).Error != nil {
			return errors.New("未查到车辆门店信息")
		}
		if brokerId != store.OwnerBrokerId {
			carEdit.AuditStatus = constant.AuditStatusWaiting
			carEdit.DealerAudit = model.DealerAuditing{
				Type:          model.DealerAuditingTypeEdit,
				DealerId:      store.Id,
				ApplyBrokerId: brokerId,
				Status:        constant.AuditStatusWaiting,
			}
			if tx.Create(&carEdit).Error != nil {
				return errors.New("车辆编辑申请失败")
			}
			carEdit.DealerAudit.LinkId = carEdit.Id
			if tx.Updates(&carEdit.DealerAudit).Error != nil {
				return errors.New("车主审核生成失败")
			}
			message.MessageLogic.MessageToBroker(mysql.Db, []uint{carEdit.BrokerId}, 0, model.BrokerMessageTypeCarEdit, message.MessageItem{
				Title:    "车辆编辑",
				Content:  "您有一个新的车辆编辑已提交至车商审批",
				LinkedId: carEdit.Id,
			})
			message.MessageLogic.MessageToBroker(mysql.Db, []uint{store.OwnerBrokerId}, 0, model.BrokerMessageTypeCarEdit, message.MessageItem{
				Title:    "车辆编辑",
				Content:  "您有一个新的车辆编辑待审批",
				LinkedId: carEdit.Id,
			})
			return nil
		}
		carEdit.AuditStatus = constant.AuditStatusAdopt
		if tx.Create(&carEdit).Error != nil {
			return errors.New("车辆编辑失败")
		}
		if err := e.edit(tx, carEdit); err != nil {
			return err
		}
		return nil
	})
}

type carEditItem struct {
	Id              uint            `json:"id"`
	CarId           uint            `json:"carId"`
	MainPhoto       string          `json:"mainPhoto"`
	ModelName       string          `json:"modelName"`
	Mileage         decimal.Decimal `json:"mileage"`
	LicensingDate   int64           `json:"licensingDate"`
	BrokerId        uint            `json:"brokerId"`
	BrokerName      string          `json:"brokerName"`
	DealerId        uint            `json:"dealerId"`
	DealerName      string          `json:"dealerName"`
	AuditStatus     uint            `json:"auditStatus"`
	AuditStatusName string          `json:"auditStatusName"`
	DealerAuditAt   int64           `json:"dealerAuditAt"`
	CreatedAt       int64           `json:"createdAt"`
}

type CarEditSearch struct {
	AuditStatus      uint
	ModelName        string
	CreatedTimeStart string
	CreatedTimeEnd   string
}

type CarEditBrokerSearch struct {
	DealerName string
	DealerTel  string
}

// BrokerLists @Title 经纪人编辑记录
func (e *editLogic) BrokerLists(brokerId uint, search CarEditSearch, searchBroker CarEditBrokerSearch, page bean.Page) (lists []carEditItem, total int64) {
	return e.Lists([]uint{brokerId}, []uint{}, search, searchBroker, CarEditDealerSearch{}, page)
}

type CarEditDealerSearch struct {
	BrokerName string
	BrokerTel  string
}

// DealerLists @Title 车商审批车辆编辑记录
func (e *editLogic) DealerLists(dealerId uint, search CarEditSearch, searchDealer CarEditDealerSearch, page bean.Page) (lists []carEditItem, total int64) {
	return e.Lists([]uint{}, []uint{dealerId}, search, CarEditBrokerSearch{}, searchDealer, page)
}

// Lists @Title 编辑记录
func (e *editLogic) Lists(brokerIds, dealerIds []uint, search CarEditSearch, searchBroker CarEditBrokerSearch, searchDealer CarEditDealerSearch, page bean.Page) (lists []carEditItem, total int64) {
	lists = []carEditItem{}
	var carEdits []model.CarEdit
	where := mysql.Db
	if len(brokerIds) > 0 {
		where = where.Where("broker_id in ?", brokerIds)
	}
	if len(dealerIds) > 0 {
		where = where.Where("dealer_id in ?", dealerIds)
	}
	if search.AuditStatus > 0 {
		where = where.Where("status = ?", search.AuditStatus)
	}
	if search.ModelName != "" {
		where = where.Where("model_id in (?)", mysql.Db.Model(&model.CarModel{}).Where("name like ?", fmt.Sprintf("%%%s%%", search.ModelName)).Select("model_id"))
	}
	if search.CreatedTimeStart != "" {
		where = where.Where("created_at > ?", search.CreatedTimeStart)
	}
	if search.CreatedTimeEnd != "" {
		where = where.Where("created_at < ?", search.CreatedTimeEnd)
	}
	{
		dealerWhere := mysql.Db.Model(&model.BrokerBusinessStore{}).Select("id")
		if searchBroker.DealerName != "" {
			dealerWhere = dealerWhere.Where("name like ?", fmt.Sprintf("%%%s%%", searchBroker.DealerName))
		}
		if searchBroker.DealerTel != "" {
			dealerWhere = dealerWhere.Where("phone like ?", fmt.Sprintf("%%%s%%", searchBroker.DealerTel))
		}
		if _, ok := dealerWhere.Statement.Clauses["WHERE"]; ok {
			where = where.Where("dealer_id in (?)", dealerWhere)
		}
	}
	{
		brokerWhere := mysql.Db.Model(&model.Broker{}).Select("id")
		if searchDealer.BrokerName != "" {
			brokerWhere = brokerWhere.Where("nickname like ?", fmt.Sprintf("%%%s%%", searchDealer.BrokerName))
		}
		if searchDealer.BrokerTel != "" {
			brokerWhere = brokerWhere.Where("phone like ?", fmt.Sprintf("%%%s%%", searchDealer.BrokerTel))
		}
		if _, ok := brokerWhere.Statement.Clauses["WHERE"]; ok {
			where = where.Where("broker_id in (?)", brokerWhere)
		}
	}
	mysql.Db.Model(&carEdits).Where(where).Count(&total)
	if page.HasPage(total) {
		mysql.Db.Preload("Car.Model").Preload("DealerAudit").
			Preload("Broker").Preload("Dealer").
			Where(where).Order("id desc").Offset(page.GetStart()).Limit(page.GetLimit()).Find(&carEdits)
		for _, item := range carEdits {
			lists = append(lists, carEditItem{
				Id:              item.Id,
				CarId:           item.CarId,
				MainPhoto:       item.Car.MainPhoto,
				ModelName:       item.Car.Model.Name,
				Mileage:         item.ShamMileage,
				LicensingDate:   item.Car.LicensingDate.Time.Unix(),
				AuditStatus:     item.AuditStatus,
				AuditStatusName: constant.GetAuditStatusName(item.AuditStatus),
				BrokerId:        item.BrokerId,
				BrokerName:      item.Broker.Nickname,
				DealerId:        item.DealerId,
				DealerName:      item.Dealer.Name,
				DealerAuditAt:   item.DealerAudit.AuditAt.Time.Unix(),
				CreatedAt:       item.CreatedAt.Unix(),
			})
		}
	}
	return
}

type carEditInfo struct {
	CarInfo         car.CarInfo `json:"carInfo"`
	AuditStatus     uint        `json:"auditStatus"`
	AuditStatusName string      `json:"auditStatusName"`
	RejectReason    string      `json:"rejectReason"`
	DealerAuditAt   int64       `json:"dealerAuditAt"`
	CreatedAt       int64       `json:"createdAt"`
}

// Info @Title 详情
func (e *editLogic) Info(carEditId uint) (result carEditInfo, err error) {
	carEdit := model.CarEdit{Id: carEditId}
	if mysql.Db.Preload("Photos").Preload("DealerAudit").First(&carEdit).Error != nil {
		return result, errors.New("车辆未找到")
	}
	carInfos, err := car.CarLogic.Info(true, carEdit.CarId)
	carInfos.Color = carEdit.Color
	carInfos.InteriorColor = carEdit.InteriorColor
	carInfos.ParkingNo = carEdit.ParkingNo
	carInfos.StockStatus = carEdit.StockStatus
	carInfos.UseCharacter = constant.GetCarUseCharacterName(carEdit.UseCharacter)
	carInfos.Mileage = carEdit.ShamMileage
	carInfos.InstallationCost = carEdit.InstallationCost
	carInfos.Location = carEdit.LocationCity.Name
	carInfos.Condition = carEdit.Condition
	carInfos.CarPhotos = []car.CarPhoto{}
	carInfos.InteriorPhotos = []car.CarPhoto{}
	carInfos.DefectPhotos = []car.CarPhoto{}
	carInfos.DataPhotos = []car.CarPhoto{}
	for _, photo := range carEdit.Photos {
		switch photo.Type {
		case model.CarPhotoTypeCar: // 车辆照片
			carInfos.CarPhotos = append(carInfos.CarPhotos, car.CarPhoto{
				Photo: photo.Path,
				Text:  photo.Text,
			})
		case model.CarPhotoTypeInterior: // 内饰照片
			carInfos.InteriorPhotos = append(carInfos.InteriorPhotos, car.CarPhoto{
				Photo: photo.Path,
				Text:  photo.Text,
			})
		case model.CarPhotoTypeDefect: // 缺陷照片
			carInfos.DefectPhotos = append(carInfos.DefectPhotos, car.CarPhoto{
				Photo: photo.Path,
				Text:  photo.Text,
			})
		case model.CarPhotoTypeData: // 资料照片
			carInfos.DataPhotos = append(carInfos.DataPhotos, car.CarPhoto{
				Photo: photo.Path,
				Text:  photo.Text,
			})
		}
	}
	result.CarInfo = carInfos
	result.AuditStatus = carEdit.AuditStatus
	result.AuditStatusName = constant.GetAuditStatusName(carEdit.AuditStatus)
	result.DealerAuditAt = carEdit.DealerAudit.AuditAt.Time.Unix()
	result.RejectReason = carEdit.DealerAudit.Reason
	result.CreatedAt = carEdit.CreatedAt.Unix()
	return
}

// Audit @Title 审核通过/驳回
func (e *editLogic) Audit(dealerId, carEditId uint, choose uint, reason string) error {
	carEdit := model.CarEdit{Id: carEditId}
	if mysql.Db.Preload("Photos").Preload("DealerAudit").First(&carEdit).Error != nil {
		return errors.New("车辆错误")
	}
	if carEdit.AuditStatus != constant.AuditStatusWaiting {
		return errors.New("不在车商审核状态")
	}
	if carEdit.DealerId != dealerId {
		return errors.New("无审核权限")
	}
	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		switch choose {
		case 1: // 审核通过
			if tx.Model(&carEdit.DealerAudit).UpdateColumns(map[string]interface{}{
				"status":   constant.AuditStatusAdopt,
				"audit_at": time.Now(),
			}).RowsAffected != 1 {
				return errors.New("审核失败")
			}
			if tx.Model(&model.CarEdit{}).Where("id = ?", carEditId).Updates(map[string]interface{}{
				"audit_status": constant.AuditStatusAdopt,
			}).RowsAffected != 1 {
				return errors.New("车辆编辑失败")
			}
			if err := e.edit(tx, carEdit); err != nil {
				return err
			}
			message.MessageLogic.MessageToBroker(mysql.Db, []uint{carEdit.BrokerId}, 0, model.BrokerMessageTypeCarEdit, message.MessageItem{
				Title:    "车辆审批提示",
				Content:  "您的一个车辆编辑已被车商审批通过",
				LinkedId: carEdit.CarId,
			})
			return nil
		case 2: // 审核拒绝
			if tx.Model(&carEdit.DealerAudit).UpdateColumns(map[string]interface{}{
				"status":   constant.AuditStatusReject,
				"reason":   reason,
				"audit_at": time.Now(),
			}).RowsAffected != 1 {
				return errors.New("审核失败")
			}
			if tx.Model(&model.CarEdit{}).Where(&carEdit).Updates(map[string]interface{}{
				"audit_status": constant.AuditStatusReject,
			}).RowsAffected != 1 {
				return errors.New("车辆编辑失败")
			}
			message.MessageLogic.MessageToBroker(mysql.Db, []uint{carEdit.BrokerId}, 0, model.BrokerMessageTypeCarEdit, message.MessageItem{
				Title:    "车辆编辑提示",
				Content:  "您的一个车辆编辑已被车商审批拒绝",
				LinkedId: carEdit.CarId,
			})
			return nil
		}
		return nil
	})
}

// edit @Title 车辆编辑完成后置逻辑
func (e *editLogic) edit(tx *gorm.DB, carEdit model.CarEdit) error {
	carData := model.Car{Id: carEdit.CarId}
	if tx.Preload("CarBase").Where(carData).First(&carData).Error != nil {
		return errors.New("车辆错误")
	}
	if carData.Status != model.CarStatusNormal {
		return errors.New("车辆不在出售状态")
	}
	var newPhotos []model.CarPhoto
	for _, photo := range carEdit.Photos {
		newPhotos = append(newPhotos, model.CarPhoto{
			OrderId: carData.OrderCarId,
			Path:    photo.Path,
			Text:    photo.Text,
			Type:    photo.Type,
		})
	}
	if tx.Where("order_id = ?", carData.OrderCarId).Delete(&model.CarPhoto{}).Error != nil {
		return errors.New("编辑失败")
	}
	if tx.Create(&newPhotos).Error != nil {
		return errors.New("编辑失败")
	}
	if tx.Model(&model.Car{}).Where("id = ?", carData.Id).Updates(map[string]interface{}{
		"main_photo":     carEdit.MainPhoto,
		"color":          carEdit.Color,
		"interior_color": carEdit.InteriorColor,
	}).Error != nil {
		return errors.New("编辑失败")
	}
	if tx.Model(&model.CarBase{}).Where("id = ?", carData.OrderCarId).Updates(map[string]interface{}{
		"main_photo":           carEdit.MainPhoto,
		"color":                carEdit.Color,
		"interior_color":       carEdit.InteriorColor,
		"parking_no":           carEdit.ParkingNo,
		"stock_status":         carEdit.StockStatus,
		"use_character":        carEdit.UseCharacter,
		"sham_mileage":         carEdit.ShamMileage,
		"installation_cost":    carEdit.InstallationCost,
		"location_province_id": carEdit.LocationProvinceId,
		"location_city_id":     carEdit.LocationCityId,
		"condition":            carEdit.Condition,
	}).Error != nil {
		return errors.New("编辑失败")
	}
	return nil
}
