package contract

import (
	"base/app/common"
	"base/app/constant"
	"base/app/lib/bean"
	"base/app/lib/tencent"
	"base/app/logic/broker/message"
	"base/app/logic/broker/user/wallet"
	"base/app/model"
	"database/sql"
	"errors"
	"fmt"
	"git.oa00.com/go/mysql"
	"github.com/shopspring/decimal"
	essbasic "github.com/tencentcloud/tencentcloud-sdk-go/tencentcloud/essbasic/v20210526"
	"gorm.io/gorm"
	"strconv"
	"time"
)

var ConsignmentLogic = &consignmentLogic{}

type consignmentLogic struct {
}

type consignmentItem struct {
	Id           uint   `json:"id"`
	ContractSn   string `json:"contractSn"`
	ModelName    string `json:"modelName"`
	CustomerName string `json:"customerName"`
	Status       uint   `json:"status"`
	StatusName   string `json:"statusName"`
	SignAt       int64  `json:"signAt"`
	EssFileUrl   string `json:"essFileUrl"`
}
type ConsignmentSearch struct {
	Keyword string
}

// Lists @Title 寄卖合同列表
func (c *consignmentLogic) Lists(brokerId uint, search ConsignmentSearch, page bean.Page) (lists []consignmentItem, total int64) {
	lists = []consignmentItem{}
	var contractConsignments []model.ContractConsignment
	where := mysql.Db.Where("broker_id = ?", brokerId)
	if search.Keyword != "" {
		keyword := fmt.Sprintf("%%%s%%", search.Keyword)
		customerWhere := mysql.Db.Model(&model.Customer{}).Select("id").Where("nickname like ?", keyword)
		where = where.Where("contract_sn like ? or customer_id in (?)", keyword, customerWhere)
	}
	mysql.Db.Model(&contractConsignments).Where(where).Count(&total)
	if page.HasPage(total) {
		mysql.Db.
			Preload("Model").Preload("Customer.Master").Preload("EssFlow").
			Where(where).Order("id desc").Offset(page.GetStart()).Limit(page.GetLimit()).Find(&contractConsignments)
		for _, contractConsignment := range contractConsignments {
			lists = append(lists, consignmentItem{
				Id:           contractConsignment.Id,
				ContractSn:   contractConsignment.ContractSn,
				ModelName:    contractConsignment.Model.Name,
				CustomerName: contractConsignment.Customer.Nickname,
				Status:       contractConsignment.Status,
				StatusName:   constant.GetContractStatusName(contractConsignment.Status),
				SignAt:       contractConsignment.SignAt.Time.Unix(),
				EssFileUrl:   contractConsignment.EssFlow.Url,
			})
		}
	}
	return
}

type ConsignmentAdd struct {
	PriceId                  uint            `binding:"required" label:"估价记录"`
	CustomerId               uint            `binding:"required" label:"客户"`
	Price                    decimal.Decimal `binding:"required" label:"合同价格"`
	MasterInfo               masterInfo      `binding:"required,dive" label:"车主信息"`
	LicensePlate             string          `binding:"required" label:"牌照"`
	KeyCount                 uint            `binding:"required" label:"钥匙数量"`
	UseCharacter             uint            `binding:"required" label:"使用性质"`
	CompulsoryInsurance      uint            // 交强险
	CompulsoryInsuranceDate  string          // 交强险到期日期
	CommercialInsurance      uint            // 商业险
	CommercialInsuranceDate  string          // 商业险到期日期
	CommercialInsurancePrice decimal.Decimal // 商业险金额
	//Exterior                 string          `binding:"required" label:"外观"`    // 外观
	//InteriorTrim             string          `binding:"required" label:"内饰"`    // 内饰
	//WorkingCondition         string          `binding:"required" label:"工况"`    // 工况
	ServiceFeeRate decimal.Decimal `binding:"required" label:"服务费比例"` // 服务费比例
}
type masterInfo struct {
	Name        string `binding:"required" label:"姓名"`
	IdCard      string `binding:"required" label:"身份证号"`
	Phone       string `binding:"required" label:"手机号"`
	BankCard    string `binding:"required" label:"银行卡号"`
	Bank        string `binding:"required" label:"开户行"`
	IdCardFront string `binding:"required" label:"身份证正面"`
	IdCardBack  string `binding:"required" label:"身份证反面"`
}

// Add @Title 添加合同
func (c *consignmentLogic) Add(brokerId uint, data ConsignmentAdd) error {
	//customer := model.Customer{BrokerId: brokerId, Id: data.CustomerId}
	customer := model.Customer{Id: data.CustomerId}
	if mysql.Db.Where(&customer).First(&customer).Error != nil {
		return errors.New("客户错误")
	}
	carPrice := model.BrokerCarPrice{Id: data.PriceId, BrokerId: brokerId}
	if mysql.Db.Where(&carPrice).First(&carPrice).Error != nil {
		return errors.New("价格错误")
	}
	//  判断线上在售无重复车辆
	car := model.Car{Vin: carPrice.Vin}
	mysql.Db.Where(&car).Where("status not in ?", []uint{model.CarStatusFinal, model.CarStatusCancel}).First(&car)
	if car.Id > 0 {
		return errors.New("车辆已存在线上")
	}
	order := model.OrderConsignment{Vin: carPrice.Vin}
	mysql.Db.Where(&order).Where("status not in ?", []uint{model.OrderConsignmentStatusFinal, model.OrderConsignmentStatusTheDown, model.OrderConsignmentStatusCancel, model.OrderConsignmentStatusReject}).First(&order)
	if order.Id > 0 {
		return errors.New("寄售订单存在vin一致订单")
	}
	carModel := model.CarModel{ModelId: carPrice.ModelId}
	if mysql.Db.Where(&carModel).First(&carModel).Error != nil {
		return errors.New("查找车型错误")
	}
	contractSn, err := common.GenNo(constant.NoPrefixContractConsignment)
	if err != nil {
		return errors.New("提交失败")
	}
	contract := model.ContractConsignment{
		CustomerId:               data.CustomerId,
		BrokerId:                 brokerId,
		BrokerPriceId:            carPrice.Id,
		ContractSn:               contractSn,
		Vin:                      carPrice.Vin,
		ModelId:                  carPrice.ModelId,
		LicensingDate:            carPrice.LicensingDate,
		LicensePlate:             data.LicensePlate,
		Color:                    carPrice.Color,
		Engine:                   carPrice.Engine,
		Mileage:                  carPrice.Mileage,
		Transfer:                 carPrice.Transfer,
		KeyCount:                 data.KeyCount,
		UseCharacter:             data.UseCharacter,
		CompulsoryInsurance:      data.CompulsoryInsurance,
		CommercialInsurance:      data.CommercialInsurance,
		CommercialInsurancePrice: data.CommercialInsurancePrice,
		ReckonPrice:              carPrice.Price,
		Price:                    data.Price,
		//Exterior:                 data.Exterior,
		//InteriorTrim:             data.InteriorTrim,
		//WorkingCondition:         data.WorkingCondition,
		ServiceFeeRate: data.ServiceFeeRate,
		Status:         constant.ContractStatusCreate,
		Master: model.ContractMaster{
			ContractType: model.ContractMasterTypeConsignment,
			Kind:         model.ContractMasterKindPerson,
			Name:         data.MasterInfo.Name,
			IdCard:       data.MasterInfo.IdCard,
			Phone:        data.MasterInfo.Phone,
			BankCard:     data.MasterInfo.BankCard,
			Bank:         data.MasterInfo.Bank,
			BankAccount:  data.MasterInfo.Name,
			IdCardFront:  data.MasterInfo.IdCardFront,
			IdCardBack:   data.MasterInfo.IdCardBack,
			//Photo:       data.MasterInfo.Photo,
		},
	}
	// 交强险
	if data.CompulsoryInsurance == model.ContractConsignmentCompulsoryInsuranceTrue {
		compulsoryInsurance, err := time.ParseInLocation("2006-01-02", data.CompulsoryInsuranceDate, time.Local)
		if err != nil {
			return errors.New("交强险到期时间错误")
		}
		contract.CompulsoryInsuranceDate = sql.NullTime{Time: compulsoryInsurance, Valid: true}
	}
	// 商业险
	if data.CommercialInsurance == model.ContractConsignmentCommercialInsuranceTrue {
		commercialInsuranceDate, err := time.ParseInLocation("2006-01-02", data.CommercialInsuranceDate, time.Local)
		if err != nil {
			return errors.New("商业险到期时间错误")
		}
		contract.CommercialInsuranceDate = sql.NullTime{Time: commercialInsuranceDate, Valid: true}
	}

	orderSn, err := common.GenNo(constant.NoPrefixOrderConsignment)
	if err != nil {
		return errors.New("提交失败")
	}

	// 腾讯电子签
	consignmentParams := model.ConsignmentParams
	var flowApproverInfos []*essbasic.FlowApproverInfo
	flowApproverInfos = append(flowApproverInfos, tencent.BuildPersonApprover(contract.Master.Name, contract.Master.Phone, consignmentParams["RecipientId1"]))
	var params map[string]string

	params = map[string]string{
		"P1A1B1": contract.Master.Name,
		"P1A1B2": contract.Master.Phone,
		"P1A1B3": contract.Master.IdCard,
		"P1A2B1": contract.Master.Name,
		"P1A2B2": contract.LicensePlate,
		"P1A2B3": carModel.Name,
		"P1A2B4": contract.Vin,
		"P1A2B5": contract.Engine,
		"P1A2B6": contract.LicensingDate.Format("2006-01-02"),
		"P1A2B7": contract.Mileage.String(),
		"P1A4B1": common.ConvertDecimalToCny(contract.Price),
		"P1A4B2": contract.Price.String(),
		"P2A1B1": contract.Master.Name,
		"P2A1B2": contract.Master.BankCard,
		"P2A1B3": contract.Master.Bank,
		"P2A2B1": time.Now().Format("2006-01-02"),
		"P2A2B2": time.Now().Add(25 * 24 * time.Hour).Format("2006-01-02"),
		"P2A3B1": "true",
		"P2A3B4": strconv.Itoa(int(contract.KeyCount)),
		//"P3A1B1"
	}
	// 车辆保险
	if contract.CommercialInsurance != 0 || contract.CompulsoryInsurance != 0 {
		params["P2A3B3"] = "true"
	}
	// 车辆使用性质选项卡
	switch contract.UseCharacter {
	case constant.CarBaseUseCharacterNonCommercial: // 非运营
		params["P1A3B1"] = "true"
	case constant.CarBaseUseCharacterCommercial: // 运营
		params["P1A3B2"] = "true"
	case constant.CarBaseUseCharacterCommercialToNonCommercial: // 营转非
		params["P1A3B3"] = "true"
	case constant.CarBaseUseCharacterTenancyNonCommercial: // 租赁非运营
		params["P1A3B4"] = "true"
	case constant.CarBaseUseCharacterTenancyCommercial: // 租赁运营
		params["P1A3B5"] = "true"
	}
	var formFields []*essbasic.FormField
	for key, value := range params {
		formFields = append(formFields, tencent.BuildFormFieldById(consignmentParams[key], value))
	}

	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		if tx.Create(&contract).Error != nil {
			return errors.New("提交失败")
		}
		orderConsignment := model.OrderConsignment{
			CustomerId:    data.CustomerId,
			BrokerId:      brokerId,
			OrderSn:       orderSn,
			ModelId:       contract.ModelId,
			Vin:           contract.Vin,
			ContractId:    contract.Id,
			Status:        model.OrderConsignmentStatusUnSign,
			ContractPrice: contract.Price,
			Histories: []model.OrderConsignmentHistory{{
				Type:     model.OrderConsignmentHistoryTypeCreate,
				UserId:   brokerId,
				UserType: model.OrderConsignmentHistoryUserTypeBroker,
			}},
		}
		if tx.Create(&orderConsignment).Error != nil {
			return errors.New("提交失败")
		}
		message.MessageLogic.MessageToCustomer(mysql.Db, []uint{data.CustomerId}, brokerId, model.CustomerMessageTypeContract, message.MessageItem{
			Title:   "合同提示",
			Content: "您有一份寄售合同等待签订"})
		if err = wallet.AssessLogic.Use(tx, model.WalletAssessKindContract, model.WalletAssessHistoryKindContractConsignment, brokerId, contract.Id, 1); err != nil {
			return err
		}
		_, err = tencent.Ess.CreateFlowsByTemplates(formFields, flowApproverInfos, contractSn, consignmentParams["FlowName"], consignmentParams["TemplateId"])
		if err != nil {
			return err
		}
		return nil
	})
}

// Cancel @Title 取消合同
func (c *consignmentLogic) Cancel(brokerId, contractId uint, reason string) error {
	contract := model.ContractConsignment{Id: contractId, BrokerId: brokerId}
	if mysql.Db.Preload("EssFlow").Where(&contract).First(&contract).Error != nil {
		return errors.New("合同错误")
	}
	if contract.Status != constant.ContractStatusCreate {
		return errors.New("合同不在待签订状态")
	}
	if err := tencent.Ess.ChannelCancelFlow([]*string{&contract.EssFlow.FlowId}); err != nil {
		return err
	}
	return nil
}
