package contract

import (
	"base/app/common"
	"base/app/config"
	"base/app/constant"
	"base/app/lib/bean"
	"base/app/lib/tencent"
	"base/app/logic/broker/message"
	"base/app/logic/broker/user/wallet"
	"base/app/model"
	"database/sql"
	"errors"
	"fmt"
	"git.oa00.com/go/mysql"
	"github.com/shopspring/decimal"
	essbasic "github.com/tencentcloud/tencentcloud-sdk-go/tencentcloud/essbasic/v20210526"
	"gorm.io/gorm"
	"time"
)

var PurchaseLogic = &purchaseLogic{}

type purchaseLogic struct {
}

type purchaseItem struct {
	Id            uint   `json:"id"`
	ContractSn    string `json:"contractSn"`
	ModelName     string `json:"modelName"`
	Status        uint   `json:"status"`
	StatusName    string `json:"statusName"`
	BrokerId      uint   `json:"brokerId"`
	BrokerName    string `json:"brokerName"`
	CustomerId    uint   `json:"customerId"`
	CustomerName  string `json:"customerName"`
	DealerId      uint   `json:"dealerId"`
	DealerName    string `json:"dealerName"`
	DealerAuditAt int64  `json:"dealerAuditAt"`
	SignAt        int64  `json:"signAt"`
	CreatedAt     int64  `json:"createdAt"`
	EssFileUrl    string `json:"essFileUrl"`
}
type PurchaseSearch struct {
	Status           uint
	ContractSn       string
	ModelName        string
	CustomerName     string
	CustomerTel      string
	CreatedTimeStart string
	CreatedTimeEnd   string
}

type PurchaseBrokerSearch struct {
	DealerName string
	DealerTel  string
}

// BrokerLists @Title 经纪人发布列表
func (p *purchaseLogic) BrokerLists(brokerId uint, search PurchaseSearch, searchBroker PurchaseBrokerSearch, page bean.Page) (lists []purchaseItem, total int64) {
	return p.Lists([]uint{brokerId}, []uint{}, search, searchBroker, PurchaseDealerSearch{}, page)
}

type PurchaseDealerSearch struct {
	BrokerName string
	BrokerTel  string
}

// DealerLists @Title 车商审批车辆发布列表
func (p *purchaseLogic) DealerLists(dealerId uint, search PurchaseSearch, searchDealer PurchaseDealerSearch, page bean.Page) (lists []purchaseItem, total int64) {
	return p.Lists([]uint{}, []uint{dealerId}, search, PurchaseBrokerSearch{}, searchDealer, page)
}

// Lists @Title 列表
func (p *purchaseLogic) Lists(brokerIds, dealerIds []uint, search PurchaseSearch, searchBroker PurchaseBrokerSearch, searchDealer PurchaseDealerSearch, page bean.Page) (lists []purchaseItem, total int64) {
	lists = []purchaseItem{}
	var contractPurchases []model.ContractPurchase
	where := mysql.Db
	if len(brokerIds) > 0 {
		where = where.Where("broker_id in ?", brokerIds)
	}
	if len(dealerIds) > 0 {
		where = where.Where("dealer_id in ?", dealerIds)
	}
	if search.Status > 0 {
		where = where.Where("status = ?", search.Status)
	}
	if search.ContractSn != "" {
		where = where.Where("contract_sn like ?", fmt.Sprintf("%%%s%%", search.ContractSn))
	}
	if search.ModelName != "" {
		where = where.Where("model_id in (?)", mysql.Db.Model(&model.CarModel{}).Where("name like ?", fmt.Sprintf("%%%s%%", search.ModelName)).Select("model_id"))
	}
	if search.CreatedTimeStart != "" {
		where = where.Where("created_at > ?", search.CreatedTimeStart)
	}
	if search.CreatedTimeEnd != "" {
		where = where.Where("created_at < ?", search.CreatedTimeEnd)
	}
	{
		customerWhere := mysql.Db.Model(&model.Customer{}).Select("id")
		if search.CustomerName != "" {
			customerWhere = customerWhere.Where("nickname like ?", fmt.Sprintf("%%%s%%", search.CustomerName))
		}
		if search.CustomerTel != "" {
			customerWhere = customerWhere.Where("phone like ?", fmt.Sprintf("%%%s%%", search.CustomerName))
		}
		if _, ok := customerWhere.Statement.Clauses["WHERE"]; ok {
			where = where.Where("customer_id in (?)", customerWhere)
		}
	}
	{
		dealerWhere := mysql.Db.Model(&model.BrokerBusinessStore{}).Select("id")
		if searchBroker.DealerName != "" {
			dealerWhere = dealerWhere.Where("name like ?", fmt.Sprintf("%%%s%%", searchBroker.DealerName))
		}
		if searchBroker.DealerTel != "" {
			dealerWhere = dealerWhere.Where("phone like ?", fmt.Sprintf("%%%s%%", searchBroker.DealerTel))
		}
		if _, ok := dealerWhere.Statement.Clauses["WHERE"]; ok {
			where = where.Where("dealer_id in (?)", dealerWhere)
		}
	}
	{
		brokerWhere := mysql.Db.Model(&model.Broker{}).Select("id")
		if searchDealer.BrokerName != "" {
			brokerWhere = brokerWhere.Where("nickname like ?", fmt.Sprintf("%%%s%%", searchDealer.BrokerName))
		}
		if searchDealer.BrokerTel != "" {
			brokerWhere = brokerWhere.Where("phone like ?", fmt.Sprintf("%%%s%%", searchDealer.BrokerTel))
		}
		if _, ok := brokerWhere.Statement.Clauses["WHERE"]; ok {
			where = where.Where("broker_id in (?)", brokerWhere)
		}
	}
	mysql.Db.Model(&contractPurchases).Where(where).Count(&total)
	if page.HasPage(total) {
		mysql.Db.
			Preload("Model").Preload("DealerAudit").
			Preload("Broker").Preload("Customer").Preload("Dealer").Preload("EssFlow").
			Where(where).Order("id desc").Offset(page.GetStart()).Limit(page.GetLimit()).Find(&contractPurchases)
		for _, item := range contractPurchases {
			lists = append(lists, purchaseItem{
				Id:            item.Id,
				ContractSn:    item.ContractSn,
				ModelName:     item.Model.Name,
				Status:        item.Status,
				StatusName:    constant.GetContractStatusName(item.Status),
				BrokerId:      item.BrokerId,
				BrokerName:    item.Broker.Nickname,
				CustomerId:    item.CustomerId,
				CustomerName:  item.Customer.Nickname,
				DealerId:      item.DealerId,
				DealerName:    item.Dealer.Name,
				DealerAuditAt: item.DealerAudit.AuditAt.Time.Unix(),
				SignAt:        item.SignAt.Time.Unix(),
				CreatedAt:     item.CreatedAt.Unix(),
				EssFileUrl:    item.EssFlow.Url,
			})
		}
	}
	return
}

type PurchaseAdd struct {
	BaseInfo   purchaseBaseAdd     `binding:"required,dive" label:"基础信息"`
	MasterInfo purchaseMasterAdd   `binding:"required,dive" label:"车主信息"`
	PriceInfo  purchasePriceAdd    `binding:"required,dive" label:"费用信息"`
	Photos     purchaseCarPhotoAdd `binding:"required,dive" label:"车辆照片"`
}
type purchaseBaseAdd struct {
	Channel                 uint            `binding:"required,oneof=1 2" label:"渠道"` // 1=入驻商 2=车商
	ModelId                 uint            `binding:"required" label:"车型id"`
	Vin                     string          `binding:"required" label:"车架号"`
	EngineNo                string          `binding:"required" label:"发动机号"`
	LicensingDate           string          `binding:"required" label:"上牌日期"`
	LicensePlate            string          `binding:"required" label:"车牌"`
	UseCharacter            uint            `binding:"required" label:"使用性质"`
	Mileage                 decimal.Decimal `binding:"required" label:"行驶里程"`
	CompulsoryInsuranceDate string          `binding:"required" label:"交强险到期日期"`
	Condition               string          `binding:"required" label:"车况"`
	Color                   string          `binding:"required" label:"车身颜色"`
}
type purchaseMasterAdd struct {
	CustomerId uint   `label:"客户id"`
	Kind       uint   `binding:"required,oneof=1 2" label:"客户身份1=个人2=公司"`
	Name       string `binding:"required" label:"姓名/公司名"`
	Phone      string `binding:"required" label:"手机号"`
	LegalName  string `label:"法人姓名"`
	IdCard     string `binding:"required" label:"身份证号/公司机构代码"`
	BankCard   string `binding:"required" label:"银行卡号"`
	Bank       string `binding:"required" label:"开户行"`
}

type purchasePriceAdd struct {
	DealPrice   decimal.Decimal `binding:"required" label:"成交金额"`
	DownPayment decimal.Decimal `binding:"required" label:"首付"`
	//DownPaymentRate    decimal.Decimal `label:"首付占比"`
	//BalancePaymentRate decimal.Decimal `label:"尾款占比"`
	DeliverDate string `binding:"required" label:"交付日期"`
}

type purchaseCarPhotoAdd struct {
	CarPhotos  []carPhoto `binding:"required,min=1" label:"车辆照片"`
	DataPhotos []carPhoto `binding:"required,min=1" label:"资料照片"`
}

type carPhoto struct {
	Photo string `binding:"required" label:"图片地址"`
	Text  string `binding:"required" label:"图片描述"`
}

// Add @Title 添加收购合同
func (p *purchaseLogic) Add(brokerId, businessId, storeId uint, data PurchaseAdd) error {
	// 车辆所有权 入驻商/车商
	store := model.BrokerBusinessStore{}
	switch data.BaseInfo.Channel {
	case model.ContractPurchaseChannelBusiness:
		business := model.BrokerBusiness{Id: businessId}
		if mysql.Db.Preload("OwnerBroker.Store.Company").Where(&business).First(&business).Error != nil {
			return errors.New("未找到入驻商信息")
		}
		store = business.OwnerBroker.Store
	case model.ContractPurchaseChannelDealer:
		store = model.BrokerBusinessStore{Id: storeId}
		if mysql.Db.Preload("Company").Where(&store).First(&store).Error != nil {
			return errors.New("未找到门店信息")
		}
	}

	carModel := model.CarModel{ModelId: data.BaseInfo.ModelId}
	if mysql.Db.Where(&carModel).First(&carModel).Error != nil {
		return errors.New("查找车型错误")
	}
	contractSn, err := common.GenNo(constant.NoPrefixContractPurchase)
	if err != nil {
		return errors.New("创建单号失败")
	}
	if data.PriceInfo.DownPayment.LessThanOrEqual(decimal.Zero) {
		return errors.New("定金错误")
	}
	balancePayment := data.PriceInfo.DealPrice.Sub(data.PriceInfo.DownPayment)
	if balancePayment.LessThanOrEqual(decimal.Zero) {
		return errors.New("金额分配异常")
	}
	contract := model.ContractPurchase{
		DealerId:     store.Id,
		BrokerId:     brokerId,
		Channel:      data.BaseInfo.Channel,
		ContractSn:   contractSn,
		Vin:          data.BaseInfo.Vin,
		ModelId:      data.BaseInfo.ModelId,
		LicensePlate: data.BaseInfo.LicensePlate,
		EngineNo:     data.BaseInfo.EngineNo,
		UseCharacter: data.BaseInfo.UseCharacter,
		Mileage:      data.BaseInfo.Mileage,
		Color:        data.BaseInfo.Color,
		Condition:    data.BaseInfo.Condition,

		DealPrice:      data.PriceInfo.DealPrice,
		DownPayment:    data.PriceInfo.DownPayment,
		BalancePayment: balancePayment,
		Status:         constant.ContractStatusCreate,
		Master: model.ContractMaster{
			ContractType: model.ContractMasterTypePurchase,
			Kind:         data.MasterInfo.Kind,
			Name:         data.MasterInfo.Name,
			LegalPerson:  data.MasterInfo.LegalName,
			IdCard:       data.MasterInfo.IdCard,
			Phone:        data.MasterInfo.Phone,
			BankAccount:  data.MasterInfo.Name,
			BankCard:     data.MasterInfo.BankCard,
			Bank:         data.MasterInfo.Bank,
		},
	}

	// 客户身份
	if data.MasterInfo.CustomerId > 0 {
		customer := model.Customer{Id: data.MasterInfo.CustomerId}
		if mysql.Db.Where(&customer).First(&customer).Error != nil {
			return errors.New("客户有误")
		}
		contract.CustomerId = customer.Id
	}
	if contract.Master.Kind == model.ContractMasterKindCompany {
		if data.MasterInfo.LegalName == "" {
			return errors.New("客户为公司请填写法人名")
		}
		contract.Master.LegalPerson = data.MasterInfo.LegalName
	}

	// 交强险到期日期
	if data.BaseInfo.CompulsoryInsuranceDate != "" {
		compulsoryInsurance, err := time.ParseInLocation("2006-01-02", data.BaseInfo.CompulsoryInsuranceDate, time.Local)
		if err != nil {
			return errors.New("交强险到期时间错误")
		}
		contract.CompulsoryInsuranceDate = sql.NullTime{Time: compulsoryInsurance, Valid: true}
	}
	// 上牌日期
	if data.BaseInfo.LicensingDate != "" {
		licensingDate, err := time.ParseInLocation("2006-01-02", data.BaseInfo.LicensingDate, time.Local)
		if err != nil {
			return errors.New("上牌日期错误")
		}
		contract.LicensingDate = sql.NullTime{Time: licensingDate, Valid: true}
	}
	// 交付时间
	if data.PriceInfo.DeliverDate != "" {
		deliverDate, err := time.ParseInLocation("2006-01-02", data.PriceInfo.DeliverDate, time.Local)
		if err != nil {
			return errors.New("交付时间错误")
		}
		contract.DeliverDate = sql.NullTime{Time: deliverDate, Valid: true}
	}
	for _, photo := range data.Photos.CarPhotos {
		if photo.Photo != "" {
			contract.Photos = append(contract.Photos, model.ContractPurchasePhoto{
				Path: photo.Photo,
				Text: photo.Text,
				Type: model.CarPhotoTypeCar,
			})
		}
	}
	for _, photo := range data.Photos.DataPhotos {
		if photo.Photo != "" {
			contract.Photos = append(contract.Photos, model.ContractPurchasePhoto{
				Path: photo.Photo,
				Text: photo.Text,
				Type: model.CarPhotoTypeData,
			})
		}
	}
	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		//  提交至车商审核
		if brokerId != store.OwnerBrokerId {
			contract.Status = constant.ContractStatusDealerAuditing
			contract.DealerAudit = model.DealerAuditing{
				DealerId:      store.Id,
				ApplyBrokerId: brokerId,
				Type:          model.DealerAuditingTypePurchase,
				Status:        constant.AuditStatusWaiting,
			}
			if tx.Create(&contract).Error != nil {
				return errors.New("车辆信息发布失败")
			}
			contract.DealerAudit.LinkId = contract.Id
			if tx.Updates(&contract.DealerAudit).Error != nil {
				return errors.New("车主审核生成失败")
			}
			message.MessageLogic.MessageToBroker(mysql.Db, []uint{contract.BrokerId}, 0, model.BrokerMessageTypeCarAudit, message.MessageItem{
				Title:    "合同提示",
				Content:  "您有一个新的收车合同已提交至车商审批",
				LinkedId: contract.Id,
				Extras: map[string]interface{}{
					"车辆名称": carModel.Name,
					"收购价格": contract.DealPrice,
					"发起时间": contract.DealerAudit.CreatedAt.Format("2006-01-02 15:04:05"),
				},
			})
			message.MessageLogic.MessageToBroker(mysql.Db, []uint{store.OwnerBrokerId}, 0, model.BrokerMessageTypeCarAudit, message.MessageItem{
				Title:    "合同提示",
				Content:  "您有一个新的收车合同待审批",
				LinkedId: contract.Id,
				Extras: map[string]interface{}{
					"车辆名称": carModel.Name,
					"收购价格": contract.DealPrice,
					"发起时间": contract.DealerAudit.CreatedAt.Format("2006-01-02 15:04:05"),
				},
			})
			return nil
		}
		contract.Status = constant.ContractStatusCreate
		if tx.Create(&contract).Error != nil {
			return errors.New("提交失败")
		}
		if err = p.purchase(tx, contract, carModel, store); err != nil {
			return err
		}
		return nil
	})
}

type purchaseInfo struct {
	Id              uint                    `json:"id"`
	Status          uint                    `json:"status"`
	StatusName      string                  `json:"statusName"`
	EssFileUrl      string                  `json:"essFileUrl"`
	CreatedAt       int64                   `json:"createdAt"`
	BaseInfo        purchaseBaseInfo        `json:"baseInfo"`
	DealerInfo      purchaseDealerInfo      `json:"dealerInfo"`
	BrokerInfo      purchaseBrokerInfo      `json:"brokerInfo"`
	CustomerInfo    purchaseCustomerInfo    `json:"customerInfo"`
	MasterInfo      purchaseMasterInfo      `json:"masterInfo"`
	PriceInfo       purchasePriceInfo       `json:"priceInfo"`
	Photos          purchaseCarPhoto        `json:"photos"`
	DealerAuditInfo purchaseDealerAuditInfo `json:"dealerAuditInfo"`
}
type purchaseBaseInfo struct {
	ModelId                 uint            `json:"modelId"`
	ModelName               string          `json:"modelName"`
	ContractSn              string          `json:"contractSn"`
	Vin                     string          `json:"vin"`
	EngineNo                string          `json:"engineNo"`
	LicensingDate           int64           `json:"licensingDate"`
	LicensePlate            string          `json:"licensePlate"`
	UseCharacter            string          `json:"useCharacter"`
	Mileage                 decimal.Decimal `json:"mileage"`
	CompulsoryInsuranceDate int64           `json:"compulsoryInsuranceDate"`
	Condition               string          `json:"condition"`
	Color                   string          `json:"color"`
}

type purchaseDealerInfo struct {
	Channel         uint   `json:"channel"`
	ChannelName     string `json:"channelName"`
	Id              uint   `json:"id"`
	Name            string `json:"name"`
	OwnerBrokerId   uint   `json:"ownerBrokerId"`
	OwnerBrokerName string `json:"ownerBrokerName"`
	OwnerBrokerTel  string `json:"ownerBrokerTel"`
}

type purchaseBrokerInfo struct {
	Id   uint   `json:"id"`
	Name string `json:"name"`
	Tel  string `json:"tel"`
}

type purchaseCustomerInfo struct {
	Id   uint   `json:"id"`
	Name string `json:"name"`
	Tel  string `json:"tel"`
}

type purchaseMasterInfo struct {
	Id          uint   `json:"id"`
	Kind        uint   `json:"kind"`
	KindName    string `json:"kindName"`
	Name        string `json:"name"`
	Phone       string `json:"phone"`
	LegalPerson string `json:"legalPerson"`
	IdCard      string `json:"idCard"`
	BankCard    string `json:"bankCard"`
	Bank        string `json:"bank"`
}

type purchasePriceInfo struct {
	DealPrice      decimal.Decimal `json:"dealPrice"`
	DownPayment    decimal.Decimal `json:"downPayment"`
	BalancePayment decimal.Decimal `json:"balancePayment"`
	DeliverDate    int64           `json:"deliverDate"`
}

type purchaseCarPhoto struct {
	CarPhotos  []carPhotoInfo `json:"carPhotos"`
	DataPhotos []carPhotoInfo `json:"dataPhotos"`
}

type carPhotoInfo struct {
	Photo string `json:"photo"`
	Text  string `json:"text"`
}

type purchaseDealerAuditInfo struct {
	Id                 uint   `json:"id"`
	DealerAuditAt      int64  `json:"dealerAuditAt"`
	DealerRejectReason string `json:"dealerRejectReason"`
}

// Info @Title 详情
func (p *purchaseLogic) Info(contractId uint) (result purchaseInfo, err error) {
	contract := model.ContractPurchase{Id: contractId}
	if mysql.Db.
		Preload("Model").Preload("Photos").Preload("Master").
		Preload("Broker").Preload("Customer").Preload("Dealer.OwnerBroker").
		Preload("EssFlow").Preload("DealerAudit").
		First(&contract).Error != nil {
		return result, errors.New("收购合同错误")
	}
	result = purchaseInfo{
		Id:         contract.Id,
		Status:     contract.Status,
		StatusName: constant.GetContractStatusName(contract.Status),
		EssFileUrl: contract.EssFlow.Url,
		CreatedAt:  contract.CreatedAt.Unix(),
		BaseInfo: purchaseBaseInfo{
			ModelId:                 contract.ModelId,
			ModelName:               contract.Model.Name,
			ContractSn:              contract.ContractSn,
			Vin:                     contract.Vin,
			EngineNo:                contract.EngineNo,
			LicensingDate:           contract.LicensingDate.Time.Unix(),
			LicensePlate:            contract.LicensePlate,
			UseCharacter:            constant.GetCarUseCharacterName(contract.UseCharacter),
			Mileage:                 contract.Mileage,
			CompulsoryInsuranceDate: contract.CompulsoryInsuranceDate.Time.Unix(),
			Condition:               contract.Condition,
			Color:                   contract.Color,
		},
		DealerInfo: purchaseDealerInfo{
			Channel:         contract.Channel,
			ChannelName:     contract.GetChannelName(contract.Channel),
			Id:              contract.DealerId,
			Name:            contract.Dealer.Name,
			OwnerBrokerId:   contract.Dealer.OwnerBrokerId,
			OwnerBrokerName: contract.Dealer.OwnerBroker.Nickname,
			OwnerBrokerTel:  contract.Dealer.OwnerBroker.Phone,
		},
		BrokerInfo: purchaseBrokerInfo{
			Id:   contract.BrokerId,
			Name: contract.Broker.Nickname,
			Tel:  contract.Broker.Phone,
		},
		MasterInfo: purchaseMasterInfo{
			Id:          contract.MasterId,
			Kind:        contract.Master.Kind,
			KindName:    contract.Master.GetKindName(contract.Master.Kind),
			Name:        contract.Master.Name,
			Phone:       contract.Master.Phone,
			LegalPerson: contract.Master.LegalPerson,
			IdCard:      contract.Master.IdCard,
			BankCard:    contract.Master.BankCard,
			Bank:        contract.Master.Bank,
		},
		PriceInfo: purchasePriceInfo{
			DealPrice:      contract.DealPrice,
			DownPayment:    contract.DownPayment,
			BalancePayment: contract.BalancePayment,
			DeliverDate:    contract.DeliverDate.Time.Unix(),
		},
		Photos: purchaseCarPhoto{
			CarPhotos:  []carPhotoInfo{},
			DataPhotos: []carPhotoInfo{},
		},
		DealerAuditInfo: purchaseDealerAuditInfo{
			Id:                 contract.DealerAuditId,
			DealerAuditAt:      contract.DealerAudit.AuditAt.Time.Unix(),
			DealerRejectReason: contract.DealerAudit.Reason,
		},
	}
	for _, photo := range contract.Photos {
		switch photo.Type {
		case model.CarPhotoTypeCar: // 车辆照片
			result.Photos.CarPhotos = append(result.Photos.CarPhotos, carPhotoInfo{
				Photo: photo.Path,
				Text:  photo.Text,
			})
		case model.CarPhotoTypeData: // 资料照片
			result.Photos.DataPhotos = append(result.Photos.DataPhotos, carPhotoInfo{
				Photo: photo.Path,
				Text:  photo.Text,
			})
		}
	}
	if contract.CustomerId > 0 {
		customer := model.Customer{Id: contract.CustomerId}
		if mysql.Db.First(&customer).Error == nil {
			result.CustomerInfo = purchaseCustomerInfo{
				Id:   customer.Id,
				Name: customer.Nickname,
				Tel:  customer.Mobile,
			}
		}
	}
	return
}

// Audit @Title 审核通过/驳回
func (p *purchaseLogic) Audit(dealerId, contractId uint, choose uint, reason string) error {
	contract := model.ContractPurchase{Id: contractId}
	if mysql.Db.Preload("DealerAudit").Preload("Master").First(&contract).Error != nil {
		return errors.New("合同id错误")
	}
	if contract.Status != constant.ContractStatusDealerAuditing {
		return errors.New("不在车商审核状态")
	}
	if contract.DealerId != dealerId {
		return errors.New("无审核权限")
	}
	store := model.BrokerBusinessStore{Id: contract.DealerId}
	if mysql.Db.Preload("Company").Where(&store).First(&store).Error != nil {
		return errors.New("未找到车商信息")
	}
	carModel := model.CarModel{ModelId: contract.ModelId}
	if mysql.Db.Where(&carModel).First(&carModel).Error != nil {
		return errors.New("查找车型错误")
	}
	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		switch choose {
		case 1: // 审核通过
			if tx.Model(&contract.DealerAudit).UpdateColumns(map[string]interface{}{
				"status":   constant.AuditStatusAdopt,
				"audit_at": time.Now(),
			}).RowsAffected != 1 {
				return errors.New("审核失败")
			}
			if tx.Model(&model.ContractPurchase{}).Where("id = ?", contractId).Updates(map[string]interface{}{
				"status": constant.ContractStatusDealerAdopt,
			}).RowsAffected != 1 {
				return errors.New("合同状态修改失败")
			}
			if err := p.purchase(tx, contract, carModel, store); err != nil {
				return err
			}
			if tx.Model(&model.ContractPurchase{}).Where("id = ?", contractId).Updates(map[string]interface{}{
				"status": constant.ContractStatusCreate,
			}).RowsAffected != 1 {
				return errors.New("合同状态修改失败")
			}
			message.MessageLogic.MessageToBroker(mysql.Db, []uint{contract.BrokerId}, 0, model.BrokerMessageTypeCarAudit, message.MessageItem{
				Title:    "合同提示",
				Content:  "您的一个收车合同已被车商审批通过",
				LinkedId: contract.Id,
				Extras: map[string]interface{}{
					"车辆名称": carModel.Name,
					"收购价格": contract.DealPrice,
					"发起时间": contract.DealerAudit.AuditAt.Time.Format("2006-01-02 15:04:05"),
				},
			})
			message.MessageLogic.MessageToBroker(mysql.Db, []uint{store.OwnerBrokerId}, 0, model.BrokerMessageTypeCarAudit, message.MessageItem{
				Title:    "合同提示",
				Content:  "您已通过收车合同的审批",
				LinkedId: contract.Id,
				Extras: map[string]interface{}{
					"车辆名称": carModel.Name,
					"收购价格": contract.DealPrice,
					"发起时间": contract.DealerAudit.AuditAt.Time.Format("2006-01-02 15:04:05"),
				},
			})
			return nil
		case 2: // 审核拒绝
			if tx.Model(&contract.DealerAudit).UpdateColumns(map[string]interface{}{
				"status":   constant.AuditStatusReject,
				"reason":   reason,
				"audit_at": time.Now(),
			}).RowsAffected != 1 {
				return errors.New("审核失败")
			}
			if tx.Model(&model.ContractPurchase{}).Where(&contract).Updates(map[string]interface{}{
				"status": constant.ContractStatusDealerReject,
			}).RowsAffected != 1 {
				return errors.New("合同状态修改失败")
			}
			message.MessageLogic.MessageToBroker(mysql.Db, []uint{contract.BrokerId}, 0, model.BrokerMessageTypeCarAudit, message.MessageItem{
				Title:    "合同提示",
				Content:  "您的一个收车合同已被车商审批拒绝",
				LinkedId: contract.Id,
				Extras: map[string]interface{}{
					"车辆名称": carModel.Name,
					"收购价格": contract.DealPrice,
					"发起时间": contract.DealerAudit.AuditAt.Time.Format("2006-01-02 15:04:05"),
				},
			})
			message.MessageLogic.MessageToBroker(mysql.Db, []uint{store.OwnerBrokerId}, 0, model.BrokerMessageTypeCarAudit, message.MessageItem{
				Title:    "合同提示",
				Content:  "您已拒绝收车合同的审批",
				LinkedId: contract.Id,
				Extras: map[string]interface{}{
					"车辆名称": carModel.Name,
					"收购价格": contract.DealPrice,
					"发起时间": contract.DealerAudit.AuditAt.Time.Format("2006-01-02 15:04:05"),
				},
			})
			return nil
		}
		return nil
	})
}

// Sale @Title 出售合同后置逻辑
func (p *purchaseLogic) purchase(tx *gorm.DB, contract model.ContractPurchase, carModel model.CarModel, dealerInfo model.BrokerBusinessStore) error {
	if err := wallet.AssessLogic.Use(tx, model.WalletAssessKindContract, model.WalletAssessHistoryKindContractPurchase, dealerInfo.OwnerBrokerId, contract.Id, 1); err != nil {
		return err
	}
	if err := p.PurchaseEss(contract, carModel, dealerInfo); err != nil {
		return err
	}
	message.MessageLogic.MessageToCustomer(mysql.Db, []uint{contract.CustomerId}, 0, model.CustomerMessageTypeContract, message.MessageItem{
		Title:   "合同提示",
		Content: "您有一个新的卖车合同待签订",
		Extras:  map[string]interface{}{"contractId": contract.Id},
	})
	message.MessageLogic.MessageToBroker(mysql.Db, []uint{dealerInfo.OwnerBrokerId}, 0, model.BrokerMessageTypeCarAudit, message.MessageItem{
		Title:    "合同提示",
		Content:  "您有一个新的收车合同待签订",
		LinkedId: contract.Id,
		Extras: map[string]interface{}{
			"车辆名称": carModel.Name,
			"收购价格": contract.DealPrice,
			"发起时间": contract.CreatedAt.Format("2006-01-02 15:04:05"),
		},
	})
	return nil
}

// PurchaseEss @Title 发起收购ess合同
func (p *purchaseLogic) PurchaseEss(contract model.ContractPurchase, carModel model.CarModel, store model.BrokerBusinessStore) error {
	var purchaseParams map[string]string
	var params map[string]string
	var flowApproverInfos []*essbasic.FlowApproverInfo

	switch contract.Master.Kind {
	case model.ContractMasterKindPerson:
		purchaseParams = model.PurchasePersonParams
		approverSalePart := tencent.BuildPersonApprover(contract.Master.Name, contract.Master.Phone, purchaseParams["RecipientId1"])
		approverPurchasePart := tencent.BuildNotChannelOrganizationApprover(store.Company.LegalPerson, store.Company.LegalTel, store.Company.Name, purchaseParams["RecipientId2"])
		flowApproverInfos = append(flowApproverInfos, approverSalePart, approverPurchasePart)
	case model.ContractMasterKindCompany:
		purchaseParams = model.PurchaseCompanyParams
		approverSalePart := tencent.BuildNotChannelOrganizationApprover(contract.Master.LegalPerson, contract.Master.Phone, contract.Master.Name, purchaseParams["RecipientId1"])
		approverPurchasePart := tencent.BuildNotChannelOrganizationApprover(store.Company.LegalPerson, store.Company.LegalTel, store.Company.Name, purchaseParams["RecipientId2"])
		flowApproverInfos = append(flowApproverInfos, approverSalePart, approverPurchasePart)
	}

	params = map[string]string{
		"P1A1B1": contract.Master.Name,
		"P1A1B2": contract.Master.LegalPerson,
		"P1A1B3": contract.Master.IdCard,
		"P1A1B4": contract.Master.Phone,
		"P1A1B5": contract.Master.Address,
		"P1A2B1": store.Company.Name,
		"P1A2B2": store.Company.BusinessLicenseNo,
		"P1A2B3": store.Address,
		"P1A2B4": store.Company.LegalTel,
		"P2A1B1": carModel.Name,
		"P2A1B2": contract.Vin,
		"P2A1B3": contract.LicensingDate.Time.Format("2006-01-02"),
		//"P2A1B4":       constant.GetCarTypeName(contract.CarType), // 车辆类型
		"P2A1B5": contract.EngineNo,
		"P2A1B6": contract.CompulsoryInsuranceDate.Time.Format("2006-01-02"),
		"P2A1B7": contract.LicensePlate, // 车辆牌照
		"P2A1B8": contract.Mileage.String() + "公里",
		"P2A1B9": constant.GetCarUseCharacterName(contract.UseCharacter),
		"P3A1B1": common.ConvertDecimalToCny(contract.DealPrice), // 成交金额中文
		"P3A1B2": contract.DealPrice.String(),                    // 成交金额
		//"P3A1B3": strconv.Itoa(30),                                    // 首付时限
		"P3A1B4": common.ConvertDecimalToCny(contract.DownPayment), //首付大写
		"P3A1B5": contract.DownPayment.String(),                    //首付小写
		//"P3A1B6": strconv.Itoa(90),                                    // 余款时限
		"P3A1B7": common.ConvertDecimalToCny(contract.BalancePayment), // 尾款大写
		"P3A1B8": contract.BalancePayment.String(),                    // 尾款小写
		//"9U0KqocTfQgF": strconv.Itoa(30), // 过户时限
		"P4A1B1": contract.Master.Name,      // 甲方户名
		"P4A1B2": contract.Master.BankCard,  // 甲方账户
		"P4A1B3": contract.Master.Bank,      // 甲方开户行
		"P4A2B1": store.Company.BankAccount, // 乙方户名
		"P4A2B2": store.Company.BankCard,    // 乙方账户
		"P4A2B3": store.Company.Bank,        // 乙方开户行
		"P5A1B1": contract.Condition,        //车况描述
	}
	cdn := config.Config.Upload.Cdn
	var files []tencent.UploadFileStruct
	for _, photo := range contract.Photos {
		files = append(files, tencent.UploadFileStruct{
			FileUrl:  cdn + photo.Path,
			FileName: photo.Text,
		})
	}
	fileIds, err := tencent.Ess.UploadFile(files)
	if err == nil {
		var photos []string
		for i, item := range fileIds {
			if i < 6 {
				photos = append(photos, *item)
			}
		}
		//params["file"] = strings.Join(photos, ",")
	}

	var formFields []*essbasic.FormField
	for key, value := range params {
		formFields = append(formFields, tencent.BuildFormFieldById(purchaseParams[key], value))
	}

	_, err = tencent.Ess.CreateFlowsByTemplates(formFields, flowApproverInfos, contract.ContractSn, purchaseParams["FlowName"], purchaseParams["TemplateId"])
	if err != nil {
		return err
	}
	return nil
}

// Cancel @Title 取消合同
func (p *purchaseLogic) Cancel(brokerId, storeId, roleId, contractId uint, reason string) error {
	contract := model.ContractPurchase{Id: contractId}
	if mysql.Db.Preload("EssFlow").Model(&contract).Where("broker_id = ? ", brokerId).First(&contract).Error != nil {
		return errors.New("合同错误")
	}
	if contract.Status != constant.ContractStatusCreate {
		return errors.New("合同不在待签订状态")
	}
	if err := tencent.Ess.ChannelCancelFlow([]*string{&contract.EssFlow.FlowId}); err != nil {
		return err
	}
	return nil
}
