package contract

import (
	"base/app/common"
	"base/app/config"
	"base/app/constant"
	"base/app/lib/bean"
	"base/app/lib/tencent"
	"base/app/logic/broker/message"
	"base/app/logic/broker/user/wallet"
	"base/app/model"
	"database/sql"
	"errors"
	"fmt"
	"git.oa00.com/go/mysql"
	"github.com/shopspring/decimal"
	essbasic "github.com/tencentcloud/tencentcloud-sdk-go/tencentcloud/essbasic/v20210526"
	"gorm.io/gorm"
	"strings"
	"time"
)

var SaleLogic = &saleLogic{}

type saleLogic struct {
}

type saleItem struct {
	Id            uint   `json:"id"`
	ContractSn    string `json:"contractSn"`
	CarId         uint   `json:"carId"`
	ModelName     string `json:"modelName"`
	BrokerId      uint   `json:"brokerId"`
	BrokerName    string `json:"brokerName"`
	DealerId      uint   `json:"dealerId"`
	DealerName    string `json:"dealerName"`
	CustomerId    uint   `json:"customerId"`
	CustomerName  string `json:"customerName"`
	Status        uint   `json:"status"`
	StatusName    string `json:"statusName"`
	DealerAuditAt int64  `json:"dealerAuditAt"`
	SignAt        int64  `json:"signAt"`
	EssFileUrl    string `json:"essFileUrl"`
	CreatedAt     int64  `json:"createdAt"`
}

type SaleSearch struct {
	Status           uint
	ContractSn       string
	ModelName        string
	CustomerName     string
	CustomerTel      string
	CreatedTimeStart string
	CreatedTimeEnd   string
}

type SaleBrokerSearch struct {
	//SourceName string
	//SourceTel string
}

// BrokerLists @Title 经纪人售车合同列表
func (s *saleLogic) BrokerLists(brokerId uint, search SaleSearch, searchBroker SaleBrokerSearch, page bean.Page) (lists []saleItem, total int64) {
	return s.Lists([]uint{brokerId}, []uint{}, search, searchBroker, SaleDealerSearch{}, page)
}

type SaleDealerSearch struct {
	BrokerName string
	BrokerTel  string
}

// DealerLists @Title 车商审批售车合同列表
func (s *saleLogic) DealerLists(dealerId uint, search SaleSearch, searchDealer SaleDealerSearch, page bean.Page) (lists []saleItem, total int64) {
	return s.Lists([]uint{}, []uint{dealerId}, search, SaleBrokerSearch{}, searchDealer, page)
}

// Lists @Title 售车合同列表
func (s *saleLogic) Lists(brokerIds, dealerIds []uint, search SaleSearch, searchBroker SaleBrokerSearch, searchDealer SaleDealerSearch, page bean.Page) (lists []saleItem, total int64) {
	lists = []saleItem{}
	var contractSales []model.ContractSale
	where := mysql.Db
	if len(brokerIds) > 0 {
		where = where.Where("broker_id in ?", brokerIds)
	}
	if len(dealerIds) > 0 {
		where = where.Where("seller_dealer_id in ?", dealerIds)
	}
	if search.Status > 0 {
		where = where.Where("status = ?", search.Status)
	}
	if search.ContractSn != "" {
		where = where.Where("contract_sn like ?", fmt.Sprintf("%%%s%%", search.ContractSn))
	}
	if search.ModelName != "" {
		where = where.Where("model_id in (?)", mysql.Db.Model(&model.CarModel{}).Where("name like ?", fmt.Sprintf("%%%s%%", search.ModelName)).Select("model_id"))
	}
	if search.CreatedTimeStart != "" {
		where = where.Where("created_at > ?", search.CreatedTimeStart)
	}
	if search.CreatedTimeEnd != "" {
		where = where.Where("created_at < ?", search.CreatedTimeEnd)
	}
	{
		customerWhere := mysql.Db.Model(&model.Customer{}).Select("id")
		if search.CustomerName != "" {
			customerWhere = customerWhere.Where("nickname like ?", fmt.Sprintf("%%%s%%", search.CustomerName))
		}
		if search.CustomerTel != "" {
			customerWhere = customerWhere.Where("phone like ?", fmt.Sprintf("%%%s%%", search.CustomerName))
		}
		if _, ok := customerWhere.Statement.Clauses["WHERE"]; ok {
			where = where.Where("customer_id in (?)", customerWhere)
		}
	}
	{
		brokerWhere := mysql.Db.Model(&model.Broker{}).Select("id")
		if searchDealer.BrokerName != "" {
			brokerWhere = brokerWhere.Where("nickname like ?", fmt.Sprintf("%%%s%%", searchDealer.BrokerName))
		}
		if searchDealer.BrokerTel != "" {
			brokerWhere = brokerWhere.Where("phone like ?", fmt.Sprintf("%%%s%%", searchDealer.BrokerTel))
		}
		if _, ok := brokerWhere.Statement.Clauses["WHERE"]; ok {
			where = where.Where("broker_id in (?)", brokerWhere)
		}
	}
	mysql.Db.Model(&contractSales).Where(where).Count(&total)
	if page.HasPage(total) {
		mysql.Db.
			Preload("Car").Preload("Model").Preload("DealerAudit").
			Preload("Broker").Preload("Customer").Preload("SellerDealer").Preload("EssFlow").
			Where(where).Order("id desc").Offset(page.GetStart()).Limit(page.GetLimit()).Find(&contractSales)
		for _, item := range contractSales {
			lists = append(lists, saleItem{
				Id:            item.Id,
				ContractSn:    item.ContractSn,
				CarId:         item.CarId,
				ModelName:     item.Model.Name,
				Status:        item.Status,
				StatusName:    constant.GetContractStatusName(item.Status),
				BrokerId:      item.BrokerId,
				BrokerName:    item.Broker.Nickname,
				DealerId:      item.SellerDealerId,
				DealerName:    item.SellerDealer.Name,
				CustomerId:    item.CustomerId,
				CustomerName:  item.Customer.Nickname,
				DealerAuditAt: item.DealerAudit.AuditAt.Time.Unix(),
				SignAt:        item.SignAt.Time.Unix(),
				CreatedAt:     item.CreatedAt.Unix(),
				EssFileUrl:    item.EssFlow.Url,
			})
		}
	}
	return
}

type saleInfo struct {
	Id                 uint                   `json:"id"`
	Status             uint                   `json:"status"`
	StatusName         string                 `json:"statusName"`
	ThirdPartKind      uint                   `json:"thirdPartKind"`
	ThirdPartKindName  string                 `json:"thirdPartKindName"`
	EssFileUrl         string                 `json:"essFileUrl"`
	CreatedAt          int64                  `json:"createdAt"`
	BaseInfo           saleBaseInfo           `json:"baseInfo"`
	BrokerInfo         saleBrokerInfo         `json:"brokerInfo"`
	CustomerInfo       saleCustomerInfo       `json:"customerInfo"`
	SellerDealerInfo   saleSellerDealerInfo   `json:"sellerDealerInfo"`
	SellerCustomerInfo saleSellerCustomerInfo `json:"sellerCustomerInfo"`
	BuyerMasterInfo    saleBuyerMasterInfo    `json:"buyerMasterInfo"`
	ThirdPartInfo      saleThirdPartInfo      `json:"thirdPartInfo"`
	DealerAuditInfo    saleDealerAuditInfo    `json:"dealerAuditInfo"`
}
type saleBaseInfo struct {
	CarId                  uint            `json:"carId"`
	ModelId                uint            `json:"modelId"`
	ModelName              string          `json:"modelName"`
	MainPhoto              string          `json:"mainPhoto"` // 车辆主图
	Transfer               uint            `json:"transfer"`
	LicensingData          int64           `json:"licensingData"`
	Mileage                decimal.Decimal `json:"mileage"`
	ContractSn             string          `json:"contractSn"`
	PayType                uint            `json:"payType"` // 支付方式
	PayTypeName            string          `json:"payTypeName"`
	TransferType           uint            `json:"transferType"` // 过户方式
	TransferTypeName       string          `json:"transferTypeName"`
	Amount                 decimal.Decimal `json:"amount"`                 // 成交价
	Deposit                decimal.Decimal `json:"deposit"`                // 定金
	DownPayment            decimal.Decimal `json:"downPayment"`            // 首付
	BalancePayment         decimal.Decimal `json:"balancePayment"`         // 尾款
	SaleServiceFee         decimal.Decimal `json:"saleServiceFee"`         // 服务费
	SaleServiceFeeRate     decimal.Decimal `json:"saleServiceFeeRate"`     // 服务费比例
	PurchaseServiceFee     decimal.Decimal `json:"purchaseServiceFee"`     // 服务费
	PurchaseServiceFeeRate decimal.Decimal `json:"purchaseServiceFeeRate"` // 服务费比例
	Remark                 string          `json:"remark"`
}

type saleBrokerInfo struct {
	Id   uint   `json:"id"`
	Name string `json:"name"`
	Tel  string `json:"tel"`
}

type saleCustomerInfo struct {
	Id   uint   `json:"id"`
	Name string `json:"name"`
	Tel  string `json:"tel"`
}

type saleBuyerMasterInfo struct {
	Id          uint   `json:"id"`
	Name        string `json:"name"`
	Phone       string `json:"phone"`
	IdCard      string `json:"idCard"`
	Address     string `json:"address"`
	BankCard    string `json:"bankCard"`
	Bank        string `json:"bank"`
	BankAccount string `json:"bankAccount"`
}

type saleSellerDealerInfo struct {
	Id              uint   `json:"id"`
	Name            string `json:"name"`
	OwnerBrokerId   uint   `json:"ownerBrokerId"`
	OwnerBrokerName string `json:"ownerBrokerName"`
	OwnerBrokerTel  string `json:"ownerBrokerTel"`
}

type saleSellerCustomerInfo struct {
	Id   uint   `json:"id"`
	Name string `json:"name"`
	Tel  string `json:"tel"`
}

type saleThirdPartInfo struct {
	Id              uint   `json:"id"`
	Name            string `json:"name"`
	OwnerBrokerId   uint   `json:"ownerBrokerId"`
	OwnerBrokerName string `json:"ownerBrokerName"`
	OwnerBrokerTel  string `json:"ownerBrokerTel"`
}

type saleDealerAuditInfo struct {
	Id                 uint   `json:"id"`
	DealerAuditAt      int64  `json:"dealerAuditAt"`
	DealerRejectReason string `json:"deaerRejectReason"`
}

// Info @Title 详情
func (s *saleLogic) Info(contractId uint) (result saleInfo, err error) {
	contract := model.ContractSale{Id: contractId}
	if mysql.Db.
		Preload("Model").Preload("Car").Preload("SellerMaster").Preload("BuyerMaster").
		Preload("SellerCustomer").Preload("SellerDealer.OwnerBroker").
		Preload("Broker").Preload("Customer").
		Preload("EssFlow").Preload("DealerAudit").
		First(&contract).Error != nil {
		return result, errors.New("出售合同错误")
	}
	result = saleInfo{
		Id:                contract.Id,
		Status:            contract.Status,
		StatusName:        constant.GetContractStatusName(contract.Status),
		CreatedAt:         contract.CreatedAt.Unix(),
		ThirdPartKind:     contract.ThirdPartKind,
		ThirdPartKindName: contract.GetThirdPartKindName(contract.ThirdPartKind),
		EssFileUrl:        contract.EssFlow.Url,
		BaseInfo: saleBaseInfo{
			CarId:                  contract.CarId,
			ModelId:                contract.ModelId,
			ModelName:              contract.Model.Name,
			MainPhoto:              contract.Car.MainPhoto,
			Transfer:               contract.Car.Transfer,
			LicensingData:          contract.Car.LicensingDate.Time.Unix(),
			Mileage:                contract.Car.Mileage,
			ContractSn:             contract.ContractSn,
			PayType:                contract.PayType,
			PayTypeName:            contract.GetPayTypeName(contract.PayType),
			TransferType:           contract.TransferType,
			TransferTypeName:       contract.GetTransferTypeName(contract.TransferType),
			Amount:                 contract.Amount,
			Deposit:                contract.Deposit,
			DownPayment:            contract.DownPayment,
			BalancePayment:         contract.BalancePayment,
			SaleServiceFee:         contract.SaleServiceFee,
			SaleServiceFeeRate:     contract.SaleServiceFeeRate,
			PurchaseServiceFee:     contract.PurchaseServiceFee,
			PurchaseServiceFeeRate: contract.PurchaseServiceFeeRate,
			Remark:                 contract.Remark,
		},
		BrokerInfo: saleBrokerInfo{
			Id:   contract.BrokerId,
			Name: contract.Broker.Nickname,
			Tel:  contract.Broker.Phone,
		},
		CustomerInfo: saleCustomerInfo{
			Id:   contract.CustomerId,
			Name: contract.Customer.Nickname,
			Tel:  contract.Customer.Mobile,
		},
		BuyerMasterInfo: saleBuyerMasterInfo{
			Id:          contract.BuyerMasterId,
			Name:        contract.BuyerMaster.Name,
			Phone:       contract.BuyerMaster.Phone,
			Address:     contract.BuyerMaster.Address,
			IdCard:      contract.BuyerMaster.IdCard,
			BankCard:    contract.BuyerMaster.BankCard,
			Bank:        contract.BuyerMaster.Bank,
			BankAccount: contract.BuyerMaster.BankAccount,
		},
		SellerDealerInfo: saleSellerDealerInfo{
			Id:              contract.SellerDealerId,
			Name:            contract.SellerDealer.Name,
			OwnerBrokerId:   contract.SellerDealer.OwnerBrokerId,
			OwnerBrokerName: contract.SellerDealer.OwnerBroker.Nickname,
			OwnerBrokerTel:  contract.SellerDealer.OwnerBroker.Phone,
		},
		SellerCustomerInfo: saleSellerCustomerInfo{
			Id:   contract.SellerCustomerId,
			Name: contract.SellerCustomer.Nickname,
			Tel:  contract.SellerCustomer.Mobile,
		},
		ThirdPartInfo: saleThirdPartInfo{},
		DealerAuditInfo: saleDealerAuditInfo{
			Id:                 contract.DealerAuditId,
			DealerAuditAt:      contract.DealerAudit.AuditAt.Time.Unix(),
			DealerRejectReason: contract.DealerAudit.Reason,
		},
	}

	switch contract.ThirdPartKind {
	case model.ContractSaleThirdPartKindPlatForm:
		result.ThirdPartInfo = saleThirdPartInfo{
			Name: "云云问车",
		}
	case model.ContractSaleThirdPartKindDealer:
		result.ThirdPartInfo = saleThirdPartInfo{
			Id:              contract.ThirdPartId,
			Name:            contract.ThirdPart.Name,
			OwnerBrokerId:   contract.ThirdPart.OwnerBrokerId,
			OwnerBrokerName: contract.ThirdPart.OwnerBroker.Nickname,
			OwnerBrokerTel:  contract.ThirdPart.OwnerBroker.Phone,
		}
	}
	return
}

type SaleAdd struct {
	CarId           uint `binding:"required" label:"车辆"`
	PayType         uint `binding:"required,oneof=1 2" label:"支付方式"`
	TransferType    uint `binding:"required,oneof=1 2" label:"过户方式"`
	PriceInfo       salePriceAdd
	CustomerChannel uint          `binding:"required,oneof=1 2" label:"客户渠道"` // 1=小程序 2=线下
	CustomerId      uint          `label:"客户"`
	MasterInfo      saleMasterAdd `binding:"required" label:"客户信息"`
	ThirdPartInfo   thirdPartAdd  `binding:"required" label:"第三方信息"`
	Remark          string        `label:"备注"`
}

type salePriceAdd struct {
	DealPrice         decimal.Decimal `binding:"required" label:"成交价"`
	Deposit           decimal.Decimal `binding:"required" label:"定金"`
	DownPayment       decimal.Decimal `binding:"required" label:"首付"`
	DeliverDate       string          `binding:"required" label:"交付日期"`
	DeliverAddress    string          `binding:"required" label:"交付地点"`
	TransferFee       decimal.Decimal `binding:"required" label:"过户费"`
	TransferFeeHolder uint            `binding:"required,oneof=1 2" label:"过户费承担方"`
	AgentFee          decimal.Decimal `binding:"required" label:"代办费"`
	AgentFeeHolder    uint            `binding:"required,oneof=1 2" label:"代办费承担方"`
}

type saleMasterAdd struct {
	Name        string `binding:"required" label:"姓名"`
	IdCard      string `binding:"required" label:"身份证号"`
	Phone       string `binding:"required" label:"手机号"`
	Address     string `binding:"required" label:"地址"`
	BankCard    string `binding:"required" label:"银行卡号"`
	Bank        string `binding:"required" label:"开户行"`
	BankAccount string `binding:"required" label:"开户账号"`
}

type thirdPartAdd struct {
	Kind                   uint            `binding:"required" label:"中间方类型"` // 中间方类型 1=无 2=自平台 3=车商
	StoreId                uint            `label:"车商id"`
	SaleServiceFeeRate     decimal.Decimal `label:"卖方服务费"`
	PurchaseServiceFeeRate decimal.Decimal `label:"买方服务费"`
}

// Add @Title 添加车辆售出合同
func (s *saleLogic) Add(brokerId uint, data SaleAdd) error {
	carInfo := model.Car{Id: data.CarId}
	if mysql.Db.Preload("CarBase.Model").Preload("CarBase.Photos").Where(&carInfo).First(&carInfo).Error != nil {
		return errors.New("车辆信息错误")
	}
	if carInfo.TheUpper != model.CarTheUpperTrue {
		return errors.New("车辆已下架")
	}
	if carInfo.Status != model.CarStatusNormal {
		return errors.New("车辆已预定或售出")
	}
	contractSn, err := common.GenNo(constant.NoPrefixContractSale)
	if err != nil {
		return errors.New("生成合同编号失败")
	}
	store := model.BrokerBusinessStore{Id: carInfo.CarBase.DealerId}
	if mysql.Db.Preload("Company").Where(&store).First(&store).Error != nil {
		return errors.New("未查到出售门店信息")
	}
	if store.Company.Id == 0 {
		return errors.New("出售门店企业信息异常")
	}

	contract := model.ContractSale{
		ContractSn:        contractSn,
		BrokerId:          brokerId,
		CustomerChannel:   data.CustomerChannel,
		CarId:             data.CarId,
		ModelId:           carInfo.ModelId,
		PayType:           data.PayType,
		TransferType:      data.TransferType,
		DeliverAddress:    data.PriceInfo.DeliverAddress,
		TransferFee:       data.PriceInfo.TransferFee,
		TransferFeeHolder: data.PriceInfo.TransferFeeHolder,
		AgentFee:          data.PriceInfo.AgentFee,
		AgentFeeHolder:    data.PriceInfo.AgentFeeHolder,
		ThirdPartKind:     data.ThirdPartInfo.Kind,
		ThirdPartId:       data.ThirdPartInfo.StoreId,
		Remark:            data.Remark,
		Status:            constant.ContractStatusCreate,
		SellerDealerId:    store.Id,
		BuyerMaster: model.ContractMaster{
			ContractType: model.ContractMasterTypeSale,
			Kind:         model.ContractMasterKindPerson,
			Name:         data.MasterInfo.Name,
			IdCard:       data.MasterInfo.IdCard,
			Address:      data.MasterInfo.Address,
			Phone:        data.MasterInfo.Phone,
			BankCard:     data.MasterInfo.BankCard,
			Bank:         data.MasterInfo.Bank,
			BankAccount:  data.MasterInfo.BankAccount,
		},
	}

	// 交付日期
	deliverDate, err := time.ParseInLocation("2006-01-02", data.PriceInfo.DeliverDate, time.Local)
	if err != nil {
		return errors.New("交付日期时间错误")
	}
	contract.DeliverDate = sql.NullTime{Time: deliverDate, Valid: true}

	if contract.ThirdPartKind == model.ContractSaleThirdPartKindDealer {
		if contract.ThirdPartId == 0 {
			return errors.New("未选择第三方车商")
		}
		thirdPartStore := model.BrokerBusinessStore{Id: contract.ThirdPartId}
		if mysql.Db.Preload("Company").Where(&thirdPartStore).First(&thirdPartStore).Error != nil {
			return errors.New("未查到中间方门店信息")
		}
		if thirdPartStore.Company.Id == 0 {
			return errors.New("中间方身份信息异常")
		}
	}
	// 卖方资料
	switch carInfo.CarBase.Source {
	case model.CarBaseSourceConsignment:
		consignment := model.OrderConsignment{Id: carInfo.CarBase.OrderId}
		if mysql.Db.Preload("Contract.Master").Where(&consignment).First(&consignment).Error != nil {
			return errors.New("出售方寄售信息异常")
		}
		if consignment.Contract.Master.Id == 0 {
			return errors.New("出售方身份信息异常")
		}
		contract.Source = model.CarBaseSourceConsignment
		contract.SellerCustomerId = consignment.CustomerId
		contract.SellerMaster = model.ContractMaster{
			ContractType: model.ContractMasterTypeSale,
			Kind:         model.ContractMasterKindPerson,
			Name:         consignment.Contract.Master.Name,
			IdCard:       consignment.Contract.Master.IdCard,
			Phone:        consignment.Contract.Master.Phone,
			BankCard:     consignment.Contract.Master.BankCard,
			Bank:         consignment.Contract.Master.Bank,
			BankAccount:  consignment.Contract.Master.BankAccount,
		}
	case model.CarBaseSourceDealerPurchase:
		contract.Source = model.CarBaseSourceDealerPurchase
		contract.SellerMaster = model.ContractMaster{
			ContractType: model.ContractMasterTypeSale,
			Kind:         model.ContractMasterKindCompany,
			Name:         store.Company.Name,
			LegalPerson:  store.Company.LegalPerson,
			IdCard:       store.Company.BusinessLicenseNo,
			Phone:        store.Company.LegalTel,
			BankCard:     store.Company.BankCard,
			Bank:         store.Company.Bank,
			BankAccount:  store.Company.BankAccount,
		}
	}

	// 计算价格
	{
		lowestFee := decimal.NewFromInt(1000) // 最低服务费1000
		contract.Amount = data.PriceInfo.DealPrice
		contract.Deposit = data.PriceInfo.Deposit
		contract.DownPayment = data.PriceInfo.DownPayment
		contract.BalancePayment = contract.Amount.Sub(contract.Deposit).Sub(contract.DownPayment)
		if contract.BalancePayment.Cmp(decimal.Zero) < 0 {
			return errors.New("金额分配有误")
		}
		saleServiceFee := contract.Amount.Mul(data.ThirdPartInfo.SaleServiceFeeRate)
		if saleServiceFee.LessThan(lowestFee) {
			saleServiceFee = lowestFee
		} else {
			contract.SaleServiceFeeRate = data.ThirdPartInfo.SaleServiceFeeRate
			contract.SaleServiceFee = saleServiceFee
		}
		purchaseServiceFee := contract.Amount.Mul(data.ThirdPartInfo.PurchaseServiceFeeRate)
		if purchaseServiceFee.LessThan(lowestFee) {
			purchaseServiceFee = lowestFee
		} else {
			contract.PurchaseServiceFeeRate = data.ThirdPartInfo.PurchaseServiceFeeRate
			contract.PurchaseServiceFee = purchaseServiceFee
		}
	}

	// 选择填写小程序客户
	if data.CustomerChannel == 1 {
		customer := model.Customer{Id: data.CustomerId}
		if mysql.Db.Where(&customer).First(&customer).Error != nil {
			return errors.New("客户有误")
		}
		contract.CustomerId = customer.Id
	}

	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		// 车商车辆出售非店长发起出售，须提交店长审核
		if contract.BrokerId != store.OwnerBrokerId {
			contract.Status = constant.ContractStatusDealerAuditing
			contract.DealerAudit = model.DealerAuditing{
				DealerId:      store.Id,
				ApplyBrokerId: brokerId,
				Type:          model.DealerAuditingTypeSale,
				Status:        constant.AuditStatusWaiting,
			}
			if tx.Create(&contract).Error != nil {
				return errors.New("合同提交失败")
			}
			contract.DealerAudit.LinkId = contract.Id
			if tx.Updates(&contract.DealerAudit).Error != nil {
				return errors.New("车主审核生成失败")
			}
			message.MessageLogic.MessageToBroker(mysql.Db, []uint{contract.BrokerId}, 0, model.BrokerMessageTypeCarAudit, message.MessageItem{
				Title:    "合同提示",
				Content:  "您有一个新的卖车合同已提交至车商审批",
				LinkedId: contract.Id,
				Extras: map[string]interface{}{
					"车辆名称": carInfo.CarBase.Model.Name,
					"出售价格": contract.Amount,
					"发起时间": contract.DealerAudit.CreatedAt.Format("2006-01-02 15:04:05"),
				},
			})
			message.MessageLogic.MessageToBroker(mysql.Db, []uint{store.OwnerBrokerId}, 0, model.BrokerMessageTypeCarAudit, message.MessageItem{
				Title:    "合同提示",
				Content:  "您有一个新的卖车合同待审批",
				LinkedId: contract.Id,
				Extras: map[string]interface{}{
					"车辆名称": carInfo.CarBase.Model.Name,
					"出售价格": contract.Amount,
					"发起时间": contract.DealerAudit.CreatedAt.Format("2006-01-02 15:04:05"),
				},
			})
			return nil
		}
		contract.Status = constant.ContractStatusCreate
		if tx.Create(&contract).Error != nil {
			return errors.New("合同提交失败")
		}
		if err = s.sale(tx, contract, carInfo, store); err != nil {
			return err
		}
		return nil
	})
}

// Audit @Title 审核通过/驳回
func (s *saleLogic) Audit(dealerId, contractId uint, choose uint, reason string) error {
	contract := model.ContractSale{Id: contractId}
	if mysql.Db.Preload("Car.CarBase.Model").Preload("Car.CarBase.Photos").Preload("DealerAudit").Preload("SellerMaster").Preload("BuyerMaster").First(&contract).Error != nil {
		return errors.New("合同id错误")
	}
	if contract.Status != constant.ContractStatusDealerAuditing {
		return errors.New("不在车商审核状态")
	}
	if contract.SellerDealerId != dealerId {
		return errors.New("无审核权限")
	}
	store := model.BrokerBusinessStore{Id: contract.SellerDealerId}
	if mysql.Db.Where(&store).First(&store).Error != nil {
		return errors.New("未找到车商信息")
	}
	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		switch choose {
		case 1: // 审核通过
			if tx.Model(&contract.DealerAudit).UpdateColumns(map[string]interface{}{
				"status":   constant.AuditStatusAdopt,
				"audit_at": time.Now(),
			}).RowsAffected != 1 {
				return errors.New("审核失败")
			}
			if tx.Model(&model.ContractSale{}).Where("id = ?", contractId).Updates(map[string]interface{}{
				"status": constant.ContractStatusDealerAdopt,
			}).RowsAffected != 1 {
				return errors.New("合同状态修改失败")
			}
			if err := s.sale(tx, contract, contract.Car, store); err != nil {
				return err
			}
			if tx.Model(&model.ContractSale{}).Where("id = ?", contractId).Updates(map[string]interface{}{
				"status": constant.ContractStatusCreate,
			}).RowsAffected != 1 {
				return errors.New("合同状态修改失败")
			}
			message.MessageLogic.MessageToBroker(mysql.Db, []uint{contract.BrokerId}, 0, model.BrokerMessageTypeCarAudit, message.MessageItem{
				Title:    "合同提示",
				Content:  "您的一个卖车合同已被车商审批通过",
				LinkedId: contract.Id,
				Extras: map[string]interface{}{
					"车辆名称": contract.Car.CarBase.Model.Name,
					"出售价格": contract.Amount,
					"审批时间": contract.DealerAudit.AuditAt.Time.Format("2006-01-02 15:04:05"),
				},
			})
			message.MessageLogic.MessageToBroker(mysql.Db, []uint{store.OwnerBrokerId}, 0, model.BrokerMessageTypeCarAudit, message.MessageItem{
				Title:    "合同提示",
				Content:  "您已通过卖车合同的审批",
				LinkedId: contract.Id,
				Extras: map[string]interface{}{
					"车辆名称": contract.Car.CarBase.Model.Name,
					"出售价格": contract.Amount,
					"审批时间": contract.DealerAudit.AuditAt.Time.Format("2006-01-02 15:04:05"),
				},
			})
			return nil
		case 2: // 审核拒绝
			if tx.Model(&contract.DealerAudit).UpdateColumns(map[string]interface{}{
				"status":   constant.AuditStatusReject,
				"reason":   reason,
				"audit_at": time.Now(),
			}).RowsAffected != 1 {
				return errors.New("审核失败")
			}
			if tx.Model(&model.ContractSale{}).Where(&contract).Updates(map[string]interface{}{
				"status": constant.ContractStatusDealerReject,
			}).RowsAffected != 1 {
				return errors.New("合同状态修改失败")
			}
			message.MessageLogic.MessageToBroker(mysql.Db, []uint{contract.BrokerId}, 0, model.BrokerMessageTypeCarAudit, message.MessageItem{
				Title:    "合同提示",
				Content:  "您的一个收车合同已被车商审批拒绝",
				LinkedId: contract.Id,
				Extras: map[string]interface{}{
					"车辆名称": contract.Car.CarBase.Model.Name,
					"出售价格": contract.Amount,
					"审批时间": contract.DealerAudit.AuditAt.Time.Format("2006-01-02 15:04:05"),
				},
			})
			message.MessageLogic.MessageToBroker(mysql.Db, []uint{store.OwnerBrokerId}, 0, model.BrokerMessageTypeCarAudit, message.MessageItem{
				Title:    "合同提示",
				Content:  "您已拒绝卖车合同的审批",
				LinkedId: contract.Id,
				Extras: map[string]interface{}{
					"车辆名称": contract.Car.CarBase.Model.Name,
					"出售价格": contract.Amount,
					"审批时间": contract.DealerAudit.AuditAt.Time.Format("2006-01-02 15:04:05"),
				},
			})
			return nil
		}
		return nil
	})
}

// Sale @Title 出售合同后置逻辑
func (s *saleLogic) sale(tx *gorm.DB, contract model.ContractSale, carInfo model.Car, dealerInfo model.BrokerBusinessStore) error {
	orderSn, err := common.GenNo(constant.NoPrefixOrderSale)
	if err != nil {
		return errors.New("生成订单编号失败")
	}
	orderSale := model.OrderSale{
		CustomerChannel: contract.CustomerChannel,
		CustomerId:      contract.CustomerId,
		BrokerId:        contract.BrokerId,
		OrderSn:         orderSn,
		CarId:           contract.CarId,
		ContractId:      contract.Id,
		Status:          model.OrderSaleStatusUnSign,
	}
	if tx.Create(&orderSale).Error != nil {
		return errors.New("提交失败")
	}
	if tx.Model(&model.Car{}).Where("id = ?", contract.CarId).UpdateColumn("status", model.CarStatusSigning).Error != nil {
		return errors.New("线上车辆状态修改失败")
	}
	switch contract.Source {
	case model.CarBaseSourceConsignment:
		if tx.Model(&model.OrderConsignment{}).Where("id = ?", carInfo.CarBase.OrderId).UpdateColumns(map[string]interface{}{
			"status": model.OrderConsignmentStatusSale,
		}).RowsAffected != 1 {
			return errors.New("寄售订单状态修改失败")
		}
	}
	if err = wallet.AssessLogic.Use(tx, model.WalletAssessKindContract, model.WalletAssessHistoryKindContractSale, dealerInfo.OwnerBrokerId, contract.Id, 1); err != nil {
		return err
	}
	if err = s.SaleEss(contract, carInfo); err != nil {
		return err
	}
	message.MessageLogic.MessageToCustomer(mysql.Db, []uint{contract.CustomerId}, 0, model.CustomerMessageTypeContract, message.MessageItem{
		Title:   "合同提示",
		Content: "您有一个新的买车合同待签订",
		Extras:  map[string]interface{}{"contractId": contract.Id},
	})
	if contract.SellerDealerId != 0 {
		dealer := model.BrokerBusinessStore{Id: contract.SellerDealerId}
		mysql.Db.Where(&dealer).First(&dealer)
		message.MessageLogic.MessageToBroker(mysql.Db, []uint{dealer.OwnerBrokerId}, 0, model.BrokerMessageTypeCarAudit, message.MessageItem{
			Title:    "合同提示",
			Content:  "您有一个新的收车合同待签订",
			LinkedId: contract.Id,
			Extras: map[string]interface{}{
				"车辆名称": carInfo.CarBase.Model.Name,
				"出售价格": contract.Amount,
				"创建时间": contract.CreatedAt.Format("2006-01-02 15:04:05"),
			},
		})
	}
	return nil
}

// SaleEss @Title 发起出售ess合同
func (s *saleLogic) SaleEss(contract model.ContractSale, carInfo model.Car) error {
	var saleParams map[string]string
	var params map[string]string
	var flowApproverInfos []*essbasic.FlowApproverInfo

	// 中间方资料
	thirdPartStore := model.BrokerBusinessStore{Id: contract.ThirdPartId}
	switch contract.ThirdPartKind {
	case model.ContractSaleThirdPartKindNone: // 无中间方
	case model.ContractSaleThirdPartKindPlatForm: // 平台
		thirdPartStore.Name = fmt.Sprintf("%s", config.Config.TencentEss.OrganizationName)
		thirdPartStore.Company.LegalTel = fmt.Sprintf("%s", "/")
	case model.ContractSaleThirdPartKindDealer: // 车商
		if mysql.Db.Preload("Company").Where(&thirdPartStore).First(&thirdPartStore).Error != nil {
			return errors.New("未查到中间方门店信息")
		}
		if thirdPartStore.Company.Id == 0 {
			return errors.New("中间方身份信息异常")
		}
	}

	// 根据卖方身份（车商/个人）和第三方身份（无/平台/车商）使用2x3=6种合同方案
	var signerName string
	switch contract.SellerMaster.Kind {
	case model.ContractMasterKindPerson:
		signerName = contract.SellerMaster.Name
		switch contract.ThirdPartKind {
		case model.ContractSaleThirdPartKindNone: // 无中间方
			saleParams = model.SaleSellerPersonThirdPartNoneParams
			approverSalePart := tencent.BuildPersonApprover(signerName, contract.SellerMaster.Phone, saleParams["RecipientId1"])
			approverPurchasePart := tencent.BuildPersonApprover(contract.BuyerMaster.Name, contract.BuyerMaster.Phone, saleParams["RecipientId2"])
			flowApproverInfos = append(flowApproverInfos, approverSalePart, approverPurchasePart)
		case model.ContractSaleThirdPartKindPlatForm: // 平台
			saleParams = model.SaleSellerPersonThirdPartPlatFormParams
			approverSalePart := tencent.BuildPersonApprover(signerName, contract.SellerMaster.Phone, saleParams["RecipientId1"])
			approverPurchasePart := tencent.BuildPersonApprover(contract.BuyerMaster.Name, contract.BuyerMaster.Phone, saleParams["RecipientId2"])
			approverThirdPart := tencent.BuildOrganizationApprover(
				config.Config.TencentEss.OrganizationName,
				config.Config.TencentEss.ProxyOrganizationOpenId,
				config.Config.TencentEss.ProxyOperatorOpenId,
				saleParams["RecipientId3"])
			flowApproverInfos = append(flowApproverInfos, approverSalePart, approverPurchasePart, approverThirdPart)
		case model.ContractSaleThirdPartKindDealer: // 车商
			saleParams = model.SaleSellerPersonThirdPartDealerParams
			approverSalePart := tencent.BuildPersonApprover(signerName, contract.SellerMaster.Phone, saleParams["RecipientId1"])
			approverPurchasePart := tencent.BuildPersonApprover(contract.BuyerMaster.Name, contract.BuyerMaster.Phone, saleParams["RecipientId2"])
			approverThirdPart := tencent.BuildNotChannelOrganizationApprover(thirdPartStore.Company.LegalPerson, thirdPartStore.Company.LegalTel, thirdPartStore.Name, saleParams["RecipientId3"])
			flowApproverInfos = append(flowApproverInfos, approverSalePart, approverPurchasePart, approverThirdPart)
		}
	case model.ContractMasterKindCompany:
		signerName = contract.SellerMaster.LegalPerson
		copmayName := contract.SellerMaster.Name
		switch contract.ThirdPartKind {
		case model.ContractSaleThirdPartKindNone: // 无中间方
			saleParams = model.SaleSellerDealerThirdPartNoneParams
			approverSalePart := tencent.BuildNotChannelOrganizationApprover(signerName, contract.SellerMaster.Phone, copmayName, saleParams["RecipientId1"])
			approverPurchasePart := tencent.BuildPersonApprover(contract.BuyerMaster.Name, contract.BuyerMaster.Phone, saleParams["RecipientId2"])
			flowApproverInfos = append(flowApproverInfos, approverSalePart, approverPurchasePart)
		case model.ContractSaleThirdPartKindPlatForm: // 平台
			saleParams = model.SaleSellerDealerThirdPartPlatFormParams
			approverSalePart := tencent.BuildNotChannelOrganizationApprover(signerName, contract.SellerMaster.Phone, copmayName, saleParams["RecipientId1"])
			approverPurchasePart := tencent.BuildPersonApprover(contract.BuyerMaster.Name, contract.BuyerMaster.Phone, saleParams["RecipientId2"])
			approverThirdPart := tencent.BuildOrganizationApprover(
				config.Config.TencentEss.OrganizationName,
				config.Config.TencentEss.ProxyOrganizationOpenId,
				config.Config.TencentEss.ProxyOperatorOpenId,
				saleParams["RecipientId3"])
			flowApproverInfos = append(flowApproverInfos, approverSalePart, approverPurchasePart, approverThirdPart)
		case model.ContractSaleThirdPartKindDealer: // 车商
			saleParams = model.SaleSellerDealerThirdPartDealerParams
			approverSalePart := tencent.BuildNotChannelOrganizationApprover(signerName, contract.SellerMaster.Phone, copmayName, saleParams["RecipientId1"])
			approverPurchasePart := tencent.BuildPersonApprover(contract.BuyerMaster.Name, contract.BuyerMaster.Phone, saleParams["RecipientId2"])
			approverThirdPart := tencent.BuildNotChannelOrganizationApprover(thirdPartStore.Company.LegalPerson, thirdPartStore.Company.LegalTel, thirdPartStore.Name, saleParams["RecipientId3"])
			flowApproverInfos = append(flowApproverInfos, approverSalePart, approverPurchasePart, approverThirdPart)
		}
	}

	params = map[string]string{
		"P1A1B1": contract.SellerMaster.Name,                                                    // 甲方名称
		"P1A1B2": contract.SellerMaster.LegalPerson,                                             // 甲方法人姓名
		"P1A1B3": contract.SellerMaster.IdCard,                                                  // 甲方身份证号码
		"P1A1B4": contract.SellerMaster.Phone,                                                   // 甲方联系方式
		"P1A1B5": contract.SellerMaster.Address,                                                 // 甲方地址
		"P1A2B1": contract.BuyerMaster.Name,                                                     // 乙方姓名
		"P1A2B2": contract.BuyerMaster.IdCard,                                                   // 乙方身份证号码
		"P1A2B3": contract.BuyerMaster.Phone,                                                    // 乙方联系方式
		"P1A3B1": "/",                                                                           // 丙方姓名
		"P1A3B2": "/",                                                                           // 丙方统一社会信用代码
		"P1A3B3": "/",                                                                           // 丙方联系方式
		"P2A1B1": carInfo.CarBase.Model.Name,                                                    // 厂牌型号
		"P2A1B2": carInfo.Vin,                                                                   // 车架号
		"P2A1B3": carInfo.CarBase.LicensingDate.Time.Format("2006-01-02"),                       // 初次上牌日期
		"P2A1B4": constant.GetCarTypeName(carInfo.CarBase.CarType),                              // 车辆类型
		"P2A1B5": carInfo.CarBase.EngineNo,                                                      // 发动机号
		"P2A1B6": common.TimeFormat(carInfo.CarBase.CompulsoryInsuranceDate.Time, "2006-01-02"), // 强制保险至
		"P2A1B7": carInfo.CarBase.TemporaryLicensePlate,                                         // 车牌号码
		"P2A1B8": carInfo.CarBase.ShamMileage.String() + "公里",                                   // 表显里程
		"P2A1B9": constant.GetCarUseCharacterName(carInfo.CarBase.UseCharacter),                 // 车辆使用性质
		"P3A1B1": common.ConvertDecimalToCny(contract.Amount),                                   // 成交价人民币大写
		"P3A1B2": contract.Amount.String(),                                                      // 成交价
		"P3A1B3": common.ConvertDecimalToCny(contract.Deposit),                                  // 定金人民币大写
		"P3A1B4": contract.Deposit.String(),                                                     // 定金
		"P3A1B5": contract.GetPayTypeName(contract.PayType),                                     // 余额支付方式
		"P3A1B6": contract.DeliverDate.Time.Format("2006-01-02"),                                // 交付时间
		"P3A1B7": contract.DeliverAddress,                                                       // 交付地点
		"P3A1B8": common.ConvertDecimalToCny(contract.DownPayment),                              // 首付金额大写
		"P3A1B9": contract.DownPayment.String(),                                                 // 首付金额
		"P3A2B1": common.ConvertDecimalToCny(contract.TransferFee),                              // 过户税费大写
		"P3A2B2": contract.TransferFee.String(),                                                 // 过户税费
		"P3A2B3": contract.GetHolderName(contract.TransferFeeHolder),                            // 过户税费承担方名称
		"P3A2B4": common.ConvertDecimalToCny(contract.AgentFee),                                 // 代办手续费大写
		"P3A2B5": contract.AgentFee.String(),                                                    // 代办手续费
		"P3A2B6": contract.GetHolderName(contract.AgentFeeHolder),                               // 代办手续费承担方名称
		"P3A4B1": contract.Remark,                                                               // 备注
		"P4A1B1": contract.SellerMaster.BankAccount,                                             // 甲方户名
		"P4A1B2": contract.SellerMaster.Bank,                                                    // 甲方开户行
		"P4A1B3": contract.SellerMaster.BankCard,                                                // 甲方账户
		"P4A2B1": contract.BuyerMaster.BankAccount,                                              // 乙方户名
		"P4A2B2": contract.BuyerMaster.Bank,                                                     // 乙方开户行
		"P4A2B3": contract.BuyerMaster.BankCard,                                                 // 乙方账户
	}

	switch contract.ThirdPartKind {
	case model.ContractSaleThirdPartKindNone: // 无中间方
	case model.ContractSaleThirdPartKindPlatForm, model.ContractSaleThirdPartKindDealer: // 平台，车商
		params["P1A3B1"] = thirdPartStore.Name
		params["P1A3B2"] = thirdPartStore.Company.BusinessLicenseNo
		params["P1A3B3"] = thirdPartStore.Company.LegalTel
		params["P1A3B4"] = thirdPartStore.Address
		params["P3A3B1"] = common.StringTernary(contract.SaleServiceFeeRate.GreaterThan(decimal.Zero), contract.SaleServiceFeeRate.Mul(decimal.NewFromFloat(100)).String(), "/")         // 成交服务费比率(甲->丙)
		params["P3A3B2"] = contract.SaleServiceFee.String()                                                                                                                              // 成交服务费(甲->丙)
		params["P3A3B3"] = common.StringTernary(contract.PurchaseServiceFeeRate.GreaterThan(decimal.Zero), contract.PurchaseServiceFeeRate.Mul(decimal.NewFromFloat(100)).String(), "/") // 成交服务费(乙->丙)
		params["P3A3B4"] = contract.PurchaseServiceFee.String()                                                                                                                          // 成交服务费(乙->丙)
		params["P4A3B1"] = thirdPartStore.Company.BankAccount                                                                                                                            // 丙方户名
		params["P4A3B2"] = thirdPartStore.Company.Bank                                                                                                                                   // 丙方开户行
		params["P4A3B3"] = thirdPartStore.Company.BankCard                                                                                                                               // 丙方账户
	}

	cdn := config.Config.Upload.Cdn
	//var carPhotos []tencent.UploadFileStruct
	//var interiorPhotos []tencent.UploadFileStruct
	//var defectPhotos []tencent.UploadFileStruct
	//var dataPhotos []tencent.UploadFileStruct
	//for _, photo := range carInfo.CarBase.Photos {
	//	switch photo.Type {
	//	case model.CarPhotoTypeCar: // 车辆照片
	//		carPhotos = append(carPhotos, tencent.UploadFileStruct{
	//			FileUrl:  cdn + photo.Path,
	//			FileName: photo.Text,
	//		})
	//	case model.CarPhotoTypeInterior: // 内饰照片
	//		interiorPhotos = append(interiorPhotos, tencent.UploadFileStruct{
	//			FileUrl:  cdn + photo.Path,
	//			FileName: photo.Text,
	//		})
	//	case model.CarPhotoTypeDefect: // 缺陷照片
	//		defectPhotos = append(defectPhotos, tencent.UploadFileStruct{
	//			FileUrl:  cdn + photo.Path,
	//			FileName: photo.Text,
	//		})
	//	case model.CarPhotoTypeData: // 资料照片
	//		dataPhotos = append(dataPhotos, tencent.UploadFileStruct{
	//			FileUrl:  cdn + photo.Path,
	//			FileName: photo.Text,
	//		})
	//	}
	//}
	var files []tencent.UploadFileStruct
	for _, photo := range carInfo.CarBase.Photos {
		if photo.Path != "" {
			files = append(files, tencent.UploadFileStruct{
				FileUrl:  cdn + photo.Path,
				FileName: photo.Text,
			})
		}
	}
	fileIds, err := tencent.Ess.UploadFile(files)
	if err == nil {
		var photos []string
		for i, item := range fileIds {
			if i < 6 {
				photos = append(photos, *item)
			}
		}
		params["carPhotos"] = strings.Join(photos, ",")
	}

	var formFields []*essbasic.FormField
	for key, value := range params {
		formFields = append(formFields, tencent.BuildFormFieldById(saleParams[key], value))
	}

	_, err = tencent.Ess.CreateFlowsByTemplates(formFields, flowApproverInfos, contract.ContractSn, saleParams["FlowName"], saleParams["TemplateId"])
	if err != nil {
		return err
	}
	return nil
}

// Cancel @Title 取消合同
func (s *saleLogic) Cancel(brokerId, contractId uint, reason string) error {
	contract := model.ContractSale{Id: contractId, BrokerId: brokerId}
	if mysql.Db.Preload("EssFlow").Where(&contract).First(&contract).Error != nil {
		return errors.New("合同错误")
	}
	if contract.Status != constant.ContractStatusCreate {
		return errors.New("合同不在待签订状态")
	}
	if err := tencent.Ess.ChannelCancelFlow([]*string{&contract.EssFlow.FlowId}); err != nil {
		return err
	}
	return nil
}
