package customer

import (
	"base/app/lib/bean"
	"base/app/model"
	"errors"
	"fmt"
	"git.oa00.com/go/mysql"
)

const (
	CustomerOrderNewTrail    = "new_trail"    // 最近跟进
	CustomerOrderNewRegister = "new_register" // 最近注册
)

var CustomerLogic = &customerLogic{}

type customerLogic struct {
}

type customerItem struct {
	Id             uint   `json:"id"`
	Nickname       string `json:"nickname"`
	Gender         uint   `json:"gender"`
	Mobile         string `json:"mobile"`
	RealName       string `json:"realName"`
	IdCard         string `json:"idCard"`
	IdCardFront    string `json:"idCardFront"`
	IdCardBack     string `json:"idCardBack"`
	BankCard       string `json:"bankCard"`
	Bank           string `json:"bank"`
	BankAccount    string `json:"bankAccount"`
	IsImportant    uint   `json:"isImportant"`
	BrokerName     string `json:"brokerName"`
	TrailContent   string `json:"trailContent"`
	TrailCreatedAt int64  `json:"trailCreatedAt"`
	CreatedAt      int64  `json:"createdAt"`
}
type CustomerSearch struct {
	Name                  string
	Status                uint
	Important             uint
	CreatedDateStart      string
	CreatedDateEnd        string
	TrailCreatedDateStart string
	TrailCreatedDateEnd   string
}

type CustomerOrder struct {
	Order string
}

// Lists @Title 客户列表
func (c *customerLogic) Lists(brokerIds []uint, search CustomerSearch, order CustomerOrder, page bean.Page) (lists []customerItem, total int64) {
	lists = []customerItem{}
	var customers []model.Customer
	where := mysql.Db.Where("broker_id in ?", brokerIds)
	orderStr := ""
	tableName := mysql.Db.NamingStrategy.TableName("Customer")
	// 客户名称
	if search.Name != "" {
		sqlKeyWord := fmt.Sprintf("%%%s%%", search.Name)
		where = where.Where("nickname like ? or mobile like ? or `yy_customer`.id in (?)", sqlKeyWord, sqlKeyWord, mysql.Db.Model(&model.CustomerMaster{}).Where("name like ?", sqlKeyWord).Select("customer_id"))
	}
	// 重要客户
	if search.Important > 0 {
		switch search.Important {
		case 1: // 重要
			where = where.Where("is_important = ?", model.CustomerIsImportantTrue)
		case 2: // 不重要
			where = where.Where("is_important = ?", model.CustomerIsImportantFalse)
		}
	}
	// 状态
	if search.Status > 0 {
		switch search.Status {
		case 1: // 注册用户
			where = where.Where("level = ?", model.CustomerLevelRegister)
		case 2: // 浏览客户
			where = where.Where("level = ?", model.CustomerLevelBrowse)
		case 3: // 意向客户
			where = where.Where("level = ?", model.CustomerLevelIntention)
		case 4: // 成交客户
			where = where.Where("level = ?", model.CustomerLevelDone)
		}
	}
	// 注册日期
	if search.CreatedDateStart != "" && search.CreatedDateEnd != "" {
		where = where.Where(fmt.Sprintf("%s.created_at > ? and %s.created_at <= ?", tableName, tableName), search.CreatedDateStart, search.CreatedDateEnd+" 23:59:59")
	}
	// 跟进日期
	if search.TrailCreatedDateStart != "" && search.TrailCreatedDateEnd != "" {
		where = where.Where("Trail.created_at > ? and Trail.created_at <= ?", search.TrailCreatedDateStart, search.TrailCreatedDateEnd+" 23:59:59")
	}
	// 排序
	if order.Order != "" {
		switch order.Order {
		case CustomerOrderNewTrail: // 最近跟进
			orderStr = "Trail.created_at desc"
		case CustomerOrderNewRegister: // 最近注册
			orderStr = fmt.Sprintf("%s.created_at desc", tableName)
		}
	}
	mysql.Db.Model(&customers).Where(where).Count(&total)
	if page.HasPage(total) {
		queryModel := mysql.Db.Preload("Broker").Preload("Master").Joins("Trail").Where(where).Offset(page.GetStart()).Limit(page.GetLimit())
		if orderStr != "" {
			queryModel = queryModel.Order(orderStr)
		}
		queryModel.Find(&customers)
		for _, customer := range customers {
			lists = append(lists, customerItem{
				Id:             customer.Id,
				Nickname:       customer.Nickname,
				Gender:         customer.Gender,
				Mobile:         customer.Mobile,
				RealName:       customer.Master.Name,
				IdCard:         customer.Master.IdCard,
				IsImportant:    customer.IsImportant,
				BrokerName:     customer.Broker.Nickname,
				IdCardFront:    customer.Master.IdCardFront,
				IdCardBack:     customer.Master.IdCardBack,
				BankCard:       customer.Master.BankCard,
				Bank:           customer.Master.Bank,
				BankAccount:    customer.Master.BankAccount,
				TrailContent:   customer.Trail.Content,
				TrailCreatedAt: customer.Trail.CreatedAt.Unix(),
				CreatedAt:      customer.CreatedAt.Unix(),
			})
		}
	}
	return
}

type customerCount struct {
	TotalCount     uint `json:"totalCount"`
	RegisterCount  uint `json:"registerCount"`
	BrowseCount    uint `json:"browseCount"`
	IntentionCount uint `json:"intentionCount"`
	DoneCount      uint `json:"doneCount"`
}
type LevelCount struct {
	Level uint
	Count uint
}

// Count @Title 客户统计
func (c *customerLogic) Count(brokerIds []uint) (result customerCount) {
	var levelCounts []LevelCount
	mysql.Db.Model(&model.Customer{}).Where("broker_id in ?", brokerIds).
		Select("level,count(id) count").
		Group("level").Find(&levelCounts)
	for _, levelCount := range levelCounts {
		result.TotalCount += levelCount.Count
		switch levelCount.Level {
		case model.CustomerLevelRegister: // 注册用户
			result.RegisterCount += levelCount.Count
		case model.CustomerLevelBrowse: // 浏览客户
			result.BrowseCount += levelCount.Count
		case model.CustomerLevelIntention: // 意向客户
			result.IntentionCount += levelCount.Count
		case model.CustomerLevelDone: // 成交客户
			result.DoneCount += levelCount.Count
		}
	}
	return
}

type customerInfo struct {
	Id             uint   `json:"id"`
	Mobile         string `json:"mobile"`
	Nickname       string `json:"nickname"`
	Gender         uint   `json:"gender"`
	RealName       string `json:"realName"`
	IdCard         string `json:"idCard"`
	IdCardFront    string `json:"idCardFront"`
	IdCardBack     string `json:"idCardBack"`
	BankCard       string `json:"bankCard"`
	Bank           string `json:"bank"`
	BankAccount    string `json:"bankAccount"`
	IsImportant    uint   `json:"isImportant"`
	BrokerName     string `json:"brokerName"`
	TrailContent   string `json:"trailContent"`
	TrailCreatedAt int64  `json:"trailCreatedAt"`
	CreatedAt      int64  `json:"createdAt"`
}

// Info @Title 客户详情
func (c *customerLogic) Info(brokerIds []uint, customerId uint) (result customerInfo, err error) {
	customer := model.Customer{}
	if mysql.Db.Preload("Broker").Preload("Trail").Preload("Master").Where("id = ? and broker_id in ?", customerId, brokerIds).First(&customer).Error != nil {
		return result, errors.New("客户错误")
	}
	result = customerInfo{
		Id:             customer.Id,
		Mobile:         customer.Mobile,
		Nickname:       customer.Nickname,
		Gender:         customer.Gender,
		IsImportant:    customer.IsImportant,
		BrokerName:     customer.Broker.Nickname,
		TrailContent:   customer.Trail.Content,
		TrailCreatedAt: customer.Trail.CreatedAt.Unix(),
		CreatedAt:      customer.CreatedAt.Unix(),
	}
	if customer.IsReal == model.CustomerIsRealTrue {
		result.RealName = customer.Master.Name
		result.IdCard = customer.Master.IdCard
		result.IdCardFront = customer.Master.IdCardFront
		result.IdCardBack = customer.Master.IdCardBack
		result.BankCard = customer.Master.BankCard
		result.Bank = customer.Master.Bank
		result.BankAccount = customer.Master.BankAccount
	}
	return
}

// Important @Title 设为重要
func (c *customerLogic) Important(brokerIds []uint, customerIds []uint) error {
	if mysql.Db.Model(&model.Customer{}).Where("id in ? and broker_id in ?", customerIds, brokerIds).UpdateColumn("is_important", model.CustomerIsImportantTrue).Error != nil {
		return errors.New("设置失败")
	}
	return nil
}

// ImportantCancel @Title 取消重要
func (c *customerLogic) ImportantCancel(brokerIds []uint, customerIds []uint) error {
	if mysql.Db.Model(&model.Customer{}).Where("id in ? and broker_id in ?", customerIds, brokerIds).UpdateColumn("is_important", model.CustomerIsImportantFalse).Error != nil {
		return errors.New("设置失败")
	}
	return nil
}

// Select @Title 客户筛选
func (c *customerLogic) Select(brokerIds []uint) (result []bean.SelectItem) {
	result = []bean.SelectItem{}
	var customers []model.Customer
	mysql.Db.Where("broker_id in ?", brokerIds).Find(&customers)
	for _, customer := range customers {
		result = append(result, bean.SelectItem{
			Id:   customer.Id,
			Name: customer.Nickname,
		})
	}
	return
}
