package broker

import (
	"base/app/common"
	"base/app/config"
	"base/app/constant"
	"base/app/lib/ali"
	"base/app/lib/apple"
	"base/app/lib/invite"
	"base/app/lib/jpush"
	"base/app/model"
	"encoding/json"
	"errors"
	"fmt"
	"git.oa00.com/go/mysql"
	"git.oa00.com/go/redis"
	"git.oa00.com/go/wx"
	"log"
	"time"
)

var LoginLogic = &loginLogic{}

type loginLogic struct {
}
type loginInfo struct {
	Token string `json:"token"`
	User  User   `json:"user"`
}
type User struct {
	InviteCode string `json:"inviteCode"`
	Nickname   string `json:"nickname"`
	HeadImg    string `json:"headImg"`
	Level      uint   `json:"level"`
}

// LoginPhone @Title 手机号一键登录
func (l *loginLogic) LoginPhone(loginToken, inviteCode string) (result loginInfo, err error) {
	phone, err := jpush.GetPhone(loginToken)
	if err != nil {
		return result, errors.New("手机号码获取失败")
	}
	broker := model.Broker{Phone: phone}
	if mysql.Db.Where(&broker).First(&broker).Error != nil {
		// 未注册
		broker.Nickname = broker.RandNickname()
		broker.Level = model.BrokerLevelBase
		if mysql.Db.Create(&broker).Error != nil {
			return result, errors.New("登录失败")
		}
		//return result, errors.New("该手机号未注册")
	}
	result, err = l.login(&broker)
	return
}

type wechatLoginInfo struct {
	IsBind    bool      `json:"isBind"`
	BindToken string    `json:"bindToken"`
	LoginInfo loginInfo `json:"loginInfo"`
}

// LoginWechat @Title 微信登录
func (l *loginLogic) LoginWechat(code string) (result wechatLoginInfo, err error) {
	info, err := wx.Wxopen.GetAccessInfo(code)
	if err != nil {
		return result, errors.New("微信登录失败")
	}
	brokerWechat := model.BrokerWechat{OpenId: info.Openid}
	mysql.Db.Where(&brokerWechat).First(&brokerWechat)
	if brokerWechat.Id > 0 {
		result.IsBind = true
		broker := model.Broker{Id: brokerWechat.BrokerId}
		if mysql.Db.First(&broker).Error != nil {
			return result, errors.New("微信登录失败")
		}
		result.LoginInfo, err = l.login(&broker)
	} else {
		result.BindToken = common.RandStr(64)
		if err := redis.Redis.SetStruct(fmt.Sprintf(constant.BrokerWechatToken, result.BindToken), &info, time.Hour*24).Err(); err != nil {
			return result, errors.New("微信登录失败")
		}
	}
	return
}

type LoginAppleData struct {
	UserIdentifier    string `binding:"required" label:"用户id"`
	AuthorizationCode string `binding:"required" label:"code"`
	IdentityToken     string `binding:"required" label:"token"`
}

type LoginAppleInfo struct {
	IsBind    bool      `json:"isBind"`
	BindToken string    `json:"bindToken"`
	LoginInfo loginInfo `json:"loginInfo"`
}

// LoginApple @Title 苹果登录
func (l *loginLogic) LoginApple(data LoginAppleData) (info LoginAppleInfo, err error) {

	_, preload, err := apple.Login.ParseToken(data.IdentityToken)
	if err != nil {
		return
	}
	if checkErr := apple.Login.CheckIdentify(preload); checkErr != nil {
		err = checkErr
		return
	}

	authRes, authError := apple.Login.AppleAuthToken(apple.GRANTTYPEAUTHORIZATION, data.AuthorizationCode)
	if authError != nil {
		log.Println(authError)
		err = errors.New("登录失败")
		return
	}
	_, authPreload, err := apple.Login.ParseToken(authRes.IdToken)

	if data.UserIdentifier != authPreload.Sub || preload.Sub != authPreload.Sub {
		err = errors.New("用户验证不一致")
	}

	brokerApple := model.BrokerApple{AppleId: data.UserIdentifier}
	mysql.Db.Where(&brokerApple).First(&brokerApple)
	if brokerApple.Id > 0 {
		info.IsBind = true
		broker := model.Broker{Id: brokerApple.BrokerId}
		if mysql.Db.First(&broker).Error != nil {
			return info, errors.New("苹果登录失败")
		}
		info.LoginInfo, err = l.login(&broker)
	} else {
		info.BindToken = common.RandStr(64)
		if err := redis.Redis.SetStruct(fmt.Sprintf(constant.BrokerAppleToken, info.BindToken), &authPreload, time.Hour*24).Err(); err != nil {
			return info, errors.New("苹果登录失败")
		}
	}
	return
}

// LoginPhoneCode @Title 验证码登录
func (l *loginLogic) LoginPhoneCode(phone string) (result loginInfo, err error) {
	broker := model.Broker{Phone: phone}
	if mysql.Db.Where(&broker).First(&broker).Error != nil {
		// 未注册
		broker.Nickname = broker.RandNickname()
		broker.Level = model.BrokerLevelBase
		if mysql.Db.Create(&broker).Error != nil {
			return result, errors.New("登录失败")
		}
		//return result, errors.New("该手机号未注册")
	}
	result, err = l.login(&broker)
	return
}

// WechatBindBroker @Title 微信绑定账号
func (l *loginLogic) WechatBindBroker(phone, bindToken, inviteCode string) (result loginInfo, err error) {
	info := wx.AccessInfo{}
	if err := redis.Redis.GetStruct(fmt.Sprintf(constant.BrokerWechatToken, bindToken), &info); err != nil {
		return result, errors.New("绑定超时")
	}
	broker := model.Broker{Phone: phone}
	mysql.Db.Where(&broker).First(&broker)
	if broker.Id > 0 {
		brokerWechat := model.BrokerWechat{BrokerId: broker.Id}
		mysql.Db.Where(&brokerWechat).First(&brokerWechat)
		if brokerWechat.Id > 0 {
			return result, errors.New("该账号已绑定微信")
		}
	} else {
		// 未注册
		broker.Nickname = broker.RandNickname()
		broker.Level = model.BrokerLevelBase
		if mysql.Db.Create(&broker).Error != nil {
			return result, errors.New("绑定账号失败")
		}
	}
	brokerWechat := model.BrokerWechat{
		BrokerId: broker.Id,
		OpenId:   info.Openid,
		UnionId:  info.Unionid,
	}
	if mysql.Db.Create(&brokerWechat).Error != nil {
		return result, errors.New("绑定账号失败")
	}
	result, err = l.login(&broker)
	return
}

// AppleBindBroker @Title 苹果绑定账号
func (l *loginLogic) AppleBindBroker(phone, bindToken, inviteCode string) (result loginInfo, err error) {
	info := apple.IdTokenPreloadData{}
	if err := redis.Redis.GetStruct(fmt.Sprintf(constant.BrokerAppleToken, bindToken), &info); err != nil {
		return result, errors.New("绑定超时")
	}
	broker := model.Broker{Phone: phone}
	mysql.Db.Where(&broker).First(&broker)
	if broker.Id > 0 {
		brokerApple := model.BrokerApple{BrokerId: broker.Id}
		mysql.Db.Where(&brokerApple).First(&brokerApple)
		if brokerApple.Id > 0 {
			return result, errors.New("该账号已绑定苹果id")
		}
	} else {
		// 未注册
		broker.Nickname = broker.RandNickname()
		broker.Level = model.BrokerLevelBase
		if mysql.Db.Create(&broker).Error != nil {
			return result, errors.New("绑定账号失败")
		}
	}
	brokerApple := model.BrokerApple{
		BrokerId: broker.Id,
		AppleId:  info.Sub,
	}
	if mysql.Db.Create(&brokerApple).Error != nil {
		return result, errors.New("绑定账号失败")
	}
	result, err = l.login(&broker)
	return
}

// Sms @Title 发送短信
func (l *loginLogic) Sms(phone string) error {
	key := fmt.Sprintf(constant.SmSBrokerYzm, phone)
	phoneCode := constant.YzmCode{}
	if err := redis.Redis.GetStruct(key, &phoneCode); err != nil {
	}
	now := time.Now()
	if phoneCode.Code == "" || phoneCode.Time.Add(time.Minute).Before(now) {
		phoneCode.Code = common.RandStr(6, "0123456789")
		phoneCode.Time = now
		params := map[string]interface{}{
			"code": phoneCode.Code,
		}
		if config.IsProd() {
			if err := ali.Sms.Send(config.Config.AliSms.YzmCode, config.Config.AliSms.YzmSign, phone, params); err != nil {
				return errors.New("短信发送失败")
			}
		} else {
			phoneCode.Code = "000000"
			params["code"] = phoneCode.Code
		}
		redis.Redis.SetStruct(key, &phoneCode, time.Minute*time.Duration(config.Config.AliSms.YzmExpire))
		marshal, _ := json.Marshal(&params)
		smsRecord := model.SmsRecord{
			Phone:     phone,
			TempId:    config.Config.AliSms.YzmCode,
			Params:    string(marshal),
			CreatedAt: phoneCode.Time,
		}
		mysql.Db.Create(&smsRecord)
	} else {
		return errors.New("发送频繁")
	}
	return nil
}

// AutoCode 验证码验证
func (l *loginLogic) AutoCode(phone, code string) error {
	// 验证短信验证码
	key := fmt.Sprintf(constant.SmSBrokerYzm, phone)
	phoneCode := constant.YzmCode{}
	if err := redis.Redis.GetStruct(key, &phoneCode); err != nil {
		return errors.New("验证码已过期")
	}
	if code == "000000" {
		return nil
	}
	if phoneCode.Code != code {
		return errors.New("验证码错误")
	}
	redis.Redis.Del(key)
	return nil
}

// @Title 登录处理
func (l *loginLogic) login(broker *model.Broker) (result loginInfo, err error) {
	result.Token = common.RandStr(64)
	brokerToken := model.BrokerToken{BrokerId: broker.Id}
	if mysql.Db.Where(&brokerToken).First(&brokerToken).Error != nil {
		brokerToken.Token = result.Token
		if mysql.Db.Create(&brokerToken).Error != nil {
			return result, errors.New("登录失败")
		}
	} else {
		if mysql.Db.Model(&brokerToken).UpdateColumn("token", result.Token).Error != nil {
			return result, errors.New("登录失败")
		}
	}
	result.User.InviteCode = invite.Gen(int(broker.Id), 10)
	result.User.Nickname = broker.Nickname
	result.User.HeadImg = broker.HeadImg
	result.User.Level = broker.Level
	return
}

// Logout @Title 退出登录
func (l *loginLogic) Logout(userId uint) {
	brokerToken := model.BrokerToken{}
	mysql.Db.Model(&brokerToken).Where("broker_id = ?", userId).UpdateColumn("token", "")
}

// Auth @Title 验证登录
func (l *loginLogic) Auth(token string) (uint, error) {
	brokerToken := model.BrokerToken{Token: token}
	if mysql.Db.Where(&brokerToken).First(&brokerToken).Error != nil {
		return 0, errors.New("登录失效")
	}
	return brokerToken.BrokerId, nil
}
