package message

import (
	"base/app/lib/bean"
	"base/app/lib/websocket"
	"base/app/model"
	"encoding/json"
	"errors"
	"fmt"
	"git.oa00.com/go/mysql"
	"gorm.io/gorm"
	"log"
)

var MessageLogic = &messageLogic{}

type messageLogic struct {
}

type MessageItem struct {
	Title    string                 `json:"title"`
	Content  string                 `json:"content"`
	LinkedId uint                   `json:"linkedId"`
	Extras   map[string]interface{} `json:"extras"`
}

type messageListItem struct {
	Id           uint   `json:"id"`
	Type         uint   `json:"type"`
	Status       uint   `json:"status"`
	SenderUserId uint   `json:"senderUserId"`
	Title        string `json:"title"`
	Content      string `json:"content"`
	LinkedId     uint   `json:"linkedId"`
	CreatedAt    int64  `json:"createdAt"`
}

// Lists @Title 消息列表
func (m *messageLogic) Lists(brokerIds []uint, messageTypes []uint, page bean.Page) (lists []messageListItem, total int64) {
	lists = []messageListItem{}
	var brokerMassage []model.BrokerMessage
	where := mysql.Db.Where("broker_id in ?", brokerIds)
	if len(messageTypes) > 0 {
		where = where.Where("type in ?", messageTypes)
	}
	mysql.Db.Model(&brokerMassage).Where(where).Count(&total)
	if page.HasPage(total) {
		queryModel := mysql.Db.Where(where).Offset(page.GetStart()).Limit(page.GetLimit())
		queryModel.Find(&brokerMassage)
		for _, item := range brokerMassage {
			lists = append(lists, messageListItem{
				Id:           item.Id,
				Type:         item.Type,
				Status:       item.Status,
				SenderUserId: item.SenderUserId,
				Title:        item.Title,
				Content:      item.Content,
				LinkedId:     item.LinkedId,
				CreatedAt:    item.CreatedAt.Unix(),
			})
		}
	}
	return
}

type messageCount struct {
	NormalCount         uint `json:"normalCount"`
	CustomerNormalCount uint `json:"customerNormalCount"`
	CustomerPayCount    uint `json:"customerPayCount"`
	CustomerTrailCount  uint `json:"customerTrailCount"`
	CustomerInviteCount uint `json:"customerInviteCount"`
	CustomerCareCount   uint `json:"customerCareCount"`
	BillCount           uint `json:"billCount"`
	CarAuditCount       uint `json:"carAuditCount"`
	ContractAuditCount  uint `json:"contractAuditCount"`
	SystemCount         uint `json:"systemCount"`
}
type MessageCount struct {
	Type    uint
	Count   uint
	Title   string
	content string
}

// Count @Title 未读消息统计
func (m *messageLogic) Count(brokerIds []uint) (result messageCount) {
	var messageCounts []MessageCount
	mysql.Db.Model(&model.BrokerMessage{}).Where("broker_id in ? and status = ?", brokerIds, model.BrokerMessageStatusNotRead).
		Select("type,count(id) count").
		Group("type").Find(&messageCounts)
	for _, item := range messageCounts {
		switch item.Type {
		case model.BrokerMessageTypeNormal: // 常规消息
			result.NormalCount += item.Count
		case model.BrokerMessageTypeCustomerNormal: // 用户消息
			result.CustomerNormalCount += item.Count
		case model.BrokerMessageTypeCustomerPay: // 用户付款
			result.CustomerPayCount += item.Count
		case model.BrokerMessageTypeCustomerTrail: // 用户跟踪
			result.CustomerInviteCount += item.Count
		case model.BrokerMessageTypeCustomerInvite: // 用户邀约
			result.CustomerInviteCount += item.Count
		case model.BrokerMessageTypeCustomerCare: // 用户关怀
			result.CustomerCareCount += item.Count
		case model.BrokerMessageTypeBill: // 分账
			result.BillCount += item.Count
		case model.BrokerMessageTypeCarAudit: // 车辆审批
			result.CarAuditCount += item.Count
		case model.BrokerMessageTypeContractConsignment, model.BrokerMessageTypeContractPurchase, model.BrokerMessageTypeContractSale: // 合同审批
			result.ContractAuditCount += item.Count
		case model.BrokerMessageTypeSystem: // 系统消息
			result.SystemCount += item.Count
		}
	}
	return
}

type messageInfo struct {
	Id           uint   `json:"id"`
	Type         uint   `json:"type"`
	Status       uint   `json:"status"`
	Title        string `json:"title"`
	Content      string `json:"content"`
	SenderUserId uint   `json:"senderUserId"`
	LinkedId     uint   `json:"linkedId"`
	Extras       map[string]interface{}
	CreatedAt    int64 `json:"createdAt"`
}

// Info @Title 消息详情
func (m *messageLogic) Info(brokerId, messageId uint) (result messageInfo, err error) {
	brokerMessage := model.BrokerMessage{}
	if mysql.Db.Where("id = ? and broker_id = ?", messageId, brokerId).First(&brokerMessage).Error != nil {
		return result, errors.New("客户错误")
	}
	result = messageInfo{
		Id:           brokerMessage.Id,
		Type:         brokerMessage.Type,
		Status:       brokerMessage.Status,
		Title:        brokerMessage.Title,
		Content:      brokerMessage.Content,
		SenderUserId: brokerMessage.SenderUserId,
		LinkedId:     brokerMessage.LinkedId,
		CreatedAt:    brokerMessage.CreatedAt.Unix(),
	}
	var extras map[string]interface{}
	err = json.Unmarshal([]byte(brokerMessage.Extras), &extras)
	if err != nil {
		return result, errors.New("消息内容解析错误")
	}
	result.Extras = extras
	if brokerMessage.Status == model.BrokerMessageStatusNotRead {
		m.read(brokerId, messageId)
	}
	return
}

// read @Title 消息标记已读
func (m *messageLogic) read(brokerId, messageId uint) error {
	if mysql.Db.Model(&model.BrokerMessage{}).Where("id = ? and broker_id = ?", messageId, brokerId).UpdateColumn("status", model.BrokerMessageStatusRead).RowsAffected != 1 {
		return errors.New("提交失败")
	}
	return nil
}

// TestSend @Title 测试发送消息
func (m *messageLogic) TestSend(BrokerId uint, item MessageItem) {
	log.Println("测试发送消息")
	log.Println(item.Content)
	m.sendToBroker([]uint{BrokerId}, item)
	return
}

// MessageToCustomer @Title 发送消息步骤
func (m *messageLogic) MessageToCustomer(tx *gorm.DB, customerIds []uint, brokerId, messageType uint, item MessageItem) error {
	if len(customerIds) == 0 {
		return nil
	}
	var customerMessages []model.CustomerMessage
	for _, customerId := range customerIds {
		customerMessages = append(customerMessages, model.CustomerMessage{
			CustomerId:   customerId,
			Type:         messageType,
			SenderUserId: brokerId,
			Status:       model.CustomerMessageStatusNotRead,
			Title:        item.Title,
			Content:      item.Content,
		})
	}
	if tx.Create(&customerMessages).Error != nil {
		return errors.New("发送消息失败")
	}
	m.sendToCustomer(customerIds, item)
	return nil
}

// sendToCustomer @Title 发送消息到客户
func (m *messageLogic) sendToCustomer(customerIds []uint, item MessageItem) {
	body, _ := json.Marshal(&item)
	for _, customerId := range customerIds {
		websocket.WebsocketManager.SendGroup(fmt.Sprintf(websocket.CustomerKey, customerId), body)
	}
}

// MessageToBroker @Title 发送消息步骤
func (m *messageLogic) MessageToBroker(tx *gorm.DB, brokerIds []uint, brokerId, messageType uint, item MessageItem) error {
	if len(brokerIds) == 0 {
		return nil
	}
	extras, _ := json.Marshal(&item.Extras)
	var brokerMessages []model.BrokerMessage
	for _, brokerIdItem := range brokerIds {
		brokerMessages = append(brokerMessages, model.BrokerMessage{
			BrokerId:     brokerIdItem,
			Type:         messageType,
			SenderUserId: brokerId,
			Status:       model.BrokerMessageStatusNotRead,
			Title:        item.Title,
			Content:      item.Content,
			LinkedId:     item.LinkedId,
			Extras:       string(extras),
		})
	}
	if tx.Create(&brokerMessages).Error != nil {
		return errors.New("发送消息失败")
	}
	m.sendToBroker(brokerIds, item)
	return nil
}

// sendToBroker @Title 发送消息到经纪人
func (m *messageLogic) sendToBroker(brokerIds []uint, item MessageItem) {
	body, _ := json.Marshal(&item)
	for _, brokerIdItem := range brokerIds {
		websocket.WebsocketManager.SendGroup(fmt.Sprintf(websocket.BrokerKey, brokerIdItem), body)
	}
}

//// Send @Title 发送消息
//func (m *messageLogic) Send(manageIds []uint, item MessageItem) {
//	log.Println(m)
//	m.sendWebsocket(manageIds, item)
//	return
//}
//
//// @Title 发送websocket消息
//func (m *messageLogic) sendWebsocket(manageIds []uint, item MessageItem) {
//	body, _ := json.Marshal(&item)
//	for _, manageId := range manageIds {
//		websocket.WebsocketManager.SendGroup(fmt.Sprintf(CustomerKey, manageId), body)
//	}
//}
