package order

import (
	"base/app/constant"
	"base/app/lib/bean"
	"base/app/model"
	"errors"
	"git.oa00.com/go/mysql"
	"github.com/shopspring/decimal"
)

var ConsignmentLogic = &consignmentLogic{}

type consignmentLogic struct {
}
type consignmentListsItem struct {
	Id            uint            `json:"id"`
	OrderSn       string          `json:"orderSn"`
	Status        uint            `json:"status"`
	StatusName    string          `json:"StatusName"`
	AuditStatus   uint            `json:"auditStatus"`
	ModeName      string          `json:"modeName"`
	LicensingDate int64           `json:"licensingDate"`
	Mileage       decimal.Decimal `json:"mileage"`
	Price         decimal.Decimal `json:"price"`
	CreatedAt     int64           `json:"createdAt"`
	CarPhoto      string          `json:"carPhoto"`
}
type ConsignmentListsSearch struct {
	Status uint
}

// Lists @Title 寄卖订单列表
func (c *consignmentLogic) Lists(brokerIds []uint, search ConsignmentListsSearch, page bean.Page) (lists []consignmentListsItem, total int64) {
	lists = []consignmentListsItem{}
	var consignments []model.OrderConsignment
	where := mysql.Db.Where("broker_id in ?", brokerIds)
	if search.Status > 0 {
		switch search.Status {
		case 1: // 待签订
			where = where.Where("status = ?", model.OrderConsignmentStatusUnSign)
		case 2: // 待发布
			where = where.Where("status = ?", model.OrderConsignmentStatusUnPublish)
		case 3: // 审核中
			where = where.Where("status = ?", model.OrderConsignmentStatusPublish)
		case 4: // 已驳回
			where = where.Where("status = ?", model.OrderConsignmentStatusReject)
		case 5: // 在售
			where = where.Where("status = ?", model.OrderConsignmentStatusTheUpper)
		case 6: // 已售
			where = where.Where("status = ?", model.OrderConsignmentStatusSale)
		case 7: // 交易取消
			where = where.Where("status = ?", model.OrderConsignmentStatusCancel)
		}
	}
	mysql.Db.Model(&consignments).Where(where).Count(&total)
	if page.HasPage(total) {
		mysql.Db.Preload("Model").Preload("Contract").Preload("CarBase.Audit").
			Where(where).Offset(page.GetStart()).Limit(page.GetLimit()).Order("id desc").Find(&consignments)
		for _, consignment := range consignments {
			lists = append(lists, consignmentListsItem{
				Id:            consignment.Id,
				OrderSn:       consignment.OrderSn,
				Status:        consignment.Status,
				StatusName:    consignment.GetStatusName(consignment.Status),
				AuditStatus:   consignment.CarBase.Audit.Status,
				ModeName:      consignment.Model.Name,
				LicensingDate: consignment.Contract.LicensingDate.Unix(),
				Mileage:       consignment.Contract.Mileage,
				Price:         consignment.Contract.Price,
				CreatedAt:     consignment.CreatedAt.Unix(),
				CarPhoto:      consignment.CarBase.MainPhoto,
			})
		}
	}
	return
}

// BusinessLists @Title 寄卖订单列表
func (c *consignmentLogic) BusinessLists(brokerId uint, search ConsignmentListsSearch, page bean.Page) (lists []consignmentListsItem, total int64) {
	// 入驻商下店长或车务可发布寄售车辆
	brokerStaff := model.BrokerBusinessStoreStaff{BrokerId: brokerId}
	if mysql.Db.Where(&brokerStaff).First(&brokerStaff).Error != nil {
		return
	}
	var businessBrokerIds []uint
	mysql.Db.Model(&model.BrokerBusinessStoreStaff{}).Where("business_id = ?", brokerStaff.BusinessId).Select("broker_id").Find(&businessBrokerIds)
	if len(businessBrokerIds) <= 0 {
		return
	}
	lists, total = c.Lists(businessBrokerIds, search, page)
	return
}

// Count @Title 个人寄卖统计
func (c *consignmentLogic) Count(brokerIds []uint) (total int64) {
	mysql.Db.Model(&model.OrderConsignment{}).Where("broker_id in ?", brokerIds).Count(&total)
	return
}

type consignmentInfo struct {
	Id         uint                `json:"id"`
	Status     uint                `json:"status"`
	StatusName string              `json:"statusName"`
	Model      modelInfo           `json:"model"`
	Car        carInfo             `json:"car"`
	Contract   contractInfo        `json:"contract"`
	Customer   consignmentCustomer `json:"customer"`
	Pay        payInfo             `json:"pay"`
}

type modelInfo struct {
	Id    uint            `json:"id"`
	Name  string          `json:"name"`
	Price decimal.Decimal `json:"price"`
}

type carInfo struct {
	Id            uint            `json:"id"`
	MainPhoto     string          `json:"mainPhoto"`
	LicensingDate int64           `json:"licensingDate"`
	Mileage       decimal.Decimal `json:"mileage"`
	Price         decimal.Decimal `json:"price"`
}
type contractInfo struct {
	ContractSn               string          `json:"contractSn"`
	ContractSignAt           int64           `json:"contractSignAt"`
	Vin                      string          `json:"vin"`
	LicensingPlate           string          `json:"licensingPlate"`
	LicensingDate            int64           `json:"licensingDate"`
	Engine                   string          `json:"engine"`
	Color                    string          `json:"color"`
	Mileage                  decimal.Decimal `json:"mileage"`
	Transfer                 uint            `json:"transfer"`                 // 过户次数
	KeyCount                 uint            `json:"keyCount"`                 // 钥匙数量
	CompulsoryInsurance      uint            `json:"compulsoryInsurance"`      // 交强险
	CompulsoryInsuranceDate  int64           `json:"compulsoryInsuranceDate"`  // 交强险到期日期
	CommercialInsurance      uint            `json:"commercialInsurance"`      // 商业险
	CommercialInsuranceDate  int64           `json:"commercialInsuranceDate"`  // 商业险到期日期
	CommercialInsurancePrice decimal.Decimal `json:"commercialInsurancePrice"` // 商业险金额
}
type consignmentCustomer struct {
	Id       uint   `json:"id"`
	Nickname string `json:"nickname"`
	RealName string `json:"realName"`
	Mobile   string `json:"mobile"`
}

type payInfo struct {
	SaleAmount       decimal.Decimal `json:"saleAmount"`
	SaleAt           int64           `json:"saleAt"`
	PaymentAmount    decimal.Decimal `json:"paymentAmount"`
	PaymentProof     string          `json:"paymentProof"`
	PaymentCreatedAt int64           `json:"paymentCreatedAt"`
}

// Info @Title 车辆详情
func (c *consignmentLogic) Info(brokerIds []uint, orderId uint) (result consignmentInfo, err error) {
	consignment := model.OrderConsignment{Id: orderId}
	if mysql.Db.
		Preload("CarBase.Car").Preload("Contract").
		Preload("Model").Preload("Payment").Preload("Customer.Master").
		//Where("broker_id in ?", brokerIds).
		First(&consignment).Error != nil {
		return result, errors.New("订单错误")
	}
	result = consignmentInfo{
		Id:         consignment.Id,
		Status:     consignment.Status,
		StatusName: consignment.GetStatusName(consignment.Status),
		Model: modelInfo{
			Id:    consignment.Model.ModelId,
			Name:  consignment.Model.Name,
			Price: consignment.Model.Price,
		},
		Car: carInfo{
			Id:            consignment.CarBase.CarId,
			MainPhoto:     consignment.CarBase.Car.MainPhoto,
			LicensingDate: consignment.CarBase.Car.LicensingDate.Time.Unix(),
			Mileage:       consignment.CarBase.Car.Mileage,
			Price:         consignment.CarBase.Car.Price,
		},
		Contract: contractInfo{
			ContractSn:               consignment.Contract.ContractSn,
			ContractSignAt:           consignment.Contract.SignAt.Time.Unix(),
			Vin:                      consignment.Contract.Vin,
			LicensingPlate:           consignment.Contract.LicensePlate,
			LicensingDate:            consignment.Contract.LicensingDate.Unix(),
			Engine:                   consignment.Contract.Engine,
			Color:                    consignment.Contract.Color,
			Mileage:                  consignment.Contract.Mileage,
			KeyCount:                 consignment.Contract.KeyCount,
			Transfer:                 consignment.Contract.Transfer,
			CompulsoryInsurance:      consignment.Contract.CompulsoryInsurance,
			CompulsoryInsuranceDate:  consignment.Contract.CompulsoryInsuranceDate.Time.Unix(),
			CommercialInsurance:      consignment.Contract.CommercialInsurance,
			CommercialInsuranceDate:  consignment.Contract.CommercialInsuranceDate.Time.Unix(),
			CommercialInsurancePrice: consignment.Contract.CommercialInsurancePrice,
		},
		Customer: consignmentCustomer{
			Id:       consignment.CustomerId,
			Nickname: consignment.Customer.Nickname,
			RealName: consignment.Customer.Master.Name,
			Mobile:   consignment.Customer.Mobile,
		},
		Pay: payInfo{
			SaleAmount:       consignment.SaleAmount,
			SaleAt:           consignment.SaleAt.Time.Unix(),
			PaymentAmount:    consignment.Payment.Amount,
			PaymentProof:     consignment.Payment.Proof,
			PaymentCreatedAt: consignment.Payment.CreatedAt.Unix(),
		},
	}
	return
}

type photos struct {
	CarPhotos      []string `binding:"required,min=1" label:"车辆照片"`
	InteriorPhotos []string `binding:"required,min=1" label:"内饰照片"`
	DefectPhotos   []string `binding:"required,min=1" label:"缺陷照片"`
	DataPhotos     []string `binding:"required,min=1" label:"资料照片"`
}
type baseInfo struct {
	Type             string          `binding:"required" label:"车辆类型"`
	InteriorColor    string          `binding:"required" label:"内饰颜色"`
	Displacement     decimal.Decimal `binding:"required" label:"排量"`
	Gearbox          string          `binding:"required" label:"变速箱"`
	EmissionStandard string          `binding:"required" label:"排放标准"`
	UseCharacter     string          `binding:"required" label:"使用性质"`
	Location         uint            `binding:"required" label:"车辆所在地"`
	Attribution      uint            `binding:"required" label:"车辆归属地"`
	ConditionIn      string          `binding:"required" label:"车况（对内）"`
	ConditionOut     string          `binding:"required" label:"车况（对外）"`
}
type report struct {
	Paints []string `binding:"required,min=1" label:"漆面数据"`
}
type PublishData struct {
	OrderId  uint     `binding:"required" label:"订单id"`
	Photos   photos   `binding:"required" label:"照片"`
	BaseInfo baseInfo `binding:"required" label:"基本信息"`
	Report   report   `binding:"required" label:"报告"`
}

type infoCar struct {
	BaseInfo carInfoBaseInfo `json:"baseInfo"`
	Photos   carInfoPhotos   `json:"photos"`
	Report   carInfoReport   `json:"report"`
}
type carInfoBaseInfo struct {
	Type             string          `json:"type"`
	InteriorColor    string          `json:"interiorColor"`
	Displacement     decimal.Decimal `json:"displacement"`
	Gearbox          string          `json:"gearbox"`
	EmissionStandard string          `json:"emissionStandard"`
	UseCharacter     string          `json:"useCharacter"`
	Location         uint            `json:"location"`
	Attribution      uint            `json:"attribution"`
	ConditionIn      string          `json:"conditionIn"`
	ConditionOut     string          `json:"conditionOut"`
}
type carInfoPhotos struct {
	CarPhotos      []string `json:"carPhotos"`
	InteriorPhotos []string `json:"interiorPhotos"`
	DefectPhotos   []string `json:"defectPhotos"`
	DataPhotos     []string `json:"dataPhotos"`
}

type carInfoReport struct {
	Paints []string `json:"paints"`
}

// InfoCar @Title 订单车辆详情
func (c *consignmentLogic) InfoCar(brokerIds []uint, orderId uint) (result infoCar, err error) {
	consignment := model.OrderConsignment{Id: orderId}
	if mysql.Db.Preload("CarBase.Photos").Preload("CarBase.Model").
		Where("broker_id in ?", brokerIds).First(&consignment).Error != nil {
		return result, errors.New("订单错误")
	}
	result = infoCar{
		carInfoBaseInfo{
			Type:             constant.GetCarTypeName(consignment.CarBase.CarType),              // 车辆类型
			InteriorColor:    consignment.CarBase.InteriorColor,                                 // 内饰颜色
			Displacement:     consignment.CarBase.Model.Liter,                                   // 排量
			Gearbox:          consignment.CarBase.Model.GearType,                                // 变速箱
			EmissionStandard: consignment.CarBase.Model.DischargeStandard,                       // 排放标准
			UseCharacter:     constant.GetCarUseCharacterName(consignment.CarBase.UseCharacter), // 使用性质
			Location:         consignment.CarBase.LocationCityId,                                // 车辆所在地
			Attribution:      consignment.CarBase.AttributionCityId,                             // 车辆归属地
			ConditionIn:      consignment.CarBase.ConditionIn,                                   // 车况（对内）
			ConditionOut:     consignment.CarBase.ConditionOut,                                  // 车况（对外）
		},
		carInfoPhotos{
			CarPhotos:      []string{},
			InteriorPhotos: []string{},
			DefectPhotos:   []string{},
			DataPhotos:     []string{},
		},
		carInfoReport{
			Paints: []string{},
		},
	}
	for _, photo := range consignment.CarBase.Photos {
		switch photo.Type {
		case model.CarPhotoTypeCar: // 车辆照片
			result.Photos.CarPhotos = append(result.Photos.CarPhotos, photo.Path)
		case model.CarPhotoTypeInterior: // 内饰照片
			result.Photos.InteriorPhotos = append(result.Photos.InteriorPhotos, photo.Path)
		case model.CarPhotoTypeDefect: // 缺陷照片
			result.Photos.DefectPhotos = append(result.Photos.DefectPhotos, photo.Path)
		case model.CarPhotoTypeData: // 资料照片
			result.Photos.DataPhotos = append(result.Photos.DataPhotos, photo.Path)
		}
	}
	return
}
