package order

import (
	"base/app/common"
	"base/app/constant"
	"base/app/lib/bean"
	"base/app/model"
	"database/sql"
	"errors"
	"git.oa00.com/go/mysql"
	"github.com/shopspring/decimal"
	"gorm.io/gorm"
	"time"
)

var DealerConsignmentLogic = &dealerConsignmentLogic{}

type dealerConsignmentLogic struct {
}
type dealerConsignmentItem struct {
	Id            uint            `json:"id"`
	OrderSn       string          `json:"orderSn"`
	Status        uint            `json:"status"`
	AuditStatus   uint            `json:"auditStatus"`
	ModeName      string          `json:"modeName"`
	LicensingDate int64           `json:"licensingDate"`
	Mileage       decimal.Decimal `json:"mileage"`
	Price         decimal.Decimal `json:"price"`
	CreatedAt     int64           `json:"createdAt"`
}
type DealerConsignmentSearch struct {
}

// Lists @Title 寄卖订单列表
func (d *dealerConsignmentLogic) Lists(brokerIds []uint, search DealerConsignmentSearch, page bean.Page) (lists []dealerConsignmentItem, total int64) {
	lists = []dealerConsignmentItem{}
	var dealerConsignments []model.OrderDealerConsignment
	where := mysql.Db.Where("broker_id in ?", brokerIds)
	mysql.Db.Model(&dealerConsignments).Where(where).Count(&total)
	if page.HasPage(total) {
		mysql.Db.Preload("Audit").Preload("Model").Preload("CarPrice").
			Where(where).Offset(page.GetStart()).Limit(page.GetLimit()).Order("id desc").Find(&dealerConsignments)
		for _, dealerConsignment := range dealerConsignments {
			lists = append(lists, dealerConsignmentItem{
				Id:            dealerConsignment.Id,
				OrderSn:       dealerConsignment.OrderSn,
				Status:        dealerConsignment.Status,
				AuditStatus:   dealerConsignment.Audit.Status,
				ModeName:      dealerConsignment.Model.Name,
				LicensingDate: dealerConsignment.CarPrice.LicensingDate.Unix(),
				Mileage:       dealerConsignment.CarPrice.Mileage,
				Price:         dealerConsignment.Price,
				CreatedAt:     dealerConsignment.CreatedAt.Unix(),
			})
		}
	}
	return
}

// Count @Title 车商寄卖统计
func (d *dealerConsignmentLogic) Count(brokerIds []uint) (total int64) {
	mysql.Db.Model(&model.OrderDealerConsignment{}).Where("broker_id in ?", brokerIds).Count(&total)
	return
}

type DealerConsignmentAdd struct {
	DealerId uint     `binding:"required" label:"车商"`
	Photos   photos   `binding:"required" label:"照片"`
	BaseInfo baseInfo `binding:"required" label:"基本信息"`
	Report   report   `binding:"required" label:"报告"`
	Other    other    `binding:"required" label:"其他资料"`
}

type other struct {
	Price                    decimal.Decimal `binding:"required" label:"展示价格"`
	KeyCount                 uint            // 钥匙数量
	CompulsoryInsurance      uint            // 交强险
	CompulsoryInsuranceDate  string          // 交强险到期日期
	CommercialInsurance      uint            // 商业险
	CommercialInsuranceDate  string          // 商业险到期日期
	CommercialInsurancePrice decimal.Decimal // 商业险金额
}

// Add @Title 添加车商寄卖订单
func (d *dealerConsignmentLogic) Add(brokerId, priceId uint, data DealerConsignmentAdd) error {
	dealer := model.Dealer{BrokerId: brokerId, Id: data.DealerId}
	if mysql.Db.Where(&dealer).First(&dealer).Error != nil {
		return errors.New("车商错误")
	}
	if dealer.Status != model.DealerStatusEnabled {
		return errors.New("车商不在启用状态")
	}
	carPrice := model.BrokerCarPrice{Id: priceId, BrokerId: brokerId}
	if mysql.Db.Where(&carPrice).First(&carPrice).Error != nil {
		return errors.New("价格错误")
	}
	orderSn, err := common.GenNo(constant.NoPrefixOrderDealerConsignment)
	if err != nil {
		return errors.New("提交失败")
	}
	location := model.Area{Id: data.BaseInfo.Location, Level: model.AreaLevelCity}
	if mysql.Db.First(&location).Error != nil {
		return errors.New("车辆所在地错误")
	}
	attribution := model.Area{Id: data.BaseInfo.Attribution, Level: model.AreaLevelCity}
	if mysql.Db.First(&attribution).Error != nil {
		return errors.New("车辆所在地错误")
	}
	dealerConsignment := model.OrderDealerConsignment{
		DealerId:                 data.DealerId,
		BrokerId:                 brokerId,
		OrderSn:                  orderSn,
		ModelId:                  carPrice.ModelId,
		PriceId:                  carPrice.Id,
		Status:                   model.OrderDealerConsignmentStatusUnTheUpper,
		Price:                    data.Other.Price,
		KeyCount:                 data.Other.KeyCount,
		CompulsoryInsurance:      data.Other.CompulsoryInsurance,
		CommercialInsurance:      data.Other.CommercialInsurance,
		CommercialInsurancePrice: data.Other.CommercialInsurancePrice,
		DealerCar: model.OrderDealerConsignmentCar{
			DealerId:              data.DealerId,
			BrokerId:              brokerId,
			ModelId:               carPrice.ModelId,
			CarId:                 0,
			Type:                  data.BaseInfo.Type,
			InteriorColor:         data.BaseInfo.InteriorColor,
			Displacement:          data.BaseInfo.Displacement,
			Gearbox:               data.BaseInfo.Gearbox,
			EmissionStandard:      data.BaseInfo.EmissionStandard,
			UseCharacter:          data.BaseInfo.UseCharacter,
			LocationProvinceId:    location.ParentId,
			LocationCityId:        location.Id,
			AttributionProvinceId: attribution.ParentId,
			AttributionCityId:     attribution.Id,
			ConditionIn:           data.BaseInfo.ConditionIn,
			ConditionOut:          data.BaseInfo.ConditionOut,
			MainPhoto:             data.Photos.CarPhotos[0],
			TheUpper:              model.OrderDealerConsignmentCarTheUpperFalse,
		},
		Audit: model.OrderDealerConsignmentAudit{
			Status: model.OrderDealerConsignmentAuditStatusAudit,
		},
	}
	// 交强险
	if data.Other.CompulsoryInsurance == model.OrderDealerConsignmentCompulsoryInsuranceTrue {
		compulsoryInsurance, err := time.ParseInLocation("2006-01-02", data.Other.CompulsoryInsuranceDate, time.Local)
		if err != nil {
			return errors.New("交强险到期时间错误")
		}
		dealerConsignment.CompulsoryInsuranceDate = sql.NullTime{Time: compulsoryInsurance, Valid: true}
	}
	// 商业险
	if data.Other.CommercialInsurance == model.OrderDealerConsignmentCommercialInsuranceTrue {
		commercialInsuranceDate, err := time.ParseInLocation("2006-01-02", data.Other.CommercialInsuranceDate, time.Local)
		if err != nil {
			return errors.New("商业险到期时间错误")
		}
		dealerConsignment.CommercialInsuranceDate = sql.NullTime{Time: commercialInsuranceDate, Valid: true}
	}
	for _, photo := range data.Photos.CarPhotos {
		dealerConsignment.DealerCar.Photos = append(dealerConsignment.DealerCar.Photos, model.OrderDealerConsignmentCarPhoto{
			Path: photo,
			Type: model.OrderDealerConsignmentCarPhotoTypeCar,
		})
	}
	for _, photo := range data.Photos.InteriorPhotos {
		dealerConsignment.DealerCar.Photos = append(dealerConsignment.DealerCar.Photos, model.OrderDealerConsignmentCarPhoto{
			Path: photo,
			Type: model.OrderDealerConsignmentCarPhotoTypeInterior,
		})
	}
	for _, photo := range data.Photos.DefectPhotos {
		dealerConsignment.DealerCar.Photos = append(dealerConsignment.DealerCar.Photos, model.OrderDealerConsignmentCarPhoto{
			Path: photo,
			Type: model.OrderDealerConsignmentCarPhotoTypeDefect,
		})
	}
	for _, photo := range data.Photos.DataPhotos {
		dealerConsignment.DealerCar.Photos = append(dealerConsignment.DealerCar.Photos, model.OrderDealerConsignmentCarPhoto{
			Path: photo,
			Type: model.OrderDealerConsignmentCarPhotoTypeData,
		})
	}
	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		if tx.Create(&dealerConsignment).Error != nil {
			return errors.New("提交失败")
		}
		dealerConsignment.Audit.OrderId = dealerConsignment.Id
		if tx.Updates(&dealerConsignment.Audit).Error != nil {
			return errors.New("提交失败")
		}
		return nil
	})
}
