package order

import (
	"base/app/lib/bean"
	"base/app/logic/broker/message"
	"base/app/model"
	"errors"
	"git.oa00.com/go/mysql"
	"github.com/shopspring/decimal"
	"gorm.io/gorm"
)

var SaleLogic = &saleLogic{}

type saleLogic struct {
}
type saleItem struct {
	Id                  uint            `json:"id"`
	CustomerChannel     uint            `json:"customerChannel"`
	CustomerChannelName string          `json:"customerChannelName"`
	Status              uint            `json:"status"`
	CreatedAt           int64           `json:"createdAt"`
	MainPhoto           string          `json:"mainPhoto"`
	ModelName           string          `json:"modelName"`
	LicensingDate       int64           `json:"licensingDate"`
	Mileage             decimal.Decimal `json:"mileage"`
	Amount              decimal.Decimal `json:"amount"`
	Deposit             decimal.Decimal `json:"deposit"`
	DownPayment         decimal.Decimal `json:"downPayment"`
	BalancePayment      decimal.Decimal `json:"balancePayment"`
}
type SaleSearchLists struct {
	Status uint // 1=待签订 2=已签订 3=待检测 4=支付首付 5=待过户 6=支付尾款 7=交易完成 8=交易取消
}

// Lists @Title 售车订单
func (s *saleLogic) Lists(brokerIds []uint, search SaleSearchLists, page bean.Page) (lists []saleItem, total int64) {
	lists = []saleItem{}
	var orderSales []model.OrderSale
	where := mysql.Db
	where = where.Where("broker_id in ?", brokerIds)
	if search.Status > 0 {
		switch search.Status {
		case 1: // 待签订
			where = where.Where("status = ?", model.OrderSaleStatusUnSign)
		case 2: // 已签订
			where = where.Where("status = ?", model.OrderSaleStatusSign)
		case 3: // 待检测
			where = where.Where("status in ?", []uint{model.OrderSaleStatusDeposit, model.OrderSaleStatusTestReport})
		case 4: // 支付首付
			where = where.Where("status in ?", []uint{model.OrderSaleStatusUploadTestReport, model.OrderSaleStatusDownPaymentAudit, model.OrderSaleStatusDownPaymentAdopt})
		case 5: // 待过户
			where = where.Where("status = ?", model.OrderSaleStatusTransfer)
		case 6: // 支付尾款
			where = where.Where("status in ?", []uint{model.OrderSaleStatusTransferFinal, model.OrderSaleStatusBalancePaymentAudit, model.OrderSaleStatusBalancePaymentAdopt})
		case 7: // 交易完成
			where = where.Where("status = ?", model.OrderSaleStatusFinal)
		case 8: // 交易取消
			where = where.Where("status = ?", model.OrderSaleStatusCancel)
		}

	}
	mysql.Db.Model(&orderSales).Where(where).Count(&total)
	if page.HasPage(total) {
		mysql.Db.Preload("Car.Model").Preload("Contract").Where(where).Order("updated_at desc").Offset(page.GetStart()).Limit(page.GetLimit()).Find(&orderSales)
		for _, orderSale := range orderSales {
			lists = append(lists, saleItem{
				Id:                  orderSale.Id,
				CustomerChannel:     orderSale.CustomerChannel,
				CustomerChannelName: orderSale.GetCustomerChannelName(orderSale.CustomerChannel),
				Status:              orderSale.Status,
				CreatedAt:           orderSale.CreatedAt.Unix(),
				MainPhoto:           orderSale.Car.MainPhoto,
				ModelName:           orderSale.Car.Model.Name,
				LicensingDate:       orderSale.Car.LicensingDate.Time.Unix(),
				Mileage:             orderSale.Car.Mileage,
				Amount:              orderSale.Contract.Amount,
				Deposit:             orderSale.Contract.Deposit,
				DownPayment:         orderSale.Contract.DownPayment,
				BalancePayment:      orderSale.Contract.BalancePayment,
			})
		}
	}
	return
}

// Count @Title 售车统计
func (s *saleLogic) Count(brokerIds []uint) (total int64) {
	mysql.Db.Model(&model.OrderSale{}).Where("broker_id in ?", brokerIds).Count(&total)
	return
}

// TestReport @Title 检车报告
func (s *saleLogic) TestReport(brokerId, orderId uint, report string) error {
	orderSale := model.OrderSale{Id: orderId, BrokerId: brokerId}
	if mysql.Db.First(&orderSale).Error != nil {
		return errors.New("订单错误")
	}
	if orderSale.Status == model.OrderSaleStatusUploadTestReport {
		return errors.New("已上传，请勿重复操作")
	}
	if orderSale.Status != model.OrderSaleStatusTestReport {
		return errors.New("订单状态错误")
	}
	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		orderSaleReport := model.OrderSaleReport{
			OrderId: orderId,
			Path:    report,
		}
		if tx.Create(&orderSaleReport).Error != nil {
			return errors.New("提交失败")
		}
		if tx.Model(&orderSale).Where(&orderSale).UpdateColumn("status", model.OrderSaleStatusUploadTestReport).RowsAffected != 1 {
			return errors.New("提交失败")
		}
		message.MessageLogic.MessageToCustomer(tx, []uint{orderSale.CustomerId}, brokerId, model.CustomerMessageTypeOrder, message.MessageItem{
			Title:   "订单提示",
			Content: "您的买车合同检车报告已生成"})
		return nil
	})
}

type SaleTransfer struct {
	OrderId                  uint   `binding:"required" label:"订单"`
	Certificate              string `binding:"required" label:"登记证书"`
	VehicleLicense           string `binding:"required" label:"行驶证"`
	Invoice                  string `binding:"required" label:"发票"`
	GuaranteeSlip            string `binding:"required" label:"保单"`
	CompulsoryInsuranceProof string `binding:"required" label:"交强险险凭证"`
	CommercialInsuranceProof string `label:"商业险凭证"`
}

// Transfer @Title 过户
func (s *saleLogic) Transfer(brokerId uint, data SaleTransfer) error {
	orderSale := model.OrderSale{Id: data.OrderId, BrokerId: brokerId}
	if mysql.Db.First(&orderSale).Error != nil {
		return errors.New("订单错误")
	}
	if orderSale.Status == model.OrderSaleStatusTransferFinal {
		return errors.New("已过户，请勿重复操作")
	}
	if orderSale.Status != model.OrderSaleStatusTransfer {
		return errors.New("订单状态错误")
	}
	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		if tx.Model(&orderSale).Where(&orderSale).UpdateColumn("status", model.OrderSaleStatusTransferFinal).RowsAffected != 1 {
			return errors.New("提交失败")
		}
		saleTransfer := model.OrderSaleTransfer{
			OrderId:                  data.OrderId,
			Certificate:              data.Certificate,
			VehicleLicense:           data.VehicleLicense,
			Invoice:                  data.Invoice,
			GuaranteeSlip:            data.GuaranteeSlip,
			CompulsoryInsuranceProof: data.CompulsoryInsuranceProof,
			CommercialInsuranceProof: data.CommercialInsuranceProof,
		}
		if tx.Create(&saleTransfer).Error != nil {
			return errors.New("提交失败")
		}
		message.MessageLogic.MessageToCustomer(tx, []uint{orderSale.CustomerId}, brokerId, model.CustomerMessageTypeOrder, message.MessageItem{
			Title:   "订单提示",
			Content: "您的买车合同已完成过户"})
		return nil
	})
}

type OfflineTransferInfo struct {
	SaleTransfer
	Report string `label:"检车报告"`
}

// OfflineTransfer @Title 线下客户订单过户
func (s *saleLogic) OfflineTransfer(brokerId uint, data OfflineTransferInfo) error {
	orderSale := model.OrderSale{Id: data.OrderId, BrokerId: brokerId}
	if mysql.Db.First(&orderSale).Error != nil {
		return errors.New("订单错误")
	}
	if orderSale.Status != model.OrderSaleStatusSign {
		return errors.New("订单状态错误")
	}
	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		orderSaleReport := model.OrderSaleReport{
			OrderId: data.OrderId,
			Path:    data.Report,
		}
		if tx.Create(&orderSaleReport).Error != nil {
			return errors.New("提交失败")
		}
		saleTransfer := model.OrderSaleTransfer{
			OrderId:                  data.OrderId,
			Certificate:              data.Certificate,
			VehicleLicense:           data.VehicleLicense,
			Invoice:                  data.Invoice,
			GuaranteeSlip:            data.GuaranteeSlip,
			CompulsoryInsuranceProof: data.CompulsoryInsuranceProof,
			CommercialInsuranceProof: data.CommercialInsuranceProof,
		}
		if tx.Create(&saleTransfer).Error != nil {
			return errors.New("提交失败")
		}
		if tx.Model(&orderSale).Where(&orderSale).UpdateColumn("status", model.OrderSaleStatusTransferFinal).RowsAffected != 1 {
			return errors.New("提交失败")
		}
		return nil
	})
}

// OfflineBalancePayment @Title 线下订单上传尾款支付
func (s *saleLogic) OfflineBalancePayment(brokerId, orderId uint, proof string) error {
	orderSale := model.OrderSale{Id: orderId, BrokerId: brokerId}
	if mysql.Db.Preload("BalancePaymentAudit").Where(&orderSale).First(&orderSale).Error != nil {
		return errors.New("订单错误")
	}
	if orderSale.Status < model.OrderSaleStatusTransferFinal {
		return errors.New("订单状态错误")
	}
	if orderSale.Status >= model.OrderSaleStatusBalancePaymentAdopt {
		return errors.New("已支付尾款，请勿重复操作")
	}
	if orderSale.BalancePaymentAudit.Status == model.OrderSaleAuditStatusAudit {
		return errors.New("系统审核中，请等待")
	}
	saleAudit := model.OrderSaleAudit{
		OrderId: orderId,
		Type:    model.OrderSaleAuditTypeBalancePayment,
		Status:  model.OrderSaleAuditStatusAudit,
		Proof:   proof,
	}
	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		if tx.Create(&saleAudit).Error != nil {
			return errors.New("提交失败")
		}
		if tx.Model(&model.OrderSale{}).Where(&orderSale).UpdateColumns(map[string]interface{}{
			"balance_payment_audit_id": saleAudit.Id,
			"status":                   model.OrderSaleStatusBalancePaymentAudit,
		}).RowsAffected != 1 {
			return errors.New("提交失败")
		}
		return nil
	})
}

// Final @Title 完结客户订单
func (s *saleLogic) Final(brokerId, orderId uint) error {
	orderSale := model.OrderSale{Id: orderId, BrokerId: brokerId}
	if mysql.Db.First(&orderSale).Error != nil {
		return errors.New("订单错误")
	}
	if orderSale.Status == model.OrderSaleStatusFinal {
		return errors.New("订单已完结")
	}
	if orderSale.Status < model.OrderSaleStatusBalancePaymentAdopt {
		return errors.New("订单尚未结束")
	}

	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		if tx.Model(&orderSale).Where(&orderSale).UpdateColumn("status", model.OrderSaleStatusFinal).RowsAffected != 1 {
			return errors.New("提交失败")
		}
		if tx.Model(&model.Car{}).Where("id = ?", orderSale.CarId).UpdateColumn("status", model.CarStatusFinal).Error != nil {
			return errors.New("提交失败")
		}
		switch orderSale.CustomerChannel {
		case model.OrderSaleCustomerChannelWxApp:
			if tx.Model(&model.Customer{}).Where("id = ? and level < ?", orderSale.CustomerId, model.CustomerLevelDone).UpdateColumn("level", model.CustomerLevelDone).Error != nil {
				return errors.New("提交失败")
			}
			message.MessageLogic.MessageToCustomer(mysql.Db, []uint{orderSale.CustomerId}, brokerId, model.CustomerMessageTypeOrder, message.MessageItem{
				Title:   "订单提示",
				Content: "您的买车合同已完结订单"})
		}
		message.MessageLogic.MessageToBroker(mysql.Db, []uint{brokerId}, 0, model.BrokerMessageTypeNormal, message.MessageItem{
			Title:    "订单提示",
			LinkedId: orderId,
			Content:  "您的卖车合同已完结订单"})
		return nil
	})
}

type saleInfo struct {
	Id                  uint               `json:"id"`
	CustomerChannel     uint               `json:"customerChannel"`
	CustomerChannelName string             `json:"customerChannelName"`
	Status              uint               `json:"status"`
	StatusName          string             `json:"statusName"`
	CreatedAt           int64              `json:"createdAt"`
	Car                 saleCar            `json:"car"`
	Contract            saleContract       `json:"contract"`
	Deposit             saleDeposit        `json:"deposit"`
	DownPayment         saleDownPayment    `json:"downPayment"`
	BalancePayment      saleBalancePayment `json:"balancePayment"`
	Report              saleReport         `json:"report"`
	Means               saleMeans          `json:"means"`
	Customer            SaleCustomer       `json:"customer"`
}

type saleCar struct {
	CarId         uint            `json:"carId"`
	MainPhoto     string          `json:"mainPhoto"`
	ModelName     string          `json:"modelName"`
	LicensingDate int64           `json:"licensingDate"`
	Mileage       decimal.Decimal `json:"mileage"`
	Transfer      uint            `json:"transfer"`
}

type saleContract struct {
	Contract           uint            `json:"contract"`
	ContractSn         string          `json:"contractSn"`
	Amount             decimal.Decimal `json:"amount"`
	Deposit            decimal.Decimal `json:"deposit"`
	DownPayment        decimal.Decimal `json:"downPayment"`
	BalancePayment     decimal.Decimal `json:"balancePayment"`
	PayTypeName        string          `json:"payTypeName"`      // 支付方式
	TransferTypeName   string          `json:"transferTypeName"` // 过户方式
	ThirdPartKind      string          `json:"thirdPartKind"`
	ThirdPartId        uint            `json:"thirdPartId"`
	ThirdPartName      string          `json:"thirdPartName"`
	SaleServiceFee     decimal.Decimal `json:"saleServiceFee"`
	PurchaseServiceFee decimal.Decimal `json:"purchaseSaleServiceFee"`
	TotalAmount        decimal.Decimal `json:"totalAmount"`
	SignAt             int64           `json:"signAt"`
}

type saleDeposit struct {
	Id        uint            `json:"id"`
	Amount    decimal.Decimal `json:"amount"`
	PayType   string          `json:"payType"`
	CreatedAt int64           `json:"createdAt"`
}

type saleDownPayment struct {
	Id        uint            `json:"id"`
	Amount    decimal.Decimal `json:"amount"`
	Proof     string          `json:"proof"`
	CreatedAt int64           `json:"createdAt"`
}

type saleBalancePayment struct {
	Id        uint            `json:"id"`
	Amount    decimal.Decimal `json:"amount"`
	Proof     string          `json:"proof"`
	CreatedAt int64           `json:"createdAt"`
}

type saleReport struct {
	Id   uint   `json:"id"`
	Path string `json:"path"`
}

type saleMeans struct {
	Id             uint   `json:"id"`
	Certificate    string `json:"certificate"`
	VehicleLicense string `json:"vehicleLicense"`
	Invoice        string `json:"invoice"`
	GuaranteeSlip  string `json:"guaranteeSlip"`
}

type SaleCustomer struct {
	Id    uint   `json:"id"`
	Name  string `json:"name"`
	Phone string `json:"phone"`
}

// Info @Title 订单详情
func (s *saleLogic) Info(brokerIds []uint, orderId uint) (result saleInfo, err error) {
	orderSale := model.OrderSale{Id: orderId}
	if mysql.Db.
		Preload("Car.Model").Preload("Contract.ThirdPart").
		Preload("DepositPay").Preload("DownPaymentAudit").Preload("BalancePaymentAudit").
		Preload("Report").Preload("Transfer").Preload("Customer").
		First(&orderSale).Error != nil {
		return result, errors.New("订单错误")
	}
	result = saleInfo{
		Id:                  orderSale.Id,
		CustomerChannel:     orderSale.CustomerChannel,
		CustomerChannelName: orderSale.GetCustomerChannelName(orderSale.CustomerChannel),
		Status:              orderSale.Status,
		StatusName:          orderSale.GetStatusName(orderSale.Status),
		CreatedAt:           orderSale.CreatedAt.Unix(),
		Car: saleCar{
			CarId:         orderSale.CarId,
			MainPhoto:     orderSale.Car.MainPhoto,
			ModelName:     orderSale.Car.Model.Name,
			LicensingDate: orderSale.Car.LicensingDate.Time.Unix(),
			Mileage:       orderSale.Car.Mileage,
			Transfer:      orderSale.Car.Transfer,
		},
		Contract: saleContract{
			Contract:           orderSale.Contract.Id,
			ContractSn:         orderSale.Contract.ContractSn,
			Amount:             orderSale.Contract.Amount,
			Deposit:            orderSale.Contract.Deposit,
			DownPayment:        orderSale.Contract.DownPayment,
			BalancePayment:     orderSale.Contract.BalancePayment,
			PayTypeName:        orderSale.Contract.GetPayTypeName(orderSale.Contract.PayType),
			TransferTypeName:   orderSale.Contract.GetTransferTypeName(orderSale.Contract.TransferType),
			ThirdPartKind:      orderSale.Contract.GetThirdPartKindName(orderSale.Contract.ThirdPartKind),
			ThirdPartId:        orderSale.Contract.ThirdPartId,
			ThirdPartName:      orderSale.Contract.ThirdPart.Name,
			SaleServiceFee:     orderSale.Contract.SaleServiceFee,
			PurchaseServiceFee: orderSale.Contract.PurchaseServiceFee,
			TotalAmount:        orderSale.Contract.Amount.Add(orderSale.Contract.SaleServiceFee).Add(orderSale.Contract.PurchaseServiceFee),
			SignAt:             orderSale.Contract.SignAt.Time.Unix(),
		},
		Deposit: saleDeposit{
			Id:        orderSale.DepositPay.Id,
			Amount:    orderSale.DepositPay.Amount,
			PayType:   orderSale.DepositPay.GetPayTypeName(),
			CreatedAt: orderSale.DepositPay.CreatedAt.Unix(),
		},
		DownPayment: saleDownPayment{
			Id:        orderSale.DownPaymentAudit.Id,
			Amount:    orderSale.Contract.DownPayment,
			Proof:     orderSale.DownPaymentAudit.Proof,
			CreatedAt: orderSale.DownPaymentAudit.CreatedAt.Unix(),
		},
		BalancePayment: saleBalancePayment{
			Id:        orderSale.BalancePaymentAudit.Id,
			Amount:    orderSale.Contract.BalancePayment,
			Proof:     orderSale.BalancePaymentAudit.Proof,
			CreatedAt: orderSale.BalancePaymentAudit.CreatedAt.Unix(),
		},
		Report: saleReport{
			Id:   orderSale.Report.Id,
			Path: orderSale.Report.Path,
		},
		Means: saleMeans{
			Id:             orderSale.Transfer.Id,
			Certificate:    orderSale.Transfer.Certificate,
			VehicleLicense: orderSale.Transfer.VehicleLicense,
			Invoice:        orderSale.Transfer.Invoice,
			GuaranteeSlip:  orderSale.Transfer.GuaranteeSlip,
		},
		Customer: SaleCustomer{
			Id:    orderSale.CustomerId,
			Name:  orderSale.Customer.Nickname,
			Phone: orderSale.Customer.Mobile,
		},
	}
	return
}
