package split

import (
	"base/app/lib/bean"
	"base/app/model"
	"database/sql"
	"errors"
	"fmt"
	"git.oa00.com/go/mysql"
	"github.com/shopspring/decimal"
	"gorm.io/gorm"
	"time"
)

var AccountLogic = accountLogic{}

type accountLogic struct {
}

type AccountItem struct {
	Id       uint   `json:"id"`
	Name     string `json:"name"`
	LastBill string `json:"lastBill"`
	CreateAt int64  `json:"createAt"`
}

// List @Title 分账列表
func (a *accountLogic) List(brokerId, status uint, page bean.Page) (lists []AccountItem, total int64) {

	where := mysql.Db.Where("id in (?)", mysql.Db.Model(&model.SplitAccountBroker{}).Where("broker_id = ?", brokerId).Select("account_id"))
	// 状态筛选
	switch status {
	case 1: // 进行中
		where = where.Where("status = ?", model.SplitAccountStatusInProgress)
	case 2: // 已完结
		where = where.Where("status = ?", model.SplitAccountStatusFinish)
	}

	var splitAccounts []model.SplitAccount
	mysql.Db.Model(&splitAccounts).Where(where).Count(&total)
	if page.HasPage(total) {
		mysql.Db.Where(where).Offset(page.GetStart()).Limit(page.GetLimit()).Find(&splitAccounts)
		for _, account := range splitAccounts {
			lists = append(lists, AccountItem{
				Id:       account.Id,
				Name:     account.Name,
				LastBill: account.LastBill,
				CreateAt: account.CreatedAt.Unix(),
			})
		}
	}
	return
}

type CreateData struct {
	OwnAmount     decimal.Decimal    `binding:"required" label:"金额"`
	Name          string             `binding:"required" label:"名称"`
	BrokerAmounts []BrokerAmountItem `binding:"required" label:"投资人"`
	ModelId       uint               `binding:"required" label:"车型id"`
	LocationId    uint               `binding:"required" label:"车辆所在地"`
	LicensingDate string             `binding:"required" label:"上牌日期"`
	Color         string             `binding:"required" label:"车身颜色"`
	Mileage       decimal.Decimal    `binding:"required" label:"里程"`
}

type BrokerAmountItem struct {
	BrokerId uint            `binding:"required" label:"投资人"`
	Amount   decimal.Decimal `binding:"required" label:"投资人金额"`
}

// Create @Title 创建分账
func (a *accountLogic) Create(brokerId uint, createData CreateData) error {

	totalAmount := createData.OwnAmount
	var brokers []model.SplitAccountBroker

	brokers = append(brokers, model.SplitAccountBroker{
		BrokerId:   brokerId,
		Amount:     createData.OwnAmount,
		BillAmount: decimal.Zero,
	})

	for _, amount := range createData.BrokerAmounts {
		totalAmount = totalAmount.Add(amount.Amount)
		brokers = append(brokers, model.SplitAccountBroker{
			BrokerId:   amount.BrokerId,
			Amount:     amount.Amount,
			BillAmount: decimal.Zero,
		})
	}
	licensingDate, err := time.ParseInLocation("2006-01-02", createData.LicensingDate, time.Local)
	if err != nil {
		return errors.New("上牌日期时间错误")
	}
	splitAccount := model.SplitAccount{
		Name:          createData.Name,
		SponsorId:     brokerId,
		LastBill:      "买车",
		Status:        model.SplitAccountStatusInProgress,
		TotalAmount:   totalAmount,
		BillAmount:    decimal.Zero,
		SaleAmount:    decimal.Zero,
		ModelId:       createData.ModelId,
		LocationId:    createData.LocationId,
		LicensingDate: sql.NullTime{Time: licensingDate, Valid: true},
		Color:         createData.Color,
		Mileage:       createData.Mileage,
		Brokers:       brokers,
	}

	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		if tx.Create(&splitAccount).Error != nil {
			return errors.New("创建失败")
		}
		return nil
	})
}

type BillData struct {
	AccountId uint            `binding:"required" label:"添加人"`
	Name      string          `binding:"required" label:"名称"`
	Amount    decimal.Decimal `binding:"required" label:"金额"`
}

// AddBill @Title 添加账单
func (a *accountLogic) AddBill(brokerId uint, data BillData) error {

	var bill = model.SplitAccountBill{
		AccountId:   data.AccountId,
		Name:        data.Name,
		Description: "",
		BrokerId:    brokerId,
		Amount:      data.Amount,
	}

	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		if tx.Create(&bill).Error != nil {
			return errors.New("添加失败")
		}
		if tx.Model(&model.SplitAccount{}).Where("id = ?", data.AccountId).Updates(map[string]interface{}{
			"last_bill":   data.Name,
			"bill_amount": gorm.Expr("bill_amount + ?", data.Amount),
		}).RowsAffected != 1 {
			return errors.New("添加失败")
		}

		if tx.Model(&model.SplitAccountBroker{}).Where("broker_id = ? and account_id = ?", brokerId, data.AccountId).Updates(map[string]interface{}{
			"bill_amount": gorm.Expr("bill_amount + ?", data.Amount),
		}).RowsAffected != 1 {
			return errors.New("添加失败")
		}

		return nil
	})
}

type AccountInfo struct {
	Amount        decimal.Decimal   `json:"amount"`
	BillAmount    decimal.Decimal   `json:"billAmount"`
	SaleAmount    decimal.Decimal   `json:"saleAmount"`
	OwnBillAmount decimal.Decimal   `json:"ownBillAmount"`
	OwnProfit     decimal.Decimal   `json:"ownProfit"`
	TotalProfit   decimal.Decimal   `json:"totalProfit"`
	Name          string            `json:"name"`
	Brokers       []BrokerItem      `json:"brokers"`
	DateBills     []BillItemsByDate `json:"dateBills"`
	Finish        bool              `json:"finish"`
	ModelName     string            `json:"modelName"`
	Location      string            `json:"location"`
	LicensingDate int64             `json:"licensingDate"`
	Color         string            `json:"color"`
	Mileage       decimal.Decimal   `json:"mileage"`
}

type BrokerItem struct {
	BrokerId   uint            `json:"brokerId"`
	Name       string          `json:"name"`
	RoleId     uint            `json:"roleId"`
	Avatar     string          `json:"avatar"`
	Proportion decimal.Decimal `json:"proportion"`
	Amount     decimal.Decimal `json:"amount"`
	BillAmount decimal.Decimal `json:"billAmount"`
	Profit     decimal.Decimal `json:"profit"`
}

type BillItem struct {
	Name       string          `json:"name"`
	BrokerName string          `json:"brokerName"`
	Amount     decimal.Decimal `json:"amount"`
	Symbol     bool            `json:"symbol"`
	CreateAt   int64           `json:"createAt"`
}

type BillItemsByDate struct {
	Date  string     `json:"date"`
	Bills []BillItem `json:"bills"`
}

// AccountInfo @Title 分账明细
func (a *accountLogic) AccountInfo(accountId, brokerId uint) (info AccountInfo, err error) {
	var splitAccount model.SplitAccount
	if mysql.Db.Preload("Broker").Preload("Model").Preload("Location").Preload("Brokers.Broker.Staff").Preload("Bills.Broker").Where("id = ?", accountId).First(&splitAccount).Error != nil {
		err = errors.New("分账错误")
		return
	}
	info = AccountInfo{
		Amount:        splitAccount.TotalAmount,
		BillAmount:    splitAccount.BillAmount,
		SaleAmount:    splitAccount.SaleAmount,
		Name:          splitAccount.Name,
		Finish:        false,
		ModelName:     splitAccount.Model.Name,
		Location:      splitAccount.Location.Name,
		LicensingDate: splitAccount.LicensingDate.Time.Unix(),
		Color:         splitAccount.Color,
		Mileage:       splitAccount.Mileage,
	}

	for _, broker := range splitAccount.Brokers {
		proportion := broker.Amount.Div(splitAccount.TotalAmount)
		if broker.BrokerId == brokerId {
			info.OwnProfit = broker.Profit
			info.OwnBillAmount = broker.BillAmount
		}
		var roleId = uint(0)
		//if broker.Broker.Staff != nil {
		roleId = broker.Broker.Staff.RoleId
		//}
		info.Brokers = append(info.Brokers, BrokerItem{
			BrokerId:   broker.BrokerId,
			Name:       broker.Broker.Nickname,
			RoleId:     roleId,
			Avatar:     broker.Broker.HeadImg,
			Proportion: proportion,
			Amount:     broker.Amount,
			BillAmount: broker.BillAmount,
			Profit:     broker.Profit,
		})
	}
	year, month, _ := splitAccount.CreatedAt.Date()
	date := fmt.Sprintf("%d年%d月", year, month)
	var mDateBill = map[string][]BillItem{}
	mDateBill[date] = []BillItem{
		{
			Name:       "买车",
			BrokerName: splitAccount.Broker.Nickname,
			Amount:     splitAccount.TotalAmount,
			Symbol:     false,
			CreateAt:   splitAccount.CreatedAt.Unix(),
		},
	}
	// 遍历账单 以日期分组
	for _, bill := range splitAccount.Bills {
		newYear, newMonth, _ := bill.CreatedAt.Date()
		if year == newYear && newMonth == month {
			mDateBill[date] = append(mDateBill[date], BillItem{
				Name:       bill.Name,
				BrokerName: bill.Broker.Nickname,
				Amount:     bill.Amount,
				Symbol:     false,
				CreateAt:   bill.CreatedAt.Unix(),
			})
		} else {
			date = fmt.Sprintf("%d年%d月", newYear, newMonth)
			mDateBill[date] = []BillItem{
				{
					Name:       bill.Name,
					BrokerName: bill.Broker.Nickname,
					Amount:     bill.Amount,
					Symbol:     false,
					CreateAt:   bill.CreatedAt.Unix(),
				},
			}
		}
	}
	// 车已出售
	if splitAccount.Status == model.SplitAccountStatusFinish {
		newYear, newMonth, _ := splitAccount.CreatedAt.Date()
		if year == newYear && newMonth == month {
			mDateBill[date] = append(mDateBill[date], BillItem{
				Name:       "卖车",
				BrokerName: splitAccount.Broker.Nickname,
				Amount:     splitAccount.SaleAmount,
				Symbol:     true,
				CreateAt:   splitAccount.SaleAt.Time.Unix(),
			})
		} else {
			date = fmt.Sprintf("%d年%d月", newYear, newMonth)
			mDateBill[date] = []BillItem{
				{
					Name:       "卖车",
					BrokerName: splitAccount.Broker.Nickname,
					Amount:     splitAccount.SaleAmount,
					Symbol:     true,
					CreateAt:   splitAccount.SaleAt.Time.Unix(),
				},
			}
		}
		info.Finish = true
		info.TotalProfit = splitAccount.SaleAmount.Sub(splitAccount.TotalAmount).Sub(splitAccount.BillAmount)
	}

	for s, items := range mDateBill {
		info.DateBills = append(info.DateBills, BillItemsByDate{
			Date:  s,
			Bills: items,
		})
	}
	return
}

// EditName @Title 编辑名字
func (a *accountLogic) EditName(accountId, brokerId uint, name string) error {
	var splitAccount = model.SplitAccount{Id: accountId}
	if mysql.Db.Where("id = ? ", accountId).First(&splitAccount).Error != nil {
		return errors.New("分账错误")
	}

	if splitAccount.SponsorId != brokerId {
		return errors.New("非创建人不可修改")
	}

	if mysql.Db.Model(&model.SplitAccount{}).Where("id = ?", accountId).Update("name", name).RowsAffected != 1 {
		return errors.New("修改失败")
	}
	return nil
}

type ProfitInfo struct {
	Total   decimal.Decimal `json:"total"`
	Profits []ProfitItem    `json:"profits"`
}

type ProfitItem struct {
	Name     string          `json:"name"`
	FinishAt int64           `json:"finishAt"`
	Profit   decimal.Decimal `json:"profit"`
}

// Profit @Title 利润统计
func (a *accountLogic) Profit(brokerId uint) (result ProfitInfo, err error) {
	//var splitAccounts []model.SplitAccount
	//
	//if mysql.Db.Where("status = ? and id in (?)", model.SplitAccountStatusFinish, mysql.Db.Where("broker_id = ?", brokerId).Select("account_id")).Find(&splitAccounts).Error != nil {
	//	err = errors.New("分账错误")
	//	return
	//}
	//total := decimal.Zero
	//for i, account := range splitAccounts {
	//	profit :=
	//	result.Profits = append(result.Profits, ProfitItem{
	//		Name:     account.Name,
	//		FinishAt: account.SaleAt.Unix(),
	//		Profit:   decimal.Decimal{},
	//	})
	//}

	var accountBrokers []model.SplitAccountBroker
	if mysql.Db.Where("broker_id = ?", brokerId).Preload("Account").Find(&accountBrokers).Error != nil {
		err = errors.New("找不到分账单")
		return
	}
	total := decimal.Zero
	for _, broker := range accountBrokers {
		if broker.Account.Status == model.SplitAccountStatusFinish {
			percent := broker.Amount.Div(broker.Account.TotalAmount)
			profit := broker.Account.SaleAmount.Sub(broker.Account.TotalAmount).Sub(broker.Account.BillAmount).Mul(percent)
			total = total.Add(profit)
			result.Profits = append(result.Profits, ProfitItem{
				Name:     broker.Account.Name,
				FinishAt: broker.Account.SaleAt.Time.Unix(),
				Profit:   profit,
			})
		}
	}
	result.Total = total
	return
}

// Finish @Title 分账结束
func (a *accountLogic) Finish(brokerId, accountId uint, amount decimal.Decimal) error {
	var splitAccount = model.SplitAccount{Id: accountId}
	if mysql.Db.Where(&splitAccount).Preload("Brokers").First(&splitAccount).Error != nil {
		return errors.New("分账错误")
	}
	if splitAccount.Status == model.SplitAccountStatusFinish {
		return errors.New("分账已完结")
	}
	if splitAccount.SponsorId != brokerId {
		return errors.New("非创建人不能结束分账")
	}
	carProfitPercent := amount.Sub(splitAccount.TotalAmount).Sub(splitAccount.BillAmount).Div(splitAccount.TotalAmount)
	var splitBrokerIds []uint
	for _, broker := range splitAccount.Brokers {
		splitBrokerIds = append(splitBrokerIds, broker.Id)
	}
	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		if tx.Model(&model.SplitAccount{}).Where("id = ?", accountId).Updates(map[string]interface{}{
			"status": model.SplitAccountStatusFinish,
			"sale_at": sql.NullTime{
				Time:  time.Now(),
				Valid: true,
			},
			"sale_amount": amount,
		}).RowsAffected != 1 {
			return errors.New("结束分账失败")
		}
		if tx.Model(&model.SplitAccountBroker{}).Where("id in ?", splitBrokerIds).Update("profit", gorm.Expr("amount * ?", carProfitPercent)).RowsAffected != int64(len(splitBrokerIds)) {
			return errors.New("结束分账失败")
		}
		return nil
	})
}
