package wallet

import (
	"base/app/common"
	"base/app/constant"
	"base/app/lib/bean"
	"base/app/logic/pay"
	"base/app/model"
	"errors"
	"fmt"
	"git.oa00.com/go/mysql"
	"github.com/shopspring/decimal"
	"gorm.io/gorm"
	"log"
)

var AssessLogic = &assessLogic{}

type assessLogic struct {
}

// Count @Title 获取剩余评估次数
func (a *assessLogic) Count(brokerInfo model.Broker, kind uint) int {
	switch brokerInfo.Level {
	case model.BrokerLevelBusiness:
		assess := model.DealerWalletAssess{DealerId: brokerInfo.StoreId, Kind: kind}
		mysql.Db.Where(&assess).First(&assess)
		return assess.Count
	default:
		assess := model.BrokerWalletAssess{BrokerId: brokerInfo.Id, Kind: kind}
		mysql.Db.Where(&assess).First(&assess)
		return assess.Count
	}
}

// Recharge @Title 充值
func (a *assessLogic) Recharge(brokerInfo model.Broker, count, payType, kind uint) (result interface{}, err error) {
	orderSn, err := common.GenNo(constant.NoPrefixOrderAssess)
	if err != nil {
		return result, errors.New("单号生成失败")
	}
	uintPrice, err := getUintPrice(count, kind)
	if err != nil {
		return
	}
	orderAssess := model.OrderAssess{
		OrderSn: orderSn,
		Kind:    kind,
		Amount:  decimal.NewFromFloat(uintPrice).Mul(decimal.NewFromInt(int64(count))),
		Count:   count,
		PayType: payType,
		Status:  model.OrderAssessStatusUnPay,
	}
	switch brokerInfo.Level {
	case model.BrokerLevelBusiness:
		orderAssess.Type = model.OrderAssessTypeDealer
		orderAssess.UserId = brokerInfo.StoreId
	default:
		orderAssess.Type = model.OrderAssessTypeBroker
		orderAssess.UserId = brokerInfo.Id
	}
	err = mysql.Db.Transaction(func(tx *gorm.DB) error {
		if tx.Create(&orderAssess).Error != nil {
			return errors.New("充值失败")
		}
		result, err = pay.PayLogic.Pay(orderAssess.Id, model.PayOrderTypeAssess, payType, orderAssess.Amount, "钱包次数充值", "")
		if err != nil {
			log.Println(err)
			return errors.New("充值失败")
		}
		return nil
	})
	return
}

type assessHistoryItem struct {
	Id        uint  `json:"id"`
	Type      uint  `json:"type"`
	Kind      uint  `json:"kind"`
	Count     int   `json:"count"`
	CreatedAt int64 `json:"createdAt"`
}
type AssessHistorySearch struct {
	Kind uint `label:"余量种类(1=评估;2=合同)"`
	Date string
}

// History @Title 评估次数记录
func (a *assessLogic) History(brokerInfo model.Broker, search AssessHistorySearch, page bean.Page) (lists []assessHistoryItem, total int64) {
	lists = []assessHistoryItem{}
	where := mysql.Db
	switch brokerInfo.Level {
	case model.BrokerLevelBusiness:
		where = where.Where("user_type = ?", model.WalletAssessHistoryUserTypeDealer).Where("user_id = ?", brokerInfo.StoreId)
	default:
		where = where.Where("user_type = ?", model.WalletAssessHistoryUserTypeBroker).Where("user_id = ?", brokerInfo.Id)
	}
	var assessHistories []model.WalletAssessHistory
	if search.Kind > 0 {
		where = where.Where("kind = ?", search.Kind)
	}
	if search.Date != "" {
		where = where.Where("created_at like ?", fmt.Sprintf("%s%%", search.Date))
	}
	mysql.Db.Model(&assessHistories).Where(where).Count(&total)
	if page.HasPage(total) {
		mysql.Db.Where(where).Offset(page.GetStart()).Limit(page.GetLimit()).Order("id desc").Find(&assessHistories)
		for _, assessHistory := range assessHistories {
			lists = append(lists, assessHistoryItem{
				Id:        assessHistory.Id,
				Type:      assessHistory.Type,
				Kind:      assessHistory.Kind,
				Count:     assessHistory.Count,
				CreatedAt: assessHistory.CreatedAt.Unix(),
			})
		}
	}
	return
}

// Add @Title 添加评估次数
func (a *assessLogic) Add(db *gorm.DB, brokerId, orderId, kind uint, count int) error {
	walletAssess := model.BrokerWalletAssess{BrokerId: brokerId, Kind: kind}
	db.Where(&walletAssess).First(&walletAssess)
	if walletAssess.Id > 0 {
		if db.Model(&walletAssess).Where(&walletAssess).UpdateColumns(map[string]interface{}{
			"total_count": gorm.Expr("total_count + ?", count),
			"count":       gorm.Expr("count + ?", count),
		}).RowsAffected != 1 {
			return errors.New("系统错误")
		}
	} else {
		walletAssess.TotalCount = count
		walletAssess.Count = count
		if db.Create(&walletAssess).Error != nil {
			return errors.New("系统错误")
		}
	}
	assessHistory := model.WalletAssessHistory{
		UserType: model.WalletAssessHistoryUserTypeBroker,
		UserId:   brokerId,
		Type:     model.WalletAssessHistoryRecharge,
		Kind:     kind,
		OrderId:  orderId,
		Count:    count,
	}
	if db.Create(&assessHistory).Error != nil {
		return errors.New("系统错误")
	}
	return nil
}

// DealerAdd @Title 添加车商评估次数
func (a *assessLogic) DealerAdd(db *gorm.DB, dealerId, orderId, kind uint, count int) error {
	walletAssess := model.DealerWalletAssess{DealerId: dealerId, Kind: kind}
	db.Where(&walletAssess).First(&walletAssess)
	if walletAssess.Id > 0 {
		if db.Model(&walletAssess).Where(&walletAssess).UpdateColumns(map[string]interface{}{
			"total_count": gorm.Expr("total_count + ?", count),
			"count":       gorm.Expr("count + ?", count),
		}).RowsAffected != 1 {
			return errors.New("系统错误")
		}
	} else {
		walletAssess.TotalCount = count
		walletAssess.Count = count
		if db.Create(&walletAssess).Error != nil {
			return errors.New("系统错误")
		}
	}
	assessHistory := model.WalletAssessHistory{
		UserType: model.WalletAssessHistoryUserTypeDealer,
		UserId:   dealerId,
		Type:     model.WalletAssessHistoryRecharge,
		Kind:     kind,
		OrderId:  orderId,
		Count:    count,
	}
	if db.Create(&assessHistory).Error != nil {
		return errors.New("系统错误")
	}
	return nil
}

// Use @Title 使用钱包次数
func (a *assessLogic) Use(db *gorm.DB, assessKind, assessHistoryKind, brokerId, orderId uint, count int) error {
	brokerInfo := model.Broker{Id: brokerId}
	if mysql.Db.Where(&brokerInfo).First(&brokerInfo).Error != nil {
		return errors.New("经纪人信息错误")
	}
	assessHistory := model.WalletAssessHistory{
		Type:    model.WalletAssessHistoryUse,
		Kind:    assessHistoryKind,
		OrderId: orderId,
		Count:   count * -1,
	}
	switch brokerInfo.Level {
	case model.BrokerLevelBusiness:
		walletAssess := model.DealerWalletAssess{DealerId: brokerInfo.StoreId, Kind: assessKind}
		var assessKindName string
		switch assessKind {
		case model.WalletAssessKindEstimate:
			assessKindName = "评估"
		case model.WalletAssessKindContract:
			assessKindName = "合同"
		}
		mysql.Db.Where(&walletAssess).First(&walletAssess)
		if walletAssess.Count <= 0 {
			return errors.New(assessKindName + "次数不足")
		}
		if db.Model(&model.DealerWalletAssess{}).Where("dealer_id = ? and kind = ?", brokerInfo.StoreId, assessKind).UpdateColumns(map[string]interface{}{
			"use_count": gorm.Expr("use_count + ?", count),
			"count":     gorm.Expr("count - ?", count),
		}).RowsAffected != 1 {
			return errors.New("使用钱包次数错误")
		}
		assessHistory.UserType = model.WalletAssessHistoryUserTypeDealer
		assessHistory.UserId = brokerInfo.StoreId
	default:
		walletAssess := model.BrokerWalletAssess{BrokerId: brokerId, Kind: assessKind}
		var assessKindName string
		switch assessKind {
		case model.WalletAssessKindEstimate:
			assessKindName = "评估"
		case model.WalletAssessKindContract:
			assessKindName = "合同"
		}
		mysql.Db.Where(&walletAssess).First(&walletAssess)
		if walletAssess.Count <= 0 {
			return errors.New(assessKindName + "次数不足")
		}
		if db.Model(&model.BrokerWalletAssess{}).Where("broker_id = ? and kind = ?", brokerId, assessKind).UpdateColumns(map[string]interface{}{
			"use_count": gorm.Expr("use_count + ?", count),
			"count":     gorm.Expr("count - ?", count),
		}).RowsAffected != 1 {
			return errors.New("使用钱包次数错误")
		}
		assessHistory.UserType = model.WalletAssessHistoryUserTypeBroker
		assessHistory.UserId = brokerId
		if db.Create(&assessHistory).Error != nil {
			return errors.New("创建使用钱包次数记录错误")
		}
	}
	if db.Create(&assessHistory).Error != nil {
		return errors.New("创建使用钱包次数记录错误")
	}
	return nil
}

// Refund @Title 返还钱包次数
func (a *assessLogic) Refund(db *gorm.DB, assessKind, assessHistoryUserType, assessHistoryUserId, assessHistoryKind, orderId uint, count int) error {
	assessHistory := model.WalletAssessHistory{
		UserType: assessHistoryUserType,
		UserId:   assessHistoryUserId,
		Type:     model.WalletAssessHistoryRefund,
		Kind:     assessHistoryKind,
		OrderId:  orderId,
		Count:    count,
	}
	switch assessHistoryUserType {
	case model.WalletAssessHistoryUserTypeDealer:
		if db.Model(&model.DealerWalletAssess{}).Where("dealer_id = ? and kind = ?", assessHistoryUserId, assessKind).UpdateColumns(map[string]interface{}{
			"use_count": gorm.Expr("use_count - ?", count),
			"count":     gorm.Expr("count + ?", count),
		}).RowsAffected != 1 {
			return errors.New("返还钱包次数错误")
		}
	case model.WalletAssessHistoryUserTypeBroker:
		if db.Model(&model.BrokerWalletAssess{}).Where("broker_id = ? and kind = ?", assessHistoryUserId, assessKind).UpdateColumns(map[string]interface{}{
			"use_count": gorm.Expr("use_count - ?", count),
			"count":     gorm.Expr("count + ?", count),
		}).RowsAffected != 1 {
			return errors.New("返还钱包次数错误")
		}
	}
	if db.Create(&assessHistory).Error != nil {
		return errors.New("创建使用钱包次数记录错误")
	}
	return nil
}

// 次数充值定价方案
func getUintPrice(count, kind uint) (uintPrice float64, err error) {
	switch kind {
	case model.OrderAssessKindEstimate:
		if count <= 10 {
			uintPrice = 3
		} else if count <= 50 {
			uintPrice = 2.5
		} else if count <= 100 {
			uintPrice = 2
		} else {
			uintPrice = 1.5
		}
	case model.OrderAssessKindContract:
		if count <= 10 {
			uintPrice = 5
		} else if count <= 50 {
			uintPrice = 3
		} else if count <= 100 {
			uintPrice = 2.8
		} else {
			uintPrice = 2.5
		}
	}
	if uintPrice == 0 {
		return uintPrice, errors.New("价位获取异常")
	}
	return
}
