package car

import (
	"base/app/constant"
	"base/app/lib/bean"
	"base/app/lib/opensearch"
	"base/app/model"
	"base/app/model/es"
	"bytes"
	"encoding/json"
	"errors"
	"fmt"
	"git.oa00.com/go/logger"
	"git.oa00.com/go/mysql"
	"github.com/shopspring/decimal"
	"go.uber.org/zap"
	"gorm.io/gorm"
	"log"
	"time"
)

const (
	CarOrderNewCreate  = "new_create"  // 最新创建
	CarOrderMaxPrice   = "max_price"   // 标价最高
	CarOrderMinPrice   = "min_price"   // 标价最低
	CarOrderMinAge     = "min_age"     // 车龄最短
	CarOrderMinMileage = "min_mileage" // 里程最少
	CarOrderNewUpdate  = "new_update"  // 最近更新
)

var CarLogic = &carLogic{}

type carLogic struct {
}

type CarItem struct {
	Id            uint            `json:"id"`
	ModelName     string          `json:"modelName"`
	Type          uint            `json:"type"`
	MainPhoto     string          `json:"mainPhoto"`
	LicensingDate int64           `json:"licensingDate"`
	Price         decimal.Decimal `json:"price"`
	InteriorPrice decimal.Decimal `json:"interiorPrice"` // 内部价格
	ExteriorPrice decimal.Decimal `json:"exteriorPrice"` // 外部价格
	DownPayment   decimal.Decimal `json:"downPayment"`
	Mileage       decimal.Decimal `json:"mileage"`
	Transfer      uint            `json:"transfer"`
	BrokerId      uint            `json:"brokerId"`
}
type CarSearch struct {
	BrokerId             uint            `json:"-"`
	Status               []uint          `json:"-"`
	TheUpper             uint            `json:"-"`
	CarType              uint            // 1=直卖车 2=最新上架 3=降价车
	Keyword              string          // 关键字
	BrandId              uint            // 品牌
	SeriesId             uint            // 车系
	MinPrice             decimal.Decimal // 最小车价
	MaxPrice             decimal.Decimal // 最大车价
	MinMileage           uint            // 最小里程
	MaxMileage           uint            // 最大里程
	MinTransfer          uint            // 最小过户次数
	MaxTransfer          uint            // 最大过户次数
	MinLiter             decimal.Decimal // 最小排量
	MaxLiter             decimal.Decimal // 最大排量
	Struct               string          // 车身结构
	FuelType             uint            // 燃油类型 1=汽油 2=柴油 3=电力 4=油电 5=油气 6=天然气 7=氢能源 8 =甲醇
	GearType             uint            // 变速箱类型 1=手动 2=自动
	DischargeStandard    string          // 排放标准 国1/欧1 等
	Color                string          // 车身颜色
	MinSeat              uint            // 最小座位数
	MaxSeat              uint            // 最大座位数
	Year                 uint            // 年款
	MinLicensingDateYear uint            // 最早上牌年份
	MinAge               uint            // 最小车龄
	MaxAge               uint            // 最大车龄
	CarIds               []uint          // 车辆编号
	StoreIds             []uint          // 门店id
	LocationProvinceId   uint            // 车辆所在地 省
	LocationCityId       uint            // 车辆所在地 市
}

type CarOrder struct {
	Order string
}

// Lists @Title 车辆列表
func (c *carLogic) Lists(search CarSearch, order CarOrder, page bean.Page) (lists []CarItem, total int64) {
	lists = []CarItem{}
	var cars []model.Car
	where := mysql.Db
	orderStr := ""
	// 筛选
	if search.BrokerId > 0 {
		where = where.Where("broker_id = ?", search.BrokerId)
	}
	if len(search.StoreIds) > 0 {
		where = where.Where("store_id in ?", search.StoreIds)
	}
	if search.CarType > 0 {
		switch search.CarType {
		case 1: // 直卖车
			where = where.Where("customer_type = ?", model.CustomerTypeCustomer)
		case 2: // 最新上架
			where = where.Where("created_at >= ?", time.Now().AddDate(0, 0, 7).Format("2006-01-02"))
		case 3: // 降价车
			where = where.Where("price_adjust_amount < 0")
		}
	}
	if len(search.Status) > 0 {
		where = where.Where("status in ?", search.Status)
	}
	if search.TheUpper > 0 {
		where = where.Where("the_upper = ?", search.TheUpper)
	}
	// 最小车价
	if !search.MinPrice.IsZero() {
		where = where.Where("price >= ?", search.MinPrice)
	}
	// 最大车价
	if !search.MaxPrice.IsZero() {
		where = where.Where("price <= ?", search.MaxPrice)
	}
	// 最小里程
	if search.MinMileage > 0 {
		where = where.Where("mileage >= ?", search.MinMileage)
	}
	// 最大里程
	if search.MaxMileage > 0 {
		where = where.Where("mileage <= ?", search.MaxMileage)
	}
	// 最小过户次数
	if search.MinTransfer > 0 {
		where = where.Where("transfer >= ?", search.MinTransfer)
	}
	// 最大过户次数
	if search.MaxTransfer > 0 {
		where = where.Where("transfer <= ?", search.MaxTransfer)
	}
	// 车身颜色
	if search.Color != "" {
		where = where.Where("color = ?", search.Color)
	}
	// 最早上牌年份
	if search.MinLicensingDateYear > 0 {
		where = where.Where("licensing_date >= ?", fmt.Sprintf("%d", search.MinLicensingDateYear))
	}
	now := time.Now()
	// 最小车龄
	if search.MinAge > 0 {
		where = where.Where("licensing_date <= ?", now.AddDate(int(search.MinAge-1)*-1, 0, 0).Format("2006-01-02"))
	}
	// 最大车龄
	if search.MaxAge > 0 {
		where = where.Where("licensing_date >= ?", now.AddDate(int(search.MaxAge)*-1, 0, 0).Format("2006-01-02"))
	}
	// 车辆编号
	if len(search.CarIds) > 0 {
		where = where.Where("id in (?)", search.CarIds)
	}
	// 车辆所在地
	{
		carBaseWhere := mysql.Db.Model(&model.CarBase{}).Select("car_id")
		if search.LocationProvinceId != 0 {
			carBaseWhere = carBaseWhere.Where("location_province_id = ?", search.LocationProvinceId)
		}
		if search.LocationCityId != 0 {
			carBaseWhere = carBaseWhere.Where("location_city_id = ?", search.LocationCityId)
		}
		if _, ok := carBaseWhere.Statement.Clauses["WHERE"]; ok {
			where = where.Where("id in (?)", carBaseWhere)
		}
	}
	// 车型
	{
		modelWhere := mysql.Db.Model(&model.CarModel{}).Select("model_id")
		// 关键字
		if search.Keyword != "" {
			modelWhere = modelWhere.Where("name like ?", fmt.Sprintf("%%%s%%", search.Keyword))
		}
		// 品牌
		if search.BrandId > 0 {
			modelWhere = modelWhere.Where("brand_id = ?", search.BrandId)
		}
		// 车型
		if search.SeriesId > 0 {
			modelWhere = modelWhere.Where("series_id = ?", search.SeriesId)
		}
		// 最小排量
		if !search.MinLiter.IsZero() {
			modelWhere = modelWhere.Where("liter >= ?", search.MinLiter)
		}
		// 最大排量
		if !search.MaxLiter.IsZero() {
			modelWhere = modelWhere.Where("liter <= ?", search.MaxLiter)
		}
		// 车身结构
		if search.Struct != "" {
			modelWhere = modelWhere.Where("car_struct like ?", fmt.Sprintf("%%%s%%", search.Struct))
		}
		// 燃油类型
		if search.FuelType > 0 {
			switch search.FuelType {
			case 1: // 汽油
				modelWhere = modelWhere.Where("fuel_type_id = ?", 0)
			case 2: // 柴油
				modelWhere = modelWhere.Where("fuel_type_id = ?", 1)
			case 3: // 电力
				modelWhere = modelWhere.Where("fuel_type_id = ?", 2)
			case 4: // 油电
				modelWhere = modelWhere.Where("fuel_type_id = ?", 3)
			case 5: // 油气
				modelWhere = modelWhere.Where("fuel_type_id = ?", 4)
			case 6: // 天然气
				modelWhere = modelWhere.Where("fuel_type_id = ?", 5)
			case 7: // 氢能源
				modelWhere = modelWhere.Where("fuel_type_id = ?", 6)
			case 8: // 甲醇
				modelWhere = modelWhere.Where("fuel_type_id = ?", 7)
			}
		}
		// 变速箱类型
		if search.GearType > 0 {
			switch search.GearType {
			case 1: // 手动
				modelWhere = modelWhere.Where("gear_type = ?", "手动")
			case 2: // 自动
				modelWhere = modelWhere.Where("gear_type = ?", "自动")
			}
		}
		// 排放标准
		if search.DischargeStandard != "" {
			modelWhere = modelWhere.Where("discharge_standard = ?", search.DischargeStandard)
		}
		// 最小座位数
		if search.MinSeat > 0 {
			modelWhere = modelWhere.Where("seat_number >= ?", search.MinSeat)
		}
		// 最大座位数
		if search.MaxSeat > 0 {
			modelWhere = modelWhere.Where("seat_number <= ?", search.MaxSeat)
		}
		// 年款
		if search.Year > 0 {
			modelWhere = modelWhere.Where("year = ?", search.Year)
		}
		if _, ok := modelWhere.Statement.Clauses["WHERE"]; ok {
			where = where.Where("model_id in (?)", modelWhere)
		}
	}
	// 排序
	if order.Order != "" {
		switch order.Order {
		case CarOrderNewCreate: // 最新创建
			orderStr = "created_at desc"
		case CarOrderMaxPrice: // 标价最高
			orderStr = "price desc"
		case CarOrderMinPrice: // 标价最低
			orderStr = "price"
		case CarOrderMinAge: // 车龄最短
			orderStr = "licensing_date desc"
		case CarOrderMinMileage: // 里程最少
			orderStr = "mileage"
		case CarOrderNewUpdate: // 最近更新
			orderStr = "updated_at desc"
		}
	}
	mysql.Db.Model(&cars).Where(where).Count(&total)
	if page.HasPage(total) {
		queryModel := mysql.Db.Preload("Model").Where(where).Offset(page.GetStart()).Limit(page.GetLimit())
		if orderStr != "" {
			queryModel = queryModel.Order(orderStr)
		} else {
			queryModel = queryModel.Order("created_at desc")
		}
		queryModel.Find(&cars)
		for _, car := range cars {
			lists = append(lists, CarItem{
				Id:            car.Id,
				ModelName:     car.Model.Name,
				Type:          car.Type,
				MainPhoto:     car.MainPhoto,
				LicensingDate: car.LicensingDate.Time.Unix(),
				Price:         car.Price,
				InteriorPrice: car.InteriorPrice,
				ExteriorPrice: car.ExteriorPrice,
				DownPayment:   car.DownPayment,
				Mileage:       car.Mileage,
				Transfer:      car.Transfer,
				BrokerId:      car.BrokerId,
			})
		}
	}
	return
}

type ListsEsSearch struct {
	Keyword  string `json:"keyword"`
	Status   []uint `json:"status"`
	TheUpper uint   `json:"theUpper"`
}

// ListsEs @Title es商品列表 目前最大10000条数据，超出不显示
func (c *carLogic) ListsEs(search ListsEsSearch, page bean.Page) (lists []CarItem, total int64) {
	// 处理es超过10000条不返回数据的方法
	newStart, newLimit := page.GetStart(), page.GetLimit()
	if newStart+newLimit > 10000 {
		newLimit = 10000 - newStart
	}

	var must []map[string]interface{}
	// 关键字搜索
	if search.Keyword != "" {
		must = append(must, map[string]interface{}{
			"multi_match": map[string]interface{}{
				"query": search.Keyword,
				"fields": []string{
					"modelName",
					"brandName",
				},
			},
		})
	}

	// 状态
	if len(search.Status) > 0 {
		must = append(must, map[string]interface{}{
			"terms": map[string]interface{}{
				"status": search.Status,
			},
		})
	}
	if search.TheUpper != 0 {
		must = append(must, map[string]interface{}{
			"term": map[string]interface{}{
				"theUpper": search.TheUpper,
			},
		})
	}

	var data []byte
	var queryData map[string]interface{}
	if len(must) > 0 {
		queryData = map[string]interface{}{
			"size": newLimit,
			"from": newStart,
			"query": map[string]interface{}{
				"bool": map[string]interface{}{
					"must": must,
				},
			},
		}
	} else {
		queryData = map[string]interface{}{
			"size": newLimit,
			"from": newStart,
			"query": map[string]interface{}{
				"match_all": map[string]interface{}{},
			},
		}
	}
	data, _ = json.Marshal(&queryData)
	log.Println(string(data))

	var cars []es.SearchCarItem
	result, err := opensearch.Api.Search([]string{constant.CarIndex}, bytes.NewReader(data), &cars)
	if err != nil {
		logger.Logger.Error("查询失败", zap.Error(err))
		return nil, 0
	}
	total = int64(result.Total)
	for _, item := range cars {
		lists = append(lists, CarItem{
			Id:            item.Source.Id,
			ModelName:     item.Source.ModelName,
			Type:          item.Source.Type,
			MainPhoto:     item.Source.MainPhoto,
			LicensingDate: item.Source.LicensingDate,
			Price:         item.Source.Price,
			InteriorPrice: item.Source.InteriorPrice,
			ExteriorPrice: item.Source.ExteriorPrice,
			DownPayment:   item.Source.DownPayment,
			Mileage:       item.Source.Mileage,
			Transfer:      item.Source.Transfer,
			BrokerId:      item.Source.BrokerId,
		})
	}
	return
}

type CarInfo struct {
	Id                    uint               `json:"id"`
	ModelId               uint               `json:"modelId"`
	ModelName             string             `json:"modelName"` // 车型名称
	Status                uint               `json:"status"`
	StatusName            string             `json:"statusName"`
	TheUpper              uint               `json:"theUpper"`
	TheUpperName          string             `json:"theUpperName"`
	Vin                   string             `json:"vin"`       // 车架号
	CarSn                 string             `json:"carSn"`     //  车辆编号
	Collect               uint               `json:"collect"`   // 收藏次数
	IsCollect             uint               `json:"isCollect"` // 是否收藏
	Browse                uint               `json:"browse"`    // 浏览次数
	Transfer              uint               `json:"transfer"`  // 过户次数
	Price                 decimal.Decimal    `json:"price"`
	LastPrice             decimal.Decimal    `json:"lastPrice"`   // 上次价格
	DownPayment           decimal.Decimal    `json:"downPayment"` // 首付
	Source                uint               `json:"source"`      // 车辆来源    1=个人直卖 2=车商 3=收购
	SourceName            string             `json:"sourceName"`
	DealerId              uint               `json:"dealerId"` // 车商id
	DealerName            string             `json:"dealerName"`
	DealerSn              string             `json:"dealerSn"`
	Type                  string             `json:"type"`                  // 车辆类型
	EngineNo              string             `json:"engineNo"`              // 发动机号
	LicensingDate         int64              `json:"licensingDate"`         // 上牌日期
	Color                 string             `json:"color"`                 // 车身颜色
	InteriorColor         string             `json:"interiorColor"`         // 内饰颜色
	TemporaryLicensePlate string             `json:"temporaryLicensePlate"` // 临时车牌
	ParkingNo             string             `json:"parkingNo"`             // 车位编号
	StockStatus           uint               `json:"stockStatus"`           // 库存状态
	StockStatusName       string             `json:"stockStatusName"`
	UseCharacter          string             `json:"useCharacter"`     // 使用性质
	Mileage               decimal.Decimal    `json:"mileage"`          // 里程
	PurchaseTax           decimal.Decimal    `json:"purchaseTax"`      // 购置税
	InstallationCost      decimal.Decimal    `json:"installationCost"` // 加装费用
	LocationCityId        uint               `json:"locationCityId"`
	Location              string             `json:"location"` // 车辆所在地
	AttributionCityId     uint               `json:"attributionCityId"`
	Attribution           string             `json:"attribution"`    // 车辆归属地
	Condition             string             `json:"condition"`      // 车况
	Remark                string             `json:"remark"`         // 备注
	CarPhotos             []CarPhoto         `json:"carPhotos"`      // 车辆照片
	InteriorPhotos        []CarPhoto         `json:"interiorPhotos"` // 内饰照片
	DefectPhotos          []CarPhoto         `json:"defectPhotos"`   // 缺陷照片
	DataPhotos            []CarPhoto         `json:"dataPhotos"`     // 资料照片
	BrokerInfo            brokerInfo         `json:"brokerInfo"`
	ModelInfo             ModelInfo          `json:"modelInfo"`
	PriceInfo             priceInfo          `json:"priceInfo"`
	PurchaseInfo          purchaseInfo       `json:"purchaseInfo"`
	CertificateInfo       certificateInfo    `json:"certificateInfo"`
	ContractMasterInfo    contractMasterInfo `json:"contractMasterInfo"`
}

type CarPhoto struct {
	Photo string `json:"photo"`
	Text  string `json:"text"`
}

type brokerInfo struct {
	BrokerId       uint   `json:"brokerId"`
	BrokerNickname string `json:"brokerNickname"`
	BrokerHeadImg  string `json:"brokerHeadImg"`
	BrokerPhone    string `json:"brokerPhone"`
}

type priceInfo struct {
	InteriorPrice decimal.Decimal `json:"interiorPrice"` //  内部价格
	ExteriorPrice decimal.Decimal `json:"exteriorPrice"` // 外部价格
}

type purchaseInfo struct {
	Price   decimal.Decimal `json:"price"`   // 采购价格
	Date    int64           `json:"date"`    // 采购日期
	Liaison string          `json:"liaison"` // 采购人
}

type certificateInfo struct {
	Transfer                 uint            `json:"transfer"`                 // 过户次数
	KeyCount                 uint            `json:"keyCount"`                 // 钥匙数量
	CompulsoryInsurance      uint            `json:"compulsoryInsurance"`      // 交强险
	CompulsoryInsuranceDate  int64           `json:"compulsoryInsuranceDate"`  // 交强险到期日期
	CommercialInsurance      uint            `json:"commercialInsurance"`      // 商业险
	CommercialInsuranceDate  int64           `json:"commercialInsuranceDate"`  // 商业险到期日期
	CommercialInsurancePrice decimal.Decimal `json:"commercialInsurancePrice"` // 商业险金额
}

type contractMasterInfo struct {
	Name     string `json:"name"`     // 姓名
	IdCard   string `json:"idCard"`   // 身份证号
	Phone    string `json:"phone"`    // 手机号
	BankCard string `json:"bankCard"` // 银行卡号
	Bank     string `json:"bank"`     // 开户行
}

// Info @Title 车辆详情
func (c *carLogic) Info(priceDet bool, carId uint) (result CarInfo, err error) {
	car := model.Car{Id: carId}
	if mysql.Db.Preload("CarBase.LocationCity").Preload("CarBase.AttributionCity").
		Preload("CarBase.Photos").Preload("Data").Preload("Model").Preload("Broker").
		First(&car).Error != nil {
		return result, errors.New("车辆不存在")
	}
	result = CarInfo{
		Id:                    car.Id,
		Status:                car.Status,
		StatusName:            car.GetStatusName(car.Status),
		TheUpper:              car.TheUpper,
		TheUpperName:          car.GetTheUpperName(car.TheUpper),
		ModelId:               car.ModelId,
		ModelName:             car.Model.Name,
		Vin:                   car.Vin,
		CarSn:                 car.CarSn,
		Collect:               car.Data.Collect,
		Browse:                car.Data.Browse,
		Transfer:              car.Transfer,
		Price:                 car.Price,
		LastPrice:             car.LastPrice,
		DownPayment:           car.DownPayment,
		Source:                car.CarBase.Source,
		SourceName:            car.CarBase.GetSourceName(car.CarBase.Source),
		Type:                  car.GetCarTypeName(car.Type),
		EngineNo:              car.CarBase.EngineNo,
		LicensingDate:         car.LicensingDate.Time.Unix(),
		Color:                 car.Color,
		InteriorColor:         car.InteriorColor,
		TemporaryLicensePlate: car.CarBase.TemporaryLicensePlate,
		ParkingNo:             car.CarBase.ParkingNo,
		StockStatus:           car.CarBase.StockStatus,
		StockStatusName:       car.CarBase.GetStockStatusName(car.CarBase.StockStatus),
		UseCharacter:          constant.GetCarUseCharacterName(car.CarBase.UseCharacter),
		Mileage:               car.Mileage,
		PurchaseTax:           car.CarBase.PurchaseTax,
		InstallationCost:      car.CarBase.InstallationCost,
		LocationCityId:        car.CarBase.LocationCityId,
		Location:              car.CarBase.LocationCity.Name,
		AttributionCityId:     car.CarBase.AttributionCityId,
		Attribution:           car.CarBase.AttributionCity.Name,
		Condition:             car.CarBase.Condition,
		Remark:                car.CarBase.Remark,
		BrokerInfo:            brokerInfo{},
		ModelInfo: ModelInfo{
			Year:              car.Model.Year,
			Price:             car.Model.Price,
			Liter:             car.Model.Liter,
			DischargeStandard: car.Model.DischargeStandard,
			FuelTypeName:      car.Model.FuelTypeName,
			EnginePower:       car.Model.EnginePower,
			EnginePowerKw:     car.Model.EnginePowerKw,
			IsGreen:           car.Model.IsGreen,
			ModelCode:         car.Model.ModelCode,
			DriveName:         car.Model.DriveName,
			DriveType:         car.Model.DriveType,
			ModelStatus:       car.Model.ModelStatus,
			GearType:          car.Model.GearType,
			MarketDate:        car.Model.MarketDate,
			MinRegYear:        car.Model.MinRegYear,
			MaxRegYear:        car.Model.MaxRegYear,
			StopMakeYear:      car.Model.StopMakeYear,
			Intake:            car.Model.Intake,
			SeatNumber:        car.Model.SeatNumber,
			BodyType:          car.Model.BodyType,
			DoorNumber:        car.Model.DoorNumber,
			CarStruct:         car.Model.CarStruct,
			IsParallel:        car.Model.IsParallel,
			PriceAllowance:    car.Model.PriceAllowance,
		},
		PriceInfo: priceInfo{},
		PurchaseInfo: purchaseInfo{
			Price:   car.CarBase.PurchasePrice,
			Date:    car.CarBase.PurchaseDate.Time.Unix(),
			Liaison: car.CarBase.PurchaseLiaison,
		},
		CertificateInfo: certificateInfo{
			Transfer:                 car.CarBase.Transfer,
			KeyCount:                 car.CarBase.KeyCount,
			CompulsoryInsurance:      car.CarBase.CompulsoryInsurance,
			CompulsoryInsuranceDate:  car.CarBase.CompulsoryInsuranceDate.Time.Unix(),
			CommercialInsurance:      car.CarBase.CommercialInsurance,
			CommercialInsuranceDate:  car.CarBase.CommercialInsuranceDate.Time.Unix(),
			CommercialInsurancePrice: car.CarBase.CommercialInsurancePrice,
		},
		CarPhotos:      []CarPhoto{},
		InteriorPhotos: []CarPhoto{},
		DefectPhotos:   []CarPhoto{},
		DataPhotos:     []CarPhoto{},
	}
	if priceDet {
		result.PriceInfo = priceInfo{
			InteriorPrice: car.CarBase.InteriorPrice,
			ExteriorPrice: car.CarBase.ExteriorPrice,
		}
	}
	for _, photo := range car.CarBase.Photos {
		switch photo.Type {
		case model.CarPhotoTypeCar: // 车辆照片
			result.CarPhotos = append(result.CarPhotos, CarPhoto{
				Photo: photo.Path,
				Text:  photo.Text,
			})
		case model.CarPhotoTypeInterior: // 内饰照片
			result.InteriorPhotos = append(result.InteriorPhotos, CarPhoto{
				Photo: photo.Path,
				Text:  photo.Text,
			})
		case model.CarPhotoTypeDefect: // 缺陷照片
			result.DefectPhotos = append(result.DefectPhotos, CarPhoto{
				Photo: photo.Path,
				Text:  photo.Text,
			})
		case model.CarPhotoTypeData: // 资料照片
			result.DataPhotos = append(result.DataPhotos, CarPhoto{
				Photo: photo.Path,
				Text:  photo.Text,
			})
		}
	}
	switch car.CarBase.Source {
	case model.CarBaseSourceConsignment: // 个人寄售
		consignmentCar := model.OrderConsignment{Id: car.CarBase.OrderId}
		if mysql.Db.Preload("Contract.Master").First(&consignmentCar).Error == nil {
			result.ContractMasterInfo = contractMasterInfo{
				Name:     consignmentCar.Contract.Master.Name,
				IdCard:   consignmentCar.Contract.Master.IdCard,
				Phone:    consignmentCar.Contract.Master.Phone,
				BankCard: consignmentCar.Contract.Master.BankCard,
				Bank:     consignmentCar.Contract.Master.Bank,
			}
		}
		switch car.Broker.Level {
		case model.BrokerLevelPartner:
			result.BrokerInfo = brokerInfo{
				BrokerId:       car.BrokerId,
				BrokerNickname: car.Broker.Nickname,
				BrokerHeadImg:  car.Broker.HeadImg,
				BrokerPhone:    car.Broker.Phone,
			}
		case model.BrokerLevelBusiness:
			staff := model.BrokerBusinessStoreStaff{BrokerId: car.BrokerId}
			if mysql.Db.Preload("Store.OwnerBroker").Where(&staff).First(&staff).Error == nil {
				result.BrokerInfo = brokerInfo{
					BrokerId:       staff.Store.OwnerBrokerId,
					BrokerNickname: staff.Store.OwnerBroker.Nickname,
					BrokerHeadImg:  staff.Store.OwnerBroker.HeadImg,
					BrokerPhone:    staff.Store.OwnerBroker.Phone,
				}
			}
		}
	case model.CarBaseSourceDealerPurchase: // 车商收购
		store := model.BrokerBusinessStore{Id: car.CarBase.DealerId}
		if mysql.Db.Preload("OwnerBroker").Where(&store).First(&store).Error == nil {
			result.DealerId = store.Id
			result.DealerName = store.Name
			result.DealerSn = store.StoreNo
			result.BrokerInfo = brokerInfo{
				BrokerId:       store.OwnerBroker.Id,
				BrokerNickname: store.OwnerBroker.Nickname,
				BrokerHeadImg:  store.OwnerBroker.HeadImg,
				BrokerPhone:    store.OwnerBroker.Phone,
			}
		}
	}
	return
}

type vinCheckResult struct {
	IsLegal uint   `json:"isLegal"`
	Message string `json:"message"`
}

// VinCheck @Title 验证vin合理性
func (c *carLogic) VinCheck(vin string) (result vinCheckResult) {
	result.IsLegal = 1
	//  判断线上在售无重复车辆
	car := model.Car{Vin: vin}
	mysql.Db.Where(&car).Where("status not in ? and the_upper = ?", []uint{model.CarStatusFinal, model.CarStatusCancel}, model.CarTheUpperTrue).First(&car)
	if car.Id > 0 {
		result.IsLegal = 0
		result.Message = "车辆已存在线上"
	}
	return result
}

// Parameter @Title 车辆参数
func (c *carLogic) Parameter(carId uint) (result string, err error) {
	car := model.Car{Id: carId}
	if mysql.Db.
		First(&car).Error != nil {
		return result, errors.New("车辆不存在")
	}
	modelDest := model.CarModelDest{ModelId: car.ModelId}
	mysql.Db.First(&modelDest)
	return modelDest.Source, nil
}

// BrowserEvent @Title 车辆被浏览
func (c *carLogic) BrowserEvent(tx *gorm.DB, carId uint) error {
	carDataUpd := tx.Model(&model.CarData{}).Where("car_id = ?", carId).UpdateColumns(map[string]interface{}{
		"browse": gorm.Expr("browse + ?", 1),
	})
	if carDataUpd.RowsAffected > 1 {
		return errors.New("车辆收藏数据更新失败")
	}
	if carDataUpd.RowsAffected == 0 {
		carData := model.CarData{CarId: carId, Browse: 1}
		if tx.Create(&carData).Error != nil {
			return errors.New("车辆收藏数据更新失败")
		}
	}
	return nil
}
