package car

import (
	"base/app/lib/bean"
	"base/app/lib/che300"
	"base/app/logic/manage/setting"
	"base/app/model"
	"errors"
	"fmt"
	"git.oa00.com/go/mysql"
	"github.com/shopspring/decimal"
	"strconv"
	"time"
)

var PriceLogic = &priceLogic{}

type priceLogic struct {
}

// Price @Title 获取价格
func (p *priceLogic) Price(priceType, priceUserId, modelId, cityId uint, licensePlate, licensingDate string, mileage decimal.Decimal) (result model.CarPrice, err error) {
	regDate, err := time.ParseInLocation("2006-01-02", licensingDate, time.Local)
	if err != nil {
		return result, errors.New("上牌日期错误")
	}
	// 查询历史估价记录
	result = model.CarPrice{
		ModelId:      modelId,
		RegDate:      regDate,
		LicensePlate: licensePlate,
		CityId:       cityId,
		Mileage:      mileage,
	}
	hisCheck := model.CarPrice{}
	if mysql.Db.Where(&result).First(&hisCheck).Error != nil {
		mileage = mileage.Div(decimal.NewFromInt(10000))
		price, err := che300.Api.Price(modelId, cityId, licensingDate, mileage)
		if err != nil {
			return result, err
		}
		result.PriceType = priceType
		result.PriceUserId = priceUserId
		tenThousand := decimal.NewFromInt(10000)
		for _, evalPrice := range price.EvalPrices {
			switch evalPrice.Condition {
			case che300.PriceConditionExcellent: // 车况优秀
				result.ExcellentMinPrice = decimal.NewFromFloat(evalPrice.IndividualLowSoldPrice).Mul(tenThousand)
				result.ExcellentNormalPrice = decimal.NewFromFloat(evalPrice.IndividualPrice).Mul(tenThousand)
				result.ExcellentMaxPrice = decimal.NewFromFloat(evalPrice.DealerLowSoldPrice).Mul(tenThousand)
			case che300.PriceConditionGood: // 车况良好
				result.GoodMinPrice = decimal.NewFromFloat(evalPrice.IndividualLowSoldPrice).Mul(tenThousand)
				result.GoodNormalPrice = decimal.NewFromFloat(evalPrice.IndividualPrice).Mul(tenThousand)
				result.GoodMaxPrice = decimal.NewFromFloat(evalPrice.DealerLowSoldPrice).Mul(tenThousand)
			case che300.PriceConditionNormal: // 车况一般
				result.NormalMinPrice = decimal.NewFromFloat(evalPrice.IndividualLowSoldPrice).Mul(tenThousand)
				result.NormalNormalPrice = decimal.NewFromFloat(evalPrice.IndividualPrice).Mul(tenThousand)
				result.NormalMaxPrice = decimal.NewFromFloat(evalPrice.DealerLowSoldPrice).Mul(tenThousand)
			}
		}
	} else {
		result.PriceType = priceType
		result.PriceUserId = priceUserId
		result.ExcellentMinPrice = hisCheck.ExcellentMinPrice
		result.ExcellentNormalPrice = hisCheck.ExcellentNormalPrice
		result.ExcellentMaxPrice = hisCheck.ExcellentMaxPrice
		result.GoodMinPrice = hisCheck.GoodMinPrice
		result.GoodNormalPrice = hisCheck.GoodNormalPrice
		result.GoodMaxPrice = hisCheck.GoodMaxPrice
		result.NormalMinPrice = hisCheck.NormalMinPrice
		result.NormalNormalPrice = hisCheck.NormalNormalPrice
		result.NormalMaxPrice = hisCheck.NormalMaxPrice
	}
	if mysql.Db.Create(&result).Error != nil {
		return result, errors.New("价格计算失败")
	}
	return
}

type EstimateData struct {
	ModelId       uint            `binding:"required" label:"车型"`
	LicensePlate  string          `label:"车牌"`
	CityId        uint            `binding:"required" label:"城市id"`
	Color         string          `binding:"required" label:"车身颜色"`
	LicensingDate string          `binding:"required" label:"上牌日期"`
	Mileage       decimal.Decimal `binding:"required" label:"里程"`
	Transfer      uint            // 过户次数
	Paint         uint            // 油漆修复面数
	Plate         uint            // 钣金修复面数
	Parts         []uint          // 更换部件
	Engine        uint            `binding:"oneof=0 1 2 3" label:"发动机"`
	Accidents     []uint          // 重大事故
	Maintain      uint            `binding:"oneof=0 1" label:"按时保养"`
}

// Estimate @Title 估算价格
func (p *priceLogic) Estimate(carPrice model.CarPrice, data EstimateData) (score float64, price, discount decimal.Decimal, err error) {
	score, discount, err = p.Calculate(data)
	if err != nil {
		return
	}
	// 选取价格
	if score < 30 { // 一般最低价
		price = carPrice.NormalMinPrice
	} else if score < 40 { // 一般普通价
		price = carPrice.NormalNormalPrice
	} else if score < 50 { // 一般最高价
		price = carPrice.NormalMaxPrice
	} else if score < 60 { // 良好最低价
		price = carPrice.GoodMinPrice
	} else if score < 70 { // 良好一般价
		price = carPrice.GoodNormalPrice
	} else if score < 80 { // 良好最高价
		price = carPrice.GoodMaxPrice
	} else if score < 90 { // 优秀最低价
		price = carPrice.ExcellentMinPrice
	} else if score < 100 { // 优秀一般价
		price = carPrice.ExcellentNormalPrice
	} else { // 优秀最高价
		price = carPrice.ExcellentMaxPrice
	}
	// 计算折扣
	price = price.Mul(discount)
	return
}

// MatchPriceValue @Title 详细车况匹配估价结果
func (p *priceLogic) MatchPriceValue(carPrice model.CarPrice, score float64, discount decimal.Decimal) (price decimal.Decimal, err error) {
	// 选取价格
	if score < 30 { // 一般最低价
		price = carPrice.NormalMinPrice
	} else if score < 40 { // 一般普通价
		price = carPrice.NormalNormalPrice
	} else if score < 50 { // 一般最高价
		price = carPrice.NormalMaxPrice
	} else if score < 60 { // 良好最低价
		price = carPrice.GoodMinPrice
	} else if score < 70 { // 良好一般价
		price = carPrice.GoodNormalPrice
	} else if score < 80 { // 良好最高价
		price = carPrice.GoodMaxPrice
	} else if score < 90 { // 优秀最低价
		price = carPrice.ExcellentMinPrice
	} else if score < 100 { // 优秀一般价
		price = carPrice.ExcellentNormalPrice
	} else { // 优秀最高价
		price = carPrice.ExcellentMaxPrice
	}
	// 计算折扣
	price = price.Mul(discount)
	return
}

// Calculate @Title 计算车辆分值
func (p *priceLogic) Calculate(data EstimateData) (score float64, discount decimal.Decimal, err error) {
	// 获取价格
	discount = decimal.NewFromInt(1)
	if len(data.Accidents) > 0 {
		// 直接最低价
		score = 20
		for _, accident := range data.Accidents {
			if accident == 4 {
				// 安全气囊弹出 最低价的7折
				discount = decimal.NewFromFloat(0.7)
				break
			}
		}
	} else {
		// 过户次数10分 没多一次扣一分
		if data.Transfer < 10 {
			score = score + float64(10-int(data.Transfer))
		}
		// 里程 1250/月正常 每增加125公里减1分 减少125公里加1分
		now := time.Now()
		licensingDate, err := time.ParseInLocation("2006-01-02", data.LicensingDate, time.Local)
		if err != nil {
			return score, discount, errors.New("上牌日期错误")
		}
		licensingMonth, _ := strconv.Atoi(licensingDate.Format("01"))
		nowMonth, _ := strconv.Atoi(now.Format("01"))
		totalMonth := (now.Year()-licensingDate.Year())*12 + nowMonth - licensingMonth + 1

		// 计算平均每月公里数
		monthMileage := data.Mileage.Mul(decimal.NewFromInt(10000)).Div(decimal.NewFromInt(int64(totalMonth)))
		tempMileageScore := decimal.NewFromInt(1250).Sub(monthMileage).Div(decimal.NewFromInt(125))
		if tempMileageScore.Cmp(decimal.Zero) >= 0 {
			tempMileageScore = tempMileageScore.Truncate(0)
		} else {
			tempMileageScore = tempMileageScore.Round(0)
		}
		mileageScore, _ := strconv.Atoi(tempMileageScore.String())

		if mileageScore+15 > 0 {
			score = score + float64(mileageScore+15)
		}

		// 油漆面11分 没多一面扣一分
		if data.Paint < 11 {
			score = score + float64(11-int(data.Paint))
		}
		// 钣金11分 没多一项扣二分
		plateScore := int(data.Plate) * 2
		if plateScore < 11 {
			score = score + float64(11-plateScore)
		}
		// 更换部件11分 大灯4分一个 尾灯1分 前翼子板2分一个 车门4一个  前后保1分
		partsScore := 0
		for _, parts := range data.Parts {
			switch parts {
			case 1: // 左大灯
				partsScore += 4
			case 2: // 右大灯
				partsScore += 4
			case 3: // 前翼子板（左）
				partsScore += 2
			case 4: // 前翼子板（右）
				partsScore += 2
			case 5: // 前保险杠
				partsScore += 1
			case 6: // 后保险杠
				partsScore += 1
			case 7: // 后尾灯（左）
				partsScore += 1
			case 8: // 后尾灯（右）
				partsScore += 1
			case 9: // 车门（左前）
				partsScore += 4
			case 10: // 车门（右前）
				partsScore += 4
			case 11: // 车门（左后）
				partsScore += 4
			case 12: // 车门（右后）
				partsScore += 4
			}
		}
		if partsScore < 13 {
			score = score + float64(13-partsScore)
		}
		// 发变状态 轻微渗油5 严重渗油20 发动机解体维修40
		engineScore := 0
		switch data.Engine {
		case 1: // 轻微渗油
			engineScore += 5
		case 2: // 严重渗油
			engineScore += 20
		case 3: // 发动机解体维修
			engineScore += 40
		}
		score = score + float64(40-engineScore)
		// 全程4s店保养5分
		if data.Maintain == 1 {
			score += 5
		}
		// 年份系数
		year := totalMonth / 12
		if totalMonth%12 != 0 {
			year++
		}
		ratio := float64(1)
		if year <= 3 {
			// 1-3年 x1
		} else if year <= 8 {
			// 4-8年
			ratio = 0.8
		} else {
			// 8年以上
			ratio = 0.7
		}
		score = score * ratio
	}
	return
}

type priceStrategy struct {
	Price      decimal.Decimal `json:"price"`
	FinalPrice decimal.Decimal `json:"finalPrice"`
}

// PriceStrategy @Title 估价转化策略
func (p *priceLogic) PriceStrategy(price decimal.Decimal) (result priceStrategy) {
	// 云云问车自定义价格方案
	carPriceRate := setting.SystemLogic.GetSystem().CarPriceRate
	price = price.Mul(carPriceRate)
	result.Price = price
	if price.Mul(decimal.NewFromFloat(0.05)).LessThan(decimal.NewFromInt(3500)) {
		result.FinalPrice = price.Sub(decimal.NewFromInt(3500))
	} else {
		result.FinalPrice = price.Mul(decimal.NewFromFloat(0.95))
	}
	return
}

type priceAmount struct {
	Deposit        decimal.Decimal `json:"deposit"`
	DownPayment    decimal.Decimal `json:"downPayment"`
	BalancePayment decimal.Decimal `json:"balancePayment"`
	ServiceFee     decimal.Decimal `json:"serviceFee"`
	ServiceFeeRate decimal.Decimal `json:"serviceFeeRate"`
}

// Amount @Title 费用计算
func (p *priceLogic) Amount(amount decimal.Decimal) (result priceAmount) {
	intAmount := amount.IntPart()
	if intAmount < 50000 {
		result.Deposit = decimal.NewFromInt(2500)
	} else if intAmount < 150000 {
		result.Deposit = decimal.NewFromInt(5000)
	} else {
		result.Deposit = decimal.NewFromInt(10000)
	}
	result.DownPayment = amount.Mul(decimal.NewFromFloat(0.2))
	result.BalancePayment = amount.Sub(result.Deposit).Sub(result.DownPayment)
	result.ServiceFeeRate = decimal.NewFromFloat(0.02)
	result.ServiceFee = amount.Mul(result.ServiceFeeRate)
	return
}

type fastPriceItem struct {
	Id            uint            `json:"id"`
	ModelName     string          `json:"modelName"`
	LicensePlate  string          `json:"licensePlate"`
	LicensingDate int64           `json:"licensingDate"`
	CityId        uint            `json:"cityId"`
	City          string          `json:"city"`
	Color         string          `json:"color"`
	Mileage       decimal.Decimal `json:"mileage"`
	Price         decimal.Decimal `json:"price"`
	CreatedAt     int64           `json:"createdAt"`
}
type PriceFastListsSearch struct {
	ModelName string
}

// FastLists @Title 快速估价列表
func (p *priceLogic) FastLists(brokerId uint, search PriceFastListsSearch, page bean.Page) (lists []fastPriceItem, total int64) {
	lists = []fastPriceItem{}
	var fastPrices []model.BrokerCarFastPrice
	where := mysql.Db.Where("broker_id = ?", brokerId)
	// 筛选
	if search.ModelName != "" {
		where = where.Where("model_id in (?)", mysql.Db.Model(&model.CarModel{}).Select("id").Where("name like ?", fmt.Sprintf("%%%s%%", search.ModelName)))
	}
	mysql.Db.Model(&fastPrices).Where(where).Count(&total)
	if page.HasPage(total) {
		mysql.Db.Preload("City").Preload("Model").Where(where).Offset(page.GetStart()).Limit(page.GetLimit()).Order("id desc").Find(&fastPrices)
		for _, fastPrice := range fastPrices {
			lists = append(lists, fastPriceItem{
				Id:            fastPrice.Id,
				ModelName:     fastPrice.Model.Name,
				LicensePlate:  fastPrice.LicensePlate,
				CityId:        fastPrice.CityId,
				City:          fastPrice.City.CityName,
				LicensingDate: fastPrice.LicensingDate.Unix(),
				Color:         fastPrice.Color,
				Mileage:       fastPrice.Mileage,
				Price:         fastPrice.Price,
				CreatedAt:     fastPrice.CreatedAt.Unix(),
			})
		}
	}
	return
}

type priceItem struct {
	Id            uint            `json:"id"`
	ModelName     string          `json:"modelName"`
	LicensePlate  string          `json:"licensePlate"`
	CityId        uint            `json:"cityId"`
	City          string          `json:"city"`
	LicensingDate int64           `json:"licensingDate"`
	Color         string          `json:"color"`
	Mileage       decimal.Decimal `json:"mileage"`
	Price         decimal.Decimal `json:"price"`
	CreatedAt     int64           `json:"createdAt"`
	Vin           string          `json:"vin"`
	Engine        string          `json:"engine"`
}
type PriceListsSearch struct {
	ModelName string
}

// Lists @Title 估价列表
func (p *priceLogic) Lists(brokerId uint, search PriceListsSearch, page bean.Page) (lists []priceItem, total int64) {
	lists = []priceItem{}
	var carPrices []model.BrokerCarPrice
	where := mysql.Db.Where("broker_id = ?", brokerId)
	// 筛选
	if search.ModelName != "" {
		where = where.Where("model_id in (?)", mysql.Db.Model(&model.CarModel{}).Select("id").Where("name like ?", fmt.Sprintf("%%%s%%", search.ModelName)))
	}
	mysql.Db.Model(&carPrices).Where(where).Count(&total)
	if page.HasPage(total) {
		mysql.Db.Preload("Model").Where(where).Offset(page.GetStart()).Limit(page.GetLimit()).Order("id desc").Find(&carPrices)
		for _, carPrice := range carPrices {
			lists = append(lists, priceItem{
				Id:            carPrice.Id,
				ModelName:     carPrice.Model.Name,
				LicensePlate:  carPrice.LicensePlate,
				CityId:        carPrice.CityId,
				City:          carPrice.City.CityName,
				LicensingDate: carPrice.LicensingDate.Unix(),
				Vin:           carPrice.Vin,
				Engine:        carPrice.Engine,
				Color:         carPrice.Color,
				Mileage:       carPrice.Mileage,
				Price:         carPrice.Price,
				CreatedAt:     carPrice.CreatedAt.Unix(),
			})
		}
	}
	return
}

type priceEstimateInfo struct {
	Id            uint            `json:"id"`
	Vin           string          `json:"vin"`
	EngineNo      string          `json:"engineNo"`
	Source        uint            `json:"source"`
	Mileage       decimal.Decimal `json:"mileage"`
	ModelId       uint            `json:"modelId"`
	ModelName     string          `json:"modelName"`
	CityId        uint            `json:"cityId"`
	LicensePlate  string          `json:"licensePlate"`
	CityName      string          `json:"cityName"`
	Color         string          `json:"color"`
	LicensingDate int64           `json:"licensingDate"`
	Transfer      uint            `json:"transfer"`
	Paint         uint            `json:"paint"`
	Plate         uint            `json:"plate"`
	Parts         string          `json:"parts"`
	Engine        uint            `json:"engine"`
	Accidents     string          `json:"accidents"`
	Maintain      uint            `json:"maintain"`
}

// Info @Title 估价详情
func (p *priceLogic) Info(brokerId, priceId uint) (result priceEstimateInfo, err error) {
	carPrice := model.BrokerCarPrice{Id: priceId, BrokerId: brokerId}
	if mysql.Db.Preload("Model").Preload("City").Where(&carPrice).First(&carPrice).Error != nil {
		return result, errors.New("估价信息错误")
	}
	result = priceEstimateInfo{
		Id:            carPrice.Id,
		Vin:           carPrice.Vin,
		EngineNo:      carPrice.Engine,
		Source:        carPrice.Source,
		Mileage:       carPrice.Mileage,
		ModelId:       carPrice.ModelId,
		ModelName:     carPrice.Model.Name,
		LicensePlate:  carPrice.LicensePlate,
		CityId:        carPrice.CityId,
		CityName:      carPrice.City.CityName,
		Color:         carPrice.Color,
		LicensingDate: carPrice.LicensingDate.Unix(),
		Transfer:      carPrice.Transfer,
		Paint:         carPrice.Paint,
		Plate:         carPrice.Plate,
		Parts:         carPrice.Parts,
		Accidents:     carPrice.Accidents,
		Maintain:      carPrice.Maintain,
	}
	return result, nil
}
