package car

import (
	"base/app/common"
	"base/app/constant"
	"base/app/lib/bean"
	"base/app/logic/car"
	"base/app/logic/customer/user/wallet"
	"base/app/logic/pay"
	"base/app/model"
	"encoding/json"
	"errors"
	"git.oa00.com/go/mysql"
	"github.com/shopspring/decimal"
	"gorm.io/gorm"
	"log"
	"strconv"
	"time"
)

var PriceLogic = &priceLogic{}

type priceLogic struct {
}

type priceItem struct {
	Id            uint            `json:"id"`
	PayType       uint            `json:"payType"`
	PayTypeName   string          `json:"payTypeName"`
	PayStatus     uint            `json:"payStatus"`
	PayStatusName string          `json:"payStatusName"`
	ModelName     string          `json:"modelName"`
	Price         decimal.Decimal `json:"price"`
	Icon          string          `json:"icon"`
	Color         string          `json:"color"`
	LicensingDate int64           `json:"licensingDate"`
	Mileage       decimal.Decimal `json:"mileage"`
	CreateAt      int64           `json:"createAt"`
}

// List @Title 估算价格记录
func (p *priceLogic) List(customerId uint, page bean.Page) (lists []priceItem, total int64) {
	lists = []priceItem{}
	var priceList []model.CustomerCarPrice
	mysql.Db.Model(&priceList).Where("customer_id = ?", customerId).Count(&total)
	if page.HasPage(total) {
		mysql.Db.Preload("Model.Brand").Where("customer_id = ?", customerId).Order("id desc").Offset(page.GetStart()).Limit(page.GetLimit()).Find(&priceList)
	}
	for _, item := range priceList {
		lists = append(lists, priceItem{
			Id:            item.Id,
			PayType:       item.PayType,
			PayTypeName:   item.GetTypeName(item.PayType),
			PayStatus:     item.PayStatus,
			PayStatusName: common.StringTernary(item.PayType == model.CustomerCarPricePayTypePay, item.GetStatusName(item.PayStatus), ""),
			ModelName:     item.Model.Name,
			Price:         item.Price,
			Icon:          item.Model.Brand.Icon,
			Color:         item.Color,
			LicensingDate: item.LicensingDate.Unix(),
			Mileage:       item.Mileage,
			CreateAt:      item.CreatedAt.Unix(),
		})
	}
	return
}

type estimateResult struct {
	Id            uint            `json:"id"`
	PayType       uint            `json:"payType"` // 1=使用次数估值 2=付费估价
	payTypeName   string          `json:"payTypeName"`
	PayStatus     uint            `json:"payStatus"` // 付费估价订单状态 1=待付款 2=付款成功 3=退款成功
	PayStatusName string          `json:"payStatusName"`
	CityId        uint            `json:"cityId"` // 车300城市ID
	CityName      string          `json:"cityName"`
	ModelId       uint            `json:"modelId"` // 车型id
	ModelName     string          `json:"modelName"`
	Color         string          `json:"color"`         // 车身颜色
	LicensingDate int64           `json:"licensingDate"` // 上牌日期
	Mileage       decimal.Decimal `json:"mileage"`       // 里程
	Transfer      uint            `json:"transfer"`      // 过户次数
	Paint         uint            `json:"paint"`         // 油漆修复面数
	Plate         uint            `json:"plate"`         // 钣金修复面数
	Parts         string          `json:"parts"`         // 更换部件
	Engine        uint            `json:"engine"`        // 发动机状态
	Accidents     string          `json:"accidents"`     // 重大事故
	Maintain      uint            `json:"maintain"`      // 按时保养 0=未按时保养 1=按时保养
	Score         decimal.Decimal `json:"score"`         // 计算分值
	Discount      decimal.Decimal `json:"discount"`      // 折扣
	Price         decimal.Decimal `json:"price"`         // 估算价格
	CreatedAt     int64           `json:"createdAt"`
}

// Info @Title 估算价格详情
func (p *priceLogic) Info(customerId, carPriceId uint) (result estimateResult, err error) {
	carPrice := model.CustomerCarPrice{Id: carPriceId, CustomerId: customerId}
	if mysql.Db.Preload("Customer").Preload("City").Preload("Model").Where(&carPrice).First(&carPrice).Error != nil {
		return result, errors.New("估价记录不存在")
	}
	result = estimateResult{
		Id:            carPrice.Id,
		PayType:       carPrice.PayType,
		payTypeName:   carPrice.GetTypeName(carPrice.PayType),
		PayStatus:     carPrice.PayStatus,
		PayStatusName: carPrice.GetStatusName(carPrice.PayStatus),
		CityId:        carPrice.CityId,
		CityName:      carPrice.City.CityName,
		ModelId:       carPrice.ModelId,
		ModelName:     carPrice.Model.Name,
		Color:         carPrice.Color,
		LicensingDate: carPrice.LicensingDate.Unix(),
		Mileage:       carPrice.Mileage,
		Transfer:      carPrice.Transfer,
		Paint:         carPrice.Paint,
		Plate:         carPrice.Plate,
		Parts:         carPrice.Parts,
		Engine:        carPrice.Engine,
		Accidents:     carPrice.Accidents,
		Maintain:      carPrice.Maintain,
		Score:         carPrice.Score,
		Discount:      carPrice.Discount,
		Price:         carPrice.Price,
		CreatedAt:     carPrice.CreatedAt.Unix(),
	}
	return
}

type priceEstimate struct {
	PriceId uint            `json:"priceId"`
	Price   decimal.Decimal `json:"price"`
}

// Result @Title 查询估算价格结果
func (p *priceLogic) Result(customerId, carPriceId uint) (result priceEstimate, err error) {
	carPrice := model.CustomerCarPrice{Id: carPriceId, CustomerId: customerId}
	if mysql.Db.Where(&carPrice).First(&carPrice).Error != nil {
		return result, errors.New("估价记录不存在")
	}
	result.PriceId = carPrice.PriceId
	result.Price = carPrice.Price
	return
}

type estimatePayData struct {
	CarPriceId uint        `json:"carPriceId"`
	PayInfo    interface{} `json:"payInfo"`
}

// EstimatePay @Title 付费估价
func (p *priceLogic) EstimatePay(customerId uint, openId string, data car.EstimateData) (result estimatePayData, err error) {
	orderSn, err := common.GenNo(constant.NoPrefixOrderEstimatePay)
	if err != nil {
		return result, errors.New("付款单号创建失败")
	}

	// 创建估价记录
	carPrice, err := p.SetCustomerCarPrice(customerId, data)
	if err != nil {
		return result, err
	}
	carPrice.PayType = model.CustomerCarPricePayTypePay
	carPrice.PayStatus = model.CustomerCarPricePayStatusUnPay

	// 创建付费估价订单
	orderEstimatePay := model.OrderEstimatePay{
		OrderSn: orderSn,
		Type:    model.OrderEstimatePayTypeCustomer,
		UserId:  customerId,
		Amount:  decimal.NewFromInt(int64(model.OrderEstimatePayAmount)),
		PayType: model.PayTypeWxapp,
		Status:  model.OrderEstimatePayStatusUnPay,
	}

	payInfo := result.PayInfo
	err = mysql.Db.Transaction(func(tx *gorm.DB) error {
		if tx.Create(&orderEstimatePay).Error != nil {
			return errors.New("付款订单创建失败")
		}
		carPrice.OrderId = orderEstimatePay.Id
		if tx.Create(&carPrice).Error != nil {
			return errors.New("用户估价记录创建失败")
		}
		payInfo, err = pay.PayLogic.Pay(orderEstimatePay.Id, model.PayOrderTypeEstimatePay, model.PayTypeWxapp, orderEstimatePay.Amount, "付费估价", openId)
		if err != nil {
			log.Println(err)
			return errors.New("付款失败")
		}
		return nil
	})
	result.CarPriceId = carPrice.Id
	result.PayInfo = payInfo
	return
}

// Estimate @Title 估算价格
func (p *priceLogic) Estimate(customerId uint, data car.EstimateData) (result priceEstimate, err error) {
	walletAssess := model.CustomerWalletAssess{CustomerId: customerId}
	mysql.Db.Where(&walletAssess).First(&walletAssess)
	if walletAssess.Count <= 0 {
		return result, errors.New("评估次数不足")
	}
	//licensingDate, err := time.ParseInLocation("2006-01-02", data.LicensingDate, time.Local)
	//if err != nil {
	//	return result, errors.New("上牌日期错误")
	//}
	//carPrice := model.CustomerCarPrice{
	//	CustomerId:    customerId,
	//	ModelId:       data.ModelId,
	//	Color:         data.Color,
	//	LicensingDate: licensingDate,
	//	Mileage:       data.Mileage,
	//	Transfer:      data.Transfer,
	//	Paint:         data.Paint,
	//	Plate:         data.Plate,
	//	Engine:        data.Engine,
	//	Maintain:      data.Maintain,
	//}
	//parts, _ := json.Marshal(data.Parts)
	//carPrice.Parts = string(parts)
	//accidents, _ := json.Marshal(data.Accidents)
	//carPrice.Accidents = string(accidents)
	carPrice, err := p.SetCustomerCarPrice(customerId, data)
	if err != nil {
		return result, err
	}
	carPrice.PayType = model.CustomerCarPricePayTypeAssess
	if err := mysql.Db.Transaction(func(tx *gorm.DB) error {
		if tx.Create(&carPrice).Error != nil {
			return errors.New("估价失败")
		}
		if err := wallet.AssessLogic.Use(tx, customerId, carPrice.Id, 1); err != nil {
			return err
		}
		carPriceResult, err := p.GetCarPriceResult(tx, carPrice)
		if err != nil {
			return err
		}
		result.PriceId = carPriceResult.PriceId
		result.Price = carPriceResult.Price
		//priceModel, err := car.PriceLogic.Price(model.CarPricePriceTypeCustomer, customerId, data.ModelId, data.CityId, data.LicensingDate, data.Mileage)
		//if err != nil {
		//	return err
		//}
		//carPrice.PriceId = priceModel.Id
		////score, price, discount, _ := car.PriceLogic.Estimate(priceModel, data)
		////carPrice.Score = decimal.NewFromFloat(score)
		////carPrice.Discount = discount
		//score, err := strconv.ParseFloat(carPrice.Score.String(), 64)
		//if err != nil {
		//    return errors.New("分数计算转型出错")
		//}
		//price, _ := car.PriceLogic.MatchPriceValue(priceModel, score, carPrice.Discount)
		//carPrice.Price = price
		//result.Price = price
		//result.PriceId = carPrice.Id
		//if tx.Updates(&carPrice).Error != nil {
		//	return errors.New("估价失败")
		//}
		return nil
	}); err != nil {
		return result, err
	}
	return
}

// SetCustomerCarPrice @Title 创建模型
func (p *priceLogic) SetCustomerCarPrice(customerId uint, data car.EstimateData) (result model.CustomerCarPrice, err error) {
	licensingDate, err := time.ParseInLocation("2006-01-02", data.LicensingDate, time.Local)
	if err != nil {
		return result, errors.New("上牌日期错误")
	}
	carPrice := model.CustomerCarPrice{
		CustomerId:    customerId,
		CityId:        data.CityId,
		ModelId:       data.ModelId,
		Color:         data.Color,
		LicensingDate: licensingDate,
		Mileage:       data.Mileage,
		Transfer:      data.Transfer,
		Paint:         data.Paint,
		Plate:         data.Plate,
		Engine:        data.Engine,
		Maintain:      data.Maintain,
	}
	parts, _ := json.Marshal(data.Parts)
	carPrice.Parts = string(parts)
	accidents, _ := json.Marshal(data.Accidents)
	carPrice.Accidents = string(accidents)
	score, discount, _ := car.PriceLogic.Calculate(data)
	carPrice.Score = decimal.NewFromFloat(score)
	carPrice.Discount = discount
	return carPrice, nil
}

// GetCarPriceResult @Title 估价结果
func (p *priceLogic) GetCarPriceResult(db *gorm.DB, carPrice model.CustomerCarPrice) (result model.CustomerCarPrice, err error) {
	priceModel, err := car.PriceLogic.Price(model.CarPricePriceTypeCustomer, carPrice.CustomerId, carPrice.ModelId, carPrice.CityId, "", carPrice.LicensingDate.Format("2006-01-02"), carPrice.Mileage)
	if err != nil {
		return result, err
	}
	carPrice.PriceId = priceModel.Id
	score, err := strconv.ParseFloat(carPrice.Score.String(), 64)
	if err != nil {
		return result, errors.New("分数计算转型出错")
	}
	price, _ := car.PriceLogic.MatchPriceValue(priceModel, score, carPrice.Discount)
	// 云云问车自定义价格方案
	strategy := car.PriceLogic.PriceStrategy(price)
	carPrice.Price = strategy.Price
	carPrice.FinalPrice = strategy.FinalPrice
	if db.Updates(&carPrice).Error != nil {
		return result, errors.New("估价失败")
	}
	return carPrice, nil
}
