package customer

import (
	"base/app/common"
	"base/app/lib/invite"
	"base/app/model"
	"errors"
	"git.oa00.com/go/mysql"
	"git.oa00.com/go/wx"
	"gorm.io/gorm"
)

var LoginLogic = &loginLogic{}

type loginLogic struct {
}
type loginInfo struct {
	Token string `json:"token"`
	User  User   `json:"user"`
}
type User struct {
	Nickname   string `json:"nickname"`
	HeadImg    string `json:"headImg"`
	Gender     uint   `json:"gender"`
	BrokerName string `json:"brokerName"`
	BrokerTel  string `json:"brokerTel"`
}

// Login @Title 登录
func (l *loginLogic) Login(code string) (result loginInfo, err error) {
	info, err := wx.Wxapp.GetLoginInfo(code)
	if err != nil {
		return result, errors.New("登录失败")
	}
	// 验证是否绑定账号
	customer := model.Customer{OpenId: info.Openid}
	if mysql.Db.Preload("Broker").Where(&customer).First(&customer).Error != nil {
		// 未注册账号
		return result, errors.New("账号未绑定账号")
	}
	result.User = User{
		Nickname:   customer.Nickname,
		HeadImg:    customer.HeadImg,
		Gender:     customer.Gender,
		BrokerName: customer.Broker.Nickname,
		BrokerTel:  customer.Broker.Phone,
	}
	err = mysql.Db.Transaction(func(tx *gorm.DB) error {
		result.Token, err = l.login(tx, customer.Id)
		if err != nil {
			return errors.New("登录失败")
		}
		return nil
	})
	return
}

type RegisterData struct {
	Code       string   `binding:"required" label:"code"`
	Userinfo   Userinfo `binding:"required,dive" label:"userinfo"`
	Phone      Phone    `binding:"required,dive" label:"phone"`
	InviteCode string
}
type Userinfo struct {
	EncryptedData string `binding:"required" label:"encryptedData"`
	Iv            string `binding:"required" label:"iv"`
}
type Phone struct {
	EncryptedData string `binding:"required" label:"encryptedData"`
	Iv            string `binding:"required" label:"iv"`
}

// Register @Title 注册绑定
func (l *loginLogic) Register(data RegisterData) (result loginInfo, err error) {
	info, err := wx.Wxapp.GetLoginInfo(data.Code)
	if err != nil {
		return result, errors.New("登录失败")
	}
	//userInfo, err := wx.Wxapp.GetUserInfo(info.SessionKey, data.Userinfo.EncryptedData, data.Userinfo.Iv)
	//if err != nil {
	//	return result, errors.New("用户信息获取失败")
	//}
	//headImg, err := file.UploadLogic.WebFile(userInfo.AvatarURL, "headimg", "")
	//if err != nil {
	//	return result, errors.New("用户信息获取失败")
	//}
	userPhone, err := wx.Wxapp.GetPhone(info.SessionKey, data.Phone.EncryptedData, data.Phone.Iv)
	if err != nil {
		return result, err
	}
	customer := model.Customer{}
	mysql.Db.Where("open_id = ?", info.Openid).First(&customer)
	customer.OpenId = info.Openid
	customer.UnionId = info.Unionid
	customer.Nickname = customer.RandNickname()
	//customer.Nickname = userInfo.NickName
	//customer.HeadImg = headImg
	//customer.Gender = uint(userInfo.Gender)

	customer.Mobile = userPhone.PhoneNumber
	customer.IsImportant = model.CustomerIsImportantFalse
	customer.Level = model.CustomerLevelRegister
	broker := model.Broker{}
	if customer.BrokerId == 0 && data.InviteCode != "" {
		brokerId := invite.Decode(data.InviteCode)
		if mysql.Db.Where("id = ?", brokerId).First(&broker).Error != nil {
			return result, errors.New("邀请码错误")
		}
		//if broker.Level != model.BrokerLevelBase {
		customer.BrokerId = broker.Id
		//}
	} else { //随机分配云云问车下经纪人
		mysql.Db.Where("business_id = ? and level = ?", 1, model.BrokerLevelBusiness).Order("rand()").First(&broker)
		customer.BrokerId = broker.Id
	}
	err = mysql.Db.Transaction(func(tx *gorm.DB) error {
		if customer.Id == 0 {
			if tx.Create(&customer).Error != nil {
				return errors.New("登录失败")
			}
		} else {
			if tx.Updates(&customer).Error != nil {
				return errors.New("登录失败")
			}
		}
		result.Token, err = l.login(tx, customer.Id)
		if err != nil {
			return errors.New("登录失败")
		}
		result.User = User{
			Nickname:   customer.Nickname,
			HeadImg:    customer.HeadImg,
			Gender:     customer.Gender,
			BrokerName: broker.Nickname,
			BrokerTel:  broker.Phone,
		}
		return nil
	})
	return
}

// @Title 登录
func (l *loginLogic) login(tx *gorm.DB, customerId uint) (token string, err error) {
	customerToken := model.CustomerToken{
		CustomerId: customerId,
	}
	token = common.RandStr(64)
	if tx.Where(&customerToken).First(&customerToken).Error != nil {
		customerToken.Token = token
		if tx.Create(&customerToken).Error != nil {
			return token, errors.New("登录失败")
		}
	} else {
		customerToken.Token = token
		if tx.Updates(&customerToken).Error != nil {
			return token, errors.New("登录失败")
		}
	}
	return
}

// Logoff @Title 注销
func (l *loginLogic) Logoff(customer *model.Customer, code string) error {
	info, err := wx.Wxapp.GetLoginInfo(code)
	if err != nil {
		return err
	}
	if customer.OpenId != info.Openid {
		return errors.New("用户openid不匹配")
	}
	if mysql.Db.Delete(&model.Customer{}, "id = ?", customer.Id).RowsAffected != 1 {
		return errors.New("注销失败")
	}
	return nil
}

// Auth @Title 验证登录
func (l *loginLogic) Auth(token string) (uint, error) {
	customerToken := model.CustomerToken{Token: token}
	if mysql.Db.Where(&customerToken).First(&customerToken).Error != nil {
		return 0, errors.New("登录超时")
	}
	return customerToken.CustomerId, nil
}
