package order

import (
	"base/app/lib/bean"
	"base/app/logic/pay"
	"base/app/model"
	"errors"
	"git.oa00.com/go/mysql"
	"github.com/shopspring/decimal"
	"gorm.io/gorm"
)

var CallCarLogic = &callCarLogic{}

type callCarLogic struct {
}
type callCarItem struct {
	Id            uint            `json:"id"`
	ModelName     string          `json:"modelName"`
	MainPhoto     string          `json:"mainPhoto"`
	LicensingDate int64           `json:"licensingDate"`
	Price         decimal.Decimal `json:"carPrice"`
	Mileage       decimal.Decimal `json:"mileage"`
	CreatedAt     int64           `json:"createdAt"`
	Status        uint            `json:"status"`
	StatusName    string          `json:"statusName"`
	Name          string          `json:"name"`
	Phone         string          `json:"phone"`     // 手机号
	ReserveAt     int64           `json:"reserveAt"` // 预定时间
	Address       string          `json:"address"`   // 预定地址
	Remark        string          `json:"remark"`    // 备注
	Amount        decimal.Decimal `json:"amount"`    // 叫车费用
}
type CallCarSearch struct {
	Status uint // 1=待付款 2=已付款 3=已完成 4=退款订单
}

// Lists @Title 叫车订单列表
func (c *callCarLogic) Lists(customerId uint, search CallCarSearch, page bean.Page) (lists []callCarItem, total int64) {
	lists = []callCarItem{}
	var callCars []model.OrderCallCar
	where := mysql.Db.Where("customer_id = ?", customerId)
	if search.Status > 0 {
		var status []uint
		switch search.Status {
		case 1: // 待付款
			status = []uint{model.OrderCallCarStatusUnPay}
		case 2: // 已付款
			status = []uint{model.OrderCallCarStatusPay}
		case 3: // 已完成
			status = []uint{model.OrderCallCarStatusFinal}
		case 4: // 退款订单
			status = []uint{model.OrderCallCarStatusRefundAudit, model.OrderCallCarStatusRefund, model.OrderCallCarStatusRefundReject}
		}
		if len(status) > 0 {
			where = where.Where("status in ?", status)
		}
	}
	mysql.Db.Model(&callCars).Where(where).Count(&total)
	if page.HasPage(total) {
		mysql.Db.Preload("Car.Model").Where(where).Offset(page.GetStart()).Order("id desc").Limit(page.GetLimit()).Find(&callCars)
		for _, callCar := range callCars {
			lists = append(lists, callCarItem{
				Id:            callCar.Id,
				ModelName:     callCar.Car.Model.Name,
				MainPhoto:     callCar.Car.MainPhoto,
				LicensingDate: callCar.Car.LicensingDate.Time.Unix(),
				Mileage:       callCar.Car.Mileage,
				Price:         callCar.Car.Price,
				CreatedAt:     callCar.Car.CreatedAt.Unix(),
				Status:        callCar.Status,
				StatusName:    callCar.GetStatusName(callCar.Status),
				Amount:        callCar.Amount,
				Name:          callCar.Customer.Nickname,
				Phone:         callCar.Phone,
				ReserveAt:     callCar.ReserveAt.Unix(),
				Address:       callCar.Address,
				Remark:        callCar.Remark,
			})
		}
	}
	return
}

type callCarInfo struct {
	Id            uint            `json:"id"`
	ModelName     string          `json:"modelName"`
	MainPhoto     string          `json:"mainPhoto"`
	LicensingDate int64           `json:"licensingDate"`
	Mileage       decimal.Decimal `json:"mileage"`
	Price         decimal.Decimal `json:"carPrice"`
	CreatedAt     int64           `json:"createdAt"`
	Status        uint            `json:"status"`
	Name          string          `json:"name"`
	Phone         string          `json:"phone"`     // 手机号
	ReserveAt     int64           `json:"reserveAt"` // 预定时间
	Address       string          `json:"address"`   // 预定地址
	Remark        string          `json:"remark"`    // 备注
	Amount        decimal.Decimal `json:"amount"`    // 叫车费用
	PayInfo       CallCarPay      `json:"payInfo"`
	RefundInfo    CallCarRefund   `json:"refundInfo"`
}
type CallCarPay struct {
	Id      uint            `json:"id"`
	channel uint            `json:"channel"`
	amount  decimal.Decimal `json:"amount"`
	PayTime int64           `json:"payTime"`
}
type CallCarRefund struct {
	Id           uint   `json:"id"`
	CreatedAt    int64  `json:"createdAt"`
	Reason       string `json:"reason"`       // 退款原因
	Proof        string `json:"proof"`        // 退款凭证
	Status       uint   `json:"status"`       // 1=未审核 2=审核通过 3=审核驳回
	AuditAt      int64  `json:"auditAt"`      // 审核时间
	RejectReason string `json:"rejectReason"` // 驳回原因
}

// Info @Title 详情
func (c *callCarLogic) Info(customerId, orderId uint) (result callCarInfo, err error) {
	callCar := model.OrderCallCar{Id: orderId}
	if mysql.Db.Preload("Car.Model").Where("customer_id = ?", customerId).First(&callCar).Error != nil {
		return result, errors.New("订单错误")
	}
	result = callCarInfo{
		Id:            callCar.Id,
		ModelName:     callCar.Car.Model.Name,
		MainPhoto:     callCar.Car.MainPhoto,
		LicensingDate: callCar.Car.LicensingDate.Time.Unix(),
		Mileage:       callCar.Car.Mileage,
		Price:         callCar.Car.Price,
		CreatedAt:     callCar.Car.CreatedAt.Unix(),
		Status:        callCar.Status,
		Amount:        callCar.Amount,
		Name:          callCar.Customer.Nickname,
		Phone:         callCar.Phone,
		ReserveAt:     callCar.ReserveAt.Unix(),
		Address:       callCar.Address,
		Remark:        callCar.Remark,
	}
	if callCar.PayId > 0 {
		payModel := model.Pay{Id: callCar.PayId}
		mysql.Db.Where(&payModel).First(&payModel)
		result.PayInfo = CallCarPay{
			Id:      callCar.PayId,
			amount:  payModel.Amount,
			PayTime: payModel.PayTime.Time.Unix(),
		}
	}
	if callCar.RefundId > 0 {
		callCarRefund := model.OrderCallCarRefund{Id: callCar.RefundId}
		mysql.Db.Where(&callCarRefund).First(&callCarRefund)
		result.RefundInfo = CallCarRefund{
			Id:           callCarRefund.Id,
			CreatedAt:    callCarRefund.CreatedAt.Unix(),
			Reason:       callCarRefund.Reason,
			Proof:        callCarRefund.Proof,
			Status:       callCarRefund.Status,
			AuditAt:      callCarRefund.AuditAt.Time.Unix(),
			RejectReason: callCarRefund.RejectReason,
		}
	}
	return
}

// Pay @Title 叫车支付
func (c *callCarLogic) Pay(customerId, orderId uint) (result interface{}, err error) {
	callCar := model.OrderCallCar{Id: orderId, CustomerId: customerId}
	if mysql.Db.Preload("Customer").Where(&callCar).First(&callCar).Error != nil {
		return result, errors.New("订单错误")
	}
	result, err = pay.PayLogic.Pay(orderId, model.PayOrderTypeCallCar, model.PayTypeWxapp, callCar.Amount, "叫车费用", callCar.Customer.OpenId)
	return
}

// Refund @Title 申请退款
func (c *callCarLogic) Refund(customerId, orderId uint, reason, proof string) error {
	callCar := model.OrderCallCar{Id: orderId, CustomerId: customerId}
	if mysql.Db.Preload("Refund").Where(&callCar).First(&callCar).Error != nil {
		return errors.New("订单错误")
	}
	if callCar.Status == model.OrderCallCarStatusRefund {
		return errors.New("已退款，请勿重复操作")
	}
	if callCar.Status != model.OrderCallCarStatusFinal {
		return errors.New("订单未完成")
	}
	if callCar.Refund.Status == model.OrderCallCarRefundStatusAudit {
		return errors.New("退款审核中，请勿重复操作")
	}
	callCarRefund := model.OrderCallCarRefund{
		OrderId: callCar.Id,
		Reason:  reason,
		Proof:   proof,
		Status:  model.OrderCallCarRefundStatusAudit,
	}
	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		if tx.Create(&callCarRefund).Error != nil {
			return errors.New("申请失败")
		}
		if tx.Model(&model.OrderCallCar{}).Where(&callCar).Updates(map[string]interface{}{
			"status":    model.OrderCallCarStatusRefundAudit,
			"refund_id": callCarRefund.Id,
		}).RowsAffected != 1 {
			return errors.New("申请失败")
		}
		return nil
	})
}
