package order

import (
	"base/app/constant"
	"base/app/lib/bean"
	"base/app/lib/tencent"
	"base/app/logic/customer/message"
	"base/app/model"
	"errors"
	"git.oa00.com/go/mysql"
	"github.com/shopspring/decimal"
	"gorm.io/gorm"
	"time"
)

var ConsignmentLogic = &consignmentLogic{}

type consignmentLogic struct {
}
type consignmentItem struct {
	Id            uint            `json:"id"`
	OrderSn       string          `json:"orderSn"`
	Status        uint            `json:"status"`
	StatusName    string          `json:"statusName"`
	MainPhoto     string          `json:"mainPhoto"`
	ModelName     string          `json:"modelName"`
	LicensingDate int64           `json:"licensingDate"`
	Mileage       decimal.Decimal `json:"mileage"`
	ContractPrice decimal.Decimal `json:"contractPrice"`
	SignAt        int64           `json:"signAt"` //  合同签订时间
	BrokerId      uint            `json:"brokerId"`
	BrokerName    string          `json:"brokerName"`
	CreatedAt     int64           `json:"createdAt"`
}
type ConsignmentListsSearch struct {
	Status uint // 1=待签订 2=待发布 3=在售 4=已售 5=已完成 6=已下架 7=已取消
}

// Lists @Title 寄卖订单
func (c *consignmentLogic) Lists(customerId uint, search ConsignmentListsSearch, page bean.Page) (lists []consignmentItem, total int64) {
	lists = []consignmentItem{}
	var consignments []model.OrderConsignment
	where := mysql.Db.Where("customer_id = ?", customerId)
	if search.Status > 0 {
		var status []uint
		switch search.Status {
		case 1: // 待签订
			status = []uint{model.OrderConsignmentStatusUnSign}
		case 2: // 待发布
			status = []uint{model.OrderConsignmentStatusUnPublish, model.OrderConsignmentStatusPublish, model.OrderConsignmentStatusReject}
		case 3: // 在售
			status = []uint{model.OrderConsignmentStatusTheUpper}
		case 4: // 已售
			status = []uint{model.OrderConsignmentStatusSale, model.OrderConsignmentStatusAccount}
		case 5: // 已完成
			status = []uint{model.OrderConsignmentStatusFinal}
		case 6: // 已下架
			status = []uint{model.OrderConsignmentStatusTheDown}
		case 7: // 已取消
			status = []uint{model.OrderConsignmentStatusCancel}
		}
		if len(status) > 0 {
			where = where.Where("status in ?", status)
		}
	}
	mysql.Db.Model(&consignments).Where(where).Count(&total)
	if page.HasPage(total) {
		mysql.Db.Preload("Broker").Preload("Model").Preload("CarBase").Preload("Contract").
			Where(where).Offset(page.GetStart()).Limit(page.GetLimit()).Order("id desc").Find(&consignments)
		for _, consignment := range consignments {
			lists = append(lists, consignmentItem{
				Id:            consignment.Id,
				OrderSn:       consignment.OrderSn,
				Status:        consignment.Status,
				StatusName:    consignment.GetStatusName(consignment.Status),
				MainPhoto:     consignment.CarBase.MainPhoto,
				ModelName:     consignment.Model.Name,
				LicensingDate: consignment.Contract.LicensingDate.Unix(),
				Mileage:       consignment.Contract.Mileage,
				ContractPrice: consignment.ContractPrice,
				SignAt:        consignment.Contract.SignAt.Time.Unix(),
				BrokerId:      consignment.BrokerId,
				BrokerName:    consignment.Broker.Nickname,
				CreatedAt:     consignment.CreatedAt.Unix(),
			})
		}
	}
	return
}

// Sign @Title 签订订单合同（已废弃：交由腾讯电子签完成该流程）
func (c *consignmentLogic) Sign(customerId, orderId uint, sign string) error {
	consignment := model.OrderConsignment{CustomerId: customerId, Id: orderId}
	if mysql.Db.Where(&consignment).First(&consignment).Error != nil {
		return errors.New("订单错误")
	}
	if consignment.Status == model.OrderConsignmentStatusCancel {
		return errors.New("订单已取消")
	}
	if consignment.Status != model.OrderConsignmentStatusUnSign {
		return errors.New("合同已签订")
	}
	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		if tx.Model(&consignment).Where(&consignment).UpdateColumns(map[string]interface{}{
			"status": model.OrderConsignmentStatusUnPublish,
		}).RowsAffected != 1 {
			return errors.New("签订失败")
		}
		if tx.Model(&model.ContractConsignment{}).Where("id = ?", consignment.ContractId).UpdateColumns(map[string]interface{}{
			"sign":    sign,
			"sign_at": time.Now(),
			"status":  constant.ContractStatusSign,
		}).RowsAffected != 1 {
			return errors.New("签订失败")
		}
		message.MessageLogic.MessageToBroker(tx, consignment.BrokerId, customerId, model.BrokerMessageTypeCustomerNormal, message.MessageItem{
			Title:   "合同提示",
			Content: "您的寄售合同已被用户签字"})
		return nil
	})
}

type consignmentInfo struct {
	Id               uint            `json:"id"`
	Status           uint            `json:"status"`
	StatusName       string          `json:"statusName"`
	CarId            uint            `json:"carId"`
	MainPhoto        string          `json:"mainPhoto"`
	ModelName        string          `json:"modelName"`
	LicensingDate    int64           `json:"licensingDate"`
	Mileage          decimal.Decimal `json:"mileage"`
	Price            decimal.Decimal `json:"price"`
	ContractSn       string          `json:"contractSn"`
	ContractSignAt   int64           `json:"contractSignAt"`
	SaleAmount       decimal.Decimal `json:"saleAmount"`
	SaleAt           int64           `json:"saleAt"`
	PaymentAmount    decimal.Decimal `json:"paymentAmount"`
	PaymentProof     string          `json:"paymentProof"`
	PaymentCreatedAt int64           `json:"paymentCreatedAt"`
}

// SignUrl @Title 签订合同地址
func (c *consignmentLogic) SignUrl(customerId, orderId uint) (result string, err error) {
	orderConsignment := model.OrderConsignment{Id: orderId, CustomerId: customerId}
	if mysql.Db.Preload("Contract.EssFlow").Where(&orderConsignment).First(&orderConsignment).Error != nil {
		return result, errors.New("订单错误")
	}
	flowId := orderConsignment.Contract.EssFlow.FlowId
	if flowId == "" {
		return result, errors.New("未找到电子签合同号")
	}
	urls, err := tencent.Ess.CreateSignUrls(3, []*string{&flowId})
	if urls == nil || len(urls) == 0 {
		return result, errors.New("电子签签署地址获取失败")
	}
	return *urls[0].SignUrl, nil
}

// Info @Title 订单详情
func (c *consignmentLogic) Info(customerId, orderId uint) (result consignmentInfo, err error) {
	orderConsignment := model.OrderConsignment{Id: orderId, CustomerId: customerId}
	if mysql.Db.
		Preload("Model").Preload("CarBase").Preload("Contract").Preload("Payment").
		Where(&orderConsignment).First(&orderConsignment).Error != nil {
		return result, errors.New("订单错误")
	}
	result = consignmentInfo{
		Id:               orderConsignment.Id,
		Status:           orderConsignment.Status,
		StatusName:       orderConsignment.GetStatusName(orderConsignment.Status),
		CarId:            orderConsignment.CarBase.CarId,
		MainPhoto:        orderConsignment.CarBase.MainPhoto,
		ModelName:        orderConsignment.Model.Name,
		LicensingDate:    orderConsignment.Contract.LicensingDate.Unix(),
		Mileage:          orderConsignment.Contract.Mileage,
		Price:            orderConsignment.ContractPrice,
		ContractSn:       orderConsignment.Contract.ContractSn,
		ContractSignAt:   orderConsignment.Contract.SignAt.Time.Unix(),
		SaleAmount:       orderConsignment.SaleAmount,
		SaleAt:           orderConsignment.SaleAt.Time.Unix(),
		PaymentAmount:    orderConsignment.Payment.Amount,
		PaymentProof:     orderConsignment.Payment.Proof,
		PaymentCreatedAt: orderConsignment.Payment.CreatedAt.Unix(),
	}
	return
}

type AdjustPriceData struct {
	ConsignmentId uint            `binding:"required" label:"寄售订单id"`
	Price         decimal.Decimal `binding:"required" label:"调价价格"`
}

// AdjustPrice @Title 调价
func (c *consignmentLogic) AdjustPrice(customerId, consignmentId uint, price decimal.Decimal) error {

	orderConsignment := model.OrderConsignment{Id: consignmentId}
	if mysql.Db.Preload("CarBase.Car").Where("customer_id = ?", customerId).First(&orderConsignment).Error != nil {
		return errors.New("寄售订单错误")
	}
	if orderConsignment.CarBase.Car.Status != model.CarStatusNormal {
		return errors.New("非在售状态")
	}
	if orderConsignment.CarBase.Source != model.CarBaseSourceConsignment {
		return errors.New("改车辆非个人所有")
	}
	carAdjustPriceHistory := model.CarAdjustPriceHistory{
		CarId:            orderConsignment.CarBase.Car.Id,
		Type:             model.CarAdjustPriceHistoryTypeConsignment,
		OperatorId:       customerId,
		ExteriorPrice:    orderConsignment.CarBase.Car.Price,
		PreExteriorPrice: price,
	}
	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		if tx.Model(&model.CarBase{}).Where("id = ?", orderConsignment.CarBase.Id).UpdateColumn("sale_price", price).RowsAffected != 1 {
			return errors.New("车辆价格修改失败")
		}
		if tx.Model(&model.Car{}).Where("id = ?", orderConsignment.CarBase.Car.Id).UpdateColumn("price", price).RowsAffected != 1 {
			return errors.New("车辆价格修改失败")
		}
		if tx.Create(&carAdjustPriceHistory).Error != nil {
			return errors.New("发布调价记录失败")
		}
		return nil
	})
}
