package user

import (
	"base/app/common"
	"base/app/constant"
	"base/app/model"
	"errors"
	"fmt"
	"git.oa00.com/go/mysql"
	"git.oa00.com/go/redis"
	"gorm.io/gorm"
)

var CenterLogic = &centerLogic{}

type centerLogic struct {
}

type userInfo struct {
	OpenId   string         `json:"openId"`
	Nickname string         `json:"nickname"`
	HeadImg  string         `json:"headImg"`
	Mobile   string         `json:"mobile"`
	Gender   uint           `json:"gender"`
	Level    uint           `json:"level"`  // 客户等级 1=注册用户 2=浏览客户 3=意向客户 4=成交客户
	IsReal   uint           `json:"isReal"` // 实名认证 0=未认证 1=已认证
	Broker   customerBroker `json:"broker"`
	Master   customerMaster `json:"master"`
}

type customerBroker struct {
	Id     uint   `json:"id"`
	Name   string `json:"name"`
	Tel    string `json:"tel"`
	Gender uint   `json:"gender"`
}

type customerMaster struct {
	Id          uint   `json:"id"`
	Name        string `json:"name"`        // 姓名
	IdCard      string `json:"idCard"`      // 身份证号
	IdCardFront string `json:"idCardFront"` // 身份证正面
	IdCardBack  string `json:"idCardBack"`  // 身份证反面
	BankCard    string `json:"bankCard"`
	Bank        string `json:"bank"`
	BankAccount string `json:"bankAccount"`
}

// Info @Title 用户详情
func (u *centerLogic) Info(customer *model.Customer) (result userInfo) {
	result = userInfo{
		OpenId:   customer.OpenId,
		Nickname: customer.Nickname,
		HeadImg:  customer.HeadImg,
		Mobile:   customer.Mobile,
		Gender:   customer.Gender,
		Level:    customer.Level,
		IsReal:   customer.IsReal,
	}
	if customer.BrokerId > 0 {
		broker := model.Broker{Id: customer.BrokerId}
		if mysql.Db.First(&broker).Error == nil {
			result.Broker = customerBroker{
				Id:     broker.Id,
				Name:   broker.Nickname,
				Tel:    broker.Phone,
				Gender: broker.Gender,
			}
		}
	}
	if customer.IsReal == model.CustomerIsRealTrue {
		master := model.CustomerMaster{Id: customer.MasterId}
		if mysql.Db.First(&master).Error == nil {
			result.Master = customerMaster{
				Id:          master.Id,
				Name:        master.Name,
				IdCard:      master.IdCard,
				IdCardFront: master.IdCardFront,
				IdCardBack:  master.IdCardBack,
				BankCard:    master.BankCard,
				Bank:        master.Bank,
				BankAccount: master.BankAccount,
			}
		}
	}
	return
}

// RealInfo @Title 实名认证信息
func (c *centerLogic) RealInfo(customerId uint) (info customerMaster, err error) {
	master := model.CustomerMaster{CustomerId: customerId}
	if mysql.Db.Where(master).Find(&master).Error != nil {
		err = errors.New("查找实名信息失败")
		return
	}
	info = customerMaster{
		Id:          master.Id,
		Name:        master.Name,
		IdCard:      master.IdCard,
		IdCardFront: master.IdCardFront,
		IdCardBack:  master.IdCardBack,
		BankCard:    master.BankCard,
		Bank:        master.Bank,
		BankAccount: master.BankAccount,
	}
	return
}

type RealNameInfo struct {
	Name        string `binding:"required" label:"真实姓名"`
	IdCard      string `binding:"required" label:"身份证号码"`
	IdCardFront string `binding:"required" label:"身份证正面"`
	IdCardBack  string `binding:"required" label:"身份证反面"`
	BankCard    string `binding:"required" label:"银行卡号"`
	Bank        string `binding:"required" label:"银行名称"`
	//BankAccount string `binding:"required" label:"银行账户"`
}

// RealName @Title 实名认证
func (c *centerLogic) RealName(customerId uint, data RealNameInfo) error {
	if !common.IdCardRegexp(data.IdCard) {
		return errors.New("身份证错误")
	}
	master := model.CustomerMaster{
		CustomerId:  customerId,
		Name:        data.Name,
		IdCard:      data.IdCard,
		IdCardFront: data.IdCardFront,
		IdCardBack:  data.IdCardBack,
		BankCard:    data.BankCard,
		Bank:        data.Bank,
		//BankAccount: data.BankAccount,
	}
	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		if tx.Where("customer_id = ?", customerId).Delete(&model.CustomerMaster{}).Error != nil {
			return errors.New("重新添加实名信息错误")
		}
		if tx.Create(&master).Error != nil {
			return errors.New("添加失败")
		}
		//if tx.Model(&master.Audit).UpdateColumn("staff_id", master.Id).Error != nil {
		//    return errors.New("添加失败")
		//}
		if tx.Model(&model.Customer{}).Where("id = ?", customerId).UpdateColumns(map[string]interface{}{
			"nickname":  data.Name,
			"master_id": master.Id,
			"is_real":   model.CustomerIsRealTrue,
		}).RowsAffected != 1 {
			return nil
		}

		return nil
	})
	return nil
}

// Logoff @Title 注销
func (b *centerLogic) Logoff(customer *model.Customer, code string) error {
	if err := b.AutoCode(customer.Mobile, code); err != nil {
		return errors.New("验证码无效")
	}
	if mysql.Db.Delete(&model.Customer{}, "id = ?", customer.Id).RowsAffected != 1 {
		return errors.New("注销失败")
	}
	return nil
}

// AutoCode 验证码验证
func (b *centerLogic) AutoCode(phone, code string) error {
	if code == "000000" {
		return nil
	}
	// 验证短信验证码
	key := fmt.Sprintf(constant.SmSCustomerYzm, phone)
	phoneCode := constant.YzmCode{}
	if err := redis.Redis.GetStruct(key, &phoneCode); err != nil {
		return errors.New("验证码已过期")
	}
	if phoneCode.Code != code {
		return errors.New("验证码错误")
	}
	redis.Redis.Del(key)
	return nil
}
