package broker

import (
	"base/app/lib/bean"
	"base/app/logic/broker/customer"
	"base/app/logic/broker/user/wallet"
	"base/app/model"
	"errors"
	"fmt"
	"git.oa00.com/go/mysql"
	"github.com/shopspring/decimal"
)

var BrokerLogic = &brokerLogic{}

type brokerLogic struct {
}
type brokerItem struct {
	Id           uint   `json:"id"`
	Nickname     string `json:"nickname"`
	HeadImg      string `json:"headImg"`
	Phone        string `json:"phone"`
	Level        uint   `json:"level"`
	LevelName    string `json:"levelName"`
	StoreId      uint   `json:"storeId"`
	StoreName    string `json:"storeName"`
	BusinessId   uint   `json:"businessId"`
	BusinessName string `json:"businessName"`
	CreatedAt    int64  `json:"createdAt"`
}
type BrokerListsSearch struct {
	Level           uint
	Name            string
	Phone           string
	StoreName       string
	BusinessName    string
	RegisterAtStart string
	RegisterAtEnd   string
}

// Lists @Title 经纪人列表
func (b *brokerLogic) Lists(search BrokerListsSearch, page bean.Page) (lists []brokerItem, total int64) {
	lists = []brokerItem{}
	var brokers []model.Broker
	where := mysql.Db
	// 等级
	if search.Level > 0 {
		switch search.Level {
		case model.BrokerLevelBase:
			where = where.Where("level = ?", model.BrokerLevelBase)
		case model.BrokerLevelPartner:
			where = where.Where("level = ?", model.BrokerLevelPartner)
		case model.BrokerLevelBusiness:
			where = where.Where("level = ?", model.BrokerLevelBusiness)
		}
	}
	// 名称
	if search.Name != "" {
		where = where.Where("nickname like ?", fmt.Sprintf("%%%s%%", search.Name))
	}
	// 手机号
	if search.Phone != "" {
		where = where.Where("phone like ?", fmt.Sprintf("%%%s%%", search.Phone))
	}
	// 门店
	if search.StoreName != "" {
		where = where.Where("store_id in (?)", mysql.Db.Model(&model.BrokerBusinessStore{}).Select("id").Where("name like ?", fmt.Sprintf("%%%s%%", search.StoreName)))
	}
	// 入驻商
	if search.BusinessName != "" {
		where = where.Where("business_id in (?)", mysql.Db.Model(&model.BrokerBusiness{}).Select("id").Where("name like ?", fmt.Sprintf("%%%s%%", search.BusinessName)))
	}
	// 注册日期
	if search.RegisterAtStart != "" && search.RegisterAtEnd != "" {
		where = where.Where("created_at > ? and created_at <= ?", search.RegisterAtStart, search.RegisterAtEnd+" 23:59:59")
	}
	mysql.Db.Model(&brokers).Where(where).Count(&total)
	if page.HasPage(total) {
		mysql.Db.
			Preload("Store").Preload("Business").
			Where(where).Offset(page.GetStart()).Limit(page.GetLimit()).Find(&brokers)
		for _, item := range brokers {
			lists = append(lists, brokerItem{
				Id:           item.Id,
				Nickname:     item.Nickname,
				Phone:        item.Phone,
				Level:        item.Level,
				LevelName:    item.GetLevelName(item.Level),
				StoreId:      item.StoreId,
				StoreName:    item.Store.Name,
				BusinessId:   item.BusinessId,
				BusinessName: item.Business.Name,
				CreatedAt:    item.CreatedAt.Unix(),
			})
		}
	}
	return
}

type brokerInfo struct {
	Id        uint         `json:"id"`
	Nickname  string       `json:"nickname"`
	Phone     string       `json:"phone"`
	Gender    uint         `json:"gender"`
	Level     uint         `json:"level"`
	LevelName string       `json:"levelName"`
	CreatedAt int64        `json:"createdAt"`
	Data      brokerData   `json:"data"`
	Master    brokerMaster `json:"master"`
	Staff     staffInfo    `json:"staff"`
	Store     storeInfo    `json:"store"`
	Business  businessInfo `json:"business"`
}

type brokerData struct {
	AssessCount         int             `json:"assessCount"`
	AssessContractCount int             `json:"assessContractCount"`
	Balance             decimal.Decimal `json:"balance"`
	InviteCount         int             `json:"inviteCount"`
}

type brokerMaster struct {
	Name        string // 姓名
	IdCard      string // 证件号
	IdCardFront string // 身份证正面
	IdCardBack  string // 身份证背面
}

type staffInfo struct {
	Id         uint            `json:"id"`
	Role       uint            `json:"role"`
	RoleName   string          `json:"roleName"`
	Commission decimal.Decimal `json:"commission"`
}
type storeInfo struct {
	Id              uint   `json:"id"`
	Name            string `json:"name"`
	OwnerBrokerId   uint   `json:"ownerBrokerId"`
	OwnerBrokerName string `json:"ownerBrokerName"`
	OwnerBrokerTel  string `json:"ownerBrokerTel"`
}

type businessInfo struct {
	Id              uint   `json:"id"`
	Name            string `json:"name"`
	OwnerBrokerId   uint   `json:"ownerBrokerId"`
	OwnerBrokerName string `json:"ownerBrokerName"`
	OwnerBrokerTel  string `json:"ownerBrokerTel"`
}

// Info @Title 经纪人详情
func (b *brokerLogic) Info(brokerId uint) (result brokerInfo, err error) {
	broker := model.Broker{Id: brokerId}
	if mysql.Db.Preload("Master").First(&broker).Error != nil {
		return result, errors.New("经纪人错误")
	}
	result = brokerInfo{
		Id:        broker.Id,
		Nickname:  broker.Nickname,
		Phone:     broker.Phone,
		Gender:    broker.Gender,
		Level:     broker.Level,
		LevelName: broker.GetLevelName(broker.Level),
		CreatedAt: broker.CreatedAt.Unix(),
		Data: brokerData{
			AssessCount:         wallet.AssessLogic.Count(broker, model.WalletAssessKindEstimate),
			AssessContractCount: wallet.AssessLogic.Count(broker, model.WalletAssessKindContract),
			Balance:             decimal.Zero,
			InviteCount:         int(customer.CustomerLogic.Count([]uint{broker.Id}).TotalCount),
		},
		Master: brokerMaster{
			Name:        broker.Master.Name,
			IdCard:      broker.Master.IdCard,
			IdCardFront: broker.Master.IdCardFront,
			IdCardBack:  broker.Master.IdCardBack,
		},
	}
	return
}

// Recharge @Title 经纪人充值
func (b *brokerLogic) Recharge(brokerId, kind uint, count int) error {
	broker := model.Broker{Id: brokerId}
	if mysql.Db.Where(&broker).First(&broker).Error != nil {
		return errors.New("经纪人错误")
	}
	switch broker.Level {
	case model.BrokerLevelBusiness:
		err := wallet.AssessLogic.DealerAdd(mysql.Db, broker.StoreId, 0, kind, count)
		if err != nil {
			return err
		}
	default:
		err := wallet.AssessLogic.Add(mysql.Db, brokerId, 0, kind, count)
		if err != nil {
			return err
		}
	}
	return nil
}
