package business

import (
	"base/app/common"
	"base/app/constant"
	"base/app/lib/bean"
	broker2 "base/app/logic/broker"
	"base/app/logic/manage/setting"
	"base/app/model"
	"database/sql"
	"errors"
	"fmt"
	"git.oa00.com/go/mysql"
	"github.com/shopspring/decimal"
	"gorm.io/gorm"
	"time"
)

var BusinessLogic = &businessLogic{}

type businessLogic struct {
}
type businessItem struct {
	Id            uint   `json:"id"`
	BusinessNo    string `json:"businessNo"`
	Name          string `json:"name"`
	Status        uint   `json:"status"`
	ExpireDate    int64  `json:"expireDate"`
	ApplyUserName string `json:"applyUserName"`
	CreatedAt     int64  `json:"createdAt"`
}
type BusinessSearch struct {
	BusinessNo      string
	Name            string
	Status          uint
	ContractStatus  uint
	ExpireDateStart string
	ExpireDateEnd   string
	CreatedAtStart  string
	CreatedAtEnd    string
}

// Lists @Title 入驻商列表
func (b *businessLogic) Lists(search BusinessSearch, page bean.Page) (lists []businessItem, total int64) {
	lists = []businessItem{}
	var businesses []model.BrokerBusiness
	where := mysql.Db.Where("auditing_status = ?", model.BrokerBusinessAuditingStatusSuc)
	// 入驻商编号
	if search.BusinessNo != "" {
		where = where.Where("business_no like ?", fmt.Sprintf("%%%s%%", search.BusinessNo))
	}
	// 入驻商名称
	if search.Name != "" {
		where = where.Where("name like ?", fmt.Sprintf("%%%s%%", search.Name))
	}
	// 账户状态
	if search.Status > 0 {
		switch search.Status {
		case 1: // 启用
			where = where.Where("status = ?", model.BrokerBusinessStatusEnabled)
		case 2: // 停用
			where = where.Where("status = ?", model.BrokerBusinessStatusDisabled)
		}
	}
	// 合同状态
	if search.ContractStatus > 0 {
		now := time.Now()
		switch search.ContractStatus {
		case 1: // 正常
			where = where.Where("expire_date > ?", now)
		case 2: // 已到期
			where = where.Where("expire_date <= ?", now)
		}
	}
	// 合同到期日
	if search.ExpireDateStart != "" && search.ExpireDateEnd != "" {
		where = where.Where("expire_date >= ? and expire_date >= ?", search.ExpireDateStart, search.ExpireDateEnd+" 23:59:59")
	}
	// 合同到期日
	if search.CreatedAtStart != "" && search.CreatedAtEnd != "" {
		where = where.Where("created_at >= ? and created_at >= ?", search.CreatedAtStart, search.CreatedAtEnd+" 23:59:59")
	}
	mysql.Db.Model(&businesses).Where(where).Count(&total)
	if page.HasPage(total) {
		mysql.Db.Preload("ApplyUser").Where(where).Offset(page.GetStart()).Limit(page.GetLimit()).Find(&businesses)
		for _, business := range businesses {
			lists = append(lists, businessItem{
				Id:            business.Id,
				BusinessNo:    business.BusinessNo,
				Name:          business.Name,
				Status:        business.Status,
				ExpireDate:    business.ExpireDate.Unix(),
				ApplyUserName: business.ApplyUser.Name,
				CreatedAt:     business.CreatedAt.Unix(),
			})
		}
	}
	return
}

type BusinessAdd struct {
	Base    base
	Company company
	Finance finance
}
type base struct {
	Name         string `binding:"required" label:"入驻商名称"`
	Address      string `binding:"required" label:"公司地址"`
	ContractNo   string `binding:"required" label:"合同编号"`
	Contract     string `binding:"required" label:"合同附件"`
	ExpireDate   string `binding:"required" label:"合同到期日"`
	Liaison      string `binding:"required" label:"联系人"`
	LiaisonPhone string `binding:"required" label:"联系人手机号"`
	Telephone    string `binding:"required" label:"入驻商电话"`
	SaleId       uint   `binding:"required" label:"所属业务"`
	Proof        string `binding:"required" label:"入驻费凭证"`
}
type company struct {
	Name                 string          `binding:"required" label:"公司名"`
	BusinessTermStart    string          `binding:"required" label:"营业期限"`
	BusinessTermEnd      string          `binding:"required" label:"营业期限"`
	RegisteredCapital    decimal.Decimal `binding:"required" label:"注册资本"`
	Quality              uint            `binding:"required,oneof=1 2" label:"公司性质"`
	BusinessLicenseNo    string          `binding:"required" label:"信用代码"`
	LegalPerson          string          `binding:"required" label:"法人"`
	LegalPersonCard      string          `binding:"required" label:"身份证"`
	StorePhoto           string          // `binding:"required" label:"门店照"`
	IdCardFront          string          `binding:"required" label:"身份证正面"`
	IdCardBack           string          `binding:"required" label:"身份证反面"`
	BusinessLicensePhoto string          `binding:"required" label:"营业执照"`
}
type finance struct {
	OpeningBank string `binding:"required" label:"开户行"`
	BankAccount string `binding:"required" label:"账号"`
	Address     string // `binding:"required" label:"地址"`
	Telephone   string // `binding:"required" label:"电话"`
	Proof       string `binding:"required" label:"开户证明"`
}

// Add @Title 添加入驻商
func (b *businessLogic) Add(manageId uint, data BusinessAdd) error {
	if !setting.SaleLogic.Ver(data.Base.SaleId) {
		return errors.New("业务员错误")
	}
	expireDate, err := time.ParseInLocation("2006-01-02", data.Base.ExpireDate, time.Local)
	if err != nil {
		return errors.New("合同到期日错误")
	}
	businessTermStart, err := time.ParseInLocation("2006-01-02", data.Company.BusinessTermStart, time.Local)
	if err != nil {
		return errors.New("营业期限错误")
	}
	businessTermEnd, err := time.ParseInLocation("2006-01-02", data.Company.BusinessTermEnd, time.Local)
	if err != nil {
		return errors.New("营业期限错误")
	}
	businessNo, err := common.GenNo(constant.NoPrefixBrokerBusiness)
	if err != nil {
		return errors.New("提交失败")
	}
	brokerBusiness := model.BrokerBusiness{
		BusinessNo:   businessNo,
		Name:         data.Base.Name,
		Address:      data.Base.Address,
		ContractNo:   data.Base.ContractNo,
		Contract:     data.Base.Contract,
		ExpireDate:   expireDate,
		Liaison:      data.Base.Liaison,
		LiaisonPhone: data.Base.LiaisonPhone,
		Telephone:    data.Base.Telephone,
		SaleId:       data.Base.SaleId,
		Proof:        data.Base.Proof,
		Company: model.BrokerBusinessCompany{
			Name:                 data.Company.Name,
			BusinessTermStart:    businessTermStart,
			BusinessTermEnd:      businessTermEnd,
			RegisteredCapital:    data.Company.RegisteredCapital,
			Quality:              data.Company.Quality,
			BusinessLicenseNo:    data.Company.BusinessLicenseNo,
			LegalPerson:          data.Company.LegalPerson,
			LegalPersonCard:      data.Company.LegalPersonCard,
			StorePhoto:           data.Company.StorePhoto,
			IdCardFront:          data.Company.IdCardFront,
			IdCardBack:           data.Company.IdCardBack,
			BusinessLicensePhoto: data.Company.BusinessLicensePhoto,
		},
		Finance: model.BrokerBusinessFinance{
			OpeningBank: data.Finance.OpeningBank,
			BankAccount: data.Finance.BankAccount,
			Address:     data.Finance.Address,
			Telephone:   data.Finance.Telephone,
			Proof:       data.Finance.Proof,
		},
		ApplyAt:        time.Now(),
		AuditingStatus: model.BrokerBusinessAuditingStatusSubmit,
		ApplyUserId:    manageId,
	}
	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		if tx.Create(&brokerBusiness).Error != nil {
			return errors.New("提交失败")
		}
		return nil
	})
}

type businessInfo struct {
	Id           uint               `json:"id"`
	BusinessNo   string             `json:"businessNo"`
	Name         string             `json:"name"`
	Address      string             `json:"address"`
	Contract     string             `json:"contract"`
	ContractNo   string             `json:"contractNo"`
	ExpireDate   int64              `json:"expireDate"`
	Liaison      string             `json:"liaison"`
	LiaisonPhone string             `json:"liaisonPhone"`
	Telephone    string             `json:"telephone"`
	SaleId       uint               `json:"saleId"`
	SaleName     string             `json:"saleName"`
	SalePhone    string             `json:"salePhone"`
	Proof        string             `json:"proof"`
	Company      businessCompany    `json:"company"`
	Finance      businessFinance    `json:"finance"`
	OwnerBroker  businessOwner      `json:"ownerBroker"`
	Auditing     []businessAuditing `json:"auditing"`
}
type businessOwner struct {
	Id      uint   `json:"id"`
	Name    string `json:"nickname"` // 昵称
	HeadImg string `json:"headImg"`  // 头像
	Phone   string `json:"phone"`    // 手机号
	Gender  uint   `json:"gender"`   // 性别 0：未填写 1：男 2：女

}
type businessCompany struct {
	Name                 string          `json:"name"`
	Quality              uint            `json:"quality"`
	RegisteredCapital    decimal.Decimal `json:"registeredCapital"`
	BusinessTermStart    int64           `json:"businessTermStart"`
	BusinessTermEnd      int64           `json:"businessTermEnd"`
	BusinessLicenseNo    string          `json:"businessLicenseNo"`
	LegalPerson          string          `json:"legalPerson"`
	LegalPersonCard      string          `json:"legalPersonCard"`
	StorePhoto           string          `json:"storePhoto"`
	IdCardFront          string          `json:"idCardFront"`
	IdCardBack           string          `json:"idCardBack"`
	BusinessLicensePhoto string          `json:"businessLicensePhoto"`
}
type businessFinance struct {
	BusinessLicenseNo string `json:"businessLicenseNo"`
	OpeningBank       string `json:"openingBank"`
	BankAccount       string `json:"bankAccount"`
	Address           string `json:"address"`
	Telephone         string `json:"telephone"`
	Proof             string `json:"proof"`
}

type businessAuditing struct {
	AuditingUserName string `json:"auditingUserName"`
	AuditingStatus   uint   `json:"auditingStatus"`
	Reason           string `json:"reason"`
	CreatedAt        int64  `json:"createdAt"`
}

// Info @Title 详情
func (b *businessLogic) Info(businessId uint) (result businessInfo, err error) {
	business := model.BrokerBusiness{Id: businessId}
	if mysql.Db.Preload("OwnerBroker").Preload("Sale").Preload("Company").Preload("Finance").Preload("Auditing", func(db *gorm.DB) *gorm.DB {
		return db.Order("id DESC")
	}).Preload("Auditing.AuditingUser").First(&business).Error != nil {
		return result, errors.New("入驻商错误")
	}
	result = businessInfo{
		Id:           business.Id,
		BusinessNo:   business.BusinessNo,
		Name:         business.Name,
		Address:      business.Address,
		Contract:     business.Contract,
		ContractNo:   business.ContractNo,
		ExpireDate:   business.ExpireDate.Unix(),
		Liaison:      business.Liaison,
		LiaisonPhone: business.LiaisonPhone,
		Telephone:    business.Telephone,
		SaleId:       business.SaleId,
		SaleName:     business.Sale.Name,
		SalePhone:    business.Sale.Phone,
		Proof:        business.Proof,
		OwnerBroker:  businessOwner{},
		Company: businessCompany{
			Name:                 business.Company.Name,
			Quality:              business.Company.Quality,
			RegisteredCapital:    business.Company.RegisteredCapital,
			BusinessTermStart:    business.Company.BusinessTermStart.Unix(),
			BusinessTermEnd:      business.Company.BusinessTermEnd.Unix(),
			BusinessLicenseNo:    business.Company.BusinessLicenseNo,
			LegalPerson:          business.Company.LegalPerson,
			LegalPersonCard:      business.Company.LegalPersonCard,
			StorePhoto:           business.Company.StorePhoto,
			IdCardFront:          business.Company.IdCardFront,
			IdCardBack:           business.Company.IdCardBack,
			BusinessLicensePhoto: business.Company.BusinessLicensePhoto,
		},
		Finance: businessFinance{
			BusinessLicenseNo: business.Company.BusinessLicenseNo,
			OpeningBank:       business.Finance.OpeningBank,
			BankAccount:       business.Finance.BankAccount,
			Address:           business.Finance.Address,
			Telephone:         business.Finance.Telephone,
			Proof:             business.Finance.Proof,
		},
		Auditing: []businessAuditing{},
	}
	for _, auditing := range business.Auditing {
		result.Auditing = append(result.Auditing, businessAuditing{
			AuditingUserName: auditing.AuditingUser.Name,
			AuditingStatus:   auditing.AuditingStatus,
			Reason:           auditing.Reason,
			CreatedAt:        auditing.CreatedAt.Unix(),
		})
	}
	if business.OwnerBrokerId > 0 {
		result.OwnerBroker = businessOwner{
			Id:      business.OwnerBrokerId,
			Name:    business.OwnerBroker.Nickname,
			HeadImg: business.OwnerBroker.HeadImg,
			Phone:   business.OwnerBroker.Phone,
			Gender:  business.OwnerBroker.Gender,
		}
	}
	return
}

// Edit @Title 编辑入驻商数据
func (b *businessLogic) Edit(businessId uint, data BusinessAdd) error {
	brokerBusiness := model.BrokerBusiness{Id: businessId}
	if mysql.Db.First(&brokerBusiness).Error != nil {
		return errors.New("入驻商错误")
	}
	if brokerBusiness.AuditingStatus != model.BrokerBusinessAuditingStatusFail {
		return errors.New("只允许修改驳回数据")
	}
	if !setting.SaleLogic.Ver(data.Base.SaleId) {
		return errors.New("业务员错误")
	}
	expireDate, err := time.ParseInLocation("2006-01-02", data.Base.ExpireDate, time.Local)
	if err != nil {
		return errors.New("合同到期日错误")
	}
	businessTermStart, err := time.ParseInLocation("2006-01-02", data.Company.BusinessTermStart, time.Local)
	if err != nil {
		return errors.New("营业期限错误")
	}
	businessTermEnd, err := time.ParseInLocation("2006-01-02", data.Company.BusinessTermEnd, time.Local)
	if err != nil {
		return errors.New("营业期限错误")
	}
	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		if tx.Model(&brokerBusiness).UpdateColumns(map[string]interface{}{
			"name":            data.Base.Name,
			"address":         data.Base.Address,
			"contract_no":     data.Base.ContractNo,
			"contract":        data.Base.Contract,
			"expire_date":     expireDate,
			"liaison":         data.Base.Liaison,
			"liaison_phone":   data.Base.LiaisonPhone,
			"telephone":       data.Base.Telephone,
			"sale_id":         data.Base.SaleId,
			"proof":           data.Base.Proof,
			"apply_at":        time.Now(),
			"auditing_status": model.BrokerBusinessAuditingStatusSubmit,
		}).Error != nil {
			return errors.New("修改失败")
		}
		if tx.Model(&model.BrokerBusinessCompany{}).Where("business_id = ?", businessId).UpdateColumns(map[string]interface{}{
			"name":                   data.Company.Name,
			"business_term_start":    businessTermStart,
			"business_term_end":      businessTermEnd,
			"registered_capital":     data.Company.RegisteredCapital,
			"quality":                data.Company.Quality,
			"business_license_no":    data.Company.BusinessLicenseNo,
			"legal_person":           data.Company.LegalPerson,
			"legal_person_card":      data.Company.LegalPersonCard,
			"store_photo":            data.Company.StorePhoto,
			"id_card_front":          data.Company.IdCardFront,
			"id_card_back":           data.Company.IdCardBack,
			"business_license_photo": data.Company.BusinessLicensePhoto,
		}).Error != nil {
			return errors.New("修改失败")
		}
		if tx.Model(&model.BrokerBusinessFinance{}).Where("business_id = ?", businessId).UpdateColumns(map[string]interface{}{
			"opening_bank": data.Finance.OpeningBank,
			"bank_account": data.Finance.BankAccount,
			"address":      data.Finance.Address,
			"telephone":    data.Finance.Telephone,
			"proof":        data.Finance.Proof,
		}).Error != nil {
			return errors.New("修改失败")
		}
		return nil
	})
}

type auditingItem struct {
	Id             uint   `json:"id"`
	Name           string `json:"name"`
	ApplyUserName  string `json:"applyUserName"`
	ApplyAt        int64  `json:"applyAt"`
	AuditingStatus uint   `json:"auditingStatus"`
}
type AuditingSearch struct {
	Name           string
	ApplyUser      string
	Phone          string
	ApplyAtStart   string
	ApplyAtEnd     string
	AuditingStatus uint
}

// AuditLists @Title 入驻商审核列表
func (b *businessLogic) AuditLists(search AuditingSearch, page bean.Page) (lists []auditingItem, total int64) {
	lists = []auditingItem{}
	var businesses []model.BrokerBusiness
	where := mysql.Db
	if search.Name != "" {
		where = where.Where("name like ?", fmt.Sprintf("%%%s%%", search.Name))
	}
	if search.Phone != "" {
		where = where.Where("phone like ?", fmt.Sprintf("%%%s%%", search.Phone))
	}
	if search.ApplyUser != "" || search.Phone != "" {
		applyUserWhere := mysql.Db.Model(&model.Manage{}).Select("id")
		if search.ApplyUser != "" {
			applyUserWhere = applyUserWhere.Where("name like ?", fmt.Sprintf("%%%s%%", search.ApplyUser))
		}
		where = where.Where("apply_user_id in (?)", applyUserWhere)
	}
	if search.ApplyAtStart != "" && search.ApplyAtEnd != "" {
		where = where.Where("apply_at >= ? and apply_at <= ?", search.ApplyAtStart, search.ApplyAtEnd+" 23:59:59")
	}
	if search.AuditingStatus > 0 {
		switch search.AuditingStatus {
		case 1: // 待审核
			where = where.Where("auditing_status = ?", model.BrokerBusinessAuditingStatusSubmit)
		case 2: // 审核通过
			where = where.Where("auditing_status = ?", model.BrokerBusinessAuditingStatusSuc)
		case 3: // 审核驳回
			where = where.Where("auditing_status = ?", model.BrokerBusinessAuditingStatusFail)
		}
	}
	mysql.Db.Model(&businesses).Where(where).Count(&total)
	if page.HasPage(total) {
		mysql.Db.Preload("ApplyUser").Where(where).Offset(page.GetStart()).Limit(page.GetLimit()).Order("apply_at desc,id desc").Find(&businesses)
		for _, business := range businesses {
			lists = append(lists, auditingItem{
				Id:             business.Id,
				Name:           business.Name,
				ApplyUserName:  business.ApplyUser.Name,
				ApplyAt:        business.ApplyAt.Unix(),
				AuditingStatus: business.AuditingStatus,
			})
		}
	}
	return
}

// Adopt @Title 审核通过
func (b *businessLogic) Adopt(manageId, businessId uint) error {
	business := model.BrokerBusiness{Id: businessId}
	if mysql.Db.Preload("Finance").Preload("Company").First(&business).Error != nil {
		return errors.New("入驻商错误")
	}
	if business.AuditingStatus != model.BrokerBusinessAuditingStatusSubmit {
		return errors.New("已审核")
	}
	broker := model.Broker{Phone: business.LiaisonPhone}
	mysql.Db.Where(&broker).First(&broker)
	auditing := model.BrokerBusinessAuditing{
		BusinessId:     businessId,
		AuditingUserId: manageId,
		AuditingStatus: model.BrokerBusinessAuditingStatusSuc,
	}
	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		store := model.BrokerBusinessStore{
			BusinessId:     businessId,
			Name:           business.Name,
			Phone:          business.LiaisonPhone,
			Address:        business.Address,
			Status:         model.BrokerBusinessStoreStatusEnabled,
			AuditingStatus: model.BrokerBusinessStoreAuditStatusSuc,
			Company: model.BrokerDealerCompany{
				Name:              business.Company.Name,
				BusinessLicenseNo: business.Company.BusinessLicenseNo,
				LegalPerson:       business.Company.LegalPerson,
				LegalTel:          business.LiaisonPhone,
				LegalPersonCard:   business.Company.LegalPersonCard,
				IdCardFront:       business.Company.IdCardFront,
				IdCardBack:        business.Company.IdCardBack,
				BankCard:          business.Finance.BankAccount,
				Bank:              business.Finance.OpeningBank,
				BankAccount:       business.Company.Name,
			},
		}
		if tx.Create(&store).Error != nil {
			return errors.New("创建门店失败")
		}
		storeNo, err := common.GenNo6(store.Id)
		if err != nil {
			return err
		}
		if tx.Model(&model.BrokerBusinessStore{}).Where("id = ?", store.Id).Updates(map[string]interface{}{
			"store_no": storeNo,
		}).RowsAffected != 1 {
			return errors.New("添加失败")
		}
		if broker.Id == 0 {
			broker.Nickname = business.Liaison
			broker.Level = model.BrokerLevelBusiness
			broker.BusinessId = business.Id
			broker.StoreId = store.Id
			if tx.Create(&broker).Error != nil {
				return errors.New("创建经纪人失败")
			}
		} else {
			if broker.Level != model.BrokerLevelBase {
				return errors.New("手机号已存在")
			}
			if tx.Model(&model.Broker{}).Where(&broker).UpdateColumns(map[string]interface{}{
				"nickname":    business.Liaison,
				"level":       model.BrokerLevelBusiness,
				"business_id": business.Id,
				"store_id":    store.Id,
			}).RowsAffected != 1 {
				return errors.New("编辑经纪人失败")
			}
			broker2.LoginLogic.Logout(broker.Id)
		}
		if tx.Model(&store).UpdateColumns(map[string]interface{}{
			"owner_broker_id": broker.Id,
		}).RowsAffected != 1 {
			return errors.New("编辑直属门店失败")
		}
		storeStaff := model.BrokerBusinessStoreStaff{
			BrokerId:   broker.Id,
			Name:       business.Liaison,
			Gender:     broker.Gender,
			Phone:      broker.Phone,
			BusinessId: business.Id,
			StoreId:    store.Id,
			RoleId:     11,
			Commission: decimal.Zero,
			Audit: model.BrokerBusinessStoreStaffAudit{
				Status:      model.BrokerBusinessStoreStaffAuditStatusSuc,
				AuditAt:     sql.NullTime{Time: time.Now(), Valid: true},
				AuditUserId: manageId,
			},
			Status: model.BrokerBusinessStoreStaffStatusEnabled,
		}
		if tx.Create(&storeStaff).Error != nil {
			return errors.New("创建员工信息失败")
		}
		if tx.Model(&storeStaff.Audit).UpdateColumn("staff_id", storeStaff.Id).Error != nil {
			return errors.New("创建员工信息审核失败")
		}
		if tx.Model(&model.BrokerBusiness{}).Where(&business).Updates(map[string]interface{}{
			"auditing_status": model.BrokerBusinessAuditingStatusSuc,
			"owner_broker_id": broker.Id,
		}).RowsAffected != 1 {
			return errors.New("更新审核状态失败")
		}
		if tx.Create(&auditing).Error != nil {
			return errors.New("创建审核单号失败")
		}
		return nil
	})
}

// Reject @Title 审核驳回
func (b *businessLogic) Reject(manageId, businessId uint, reason string) error {
	business := model.BrokerBusiness{Id: businessId}
	if mysql.Db.First(&business).Error != nil {
		return errors.New("入驻商错误")
	}
	if business.AuditingStatus != model.BrokerBusinessAuditingStatusSubmit {
		return errors.New("已审核")
	}
	auditing := model.BrokerBusinessAuditing{
		BusinessId:     businessId,
		AuditingUserId: manageId,
		AuditingStatus: model.BrokerBusinessAuditingStatusFail,
		Reason:         reason,
	}
	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		if tx.Model(&business).Where(&business).UpdateColumns(map[string]interface{}{
			"auditing_status": model.BrokerBusinessAuditingStatusFail,
		}).RowsAffected != 1 {
			return errors.New("审核失败")
		}
		if tx.Create(&auditing).Error != nil {
			return errors.New("审核失败")
		}
		return nil
	})
}
