package business

import (
	"base/app/lib/bean"
	broker2 "base/app/logic/broker"
	"base/app/model"
	"errors"
	"fmt"
	"git.oa00.com/go/mysql"
	"github.com/shopspring/decimal"
	"gorm.io/gorm"
	"time"
)

var StaffLogic = &staffLogic{}

type staffLogic struct {
}
type staffItem struct {
	Id           uint   `json:"id"`
	BusinessNo   string `json:"businessNo"` // 入驻商编号
	BusinessId   uint   `json:"businessId"`
	BusinessName string `json:"businessName"`
	StoreId      uint   `json:"storeId"`
	StoreName    string `json:"storeName"`
	RoleName     string `json:"roleName"`
	Name         string `json:"name"`
	Phone        string `json:"phone"`
	CreatedAt    int64  `json:"createdAt"`
	AuditId      uint   `json:"auditId"`
	AuditStatus  uint   `json:"auditStatus"`
}
type StaffSearch struct {
	BusinessNo   uint
	BusinessName string
	//StaffType   uint
	Status           uint
	Name             string
	Phone            string
	CreatedDateStart string
	CreatedDateEnd   string
}

// Lists @Title 子账户审核列表
func (s *staffLogic) Lists(search StaffSearch, page bean.Page) (lists []staffItem, total int64) {
	lists = []staffItem{}
	var storeStaffs []model.BrokerBusinessStoreStaff
	where := mysql.Db
	{
		// 入驻商编号
		if search.BusinessNo > 0 {
			where = where.Where("business_id = ?", search.BusinessNo)
		}
		// 入驻商名称
		if search.BusinessName != "" {
			where = where.Where("business_id in (?)", mysql.Db.Model(&model.BrokerBusiness{}).Select("id").Where("name = ?", search.BusinessName))
		}
		// 姓名
		if search.Name != "" {
			where = where.Where("name like ?", fmt.Sprintf("%%%s%%", search.Name))
		}
		// 手机号
		if search.Phone != "" {
			where = where.Where("phone like ?", fmt.Sprintf("%%%s%%", search.Phone))
		}
	}
	{
		staffAuditWhere := mysql.Db.Model(&model.BrokerBusinessStoreStaffAudit{}).Select("id")
		if search.Status > 0 {
			switch search.Status {
			case 1: // 提交审核
				staffAuditWhere = staffAuditWhere.Where("status = ?", model.BrokerBusinessStoreStaffAuditStatusSubmit)
			case 2: // 审核通过
				staffAuditWhere = staffAuditWhere.Where("status = ?", model.BrokerBusinessStoreStaffAuditStatusSuc)
			case 3: // 审核驳回
				staffAuditWhere = staffAuditWhere.Where("status = ?", model.BrokerBusinessStoreStaffAuditStatusFail)
			}
		}
		if search.CreatedDateStart != "" && search.CreatedDateEnd != "" {
			staffAuditWhere = staffAuditWhere.Where("created_at >= ? and created_at <= ?", search.CreatedDateStart, search.CreatedDateEnd+" 23:59:59")
		}
		if _, ok := staffAuditWhere.Statement.Clauses["WHERE"]; ok {
			where = where.Where("audit_id in (?)", staffAuditWhere)
		}
	}

	mysql.Db.Model(&storeStaffs).Where(where).Count(&total)
	if page.HasPage(total) {
		mysql.Db.
			Preload("Audit", func(db *gorm.DB) *gorm.DB {
				return db.Unscoped()
			}).
			Preload("Store").Preload("Business").Preload("Role").
			Where(where).Order("updated_at desc, created_at desc").Offset(page.GetStart()).Limit(page.GetLimit()).Find(&storeStaffs)
		for _, staff := range storeStaffs {
			lists = append(lists, staffItem{
				Id:           staff.Id,
				BusinessNo:   staff.Business.BusinessNo,
				BusinessId:   staff.BusinessId,
				BusinessName: staff.Business.Name,
				StoreId:      staff.Store.Id,
				StoreName:    staff.Store.Name,
				RoleName:     staff.Role.Name,
				Name:         staff.Name,
				Phone:        staff.Phone,
				CreatedAt:    staff.CreatedAt.Unix(),
				AuditId:      staff.Id,
				AuditStatus:  staff.Audit.Status,
			})
		}
	}
	return
}

type staffInfo struct {
	Base     staffBaseInfo     `json:"base"`
	Audit    staffAuditInfo    `json:"audit"`
	Store    staffStoreInfo    `json:"store"`
	Business staffBusinessInfo `json:"business"`
}

type staffBaseInfo struct {
	Id         uint            `json:"id"`
	Name       string          `json:"name"`
	Gender     uint            `json:"gender"`
	Tel        string          `json:"tel"`
	RoleId     uint            `json:"roleId"` // 权限
	RoleName   string          `json:"roleName"`
	Commission decimal.Decimal `json:"commission"` // 销售提成
}

type staffAuditInfo struct {
	Id               uint   `json:"id"`
	Status           uint   `json:"status"`
	Reason           string `json:"reason"`
	AuditingUserName string `json:"auditingUserName"`
	CreatedAt        int64  `json:"createdAt"`
}

type staffStoreInfo struct {
	Id      uint   `json:"id"`
	Name    string `json:"name"`
	Address string `json:"address"`
}

type staffBusinessInfo struct {
	Id         uint   `json:"id"`
	BusinessNo string `json:"businessNo"` // 入驻商编号
	Name       string `json:"name"`
	Tel        string `json:"tel"`
	Address    string `json:"address"`
}

// Info @Title 详情
func (s *staffLogic) Info(staffId uint) (result staffInfo, err error) {
	staff := model.BrokerBusinessStoreStaff{Id: staffId}
	if mysql.Db.Preload("Role").Preload("Store").Preload("Business").Preload("Audit.AuditingUser").First(&staff).Error != nil {
		return result, errors.New("员工错误")
	}
	result = staffInfo{
		Base: staffBaseInfo{
			Id:         staff.Id,
			Name:       staff.Name,
			Gender:     staff.Gender,
			Tel:        staff.Phone,
			RoleId:     staff.Role.Id,
			RoleName:   staff.Role.Name,
			Commission: staff.Commission,
		},
		Audit: staffAuditInfo{
			Id:               staff.Audit.Id,
			Status:           staff.Audit.Status,
			Reason:           staff.Audit.Reason,
			AuditingUserName: staff.Audit.AuditingUser.Name,
			CreatedAt:        staff.Audit.CreatedAt.Unix(),
		},
		Store: staffStoreInfo{
			Id:      staff.Store.Id,
			Name:    staff.Store.Name,
			Address: staff.Store.Address,
		},
		Business: staffBusinessInfo{
			Id:         staff.Business.Id,
			BusinessNo: staff.Business.BusinessNo,
			Name:       staff.Business.Name,
			Tel:        staff.Business.Telephone,
			Address:    staff.Business.Address,
		},
	}
	return
}

// Adopt @Title 审核通过
func (s *staffLogic) Adopt(manageId, staffId uint) error {
	staff := model.BrokerBusinessStoreStaff{Id: staffId}
	if mysql.Db.Preload("Store").Preload("Audit").First(&staff).Error != nil {
		return errors.New("子账户错误")
	}
	if staff.Audit.Status != model.BrokerBusinessStoreStaffAuditStatusSubmit {
		return errors.New("不在待审核状态")
	}
	broker := model.Broker{Phone: staff.Phone}
	mysql.Db.Where(&broker).First(&broker)
	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		if tx.Model(&model.BrokerBusinessStoreStaffAudit{}).Where(&staff.Audit).UpdateColumns(map[string]interface{}{
			"status":        model.BrokerBusinessStoreStaffAuditStatusSuc,
			"audit_at":      time.Now(),
			"audit_user_id": manageId,
		}).RowsAffected != 1 {
			return errors.New("审核失败")
		}
		if broker.Id == 0 {
			broker.Nickname = staff.Name
			broker.Level = model.BrokerLevelBusiness
			broker.BusinessId = staff.BusinessId
			broker.StoreId = staff.StoreId
			if tx.Create(&broker).Error != nil {
				return errors.New("审核失败")
			}
		} else {
			if broker.Level != model.BrokerLevelBase {
				return errors.New("手机号已存在")
			}
			if tx.Model(&model.Broker{}).Where(&broker).UpdateColumns(map[string]interface{}{
				"level":       model.BrokerLevelBusiness,
				"business_id": staff.BusinessId,
				"store_id":    staff.StoreId,
			}).RowsAffected != 1 {
				return errors.New("审核失败")
			}
			broker2.LoginLogic.Logout(broker.Id)
		}
		if tx.Model(&model.BrokerBusinessStoreStaff{}).Where(&staff).Updates(map[string]interface{}{
			"broker_id": broker.Id,
			"status":    model.BrokerBusinessStoreStaffStatusEnabled,
		}).RowsAffected != 1 {
			return errors.New("审核失败")
		}
		return nil
	})
}

// Reject @Title 审核驳回
func (s *staffLogic) Reject(manageId, staffId uint, reason string) error {
	staff := model.BrokerBusinessStoreStaff{Id: staffId}
	if mysql.Db.Preload("Audit").First(&staff).Error != nil {
		return errors.New("子账户错误")
	}
	if staff.Audit.Status != model.BrokerBusinessStoreStaffAuditStatusSubmit {
		return errors.New("不在待审核状态")
	}
	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		if tx.Model(&model.BrokerBusinessStoreStaffAudit{}).Where(&staff.Audit).UpdateColumns(map[string]interface{}{
			"status":        model.BrokerBusinessStoreStaffAuditStatusFail,
			"audit_at":      time.Now(),
			"reason":        reason,
			"audit_user_id": manageId,
		}).Error != nil {
			return errors.New("审核失败")
		}
		return nil
	})
}

// Direct @Title 直营经纪人
func (s *staffLogic) Direct() (result []bean.SelectItem) {
	var storeStaffs []model.BrokerBusinessStoreStaff
	mysql.Db.Where("business_id = ?", 1).Find(&storeStaffs)
	for _, storeStaff := range storeStaffs {
		result = append(result, bean.SelectItem{
			Id:   storeStaff.BrokerId,
			Name: storeStaff.Name,
		})
	}
	return
}
