package business

import (
	"base/app/lib/bean"
	broker2 "base/app/logic/broker"
	"base/app/model"
	"database/sql"
	"errors"
	"fmt"
	"git.oa00.com/go/mysql"
	"github.com/shopspring/decimal"
	"gorm.io/gorm"
	"time"
)

var StoreLogic = &storeLogic{}

type storeLogic struct {
}

type storeItem struct {
	Id              uint            `json:"id"`
	BusinessId      uint            `json:"businessId"`
	BusinessNo      string          `json:"businessNo"` // 入驻商编号
	BusinessName    string          `json:"businessName"`
	StoreNo         string          `json:"storeNo"`
	Name            string          `json:"name"`
	Address         string          `json:"address"`
	Commission      decimal.Decimal `json:"commission"`
	Status          uint            `json:"status"`
	StatusName      string          `json:"statusName"`
	OwnerBrokerId   uint            `json:"ownerBrokerId"`
	OwnerBrokerName string          `json:"ownerBrokerName"`
	OwnerBrokerTel  string          `json:"ownerBrokerTel"`
	CreatedAt       int64           `json:"createdAt"`
	UpdateAt        int64           `json:"updateAt"`
}
type StoreSearch struct {
	BusinessNo     uint
	BusinessName   string
	StoreNo        uint
	Name           string
	Phone          string
	Status         uint
	CreatedAtStart string
	CreatedAtEnd   string
}

// Lists @Title 门店列表
func (s *storeLogic) Lists(search StoreSearch, page bean.Page) (lists []storeItem, total int64) {
	lists = []storeItem{}
	var stores []model.BrokerBusinessStore
	where := mysql.Db.Where("auditing_status = ?", model.BrokerBusinessStoreAuditStatusSuc)
	{
		businessWhere := mysql.Db.Model(&model.BrokerBusiness{}).Select("id")
		// 入驻商编号
		if search.BusinessNo > 0 {
			businessWhere = businessWhere.Where("business_no = ?", search.BusinessNo)
		}
		// 入驻商名称
		if search.BusinessName != "" {
			businessWhere = businessWhere.Where("name = ?", search.BusinessName)
		}
		if _, ok := businessWhere.Statement.Clauses["WHERE"]; ok {
			where = where.Where("business_id in (?)", businessWhere)
		}
	}
	// 门店编号
	if search.StoreNo > 0 {
		where = where.Where("store_no like ?", fmt.Sprintf("%%%s%%", search.StoreNo))
	}
	// 姓名
	if search.Name != "" {
		where = where.Where("name like ?", fmt.Sprintf("%%%s%%", search.Name))
	}
	// 手机号
	if search.Phone != "" {
		where = where.Where("phone like ?", fmt.Sprintf("%%%s%%", search.Phone))
	}
	if search.Status > 0 {
		switch search.Status {
		case model.BrokerBusinessStoreStatusEnabled:
			where = where.Where("status = ?", model.BrokerBusinessStoreStatusEnabled)
		case model.BrokerBusinessStoreStatusDisabled:
			where = where.Where("status = ?", model.BrokerBusinessStoreStatusDisabled)
		}
	}
	if search.CreatedAtStart != "" && search.CreatedAtEnd != "" {
		where = where.Where("created_at >= ? and created_at <= ?", search.CreatedAtStart, search.CreatedAtEnd+" 23:59:59")
	}

	mysql.Db.Model(&stores).Where(where).Count(&total)
	if page.HasPage(total) {
		mysql.Db.Preload("Business").Preload("OwnerBroker").
			Where(where).Order("updated_at desc, created_at desc").Offset(page.GetStart()).Limit(page.GetLimit()).Find(&stores)
		for _, store := range stores {
			lists = append(lists, storeItem{
				Id:              store.Id,
				BusinessNo:      store.Business.BusinessNo,
				BusinessId:      store.BusinessId,
				BusinessName:    store.Business.Name,
				StoreNo:         store.StoreNo,
				Name:            store.Name,
				Address:         store.Address,
				Commission:      store.Commission,
				Status:          store.Status,
				StatusName:      store.GetStatusName(store.Status),
				OwnerBrokerId:   store.OwnerBroker.Id,
				OwnerBrokerName: store.OwnerBroker.Nickname,
				OwnerBrokerTel:  store.OwnerBroker.Phone,
				CreatedAt:       store.CreatedAt.Unix(),
				UpdateAt:        store.UpdatedAt.Unix(),
			})
		}
	}
	return
}

type storeAuditItem struct {
	Id                uint            `json:"id"`
	BusinessId        uint            `json:"businessId"`
	BusinessNo        string          `json:"businessNo"` // 入驻商编号
	BusinessName      string          `json:"businessName"`
	BusinessOwnerName string          `json:"businessOwnerName"`
	StoreNo           string          `json:"storeNo"`
	Name              string          `json:"name"`
	Liaison           string          `json:"liaison"`
	Phone             string          `json:"phone"`
	Address           string          `json:"address"`
	Commission        decimal.Decimal `json:"commission"`
	CreatedAt         int64           `json:"createdAt"`
	AuditStatus       uint            `json:"auditStatus"`
	AuditStatusName   string          `json:"auditStatusName"`
}
type StoreAuditSearch struct {
	BusinessNo       uint
	BusinessName     string
	Name             string
	Phone            string
	CreatedDateStart string
	CreatedDateEnd   string
	AuditingStatus   uint
}

// AuditLists @Title 门店审核列表
func (s *storeLogic) AuditLists(search StoreAuditSearch, page bean.Page) (lists []storeAuditItem, total int64) {
	lists = []storeAuditItem{}
	var stores []model.BrokerBusinessStore
	where := mysql.Db

	// 入驻商编号
	if search.BusinessNo > 0 {
		where = where.Where("business_id = ?", search.BusinessNo)
	}
	// 入驻商名称
	if search.BusinessName != "" {
		where = where.Where("business_id in (?)", mysql.Db.Model(&model.BrokerBusiness{}).Select("id").Where("name = ?", search.BusinessName))
	}
	// 姓名
	if search.Name != "" {
		where = where.Where("name like ?", fmt.Sprintf("%%%s%%", search.Name))
	}
	// 手机号
	if search.Phone != "" {
		where = where.Where("phone like ?", fmt.Sprintf("%%%s%%", search.Phone))
	}
	if search.AuditingStatus > 0 {
		switch search.AuditingStatus {
		case model.BrokerBusinessStoreAuditStatusSubmit:
			where = where.Where("auditing_status = ?", model.BrokerBusinessStoreAuditStatusSubmit)
		case model.BrokerBusinessStoreAuditStatusSuc:
			where = where.Where("auditing_status = ?", model.BrokerBusinessStoreAuditStatusSuc)
		case model.BrokerBusinessStoreAuditStatusFail:
			where = where.Where("auditing_status = ?", model.BrokerBusinessStoreAuditStatusFail)
		}
	}
	if search.CreatedDateStart != "" && search.CreatedDateEnd != "" {
		where = where.Where("created_at >= ? and created_at <= ?", search.CreatedDateStart, search.CreatedDateEnd+" 23:59:59")
	}

	mysql.Db.Model(&stores).Where(where).Count(&total)
	if page.HasPage(total) {
		mysql.Db.Preload("Business.OwnerBroker").
			Where(where).Order("updated_at desc, created_at desc").Offset(page.GetStart()).Limit(page.GetLimit()).Find(&stores)
		for _, store := range stores {
			lists = append(lists, storeAuditItem{
				Id:                store.Id,
				BusinessNo:        store.Business.BusinessNo,
				BusinessId:        store.BusinessId,
				BusinessName:      store.Business.Name,
				BusinessOwnerName: store.Business.OwnerBroker.Nickname,
				StoreNo:           store.StoreNo,
				Name:              store.Name,
				Address:           store.Address,
				Commission:        store.Commission,
				Liaison:           store.Liaison,
				Phone:             store.Phone,
				CreatedAt:         store.CreatedAt.Unix(),
				AuditStatus:       store.AuditingStatus,
				AuditStatusName:   store.GetAuditStatusName(store.AuditingStatus),
			})
		}
	}
	return
}

type storeInfo struct {
	Base     storeBaseInfo       `json:"base"`
	Company  storeCompanyInfo    `json:"company"`
	Owner    storeOwnerInfo      `json:"owner"`
	Staffs   []storeStaffInfo    `json:"staffs"`
	Auditing []storeAuditingInfo `json:"auditing"`
	Business storeBusinessInfo   `json:"business"`
}

type storeBaseInfo struct {
	Id          uint            `json:"id"`
	StoreNo     string          `json:"storeNo"`
	Name        string          `json:"name"`
	Liaison     string          `json:"liaison"`
	Phone       string          `json:"phone"`
	Address     string          `json:"address"`
	Commission  decimal.Decimal `json:"commission"`
	AuditStatus uint            `json:"auditStatus"`
}

type storeCompanyInfo struct {
	Id                uint   `json:"id"`
	BusinessLicenseNo string `json:"businessLicenseNo"` // 营业执照号
	LegalPerson       string `json:"legalPerson"`       // 法人
	LegalPersonCard   string `json:"legalPersonCard"`   // 法人身份证号
}

type storeOwnerInfo struct {
	Id      uint   `json:"id"`
	Name    string `json:"name"`
	HeadImg string `json:"headImg"`
	Phone   string `json:"phone"`
	Gender  uint   `json:"gender"`
}

type storeStaffInfo struct {
	Id       uint   `json:"id"`
	Name     string `json:"name"`
	Tel      string `json:"tel"`
	Gender   uint   `json:"gender"`
	RoleId   uint   `json:"roleId"`
	RoleName string `json:"roleName"`
}

type storeAuditingInfo struct {
	Id               uint   `json:"id"`
	Status           uint   `json:"status"`
	Reason           string `json:"reason"`
	AuditingUserName string `json:"auditingUserName"`
	CreatedAt        int64  `json:"createdAt"`
}

type storeBusinessInfo struct {
	Id         uint   `json:"id"`
	BusinessNo string `json:"businessNo"` // 入驻商编号
	Name       string `json:"name"`
	Tel        string `json:"tel"`
	Address    string `json:"address"`
}

// Info @Title 详情
func (s *storeLogic) Info(storeId uint) (result storeInfo, err error) {
	store := model.BrokerBusinessStore{Id: storeId}
	if mysql.Db.Preload("Business").Preload("Company").
		Preload("Staffs.Role", func(db *gorm.DB) *gorm.DB {
			return db.Order("id DESC")
		}).
		Preload("Auditing", func(db *gorm.DB) *gorm.DB {
			return db.Order("id DESC")
		}).Preload("Auditing.AuditingUser").First(&store).Error != nil {
		return result, errors.New("员工错误")
	}
	result = storeInfo{
		Base: storeBaseInfo{
			Id:          store.Id,
			StoreNo:     store.StoreNo,
			Name:        store.Name,
			Liaison:     store.Liaison,
			Phone:       store.Phone,
			Address:     store.Address,
			Commission:  store.Commission,
			AuditStatus: store.AuditingStatus,
		},
		Company: storeCompanyInfo{
			Id:                store.Company.Id,
			BusinessLicenseNo: store.Company.BusinessLicenseNo,
			LegalPerson:       store.Company.LegalPerson,
			LegalPersonCard:   store.Company.LegalPersonCard,
		},
		Owner: storeOwnerInfo{
			Id: store.OwnerBrokerId,
		},
		Staffs:   []storeStaffInfo{},
		Auditing: []storeAuditingInfo{},
		Business: storeBusinessInfo{
			Id:         store.Business.Id,
			BusinessNo: store.Business.BusinessNo,
			Name:       store.Business.Name,
			Tel:        store.Business.Telephone,
			Address:    store.Business.Address,
		},
	}
	if store.OwnerBrokerId > 0 {
		storeOwner := model.Broker{Id: store.OwnerBrokerId}
		if mysql.Db.Model(&storeOwner).Where(&storeOwner).First(&storeOwner).Error != nil {
			return result, errors.New("门店店长信息错误")
		}
		result.Owner = storeOwnerInfo{
			Id:      storeOwner.Id,
			Name:    storeOwner.Nickname,
			HeadImg: storeOwner.HeadImg,
			Phone:   storeOwner.Phone,
			Gender:  storeOwner.Gender,
		}
	}
	for _, staff := range store.Staffs {
		result.Staffs = append(result.Staffs, storeStaffInfo{
			Id:       staff.Id,
			Name:     staff.Name,
			Tel:      staff.Phone,
			Gender:   staff.Gender,
			RoleId:   staff.Role.Id,
			RoleName: staff.GetRoleName(staff.RoleId),
		})
	}
	for _, auditing := range store.Auditing {
		result.Auditing = append(result.Auditing, storeAuditingInfo{
			Id:               auditing.Id,
			Status:           auditing.AuditingStatus,
			Reason:           auditing.Reason,
			AuditingUserName: auditing.AuditingUser.Name,
			CreatedAt:        auditing.CreatedAt.Unix(),
		})
	}
	return
}

// Adopt @Title 审核通过
func (s *storeLogic) Adopt(manageId, storeId uint) error {
	store := model.BrokerBusinessStore{Id: storeId}
	if mysql.Db.Preload("Company").Preload("OwnerBroker").First(&store).Error != nil {
		return errors.New("门店错误")
	}
	if store.AuditingStatus != model.BrokerBusinessStoreAuditStatusSubmit {
		return errors.New("已审核")
	}
	auditing := model.BrokerBusinessStoreAuditing{
		StoreId:        storeId,
		AuditingUserId: manageId,
		AuditingStatus: model.BrokerBusinessStoreAuditingStatusSuc,
	}
	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		brokerReal := model.BrokerReal{
			BrokerId:    store.OwnerBrokerId,
			Name:        store.Company.LegalPerson,
			IdCard:      store.Company.LegalPersonCard,
			IdCardFront: store.Company.IdCardFront,
			IdCardBack:  store.Company.IdCardBack,
		}
		if tx.Create(&brokerReal).Error != nil {
			return errors.New("添加实名认证信息失败")
		}
		if store.OwnerBroker.Level != model.BrokerLevelBase {
			return errors.New("手机号已存在")
		}
		if tx.Model(&model.Broker{}).Where(&store.OwnerBroker).UpdateColumns(map[string]interface{}{
			"level":       model.BrokerLevelBusiness,
			"business_id": store.BusinessId,
			"store_id":    store.Id,
			"is_real":     model.BrokerIsRealTrue,
			"master_id":   brokerReal.Id,
		}).RowsAffected != 1 {
			return errors.New("编辑经纪人失败")
		}
		broker2.LoginLogic.Logout(store.OwnerBrokerId)

		if tx.Model(&model.BrokerBusinessStore{}).Where(&store).Updates(map[string]interface{}{
			"status":          model.BrokerBusinessStoreStatusEnabled,
			"auditing_status": model.BrokerBusinessStoreAuditingStatusSuc,
		}).RowsAffected != 1 {
			return errors.New("更新审核状态失败")
		}
		if tx.Create(&auditing).Error != nil {
			return errors.New("创建审核单号失败")
		}
		storeStaff := model.BrokerBusinessStoreStaff{
			BrokerId:   store.OwnerBrokerId,
			Name:       store.Company.LegalPerson,
			Phone:      store.OwnerBroker.Phone,
			BusinessId: store.BusinessId,
			StoreId:    store.Id,
			Gender:     store.OwnerBroker.Gender,
			RoleId:     model.BrokerRoleStoreOwner,
			Commission: decimal.Zero,
			Audit: model.BrokerBusinessStoreStaffAudit{
				Status:      model.BrokerBusinessStoreStaffAuditStatusSuc,
				AuditAt:     sql.NullTime{Time: time.Now(), Valid: true},
				AuditUserId: manageId,
			},
			Status: model.BrokerBusinessStoreStaffStatusEnabled,
		}
		if tx.Create(&storeStaff).Error != nil {
			return errors.New("创建员工信息失败")
		}
		if tx.Model(&storeStaff.Audit).UpdateColumn("staff_id", storeStaff.Id).Error != nil {
			return errors.New("创建员工信息审核失败")
		}
		return nil
	})
}

// Reject @Title 审核驳回
func (s *storeLogic) Reject(manageId, storeId uint, reason string) error {
	store := model.BrokerBusinessStore{Id: storeId}
	if mysql.Db.First(&store).Error != nil {
		return errors.New("子账户错误")
	}
	if store.AuditingStatus != model.BrokerBusinessStoreAuditStatusSubmit {
		return errors.New("不在待审核状态")
	}
	auditing := model.BrokerBusinessStoreAuditing{
		StoreId:        storeId,
		AuditingUserId: manageId,
		AuditingStatus: model.BrokerBusinessStoreAuditingStatusFail,
		Reason:         reason,
	}
	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		if tx.Model(&store).Where(&store).UpdateColumns(map[string]interface{}{
			"auditing_status": model.BrokerBusinessStoreAuditStatusFail,
		}).RowsAffected != 1 {
			return errors.New("审核失败")
		}
		if tx.Create(&auditing).Error != nil {
			return errors.New("审核失败")
		}
		return nil
	})
}
