package car

import (
	"base/app/common"
	"base/app/constant"
	"base/app/lib/bean"
	"base/app/logic/car"
	"base/app/logic/manage/message"
	"base/app/model"
	"errors"
	"fmt"
	"git.oa00.com/go/mysql"
	"github.com/shopspring/decimal"
	"gorm.io/gorm"
	"time"
)

var CarBaseLogic = &carBaseLogic{}

type carBaseLogic struct {
}
type carBaseItem struct {
	Id              uint   `json:"id"`
	MainPhoto       string `json:"mainPhoto"`
	ModelName       string `json:"modelName"`
	CarSn           string `json:"carSn"`
	CarType         uint   `json:"carType"`
	CarTypeName     string `json:"carTypeName"`
	BrokerName      string `json:"brokerName"`
	BrokerPhone     string `json:"brokerPhone"`
	BrokerBusiness  string `json:"brokerBusiness"`
	AuditType       uint   `json:"auditType"`
	AuditTypeName   string `json:"auditTypeName"`
	AuditStatus     uint   `json:"auditStatus"`
	AuditStatusName string `json:"auditStatusName"`
	ApplyAt         int64  `json:"applyAt"`
}
type CarBaseSearch struct {
	Kind           uint `binding:"oneof=0 1 2 3" label:"申请类型"`
	ModelName      string
	CarSn          string
	BrokerName     string
	BrokerTel      string
	ApplyTimeStart string
	ApplyTimeEnd   string
}

// Lists @Title 车辆审核列表
func (c *carBaseLogic) Lists(search CarBaseSearch, page bean.Page) (lists []carBaseItem, total int64) {
	lists = []carBaseItem{}
	var carBases []model.CarBase
	where := mysql.Db
	{
		auditWhere := mysql.Db.Model(&model.CarAudit{}).Select("id")
		auditWhere = auditWhere.Where("status = ?", constant.AuditStatusWaiting)
		if search.Kind > 0 {
			auditWhere = auditWhere.Where("type = ?", search.Kind)
		}
		if search.ApplyTimeStart != "" {
			auditWhere = auditWhere.Where("created_at > ?", search.ApplyTimeStart)
		}
		if search.ApplyTimeEnd != "" {
			auditWhere = auditWhere.Where("created_at < ?", search.ApplyTimeEnd)
		}
		if _, ok := auditWhere.Statement.Clauses["WHERE"]; ok {
			where = where.Where("audit_id in (?)", auditWhere)
		}
	}
	if search.ModelName != "" {
		where = where.Where("model_id in (?)", mysql.Db.Model(&model.CarModel{}).Where("name like ?", fmt.Sprintf("%%%s%%", search.ModelName)).Select("model_id"))
	}
	if search.CarSn != "" {
		where = where.Where("car_id in (?)", mysql.Db.Model(&model.Car{}).Where("car_sn like ?", fmt.Sprintf("%%%s%%", search.CarSn)).Select("id"))
	}
	{
		brokerWhere := mysql.Db.Model(&model.Broker{}).Select("id")
		if search.BrokerName != "" {
			brokerWhere = brokerWhere.Where("nickname like ?", fmt.Sprintf("%%%s%%", search.BrokerName))
		}
		if search.BrokerTel != "" {
			brokerWhere = brokerWhere.Where("phone like ?", fmt.Sprintf("%%%s%%", search.BrokerTel))
		}
		if _, ok := brokerWhere.Statement.Clauses["WHERE"]; ok {
			where = where.Where("broker_id in (?)", brokerWhere)
		}
	}
	mysql.Db.Model(&carBases).Where(where).Count(&total)
	if page.HasPage(total) {
		mysql.Db.Preload("Car").Preload("Model").Preload("Broker.Business").Preload("Audit").
			Where(where).Order("id desc").Offset(page.GetStart()).Limit(page.GetLimit()).Find(&carBases)
		for _, item := range carBases {
			lists = append(lists, carBaseItem{
				Id:              item.Id,
				MainPhoto:       item.MainPhoto,
				ModelName:       item.Model.Name,
				CarSn:           item.Car.CarSn,
				CarType:         item.Car.Type,
				CarTypeName:     item.Car.GetCarTypeName(item.Car.Type),
				BrokerName:      item.Broker.Nickname,
				BrokerPhone:     item.Broker.Phone,
				BrokerBusiness:  item.Broker.Business.Name,
				AuditType:       item.Audit.Type,
				AuditTypeName:   item.Audit.GetTypeName(item.Audit.Type),
				AuditStatus:     item.Audit.Status,
				AuditStatusName: constant.GetAuditStatusName(item.Audit.Status),
				ApplyAt:         item.Audit.CreatedAt.Unix(),
			})
		}
	}
	return
}

type carInfo struct {
	Id           uint            `json:"id"`
	TheUpper     uint            `json:"theUpper"`
	TheUpperName string          `json:"theUpperName"`
	Base         baseInfo        `json:"base"`
	Model        modelInfo       `json:"model"`
	Price        priceInfo       `json:"price"`
	PurchaseInfo purchaseInfo    `json:"purchaseInfo"`
	Broker       brokerInfo      `json:"broker"`
	Business     businessInfo    `json:"business"`
	Audit        auditInfo       `json:"audit"`
	Certificate  certificateInfo `json:"certificate"`
	Master       masterInfo      `json:"master"`
}

type baseInfo struct {
	ModelId               uint            `json:"modelId"`
	ModelName             string          `json:"modelName"` // 车型名称
	Type                  string          `json:"type"`
	Vin                   string          `json:"vin"`    // 车架号
	Source                string          `json:"source"` // 车辆来源    1=个人直卖 2=车商 3=收购
	SourceName            string          `json:"storeName"`
	EngineNo              string          `json:"engineNo"` // 发动机号
	LicensingDate         int64           `json:"licensingDate"`
	Color                 string          `json:"color"`                 // 车身颜色
	InteriorColor         string          `json:"interiorColor"`         // 内饰颜色
	TemporaryLicensePlate string          `json:"temporaryLicensePlate"` // 临时车牌
	ParkingNo             string          `json:"parkingNo"`             // 车位编号
	UseCharacter          string          `json:"useCharacter"`          // 使用性质
	Mileage               decimal.Decimal `json:"mileage"`               // 里程
	NewCarGuidePrice      decimal.Decimal `json:"newCarGuidePrice"`      // 新车指导价
	PurchaseTax           decimal.Decimal `json:"purchaseTax"`           // 购置税
	InstallationCost      decimal.Decimal `json:"installationCost"`      // 加装费用
	Location              string          `json:"location"`              // 车辆所在地
	Condition             string          `json:"condition"`             // 车况
	Photos                photos          `json:"carPhoto"`
}

type brokerInfo struct {
	Id    uint   `json:"id"`
	Name  string `json:"name"`
	Phone string `json:"phone"`
}

type businessInfo struct {
	Id    uint   `json:"id"`
	Name  string `json:"name"`
	Phone string `json:"phone"`
}

type modelInfo struct {
	Year              uint            `json:"year"`              // 年份
	Price             decimal.Decimal `json:"price"`             // 指导价
	Liter             decimal.Decimal `json:"liter"`             // 排量
	DischargeStandard string          `json:"dischargeStandard"` // 排放标准
	FuelTypeName      string          `json:"fuelTypeName"`      // 燃料类型名称
	EnginePower       uint            `json:"enginePower"`       // 马力
	EnginePowerKw     uint            `json:"enginePowerKw"`     // 马力转换功率
	IsGreen           uint            `json:"isGreen"`           // 是否新能源车 0=否 1=是
	ModelCode         string          `json:"modelCode"`         // 汽车公告型号
	DriveName         string          `json:"driveName"`         // 驱动类型
	DriveType         string          `json:"driveType"`         // 驱动类型
	ModelStatus       string          `json:"modelStatus"`       // 车型状态 在售/停售等
	GearType          string          `json:"gearType"`          // 变速箱类型 手动/自动
	MarketDate        string          `json:"marketDate"`        // 生产日期
	MinRegYear        string          `json:"minRegYear"`        // 最小注册年
	MaxRegYear        string          `json:"maxRegYear"`        // 最大注册年
	StopMakeYear      string          `json:"stopMakeYear"`      // 车型停产年份
	Intake            string          `json:"intake"`            // 进气形式
	SeatNumber        string          `json:"seatNumber"`        // 座位数
	BodyType          string          `json:"bodyType"`          // 车身类型
	DoorNumber        uint            `json:"doorNumber"`        // 门数
	CarStruct         string          `json:"carStruct"`         // 车辆结构
	IsParallel        uint            `json:"isParallel"`        // 是否平行进口车 0=否 1=是
	PriceAllowance    string          `json:"priceAllowance"`    // 新能源车补贴
}
type auditInfo struct {
	Status        uint   `json:"status"`
	AuditAt       int64  `json:"auditAt"`
	Reason        string `json:"reason"`
	AuditUserName string `json:"auditUserName"`
}

type priceInfo struct {
	InteriorPrice decimal.Decimal `json:"interiorPrice"` // 内部价格
	ExteriorPrice decimal.Decimal `json:"exteriorPrice"` //外部价格
}

type purchaseInfo struct {
	Price   decimal.Decimal `json:"price"`   // 采购价格
	Date    int64           `json:"date"`    // 采购日期
	Liaison string          `json:"liaison"` // 采购人
}

type certificateInfo struct {
	Transfer                 uint            `json:"transfer"`                 // 过户次数
	KeyCount                 uint            `json:"keyCount"`                 // 钥匙数量
	CompulsoryInsurance      uint            `json:"compulsoryInsurance"`      // 交强险
	CompulsoryInsuranceDate  int64           `json:"compulsoryInsuranceDate"`  // 交强险到期日期
	CommercialInsurance      uint            `json:"commercialInsurance"`      // 商业险
	CommercialInsuranceDate  int64           `json:"commercialInsuranceDate"`  // 商业险到期日期
	CommercialInsurancePrice decimal.Decimal `json:"commercialInsurancePrice"` // 商业险金额
}

type photos struct {
	Cars      []carPhoto `json:"cars"`      // 车辆照片
	Interiors []carPhoto `json:"interiors"` // 内饰照片
	Defects   []carPhoto `json:"defects"`   // 缺陷照片
	Datas     []carPhoto `json:"datas"`     // 资料照片
}

type carPhoto struct {
	Photo string `binding:"required" label:"图片地址"`
	Text  string `binding:"required" label:"图片描述"`
}

type masterInfo struct {
	Id          uint   `json:"id"`
	Name        string `json:"name"`
	IdCard      string `json:"idCard"`
	Phone       string `json:"phone"`
	BankCard    string `json:"bankCard"`
	Bank        string `json:"bank"`
	IdCardFront string `json:"idCardFront"`
	IdCardBack  string `json:"idCardBack"`
	Photo       string `json:"photo"`
}

// Info @Title 车辆详情
func (c *carBaseLogic) Info(carBaseId uint) (result carInfo, err error) {
	carBase := model.CarBase{Id: carBaseId}
	if mysql.Db.
		Preload("Broker.Business").
		//Preload("Order.Contract.Master").
		Preload("Model").
		Preload("LocationCity").
		Preload("AttributionCity").
		Preload("Photos").
		Preload("Audit.AuditUser").
		First(&carBase).Error != nil {
		return result, errors.New("车辆错误")
	}
	result = carInfo{
		Id:           carBase.Id,
		TheUpper:     carBase.TheUpper,
		TheUpperName: carBase.GetTheUpperName(carBase.TheUpper),
		Broker: brokerInfo{
			Id:    carBase.Broker.Id,
			Name:  carBase.Broker.Nickname,
			Phone: carBase.Broker.Phone,
		},
		Business: businessInfo{
			Id:    carBase.Broker.Business.Id,
			Name:  carBase.Broker.Business.Name,
			Phone: carBase.Broker.Business.Telephone,
		},
		Base: baseInfo{
			ModelId:               carBase.Model.Id,
			ModelName:             carBase.Model.Name,
			Type:                  constant.GetCarTypeName(carBase.CarType),
			Vin:                   carBase.Vin,
			Source:                carBase.GetSourceName(carBase.Source),
			EngineNo:              carBase.EngineNo,
			LicensingDate:         carBase.LicensingDate.Time.Unix(),
			Color:                 carBase.Color,
			InteriorColor:         carBase.InteriorColor,
			TemporaryLicensePlate: carBase.TemporaryLicensePlate,
			ParkingNo:             carBase.ParkingNo,
			UseCharacter:          constant.GetCarUseCharacterName(carBase.UseCharacter),
			NewCarGuidePrice:      carBase.Model.Price,
			Mileage:               carBase.ShamMileage,
			PurchaseTax:           carBase.PurchaseTax,
			InstallationCost:      carBase.InstallationCost,
			Location:              carBase.LocationCity.Name,
			Condition:             carBase.Condition,
			Photos: photos{
				Cars:      []carPhoto{},
				Interiors: []carPhoto{},
				Defects:   []carPhoto{},
				Datas:     []carPhoto{},
			},
		},
		Price: priceInfo{
			InteriorPrice: carBase.InteriorPrice,
			ExteriorPrice: carBase.ExteriorPrice,
		},
		PurchaseInfo: purchaseInfo{
			Price:   carBase.PurchasePrice,
			Date:    carBase.PurchaseDate.Time.Unix(),
			Liaison: carBase.PurchaseLiaison,
		},
		Model: modelInfo{
			Year:              carBase.Model.Year,
			Price:             carBase.Model.Price,
			Liter:             carBase.Model.Liter,
			DischargeStandard: carBase.Model.DischargeStandard,
			FuelTypeName:      carBase.Model.FuelTypeName,
			EnginePower:       carBase.Model.EnginePower,
			EnginePowerKw:     carBase.Model.EnginePowerKw,
			IsGreen:           carBase.Model.IsGreen,
			ModelCode:         carBase.Model.ModelCode,
			DriveName:         carBase.Model.DriveName,
			DriveType:         carBase.Model.DriveType,
			ModelStatus:       carBase.Model.ModelStatus,
			GearType:          carBase.Model.GearType,
			MarketDate:        carBase.Model.MarketDate,
			MinRegYear:        carBase.Model.MinRegYear,
			MaxRegYear:        carBase.Model.MaxRegYear,
			StopMakeYear:      carBase.Model.StopMakeYear,
			Intake:            carBase.Model.Intake,
			SeatNumber:        carBase.Model.SeatNumber,
			BodyType:          carBase.Model.BodyType,
			DoorNumber:        carBase.Model.DoorNumber,
			CarStruct:         carBase.Model.CarStruct,
			IsParallel:        carBase.Model.IsParallel,
			PriceAllowance:    carBase.Model.PriceAllowance,
		},
		Certificate: certificateInfo{
			Transfer:                 carBase.Transfer,
			KeyCount:                 carBase.KeyCount,
			CompulsoryInsurance:      carBase.CompulsoryInsurance,
			CompulsoryInsuranceDate:  carBase.CompulsoryInsuranceDate.Time.Unix(),
			CommercialInsurance:      carBase.CommercialInsurance,
			CommercialInsuranceDate:  carBase.CommercialInsuranceDate.Time.Unix(),
			CommercialInsurancePrice: carBase.CommercialInsurancePrice,
		},
		Audit: auditInfo{
			Status:        carBase.Audit.Status,
			AuditAt:       carBase.Audit.AuditAt.Time.Unix(),
			Reason:        carBase.Audit.Reason,
			AuditUserName: carBase.Audit.AuditUser.Name,
		},
	}
	for _, photo := range carBase.Photos {
		switch photo.Type {
		case model.CarPhotoTypeCar: // 车辆照片
			result.Base.Photos.Cars = append(result.Base.Photos.Cars, carPhoto{
				Photo: photo.Path,
				Text:  photo.Text,
			})
		case model.CarPhotoTypeInterior: // 内饰照片
			result.Base.Photos.Interiors = append(result.Base.Photos.Interiors, carPhoto{
				Photo: photo.Path,
				Text:  photo.Text,
			})
		case model.CarPhotoTypeDefect: // 缺陷照片
			result.Base.Photos.Defects = append(result.Base.Photos.Defects, carPhoto{
				Photo: photo.Path,
				Text:  photo.Text,
			})
		case model.CarPhotoTypeData: // 资料照片
			result.Base.Photos.Datas = append(result.Base.Photos.Datas, carPhoto{
				Photo: photo.Path,
				Text:  photo.Text,
			})
		}
	}
	switch carBase.Source {
	case model.CarBaseSourceConsignment: // 个人寄售
		orderConsignment := model.OrderConsignment{Id: carBase.OrderId}
		if mysql.Db.Preload("Contract.Master").First(&orderConsignment).Error == nil {
			result.Master = masterInfo{
				Id:       orderConsignment.Contract.Master.Id,
				Name:     orderConsignment.Contract.Master.Name,
				IdCard:   orderConsignment.Contract.Master.IdCard,
				Phone:    orderConsignment.Contract.Master.Phone,
				BankCard: orderConsignment.Contract.Master.BankCard,
				Bank:     orderConsignment.Contract.Master.Bank,
			}
		}
	case model.CarBaseSourceDealerPurchase: // 车商收购
	}
	store := model.BrokerBusinessStore{Id: carBase.DealerId}
	if mysql.Db.Where(&store).First(&store).Error == nil {
		result.Base.SourceName = store.Name
	}

	return
}

// Adopt @Title 审核通过
func (c *carBaseLogic) Adopt(manageId, carBaseId uint) error {
	carBase := model.CarBase{Id: carBaseId}
	if mysql.Db.Preload("Model").Preload("Audit").First(&carBase).Error != nil {
		return errors.New("车辆错误")
	}
	if carBase.Audit.Status != constant.AuditStatusWaiting {
		return errors.New("已审核，请勿重复操作")
	}
	// 审核发布车辆
	if carBase.Audit.Type == model.CarAuditTypePublish {
		return c.PublishAdopt(manageId, carBase)
	} else
	// 审核上架车辆
	if carBase.Audit.Type == model.CarAuditTypeUpper {
		return c.UpperDownAdopt(manageId, carBase, 1)
	} else
	// 审核下架车辆
	if carBase.Audit.Type == model.CarAuditTypeDown {
		return c.UpperDownAdopt(manageId, carBase, 2)
	} else {
		return errors.New("未知审核种类")
	}
}

// Reject @Title 审核驳回
func (c *carBaseLogic) Reject(manageId, carBaseId uint, reason string) error {
	carBase := model.CarBase{Id: carBaseId}
	if mysql.Db.Preload("Model").Preload("Audit").First(&carBase).Error != nil {
		return errors.New("车辆错误")
	}
	if carBase.Audit.Status != constant.AuditStatusWaiting {
		return errors.New("已审核，请勿重复操作")
	}
	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		if tx.Model(&carBase.Audit).UpdateColumns(map[string]interface{}{
			"status":        constant.AuditStatusReject,
			"reason":        reason,
			"audit_at":      time.Now(),
			"audit_user_id": manageId,
		}).RowsAffected != 1 {
			return errors.New("审核失败")
		}
		if tx.Model(&model.CarBase{}).Where(&carBase).Updates(map[string]interface{}{
			"status": model.CarBaseStatusReject,
		}).RowsAffected != 1 {
			return errors.New("审核失败")
		}
		message.MessageLogic.MessageToBroker(mysql.Db, []uint{carBase.BrokerId}, model.BrokerMessageTypeCarAudit, message.MessageItem{
			Title:    "车辆发布提示",
			Content:  "您的一个车辆发布已被后台审批拒绝",
			LinkedId: carBase.Id,
			Extras: map[string]interface{}{
				"车辆名称": carBase.Model.Name,
				"发布价格": fmt.Sprintf("内部价%s|外部价%s", carBase.InteriorPrice, carBase.ExteriorPrice),
				"审核时间": carBase.Audit.AuditAt.Time.Unix(),
			},
		})
		return nil
	})
}

// PublishAdopt @Title 审核车辆发布
func (c *carBaseLogic) PublishAdopt(manageId uint, carBase model.CarBase) error {
	// vin存在自动驳回
	vinCheck := car.CarLogic.VinCheck(carBase.Vin)
	if vinCheck.IsLegal == 0 {
		mysql.Db.Model(&carBase.Audit).UpdateColumns(map[string]interface{}{
			"status":        constant.AuditStatusReject,
			"reason":        vinCheck.Message,
			"audit_at":      time.Now(),
			"audit_user_id": manageId,
		})
		return errors.New(vinCheck.Message)
	}

	carSn, err := common.GenNo(constant.NoPrefixCar)
	if err != nil {
		return errors.New("审核失败")
	}
	carModel := model.Car{
		ModelId:    carBase.ModelId,
		CarSn:      carSn,
		Vin:        carBase.Vin,
		OrderId:    carBase.OrderId,
		OrderCarId: carBase.Id,
		StoreId:    carBase.DealerId,
		CustomerId: carBase.CustomerId,
		BrokerId:   carBase.BrokerId,
		MainPhoto:  carBase.MainPhoto,
		Status:     model.CarStatusNormal,
		//ReckonPrice:   consignmentCar.Order.Contract.ReckonPrice,
		Price:         carBase.ExteriorPrice,
		InteriorPrice: carBase.InteriorPrice,
		ExteriorPrice: carBase.ExteriorPrice,
		LicensingDate: carBase.LicensingDate,
		Mileage:       carBase.ShamMileage,
		Transfer:      carBase.Transfer,
		Color:         carBase.Color,
		InteriorColor: carBase.InteriorColor,
		TheUpper:      model.CarTheUpperTrue,
		//LastPrice:     consignmentCar.Order.ContractPrice,
	}

	priceAmount := car.PriceLogic.Amount(carModel.ExteriorPrice)
	carModel.DownPayment = priceAmount.DownPayment

	switch carBase.CarType {
	case constant.CarBaseTypeSecondCarNormal, constant.CarBaseTypeSecondCarImport, constant.CarBaseTypeFirstCar:
		carModel.Type = model.CarTypeSecond
	case constant.CarBaseTypeNewCarNormal, constant.CarBaseTypeNewCarImport:
		carModel.Type = model.CarTypeNew
	}
	switch carBase.Source {
	case model.CarBaseSourceConsignment: // 个人寄售
		carModel.CustomerType = model.CustomerTypeCustomer
	case model.CarBaseSourceDealerPurchase: // 车商收购
		carModel.CustomerType = model.CustomerTypeDealer
	}
	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		if tx.Create(&carModel).Error != nil {
			return errors.New("车辆发布失败")
		}
		carAdjustPriceHistory := model.CarAdjustPriceHistory{
			CarId:         carModel.Id,
			Type:          model.CarAdjustPriceHistoryTypeInit,
			InteriorPrice: carModel.InteriorPrice,
			ExteriorPrice: carModel.ExteriorPrice,
		}
		if tx.Create(&carAdjustPriceHistory).Error != nil {
			return errors.New("车辆发布失败")
		}
		if tx.Model(&carBase.Audit).UpdateColumns(map[string]interface{}{
			"status":        constant.AuditStatusAdopt,
			"audit_at":      time.Now(),
			"audit_user_id": manageId,
		}).RowsAffected != 1 {
			return errors.New("审核失败")
		}
		if tx.Model(&model.CarBase{}).Where(&carBase).Updates(map[string]interface{}{
			"status":    model.CarBaseStatusRelease,
			"the_upper": model.CarBaseTheUpperTrue,
			"car_id":    carModel.Id,
		}).RowsAffected != 1 {
			return errors.New("车辆上架失败")
		}
		if carBase.Source == model.CarBaseSourceConsignment {
			if tx.Model(&model.OrderConsignment{}).Where("id = ?", carBase.OrderId).Updates(map[string]interface{}{
				"status": model.OrderConsignmentStatusTheUpper,
			}).RowsAffected != 1 {
				return errors.New("寄售订单修改上架状态失败")
			}
		}
		message.MessageLogic.MessageToBroker(mysql.Db, []uint{carBase.BrokerId}, model.BrokerMessageTypeCarAudit, message.MessageItem{
			Title:    "车辆发布提示",
			Content:  "您已成功完成车辆发布",
			LinkedId: carBase.Id,
			Extras: map[string]interface{}{
				"车辆名称": carBase.Model.Name,
				"发布价格": fmt.Sprintf("内部价%s|外部价%s", carBase.InteriorPrice, carBase.ExteriorPrice),
				"审核时间": carBase.Audit.AuditAt.Time.Unix(),
			},
		})
		return nil
	})
}

// UpperDownAdopt @Title 审核车辆上下架
func (c *carBaseLogic) UpperDownAdopt(manageId uint, carBase model.CarBase, kind uint) error {
	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		if tx.Model(&carBase.Audit).UpdateColumns(map[string]interface{}{
			"status":        constant.AuditStatusAdopt,
			"audit_at":      time.Now(),
			"audit_user_id": manageId,
		}).RowsAffected != 1 {
			return errors.New("审核失败")
		}
		switch kind {
		case 1:
			vinCheck := car.CarLogic.VinCheck(carBase.Vin)
			if vinCheck.IsLegal == 0 {
				return errors.New(vinCheck.Message)
			}
			SecondLogic.UpperCar(tx, carBase.CarId, carBase.Id, carBase.Source, carBase.OrderId)
		case 2:
			SecondLogic.DownCar(tx, carBase.CarId, carBase.Id, carBase.Source, carBase.OrderId)
		}
		return nil
	})
}
