package car

import (
	"base/app/constant"
	"base/app/lib/bean"
	"base/app/logic/manage/order"
	"base/app/model"
	"database/sql"
	"errors"
	"fmt"
	"git.oa00.com/go/mysql"
	"github.com/shopspring/decimal"
	"gorm.io/gorm"
	"time"
)

var SecondLogic = &secondLogic{}

type secondLogic struct {
}

type secondItem struct {
	Id            uint            `json:"id"`
	MainPhoto     string          `json:"mainPhoto"`
	ModelName     string          `json:"modelName"`
	CarSn         string          `json:"carSn"`
	CustomerType  uint            `json:"customerType"`
	BusinessName  string          `json:"businessName"`
	LicensingDate int64           `json:"licensingDate"`
	Mileage       decimal.Decimal `json:"mileage"`
	ReckonPrice   decimal.Decimal `json:"reckonPrice"`
	Price         decimal.Decimal `json:"price"`
	CreatedAt     int64           `json:"createdAt"`
	Status        uint            `json:"status"`
	StatusName    string          `json:"statusName"`
	TheUpper      uint            `json:"theUpper"`
	TheUpperName  string          `json:"theUpperName"`
}
type SecondSearch struct {
	ModelName        string
	CarSn            string
	PriceType        uint
	Price            decimal.Decimal
	CustomerType     uint
	BusinessId       uint
	CreatedDateStart string
	CreatedDateEnd   string
	Status           uint
}

// Lists @Title 二手车列表
func (s *secondLogic) Lists(search SecondSearch, page bean.Page) (lists []secondItem, total int64) {
	lists = []secondItem{}
	var cars []model.Car
	where := mysql.Db.Where("type = ?", model.CarTypeSecond)
	// 车型筛选
	if search.ModelName != "" {
		where = where.Where("model_id in (?)", mysql.Db.Model(&model.CarModel{}).Select("model_id").Where("name like ?", fmt.Sprintf("%%%s%%", search.ModelName)))
	}
	// 车辆编号
	if search.CarSn != "" {
		where = where.Where("car_sn like ?", fmt.Sprintf("%%%s%%", search.CarSn))
	}
	// 展示价格
	if !search.Price.IsZero() {
		switch search.PriceType {
		case 1: // 大于等于
			where = where.Where("price >= ?", search.Price)
		case 2: // 小于等于
			where = where.Where("price <= ?", search.Price)
		}
	}
	// 车源类型
	if search.CustomerType > 0 {
		switch search.CustomerType {
		case 1: // 个人
			where = where.Where("customer_type = ?", model.CustomerTypeCustomer)
		case 2: // 车商
			where = where.Where("customer_type = ?", model.CustomerTypeDealer)
		}
	}
	// 入驻商
	if search.BusinessId > 0 {
		where = where.Where("broker_id in (?)", mysql.Db.Model(&model.Broker{}).Select("id").Where("business_id = ?", search.BusinessId))
	}
	// 发布日期
	if search.CreatedDateStart != "" && search.CreatedDateEnd != "" {
		where = where.Where("created_at >= ? and created_at <= ?", search.CreatedDateStart, search.CreatedDateEnd+" 23:59:59")
	}
	// 车辆状态
	if search.Status > 0 {
		switch search.Status {
		case 1: // 已上架
			where = where.Where("the_upper = ? and status = ?", model.CarTheUpperTrue, model.CarStatusNormal)
		case 2: // 已下架
			where = where.Where("the_upper = ?", model.CarTheUpperFalse)
		case 3: // 在售
			where = where.Where("the_upper = ? and status = ?", model.CarTheUpperTrue, model.CarStatusDeposit)
		case 4: // 已售
			where = where.Where("the_upper = ? and status = ?", model.CarTheUpperFalse, model.CarStatusFinal)
		}
	}
	mysql.Db.Model(&cars).Where(where).Count(&total)
	if page.HasPage(total) {
		mysql.Db.
			Preload("Model").Preload("Broker.Business").
			Where(where).Offset(page.GetStart()).Limit(page.GetLimit()).Order("id desc").Find(&cars)
		for _, car := range cars {
			lists = append(lists, secondItem{
				Id:            car.Id,
				MainPhoto:     car.MainPhoto,
				ModelName:     car.Model.Name,
				CarSn:         car.CarSn,
				CustomerType:  car.CustomerType,
				BusinessName:  car.Broker.Business.Name,
				LicensingDate: car.LicensingDate.Time.Unix(),
				Mileage:       car.Mileage,
				ReckonPrice:   car.ReckonPrice,
				Price:         car.Price,
				CreatedAt:     car.CreatedAt.Unix(),
				Status:        car.Status,
				StatusName:    car.GetStatusName(car.Status),
				TheUpper:      car.TheUpper,
				TheUpperName:  car.GetTheUpperName(car.TheUpper),
			})
		}
	}
	return
}

type secondCarInfo struct {
	Id           uint                `json:"id"`
	CarSn        string              `json:"carSn"`
	TheUpper     uint                `json:"theUpper"`
	TheUpperName string              `json:"theUpperName"`
	Base         baseInfo            `json:"base"`
	Model        modelInfo           `json:"model"`
	Price        priceInfo           `json:"price"`
	PurchaseInfo purchaseInfo        `json:"purchaseInfo"`
	Broker       brokerInfo          `json:"broker"`
	Business     businessInfo        `json:"business"`
	Audit        auditInfo           `json:"audit"`
	Certificate  certificateInfo     `json:"certificate"`
	Master       masterInfo          `json:"master"`
	SaleInfo     order.SaleOrderInfo `json:"saleInfo"`
}

// Info @Title 二手车详情
func (s *secondLogic) Info(carId uint) (result secondCarInfo, err error) {
	car := model.Car{Id: carId}
	mysql.Db.
		Preload("Store.Business").
		Preload("Model").
		Preload("CarBase.LocationCity").
		Preload("CarBase.AttributionCity").
		Preload("Broker.Business").
		Preload("CarBase.Photos").
		First(&car)
	if car.Type != model.CarTypeSecond {
		return result, errors.New("车辆非二手车")
	}
	result = secondCarInfo{
		Id:           car.Id,
		CarSn:        car.CarSn,
		TheUpper:     car.TheUpper,
		TheUpperName: car.GetTheUpperName(car.TheUpper),
		Broker: brokerInfo{
			Id:    car.Broker.Id,
			Name:  car.Broker.Nickname,
			Phone: car.Broker.Phone,
		},
		Business: businessInfo{
			Id:    car.Broker.Business.Id,
			Name:  car.Broker.Business.Name,
			Phone: car.Broker.Business.Telephone,
		},
		Base: baseInfo{
			ModelId:               car.Model.Id,
			ModelName:             car.Model.Name,
			Type:                  constant.GetCarTypeName(car.CarBase.CarType),
			Vin:                   car.CarBase.Vin,
			Source:                car.CarBase.GetSourceName(car.CarBase.Source),
			EngineNo:              car.CarBase.EngineNo,
			LicensingDate:         car.CarBase.LicensingDate.Time.Unix(),
			Color:                 car.CarBase.Color,
			InteriorColor:         car.CarBase.InteriorColor,
			TemporaryLicensePlate: car.CarBase.TemporaryLicensePlate,
			ParkingNo:             car.CarBase.ParkingNo,
			UseCharacter:          constant.GetCarUseCharacterName(car.CarBase.UseCharacter),
			NewCarGuidePrice:      car.Model.Price,
			Mileage:               car.CarBase.ShamMileage,
			PurchaseTax:           car.CarBase.PurchaseTax,
			InstallationCost:      car.CarBase.InstallationCost,
			Location:              car.CarBase.LocationCity.Name,
			Condition:             car.CarBase.Condition,
			Photos: photos{
				Cars:      []carPhoto{},
				Interiors: []carPhoto{},
				Defects:   []carPhoto{},
				Datas:     []carPhoto{},
			},
		},
		Price: priceInfo{
			InteriorPrice: car.CarBase.InteriorPrice,
			ExteriorPrice: car.CarBase.ExteriorPrice,
		},
		PurchaseInfo: purchaseInfo{
			Price:   car.CarBase.PurchasePrice,
			Date:    car.CarBase.PurchaseDate.Time.Unix(),
			Liaison: car.CarBase.PurchaseLiaison,
		},
		Model: modelInfo{
			Year:              car.Model.Year,
			Price:             car.Model.Price,
			Liter:             car.Model.Liter,
			DischargeStandard: car.Model.DischargeStandard,
			FuelTypeName:      car.Model.FuelTypeName,
			EnginePower:       car.Model.EnginePower,
			EnginePowerKw:     car.Model.EnginePowerKw,
			IsGreen:           car.Model.IsGreen,
			ModelCode:         car.Model.ModelCode,
			DriveName:         car.Model.DriveName,
			DriveType:         car.Model.DriveType,
			ModelStatus:       car.Model.ModelStatus,
			GearType:          car.Model.GearType,
			MarketDate:        car.Model.MarketDate,
			MinRegYear:        car.Model.MinRegYear,
			MaxRegYear:        car.Model.MaxRegYear,
			StopMakeYear:      car.Model.StopMakeYear,
			Intake:            car.Model.Intake,
			SeatNumber:        car.Model.SeatNumber,
			BodyType:          car.Model.BodyType,
			DoorNumber:        car.Model.DoorNumber,
			CarStruct:         car.Model.CarStruct,
			IsParallel:        car.Model.IsParallel,
			PriceAllowance:    car.Model.PriceAllowance,
		},
		Certificate: certificateInfo{
			Transfer:                 car.CarBase.Transfer,
			KeyCount:                 car.CarBase.KeyCount,
			CompulsoryInsurance:      car.CarBase.CompulsoryInsurance,
			CompulsoryInsuranceDate:  car.CarBase.CompulsoryInsuranceDate.Time.Unix(),
			CommercialInsurance:      car.CarBase.CommercialInsurance,
			CommercialInsuranceDate:  car.CarBase.CommercialInsuranceDate.Time.Unix(),
			CommercialInsurancePrice: car.CarBase.CommercialInsurancePrice,
		},
		Audit: auditInfo{
			Status:        car.CarBase.Audit.Status,
			AuditAt:       car.CarBase.Audit.AuditAt.Time.Unix(),
			Reason:        car.CarBase.Audit.Reason,
			AuditUserName: car.CarBase.Audit.AuditUser.Name,
		},
	}
	for _, photo := range car.CarBase.Photos {
		switch photo.Type {
		case model.CarPhotoTypeCar: // 车辆照片
			result.Base.Photos.Cars = append(result.Base.Photos.Cars, carPhoto{
				Photo: photo.Path,
				Text:  photo.Text,
			})
		case model.CarPhotoTypeInterior: // 内饰照片
			result.Base.Photos.Interiors = append(result.Base.Photos.Interiors, carPhoto{
				Photo: photo.Path,
				Text:  photo.Text,
			})
		case model.CarPhotoTypeDefect: // 缺陷照片
			result.Base.Photos.Defects = append(result.Base.Photos.Defects, carPhoto{
				Photo: photo.Path,
				Text:  photo.Text,
			})
		case model.CarPhotoTypeData: // 资料照片
			result.Base.Photos.Datas = append(result.Base.Photos.Datas, carPhoto{
				Photo: photo.Path,
				Text:  photo.Text,
			})
		}
	}
	switch car.CarBase.Source {
	case model.CarBaseSourceConsignment: // 个人寄售
		orderConsignment := model.OrderConsignment{Id: car.CarBase.OrderId}
		if mysql.Db.Preload("Contract.Master").First(&orderConsignment).Error == nil {
			result.Master = masterInfo{
				Id:       orderConsignment.Contract.Master.Id,
				Name:     orderConsignment.Contract.Master.Name,
				IdCard:   orderConsignment.Contract.Master.IdCard,
				Phone:    orderConsignment.Contract.Master.Phone,
				BankCard: orderConsignment.Contract.Master.BankCard,
				Bank:     orderConsignment.Contract.Master.Bank,
			}
		}
	case model.CarBaseSourceDealerPurchase: // 车商收购
		store := model.BrokerBusinessStore{Id: car.CarBase.DealerId}
		if mysql.Db.Where(&store).First(&store).Error == nil {
			result.Base.SourceName = store.Name
		}
	}
	if car.Sale != nil && car.Sale.Id > 0 {
		saleInfo, _ := order.SaleLogic.Info(car.Sale.Id)
		result.SaleInfo = saleInfo
	}
	return
}

// Upper @Title 后台上架车辆
func (s *secondLogic) Upper(manageId, carId uint) error {
	car := model.Car{Id: carId}
	if mysql.Db.Preload("CarBase.Order").First(&car).Error != nil {
		return errors.New("车辆错误")
	}
	if car.CarBase.Id == 0 {
		return errors.New("车辆寄售订单错误")
	}
	if car.TheUpper != model.CarTheUpperFalse {
		return errors.New("非下架状态")
	}

	audit := model.CarAudit{
		CarBaseId:   car.OrderCarId,
		CarId:       carId,
		Type:        model.CarAuditTypeManageUpper,
		Status:      constant.AuditStatusAdopt,
		AuditAt:     sql.NullTime{Time: time.Now(), Valid: true},
		AuditUserId: manageId,
	}
	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		if tx.Create(&audit).Error != nil {
			return errors.New("提交失败")
		}
		s.UpperCar(tx, carId, car.CarBase.Id, car.CarBase.Source, car.CarBase.Order.Id)
		return nil
	})
}

// Down @Title 后台下架车辆
func (s *secondLogic) Down(manageId, carId uint) error {
	car := model.Car{Id: carId}
	if mysql.Db.Preload("CarBase.Order").First(&car).Error != nil {
		return errors.New("车辆错误")
	}
	if car.CarBase.Id == 0 {
		return errors.New("车辆寄售订单错误")
	}
	if car.TheUpper != model.CarTheUpperTrue {
		return errors.New("非上架状态")
	}

	audit := model.CarAudit{
		CarBaseId:   car.OrderCarId,
		CarId:       carId,
		Type:        model.CarAuditTypeManageDown,
		Status:      constant.AuditStatusAdopt,
		AuditAt:     sql.NullTime{Time: time.Now(), Valid: true},
		AuditUserId: manageId,
	}
	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		if tx.Create(&audit).Error != nil {
			return errors.New("提交失败")
		}
		s.DownCar(tx, carId, car.CarBase.Id, car.CarBase.Source, car.CarBase.Order.Id)
		return nil
	})
}

// UpperCar @Title 车辆上架
func (s *secondLogic) UpperCar(tx *gorm.DB, carId, consignmentCarId, source, consignmentId uint) error {
	if tx.Model(&model.Car{}).Where("id = ?", carId).UpdateColumns(map[string]interface{}{
		"the_upper": model.CarTheUpperTrue,
	}).RowsAffected != 1 {
		return errors.New("车辆上架失败")
	}
	if tx.Model(&model.CarBase{}).Where("id = ?", consignmentCarId).UpdateColumns(map[string]interface{}{
		"the_upper": model.CarBaseTheUpperTrue,
	}).RowsAffected != 1 {
		return errors.New("寄售车辆上架失败")
	}
	if source == model.CarBaseSourceConsignment {
		if tx.Model(&model.OrderConsignment{}).Where("id = ?", consignmentId).UpdateColumns(map[string]interface{}{
			"status": model.OrderConsignmentStatusTheUpper,
		}).RowsAffected != 1 {
			return errors.New("寄售订单修改上架状态失败")
		}
	}
	return nil
}

// DownCar @Title 车辆下架
func (s *secondLogic) DownCar(tx *gorm.DB, carId, consignmentCarId, source, consignmentId uint) error {
	if tx.Model(&model.Car{}).Where("id = ?", carId).UpdateColumns(map[string]interface{}{
		"the_upper": model.CarTheUpperFalse,
	}).RowsAffected != 1 {
		return errors.New("车辆下架失败")
	}
	if tx.Model(&model.CarBase{}).Where("id = ?", consignmentCarId).UpdateColumns(map[string]interface{}{
		"the_upper": model.CarBaseTheUpperFalse,
	}).RowsAffected != 1 {
		return errors.New("寄售车辆下架失败")
	}
	if source == model.CarBaseSourceConsignment {
		if tx.Model(&model.OrderConsignment{}).Where("id = ?", consignmentId).UpdateColumns(map[string]interface{}{
			"status": model.OrderConsignmentStatusTheDown,
		}).RowsAffected != 1 {
			return errors.New("寄售订单修改下架状态失败")
		}
	}
	return nil
}
