package dealer

import (
	"base/app/lib/bean"
	"base/app/model"
	"errors"
	"fmt"
	"git.oa00.com/go/mysql"
	"gorm.io/gorm"
	"time"
)

var AuditLogic = &auditLogic{}

type auditLogic struct {
}
type auditItem struct {
	Id          uint   `json:"id"`
	Name        string `json:"name"`
	BrokerName  string `json:"brokerName"`
	BrokerPhone string `json:"brokerPhone"`
	CreatedName string `json:"createdName"`
	AuditStatus uint   `json:"auditStatus"`
	ApplyAt     int64  `json:"applyAt"`
}
type AuditSearch struct {
	Name           string
	ApplyUser      string
	Phone          string
	ApplyDataStart string
	ApplyDataEnd   string
	AuditStatus    uint
}

// Lists @Title 车商审核列表
func (a *auditLogic) Lists(search AuditSearch, page bean.Page) (lists []auditItem, total int64) {
	lists = []auditItem{}
	var dealerAudits []model.DealerAudit
	where := mysql.Db
	// 车商筛选
	{
		dealerWhere := mysql.Db.Model(&model.Dealer{}).Select("id")
		// 车商名称
		if search.Name != "" {
			dealerWhere = dealerWhere.Where("name like ?", fmt.Sprintf("%%%s%%", search.Name))
		}
		// 申请人筛选
		{
			manageWhere := mysql.Db.Model(&model.Manage{}).Select("id")
			if search.ApplyUser != "" {
				manageWhere = manageWhere.Where("name like ?", fmt.Sprintf("%%%s%%", search.ApplyUser))
			}
			if search.Phone != "" {
				manageWhere = manageWhere.Where("phone like ?", fmt.Sprintf("%%%s%%", search.Phone))
			}
			if _, ok := manageWhere.Statement.Clauses["WHERE"]; ok {
				dealerWhere = dealerWhere.Where("apply_user_id in (?)", manageWhere)
			}
		}
		if _, ok := dealerWhere.Statement.Clauses["WHERE"]; ok {
			where = where.Where("dealer_id in (?)", dealerWhere)
		}
	}
	// 申请日期
	if search.ApplyDataStart != "" && search.ApplyDataEnd != "" {
		where = where.Where("created_at >= ? and created_at <= ?", search.ApplyDataStart, search.ApplyDataEnd+" 23:59:59")
	}
	// 审核状态
	if search.AuditStatus > 0 {
		switch search.AuditStatus {
		case 1: // 待审核
			where = where.Where("status = ?", model.DealerAuditStatusAudit)
		case 2: // 已审核
			where = where.Where("status in ?", []uint{
				model.DealerAuditStatusAdopt,
				model.DealerAuditStatusReject,
			})
		}
	}
	mysql.Db.Model(&dealerAudits).Where(where).Count(&total)
	if page.HasPage(total) {
		mysql.Db.
			Preload("Dealer.Broker").Preload("Dealer.ApplyUser").
			Where(where).Offset(page.GetStart()).Limit(page.GetLimit()).Order("id desc").Find(&dealerAudits)
		for _, dealer := range dealerAudits {
			lists = append(lists, auditItem{
				Id:          dealer.Id,
				Name:        dealer.Dealer.Name,
				BrokerName:  dealer.Dealer.Broker.Nickname,
				BrokerPhone: dealer.Dealer.Broker.Phone,
				CreatedName: dealer.Dealer.ApplyUser.Name,
				AuditStatus: dealer.Status,
				ApplyAt:     dealer.CreatedAt.Unix(),
			})
		}
	}
	return
}

// Adopt @Title 审核通过
func (a *auditLogic) Adopt(manageId, auditId uint) error {
	dealerAudit := model.DealerAudit{Id: auditId}
	if mysql.Db.First(&dealerAudit).Error != nil {
		return errors.New("车商错误")
	}
	if dealerAudit.Status != model.DealerAuditStatusAudit {
		return errors.New("已审核，请勿重复操作")
	}
	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		if tx.Model(&dealerAudit).Where(&dealerAudit).UpdateColumns(map[string]interface{}{
			"status":        model.DealerAuditStatusAdopt,
			"audit_at":      time.Now(),
			"audit_user_id": manageId,
		}).RowsAffected != 1 {
			return errors.New("审核失败")
		}
		if tx.Model(&model.Dealer{}).Where("id = ?", dealerAudit.DealerId).UpdateColumn("status", model.DealerStatusEnabled).Error != nil {
			return errors.New("审核失败")
		}
		return nil
	})
}

// Reject @Title 审核驳回
func (a *auditLogic) Reject(manageId, auditId uint, reason string) error {
	dealerAudit := model.DealerAudit{Id: auditId}
	if mysql.Db.First(&dealerAudit).Error != nil {
		return errors.New("审核单错误")
	}
	if dealerAudit.Status != model.DealerAuditStatusAudit {
		return errors.New("已审核，请勿重复操作")
	}
	if mysql.Db.Model(&dealerAudit).Where(&dealerAudit).UpdateColumns(map[string]interface{}{
		"status":        model.DealerAuditStatusReject,
		"audit_at":      time.Now(),
		"reason":        reason,
		"audit_user_id": manageId,
	}).RowsAffected != 1 {
		return errors.New("审核失败")
	}
	return nil
}

type auditInfo struct {
	Id           uint              `json:"id"`
	DealerNo     string            `json:"dealerNo"`
	Name         string            `json:"name"`
	Address      string            `json:"address"`
	ContractNo   string            `json:"contractNo"`
	Contract     string            `json:"contract"`
	ExpireDate   int64             `json:"expireDate"`
	Liaison      string            `json:"liaison"`
	LiaisonPhone string            `json:"liaisonPhone"`
	Telephone    string            `json:"telephone"`
	StoreName    string            `json:"storeName"`
	StaffName    string            `json:"staffName"`
	StaffPhone   string            `json:"staffPhone"`
	Company      dealerInfoCompany `json:"company"`
	Finance      dealerInfoFinance `json:"finance"`
	Audit        dealerInfoAudit   `json:"audit"`
}

// Info @Title 车商审核详情
func (a *auditLogic) Info(auditId uint) (result auditInfo, err error) {
	dealerAudit := model.DealerAudit{Id: auditId}
	if mysql.Db.
		Preload("AuditUser").Preload("Dealer.Broker.Staff.Store").
		Preload("Dealer.Company").Preload("Dealer.Finance").
		Where(&dealerAudit).First(&dealerAudit).Error != nil {
		return result, errors.New("数据错误")
	}
	result = auditInfo{
		Id:           dealerAudit.Id,
		DealerNo:     dealerAudit.Dealer.DealerNo,
		Name:         dealerAudit.Dealer.Name,
		Address:      dealerAudit.Dealer.Address,
		ContractNo:   dealerAudit.Dealer.ContractNo,
		Contract:     dealerAudit.Dealer.Contract,
		ExpireDate:   dealerAudit.Dealer.ExpireDate.Unix(),
		Liaison:      dealerAudit.Dealer.Liaison,
		LiaisonPhone: dealerAudit.Dealer.LiaisonPhone,
		Telephone:    dealerAudit.Dealer.Telephone,
		StoreName:    dealerAudit.Dealer.Broker.Staff.Store.Name,
		StaffName:    dealerAudit.Dealer.Broker.Staff.Name,
		StaffPhone:   dealerAudit.Dealer.Broker.Staff.Phone,
		Company: dealerInfoCompany{
			Name:                 dealerAudit.Dealer.Company.Name,
			Quality:              dealerAudit.Dealer.Company.Quality,
			RegisteredCapital:    dealerAudit.Dealer.Company.RegisteredCapital,
			BusinessTermStart:    dealerAudit.Dealer.Company.BusinessTermStart.Unix(),
			BusinessTermEnd:      dealerAudit.Dealer.Company.BusinessTermEnd.Unix(),
			BusinessLicenseNo:    dealerAudit.Dealer.Company.BusinessLicenseNo,
			LegalPerson:          dealerAudit.Dealer.Company.LegalPerson,
			LegalPersonCard:      dealerAudit.Dealer.Company.LegalPersonCard,
			IdCardFront:          dealerAudit.Dealer.Company.IdCardFront,
			IdCardBack:           dealerAudit.Dealer.Company.IdCardBack,
			BusinessLicensePhoto: dealerAudit.Dealer.Company.BusinessLicensePhoto,
		},
		Finance: dealerInfoFinance{
			OpeningBank: dealerAudit.Dealer.Finance.OpeningBank,
			BankAccount: dealerAudit.Dealer.Finance.BankAccount,
			Address:     dealerAudit.Dealer.Finance.Address,
			Telephone:   dealerAudit.Dealer.Finance.Telephone,
			Proof:       dealerAudit.Dealer.Finance.Proof,
		},
		Audit: dealerInfoAudit{
			Status:        dealerAudit.Status,
			AuditAt:       dealerAudit.AuditAt.Time.Unix(),
			Reason:        dealerAudit.Reason,
			AuditUserName: dealerAudit.AuditUser.Name,
		},
	}
	return
}
