package dealer

import (
	"base/app/common"
	"base/app/constant"
	"base/app/lib/bean"
	"base/app/model"
	"errors"
	"fmt"
	"git.oa00.com/go/mysql"
	"github.com/shopspring/decimal"
	"gorm.io/gorm"
	"time"
)

var DealerLogic = &dealerLogic{}

type dealerLogic struct {
}
type dealerItem struct {
	Id          uint   `json:"id"`
	DealerNo    string `json:"dealerNo"`
	Name        string `json:"name"`
	Status      uint   `json:"status"`
	StatusName  string `json:"statusName"`
	ExpireDate  int64  `json:"expireDate"`
	BrokerName  string `json:"brokerName"`
	BrokerPhone string `json:"brokerPhone"`
	CreatedName string `json:"createdName"`
	CreatedAt   int64  `json:"createdAt"`
	AuditStatus uint   `json:"auditStatus"`
}
type DealerSearch struct {
	No               string
	Name             string
	SaleName         string
	SalePhone        string
	ContractStatus   uint
	ExpireDateStart  string
	ExpireDateEnd    string
	CreatedDateStart string
	CreatedDateEnd   string
}

// Lists @Title 车商列表
func (d *dealerLogic) Lists(manageId uint, search DealerSearch, page bean.Page) (lists []dealerItem, total int64) {
	lists = []dealerItem{}
	var dealers []model.Dealer
	where := mysql.Db
	// 车商编号
	if search.No != "" {
		where = where.Where("dealer_no like ?", fmt.Sprintf("%%%s%%", search.No))
	}
	// 车商名称
	if search.Name != "" {
		where = where.Where("name like ?", fmt.Sprintf("%%%s%%", search.Name))
	}
	// 车务
	if search.SaleName != "" || search.SalePhone != "" {
		saleWhere := mysql.Db.Model(&model.Broker{}).Select("id")
		if search.SaleName != "" {
			saleWhere = saleWhere.Where("nickname like ?", fmt.Sprintf("%%%s%%", search.SaleName))
		}
		if search.SalePhone != "" {
			saleWhere = saleWhere.Where("phone like ?", fmt.Sprintf("%%%s%%", search.SalePhone))
		}
		where = where.Where("sale_id in (?)", saleWhere)
	}
	// 合同状态
	if search.ContractStatus > 0 {

	}
	// 合同到期日
	if search.ExpireDateStart != "" && search.ExpireDateEnd != "" {
		where = where.Where("expire_date > ? and expire_date <= ?", search.ExpireDateStart, search.ExpireDateEnd+" 23:59:59")
	}
	// 创建日期
	if search.CreatedDateStart != "" && search.CreatedDateEnd != "" {
		where = where.Where("created_at > ? and created_at <= ?", search.CreatedDateStart, search.CreatedDateEnd+" 23:59:59")
	}
	mysql.Db.Model(&dealers).Where(where).Count(&total)
	if page.HasPage(total) {
		mysql.Db.
			Preload("Broker").Preload("Audit").Preload("ApplyUser").
			Where(where).Offset(page.GetStart()).Limit(page.GetLimit()).Find(&dealers)
		for _, dealer := range dealers {
			lists = append(lists, dealerItem{
				Id:          dealer.Id,
				DealerNo:    dealer.DealerNo,
				Name:        dealer.Name,
				Status:      dealer.Status,
				StatusName:  dealer.GetStatusName(dealer.Status),
				ExpireDate:  dealer.ExpireDate.Unix(),
				BrokerName:  dealer.Broker.Nickname,
				BrokerPhone: dealer.Broker.Phone,
				AuditStatus: dealer.Audit.Status,
				CreatedName: dealer.ApplyUser.Name,
				CreatedAt:   dealer.CreatedAt.Unix(),
			})
		}
	}
	return
}

type DealerAdd struct {
	Base    base
	Company company
	Finance finance
}
type base struct {
	Name         string `binding:"required" label:"车商名称"`
	Address      string `binding:"required" label:"公司地址"`
	ContractNo   string `binding:"required" label:"合同编号"`
	Contract     string `binding:"required" label:"合同附件"`
	ExpireDate   string `binding:"required" label:"合同到期日"`
	Liaison      string `binding:"required" label:"联系人"`
	LiaisonPhone string `binding:"required" label:"联系人手机号"`
	Telephone    string `binding:"required" label:"车商电话"`
	BrokerId     uint   `binding:"required" label:"所属车务"`
}
type company struct {
	Name                 string          `binding:"required" label:"公司名"`
	BusinessTermStart    string          `binding:"required" label:"营业期限"`
	BusinessTermEnd      string          `binding:"required" label:"营业期限"`
	RegisteredCapital    decimal.Decimal //`binding:"required" label:"注册资本"`
	Quality              uint            `binding:"required,oneof=1 2" label:"公司性质"`
	BusinessLicenseNo    string          `binding:"required" label:"信用代码"`
	LegalPerson          string          `binding:"required" label:"法人"`
	LegalPersonCard      string          `binding:"required" label:"身份证"`
	IdCardFront          string          `binding:"required" label:"身份证正面"`
	IdCardBack           string          `binding:"required" label:"身份证反面"`
	BusinessLicensePhoto string          `binding:"required" label:"营业执照"`
}
type finance struct {
	OpeningBank string `binding:"required" label:"开户行"`
	BankAccount string `binding:"required" label:"账号"`
	Address     string // `binding:"required" label:"地址"`
	Telephone   string // `binding:"required" label:"电话"`
	Proof       string `binding:"required" label:"开户证明"`
}

// Add @Title 添加车商
func (d *dealerLogic) Add(manageId uint, data DealerAdd) error {
	expireDate, err := time.ParseInLocation("2006-01-02", data.Base.ExpireDate, time.Local)
	if err != nil {
		return errors.New("合同到期日错误")
	}
	businessTermStart, err := time.ParseInLocation("2006-01-02", data.Company.BusinessTermStart, time.Local)
	if err != nil {
		return errors.New("营业期限错误")
	}
	businessTermEnd, err := time.ParseInLocation("2006-01-02", data.Company.BusinessTermEnd, time.Local)
	if err != nil {
		return errors.New("营业期限错误")
	}
	dealerNo, err := common.GenNo(constant.NoPrefixDealer)
	if err != nil {
		return errors.New("提交失败")
	}
	if mysql.Db.Where("id = ?", data.Base.BrokerId).First(&model.Broker{}).Error != nil {
		return errors.New("经纪人错误")
	}
	brokerBusiness := model.Dealer{
		DealerNo:     dealerNo,
		Name:         data.Base.Name,
		Address:      data.Base.Address,
		ContractNo:   data.Base.ContractNo,
		Contract:     data.Base.Contract,
		ExpireDate:   expireDate,
		Liaison:      data.Base.Liaison,
		LiaisonPhone: data.Base.LiaisonPhone,
		Telephone:    data.Base.Telephone,
		BrokerId:     data.Base.BrokerId,
		Company: model.DealerCompany{
			Name:                 data.Company.Name,
			BusinessTermStart:    businessTermStart,
			BusinessTermEnd:      businessTermEnd,
			RegisteredCapital:    data.Company.RegisteredCapital,
			Quality:              data.Company.Quality,
			BusinessLicenseNo:    data.Company.BusinessLicenseNo,
			LegalPerson:          data.Company.LegalPerson,
			LegalPersonCard:      data.Company.LegalPersonCard,
			IdCardFront:          data.Company.IdCardFront,
			IdCardBack:           data.Company.IdCardBack,
			BusinessLicensePhoto: data.Company.BusinessLicensePhoto,
		},
		Finance: model.DealerFinance{
			OpeningBank: data.Finance.OpeningBank,
			BankAccount: data.Finance.BankAccount,
			Address:     data.Finance.Address,
			Telephone:   data.Finance.Telephone,
			Proof:       data.Finance.Proof,
		},
		ApplyAt:     time.Now(),
		ApplyUserId: manageId,
		Status:      model.DealerStatusDisabled,
		Audit: model.DealerAudit{
			Status: model.DealerAuditStatusAudit,
		},
	}
	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		if tx.Create(&brokerBusiness).Error != nil {
			return errors.New("提交失败")
		}
		brokerBusiness.Audit.DealerId = brokerBusiness.Id
		if tx.Updates(&brokerBusiness.Audit).Error != nil {
			return errors.New("提交失败")
		}
		return nil
	})
}

type dealerInfo struct {
	Id           uint              `json:"id"`
	DealerNo     string            `json:"dealerNo"`
	Name         string            `json:"name"`
	Address      string            `json:"address"`
	ContractNo   string            `json:"contractNo"`
	Contract     string            `json:"contract"`
	ExpireDate   int64             `json:"expireDate"`
	Liaison      string            `json:"liaison"`
	LiaisonPhone string            `json:"liaisonPhone"`
	Telephone    string            `json:"telephone"`
	StoreName    string            `json:"storeName"`
	StaffName    string            `json:"staffName"`
	StaffPhone   string            `json:"staffPhone"`
	Company      dealerInfoCompany `json:"company"`
	Finance      dealerInfoFinance `json:"finance"`
	Audit        dealerInfoAudit   `json:"audit"`
}

type dealerInfoCompany struct {
	Name                 string          `json:"name"`
	Quality              uint            `json:"quality"`
	RegisteredCapital    decimal.Decimal `json:"registeredCapital"`
	BusinessTermStart    int64           `json:"businessTermStart"`
	BusinessTermEnd      int64           `json:"businessTermEnd"`
	BusinessLicenseNo    string          `json:"businessLicenseNo"`
	LegalPerson          string          `json:"legalPerson"`
	LegalPersonCard      string          `json:"legalPersonCard"`
	IdCardFront          string          `json:"idCardFront"`
	IdCardBack           string          `json:"idCardBack"`
	BusinessLicensePhoto string          `json:"businessLicensePhoto"`
}

type dealerInfoFinance struct {
	OpeningBank string `json:"openingBank"`
	BankAccount string `json:"bankAccount"`
	Address     string `json:"address"`
	Telephone   string `json:"telephone"`
	Proof       string `json:"proof"`
}

type dealerInfoAudit struct {
	Status        uint   `json:"status"`
	AuditAt       int64  `json:"auditAt"`
	Reason        string `json:"reason"`
	AuditUserName string `json:"auditUserName"`
}

// Info @Title 车商详情
func (d *dealerLogic) Info(dealerId uint) (result dealerInfo, err error) {
	dealer := model.Dealer{Id: dealerId}
	if mysql.Db.Preload("Broker.Staff.Store").Preload("Company").Preload("Finance").Preload("Audit.AuditUser").First(&dealer).Error != nil {
		return result, errors.New("车商错误")
	}
	result = dealerInfo{
		Id:           dealer.Id,
		DealerNo:     dealer.DealerNo,
		Name:         dealer.Name,
		Address:      dealer.Address,
		ContractNo:   dealer.ContractNo,
		Contract:     dealer.Contract,
		ExpireDate:   dealer.ExpireDate.Unix(),
		Liaison:      dealer.Liaison,
		LiaisonPhone: dealer.LiaisonPhone,
		Telephone:    dealer.Telephone,
		StoreName:    dealer.Broker.Staff.Store.Name,
		StaffName:    dealer.Broker.Staff.Name,
		StaffPhone:   dealer.Broker.Staff.Phone,
		Company: dealerInfoCompany{
			Name:                 dealer.Company.Name,
			Quality:              dealer.Company.Quality,
			RegisteredCapital:    dealer.Company.RegisteredCapital,
			BusinessTermStart:    dealer.Company.BusinessTermStart.Unix(),
			BusinessTermEnd:      dealer.Company.BusinessTermEnd.Unix(),
			BusinessLicenseNo:    dealer.Company.BusinessLicenseNo,
			LegalPerson:          dealer.Company.LegalPerson,
			LegalPersonCard:      dealer.Company.LegalPersonCard,
			IdCardFront:          dealer.Company.IdCardFront,
			IdCardBack:           dealer.Company.IdCardBack,
			BusinessLicensePhoto: dealer.Company.BusinessLicensePhoto,
		},
		Finance: dealerInfoFinance{
			OpeningBank: dealer.Finance.OpeningBank,
			BankAccount: dealer.Finance.BankAccount,
			Address:     dealer.Finance.Address,
			Telephone:   dealer.Finance.Telephone,
			Proof:       dealer.Finance.Proof,
		},
		Audit: dealerInfoAudit{
			Status:        dealer.Audit.Status,
			AuditAt:       dealer.Audit.AuditAt.Time.Unix(),
			Reason:        dealer.Audit.Reason,
			AuditUserName: dealer.Audit.AuditUser.Name,
		},
	}
	return
}

// Able @Title 车商启用停用
func (d *dealerLogic) Able(dealerId, status uint) error {
	dealer := model.Dealer{Id: dealerId}
	if mysql.Db.First(&dealer).Error != nil {
		return nil
	}
	switch status {
	case 1:
		if dealer.Status == model.DealerStatusEnabled {
			return errors.New("账号已启用")
		}
	case 2:
		if dealer.Status == model.DealerStatusDisabled {
			return errors.New("账号已停用")
		}
	default:
		return errors.New("未知状态")
	}
	if mysql.Db.Model(&dealer).Where(&dealer).UpdateColumns(map[string]interface{}{
		"status": status,
	}).RowsAffected != 1 {
		return errors.New("更改状态失败")
	}
	return nil
}

// Delete @Title 删除车商
func (d *dealerLogic) Delete(dealerId uint) error {
	dealer := model.Dealer{Id: dealerId}
	if mysql.Db.First(&dealer).Error != nil {
		return nil
	}
	if dealer.Status == model.DealerStatusEnabled {
		return errors.New("账号已启用")
	}
	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		if tx.Where(&dealer).Delete(&dealer).Error != nil {
			return errors.New("删除失败")
		}
		return nil
	})
}
