package order

import (
	"base/app/lib/bean"
	"base/app/model"
	"errors"
	"fmt"
	"git.oa00.com/go/mysql"
	"github.com/shopspring/decimal"
)

var CallCarLogic = &callCarLogic{}

type callCarLogic struct {
}

type callCarItem struct {
	Id         uint            `json:"id"`
	OrderSn    string          `json:"orderSn"`
	MainPhoto  string          `json:"mainPhoto"`
	ModelName  string          `json:"modelName"`
	Amount     decimal.Decimal `json:"amount"`
	Status     uint            `json:"status"`
	StatusName string          `json:"statusName"`
	CreatedAt  int64           `json:"createdAt"`
	FinalAt    int64           `json:"finalAt"`
}
type CallCarSearch struct {
	OrderSn          string
	TotalAmountType  uint
	TotalAmount      decimal.Decimal
	Status           uint
	CreatedDateStart string
	CreatedDateEnd   string
	FinalDateStart   string
	FinalDateEnd     string
}

// Lists @Title 叫车订单列表
func (c *callCarLogic) Lists(search CallCarSearch, page bean.Page) (lists []callCarItem, total int64) {
	lists = []callCarItem{}
	var callCars []model.OrderCallCar
	where := mysql.Db.Where("status > ?", model.OrderCallCarStatusUnPay)
	// 订单编号
	if search.OrderSn != "" {
		where = where.Where("order_sn like ?", fmt.Sprintf("%%%s%%", search.OrderSn))
	}
	// 订单总价
	if !search.TotalAmount.IsZero() {
		switch search.TotalAmountType {
		case 1: // 大于等于
			where = where.Where("amount >= ?", search.TotalAmount)
		case 2: // 小于等于
			where = where.Where("amount <= ?", search.TotalAmount)
		}
	}
	// 订单状态
	if search.Status > 0 {
		switch search.Status {
		case 1: // 进行中
			where = where.Where("status = ?", model.OrderCallCarStatusPay)
		case 2: // 已完成
			where = where.Where("status = ?", model.OrderCallCarStatusFinal)
		case 3: // 已退款
			where = where.Where("status = ?", model.OrderCallCarStatusRefund)
		case 4: // 退款申请
			where = where.Where("status = ?", model.OrderCallCarStatusRefundAudit)
		case 5: // 已退款
			where = where.Where("status = ?", model.OrderCallCarStatusRefund)
		case 6: // 退款驳回
			where = where.Where("status = ?", model.OrderCallCarStatusRefundReject)
		}
	}
	// 下单时间
	if search.CreatedDateStart != "" && search.CreatedDateEnd != "" {
		where = where.Where("created_at >= ? and created_at <= ?", search.CreatedDateStart, search.CreatedDateEnd+" 23:59:59")
	}
	// 完成时间
	if search.FinalDateStart != "" && search.FinalDateEnd != "" {
		where = where.Where("final_at >= ? and final_at <= ?", search.FinalDateStart, search.FinalDateEnd+" 23:59:59")
	}
	mysql.Db.Model(&callCars).Where(where).Count(&total)
	if page.HasPage(total) {
		mysql.Db.Preload("Car.Model").Where(where).Offset(page.GetStart()).Limit(page.GetLimit()).Order("id desc").Find(&callCars)
		for _, callCar := range callCars {
			lists = append(lists, callCarItem{
				Id:         callCar.Id,
				OrderSn:    callCar.OrderSn,
				MainPhoto:  callCar.Car.MainPhoto,
				ModelName:  callCar.Car.Model.Name,
				Amount:     callCar.Amount,
				Status:     callCar.Status,
				StatusName: callCar.GetStatusName(callCar.Status),
				CreatedAt:  callCar.CreatedAt.Unix(),
				FinalAt:    callCar.FinalAt.Time.Unix(),
			})
		}
	}
	return
}

type callCarInfo struct {
	Id         uint                 `json:"id"`
	Customer   callCarInfoCustomer  `json:"customer"`
	Broker     callCarInfoBroker    `json:"broker"`
	CarBroker  callCarInfoCarBroker `json:"carBroker"`
	OrderSn    string               `json:"orderSn"`
	ModelName  string               `json:"modelName"`
	Amount     decimal.Decimal      `json:"amount"`
	ReserveAt  int64                `json:"reserveAt"`
	Address    string               `json:"address"`
	Remark     string               `json:"remark"`
	Status     uint                 `json:"status"`
	StatusName string               `json:"statusName"`
}
type callCarInfoCustomer struct {
	Id       uint   `json:"id"`
	Nickname string `json:"nickname"`
	Mobile   string `json:"mobile"`
}

type callCarInfoBroker struct {
	Id       uint   `json:"id"`
	Nickname string `json:"nickname"`
	Phone    string `json:"phone"`
}

type callCarInfoCarBroker struct {
	Id       uint   `json:"id"`
	Nickname string `json:"nickname"`
	Phone    string `json:"phone"`
}

// Info @Title 订单详情
func (c *callCarLogic) Info(orderId uint) (result callCarInfo, err error) {
	callCar := model.OrderCallCar{Id: orderId}
	if mysql.Db.
		Preload("Customer").Preload("Broker").Preload("CarBroker").
		Preload("Car.Model").
		First(&callCar).Error != nil {
		return result, errors.New("订单错误")
	}
	result = callCarInfo{
		Id: callCar.Id,
		Customer: callCarInfoCustomer{
			Id:       callCar.CustomerId,
			Nickname: callCar.Customer.Nickname,
			Mobile:   callCar.Customer.Mobile,
		},
		Broker: callCarInfoBroker{
			Id:       callCar.BrokerId,
			Nickname: callCar.Broker.Nickname,
			Phone:    callCar.Broker.Phone,
		},
		CarBroker: callCarInfoCarBroker{
			Id:       callCar.BrokerId,
			Nickname: callCar.CarBroker.Nickname,
			Phone:    callCar.CarBroker.Phone,
		},
		OrderSn:    callCar.OrderSn,
		ModelName:  callCar.Car.Model.Name,
		Amount:     callCar.Amount,
		ReserveAt:  callCar.ReserveAt.Unix(),
		Address:    callCar.Address,
		Remark:     callCar.Remark,
		Status:     callCar.Status,
		StatusName: callCar.GetStatusName(callCar.Status),
	}
	return
}
