package order

import (
	"base/app/lib/bean"
	"base/app/logic/pay"
	"base/app/model"
	"errors"
	"fmt"
	"git.oa00.com/go/mysql"
	"github.com/shopspring/decimal"
	"gorm.io/gorm"
	"time"
)

var CallCarRefundLogic = &callCarRefundLogic{}

type callCarRefundLogic struct {
}

type callCarRefundItem struct {
	Id            uint            `json:"id"`
	OrderSn       string          `json:"orderSn"`
	MainPhoto     string          `json:"mainPhoto"`
	ModelName     string          `json:"modelName"`
	Amount        decimal.Decimal `json:"amount"`
	CreatedAt     int64           `json:"createdAt"`
	RefundApplyAt int64           `json:"refundApplyAt"`
	OrderStatus   uint            `json:"orderStatus"`
	AuditStatus   uint            `json:"auditStatus"`
}

type CallCarRefundSearch struct {
	OrderSn          string
	CarSn            string
	TotalAmountType  uint
	TotalAmount      decimal.Decimal
	Status           uint
	CreatedDateStart string
	CreatedDateEnd   string
}

// Lists @Title 叫车退款审核
func (c *callCarRefundLogic) Lists(search CallCarRefundSearch, page bean.Page) (lists []callCarRefundItem, total int64) {
	lists = []callCarRefundItem{}
	var callCarRefunds []model.OrderCallCarRefund
	where := mysql.Db
	// 订单筛选
	{
		orderWhere := mysql.Db.Model(&model.OrderCallCar{}).Select("id")
		// 订单编号
		if search.OrderSn != "" {
			orderWhere = orderWhere.Where("order_sn like ?", fmt.Sprintf("%%%s%%", search.OrderSn))
		}
		// 订单总价
		if !search.TotalAmount.IsZero() {
			switch search.TotalAmountType {
			case 1: // 大于等于
				orderWhere = orderWhere.Where("amount >= ?", search.TotalAmount)
			case 2: // 小于等于
				orderWhere = orderWhere.Where("amount <= ?", search.TotalAmount)
			}
		}
		if _, ok := orderWhere.Statement.Clauses["WHERE"]; ok {
			where = where.Where("order_id in (?)", orderWhere)
		}
	}
	// 车辆编号
	if search.CarSn != "" {
		where = where.Where("order_id in (?)",
			mysql.Db.Model(&model.OrderCallCar{}).Select("id").
				Where("car_id in (?)", mysql.Db.Model(&model.Car{}).Select("id").
					Where("car_sn like ?", fmt.Sprintf("%%%s%%", search.CarSn))))
	}
	// 审核状态
	if search.Status > 0 {
		switch search.Status {
		case 1: // 未审核
			where = where.Where("status = ?", model.OrderCallCarRefundStatusAudit)
		case 2: // 已审核
			where = where.Where("status in ?", []int{
				model.OrderCallCarRefundStatusAdopt,
				model.OrderCallCarRefundStatusReject,
			})
		}
	}
	// 申请日期
	if search.CreatedDateStart != "" && search.CreatedDateEnd != "" {
		where = where.Where("created_at >= ? and created_at <= ?", search.CreatedDateStart, search.CreatedDateEnd+" 23:59:59")
	}
	mysql.Db.Model(&callCarRefunds).Where(where).Count(&total)
	if page.HasPage(total) {
		mysql.Db.Preload("Order.Car.Model").Where(where).Offset(page.GetStart()).Limit(page.GetLimit()).Order("id desc").Find(&callCarRefunds)
		for _, orderCallCarRefund := range callCarRefunds {
			lists = append(lists, callCarRefundItem{
				Id:            orderCallCarRefund.Id,
				OrderSn:       orderCallCarRefund.Order.OrderSn,
				MainPhoto:     orderCallCarRefund.Order.Car.MainPhoto,
				ModelName:     orderCallCarRefund.Order.Car.Model.Name,
				Amount:        orderCallCarRefund.Order.Amount,
				CreatedAt:     orderCallCarRefund.Order.CreatedAt.Unix(),
				RefundApplyAt: orderCallCarRefund.CreatedAt.Unix(),
				OrderStatus:   orderCallCarRefund.Order.Status,
				AuditStatus:   orderCallCarRefund.Status,
			})
		}
	}
	return
}

type callRefundCarInfo struct {
	Id            uint                       `json:"id"`
	Customer      callCarRefundInfoCustomer  `json:"customer"`
	Broker        callCarRefundInfoBroker    `json:"broker"`
	CarBroker     callCarRefundInfoCarBroker `json:"carBroker"`
	OrderSn       string                     `json:"orderSn"`
	ModelName     string                     `json:"modelName"`
	Amount        decimal.Decimal            `json:"amount"`
	ReserveAt     int64                      `json:"reserveAt"`
	Address       string                     `json:"address"`
	Remark        string                     `json:"remark"`
	Status        uint                       `json:"status"`
	Reason        string                     `json:"reason"`
	Proof         string                     `json:"proof"`
	AuditUserName string                     `json:"auditUserName"`
	AuditStatus   uint                       `json:"auditStatus"`
	RejectReason  string                     `json:"rejectReason"`
	AuditAt       int64
}
type callCarRefundInfoCustomer struct {
	Id       uint   `json:"id"`
	Nickname string `json:"nickname"`
	Mobile   string `json:"mobile"`
}

type callCarRefundInfoBroker struct {
	Id       uint   `json:"id"`
	Nickname string `json:"nickname"`
	Phone    string `json:"phone"`
}

type callCarRefundInfoCarBroker struct {
	Id       uint   `json:"id"`
	Nickname string `json:"nickname"`
	Phone    string `json:"phone"`
}

// Info @Title 订单详情
func (c *callCarRefundLogic) Info(refundId uint) (result callRefundCarInfo, err error) {
	callCarRefund := model.OrderCallCarRefund{Id: refundId}
	if mysql.Db.
		Preload("Order.Customer").Preload("Order.Broker").Preload("Order.CarBroker").
		Preload("Order.Car.Model").Preload("AuditUser").
		First(&callCarRefund).Error != nil {
		return result, errors.New("订单错误")
	}
	result = callRefundCarInfo{
		Id: callCarRefund.Id,
		Customer: callCarRefundInfoCustomer{
			Id:       callCarRefund.Order.CustomerId,
			Nickname: callCarRefund.Order.Customer.Nickname,
			Mobile:   callCarRefund.Order.Customer.Mobile,
		},
		Broker: callCarRefundInfoBroker{
			Id:       callCarRefund.Order.BrokerId,
			Nickname: callCarRefund.Order.Broker.Nickname,
			Phone:    callCarRefund.Order.Broker.Phone,
		},
		CarBroker: callCarRefundInfoCarBroker{
			Id:       callCarRefund.Order.BrokerId,
			Nickname: callCarRefund.Order.CarBroker.Nickname,
			Phone:    callCarRefund.Order.CarBroker.Phone,
		},
		OrderSn:       callCarRefund.Order.OrderSn,
		ModelName:     callCarRefund.Order.Car.Model.Name,
		Amount:        callCarRefund.Order.Amount,
		ReserveAt:     callCarRefund.Order.ReserveAt.Unix(),
		Address:       callCarRefund.Order.Address,
		Remark:        callCarRefund.Order.Remark,
		Reason:        callCarRefund.Reason,
		Proof:         callCarRefund.Proof,
		AuditUserName: callCarRefund.AuditUser.Name,
		AuditStatus:   callCarRefund.Status,
		RejectReason:  callCarRefund.RejectReason,
		AuditAt:       callCarRefund.AuditAt.Time.Unix(),
	}
	return
}

// Adopt @Title 审核通过
func (c *callCarRefundLogic) Adopt(manageId, refundId uint) error {
	callCarRefund := model.OrderCallCarRefund{Id: refundId}
	if mysql.Db.Preload("Order").First(&callCarRefund).Error != nil {
		return errors.New("审核单错误")
	}
	if callCarRefund.Status != model.OrderCallCarRefundStatusAudit {
		return errors.New("已审核，请勿重复操作")
	}
	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		if err := pay.PayLogic.Refund(tx, callCarRefund.Order.PayId); err != nil {
			return err
		}
		if tx.Model(&model.OrderCallCarRefund{}).Where(&callCarRefund).UpdateColumns(map[string]interface{}{
			"status":        model.OrderCallCarRefundStatusAdopt,
			"audit_at":      time.Now(),
			"audit_user_id": manageId,
		}).RowsAffected != 1 {
			return errors.New("审核失败")
		}
		if tx.Model(&model.OrderCallCar{}).Where("id  = ? and status = ?", callCarRefund.OrderId, model.OrderCallCarStatusRefundAudit).UpdateColumn("status", model.OrderCallCarStatusRefund).RowsAffected != 1 {
			return errors.New("审核失败")
		}
		return nil
	})
}

// Reject @Title 审核驳回
func (c *callCarRefundLogic) Reject(manageId, refundId uint, reason string) error {
	callCarRefund := model.OrderCallCarRefund{Id: refundId}
	if mysql.Db.First(&callCarRefund).Error != nil {
		return errors.New("审核单错误")
	}
	if callCarRefund.Status != model.OrderCallCarRefundStatusAudit {
		return errors.New("已审核，请勿重复操作")
	}
	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		if tx.Model(&callCarRefund).Where(&callCarRefund).UpdateColumns(map[string]interface{}{
			"status":        model.OrderCallCarRefundStatusReject,
			"audit_at":      time.Now(),
			"reject_reason": reason,
			"audit_user_id": manageId,
		}).RowsAffected != 1 {
			return errors.New("审核失败")
		}
		if tx.Model(&model.OrderCallCar{}).Where("id  = ? and status = ?", callCarRefund.OrderId, model.OrderCallCarStatusRefundAudit).UpdateColumn("status", model.OrderCallCarStatusRefundReject).RowsAffected != 1 {
			return errors.New("审核失败")
		}
		return nil
	})
}
