package order

import (
	"base/app/common"
	"base/app/lib/bean"
	"base/app/model"
	"errors"
	"fmt"
	"git.oa00.com/go/mysql"
	"github.com/shopspring/decimal"
	"gorm.io/gorm"
)

var ConsignmentLogic = &consignmentLogic{}

type consignmentLogic struct {
}
type consignmentItem struct {
	Id         uint            `json:"id"`
	OrderSn    string          `json:"orderSn"`
	MainPhoto  string          `json:"mainPhoto"`
	ModelName  string          `json:"modelName"`
	CarSn      string          `json:"carSn"`
	Price      decimal.Decimal `json:"price"`
	ContractSn string          `json:"contractSn"`
	Status     uint            `json:"status"`
	StatusName string          `json:"statusName"`
	CreatedAt  int64           `json:"createdAt"`
	CancelAt   int64           `json:"cancelAt"`
	SaleAt     int64           `json:"saleAt"`
}
type ConsignmentListsSearch struct {
	Status           uint `binding:"oneof=0 1 2 3" label:"状态"` // 1=交易中 2=交易完成 3=交易取消
	OrderSn          string
	CarSn            string
	TotalAmountType  uint
	TotalAmount      decimal.Decimal
	ContractSn       string
	SearchStatus     uint
	CreatedDateStart string
	CreatedDateEnd   string
	CancelDateStart  string
	CancelDateEnd    string
	FinishDateStart  string
	FinishDateEnd    string
}

// Lists @Title 寄卖订单列表
func (c *consignmentLogic) Lists(search ConsignmentListsSearch, page bean.Page) (lists []consignmentItem, total int64) {
	var consignments []model.OrderConsignment
	where := mysql.Db
	// 交易状态
	switch search.Status {
	case 1: // 交易中
		where = where.Where("status not in ?", []int{
			model.OrderConsignmentStatusFinal,
			model.OrderConsignmentStatusCancel,
		})
	case 2: // 交易完成
		where = where.Where("status = ?", model.OrderConsignmentStatusFinal)
	case 3: // 交易取消
		where = where.Where("status = ?", model.OrderConsignmentStatusCancel)
	}
	// 车辆编号
	if search.CarSn != "" {
		where = where.Where("id in (?)",
			mysql.Db.Model(&model.CarBase{}).Select("order_id").
				Where("car_id in (?)", mysql.Db.Model(&model.Car{}).Select("id").
					Where("car_sn like ?", fmt.Sprintf("%%%s%%", search.CarSn))))
	}
	// 成交价格
	if !search.TotalAmount.IsZero() {
		switch search.TotalAmountType {
		case 1: // 大于等于
			where = where.Where("sale_amount >= ?", search.TotalAmount)
		case 2: // 小于等于
			where = where.Where("sale_amount <= ?", search.TotalAmount)
		}
	}
	// 合同编号
	if search.OrderSn != "" {
		where = where.Where("order_sn like ?", fmt.Sprintf("%%%s%%", search.OrderSn))
	}
	// 当前进度
	if search.SearchStatus > 0 {
		switch search.SearchStatus {
		case 1: // 待签订
			where = where.Where("status = ?", model.OrderConsignmentStatusUnSign)
		case 2: // 待发布
			where = where.Where("status = ?", model.OrderConsignmentStatusUnPublish)
		case 3: // 审核中
			where = where.Where("status = ?", model.OrderConsignmentStatusPublish)
		case 4: // 已上架
			where = where.Where("status = ?", model.OrderConsignmentStatusTheUpper)
		case 5: // 已出售
			where = where.Where("status = ?", model.OrderConsignmentStatusSale)
		case 6: // 已到账
			where = where.Where("status = ?", model.OrderConsignmentStatusAccount)
		case 7: // 已成交
			where = where.Where("status = ?", model.OrderConsignmentStatusFinal)
		case 11: // 已下架
			where = where.Where("status = ?", model.OrderConsignmentStatusAccount)
		case 12: // 订单取消
			where = where.Where("status = ?", model.OrderConsignmentStatusCancel)
		}
	}
	// 创建日期
	if search.CreatedDateStart != "" && search.CreatedDateEnd != "" {
		where = where.Where("created_at >= ? and created_at <= ?", search.CreatedDateStart, search.CreatedDateEnd+" 23:59:59")
	}
	// 取消日期
	if search.CancelDateStart != "" && search.CancelDateEnd != "" {
		where = where.Where("cancel_at >= ? and cancel_at <= ?", search.CancelDateStart, search.CancelDateEnd+" 23:59:59")
	}
	// 售出日期
	if search.FinishDateStart != "" && search.FinishDateEnd != "" {
		where = where.Where("sale_at >= ? and sale_at <= ?", search.FinishDateStart, search.FinishDateEnd+" 23:59:59")
	}
	mysql.Db.Model(&consignments).Where(where).Count(&total)
	if page.HasPage(total) {
		mysql.Db.Preload("CarBase.Model").Preload("CarBase.Car").Preload("Contract").
			Where(where).Offset(page.GetStart()).Limit(page.GetLimit()).Order("id desc").Find(&consignments)
		for _, consignment := range consignments {
			lists = append(lists, consignmentItem{
				Id:         consignment.Id,
				OrderSn:    consignment.OrderSn,
				MainPhoto:  consignment.CarBase.MainPhoto,
				ModelName:  consignment.CarBase.Model.Name,
				CarSn:      consignment.CarBase.Car.CarSn,
				Price:      consignment.CarBase.Car.Price,
				ContractSn: consignment.Contract.ContractSn,
				Status:     consignment.Status,
				StatusName: consignment.GetStatusName(consignment.Status),
				CreatedAt:  consignment.CreatedAt.Unix(),
				SaleAt:     consignment.SaleAt.Time.Unix(),
				CancelAt:   consignment.CancelAt.Time.Unix(),
			})
		}
	}
	return
}

type consignmentInfo struct {
	Id             uint                      `json:"id"`
	OrderSn        string                    `json:"orderSn"`
	BrokerName     string                    `json:"brokerName"`
	CarModel       string                    `json:"carModel"`
	CreatedAt      int64                     `json:"createdAt"`
	Status         uint                      `json:"status"`
	StatusName     string                    `json:"statusName"`
	Price          decimal.Decimal           `json:"price"`
	CarSn          string                    `json:"carSn"`
	ContractSn     string                    `json:"contractSn"`
	BrokerBusiness consignmentBrokerBusiness `json:"brokerBusiness"`
	Customer       consignmentCustomer       `json:"customer"`
	ContractMaster consignmentContractMaster `json:"contractMaster"`
	History        []consignmentHis          `json:"history"`
}

type consignmentBrokerBusiness struct {
	Broker   consignmentBroker
	Business consignmentBusiness
}

type consignmentBroker struct {
	Id         uint            `json:"id"`
	Name       string          `json:"name"`
	Tel        string          `json:"tel"`
	Commission decimal.Decimal `json:"commission"`
}

type consignmentBusiness struct {
	Id   uint   `json:"id"`
	Name string `json:"name"`
	Tel  string `json:"tel"`
}
type consignmentCustomer struct {
	Id   uint   `json:"id"`
	Name string `json:"name"`
	Tel  string `json:"tel"`
}

type consignmentContractMaster struct {
	Name     string `json:"name"`     // 姓名
	Tel      string `json:"tel"`      // 手机号
	IdCard   string `json:"idCard"`   // 身份证号
	BankCard string `json:"bankCard"` // 银行卡号
	Bank     string `json:"bank"`     // 开户行
}
type consignmentHis struct {
	Type      uint  `json:"type"`     // 操作类型 1=创建订单
	UserId    uint  `json:"userId"`   // 操作人员id
	UserType  uint  `json:"userType"` // 操作人员类型  1=车务 2=客户
	CreatedAt int64 `json:"createdAt"`
	UpdatedAt int64 `json:"updatedAt"`
}

// Info @Title 寄卖订单详情
func (c *consignmentLogic) Info(consignmentId uint) (result consignmentInfo, err error) {
	consignment := model.OrderConsignment{Id: consignmentId}
	if mysql.Db.
		Preload("Broker.Staff.Business").
		Preload("Customer").
		Preload("Contract.Master").
		Preload("CarBase.Car.Model").
		Preload("Histories").
		First(&consignment).Error != nil {
		return result, errors.New("订单错误")
	}
	var his []consignmentHis
	for _, consignmentHistory := range consignment.Histories {
		his = append(his, consignmentHis{
			Type:      consignmentHistory.Type,
			UserId:    consignmentHistory.UserId,
			UserType:  consignmentHistory.UserType,
			CreatedAt: consignmentHistory.CreatedAt.Unix(),
			UpdatedAt: consignmentHistory.UpdatedAt.Unix(),
		})
	}
	result = consignmentInfo{
		Id:         consignment.Id,
		OrderSn:    consignment.OrderSn,
		BrokerName: consignment.Broker.Nickname,
		CarModel:   consignment.CarBase.Car.Model.Name,
		CreatedAt:  consignment.CreatedAt.Unix(),
		Status:     consignment.Status,
		StatusName: consignment.GetStatusName(consignment.Status),
		Price:      consignment.ContractPrice,
		CarSn:      consignment.CarBase.Car.CarSn,
		ContractSn: consignment.Contract.ContractSn,
		BrokerBusiness: consignmentBrokerBusiness{
			Broker: consignmentBroker{
				Id:         consignment.Broker.Id,
				Name:       consignment.Broker.Nickname,
				Tel:        consignment.Broker.Phone,
				Commission: consignment.Broker.Staff.Commission,
			},
			Business: consignmentBusiness{
				Id:   consignment.Broker.Staff.Business.Id,
				Name: consignment.Broker.Staff.Business.Name,
				Tel:  consignment.Broker.Staff.Business.Telephone,
			},
		},
		Customer: consignmentCustomer{
			Id:   consignment.Customer.Id,
			Name: consignment.Customer.Nickname,
			Tel:  consignment.Customer.Mobile,
		},
		ContractMaster: consignmentContractMaster{
			Name:     consignment.Contract.Master.Name,
			Tel:      consignment.Contract.Master.Phone,
			IdCard:   consignment.Contract.Master.IdCard,
			BankCard: consignment.Contract.Master.BankCard,
			Bank:     consignment.Contract.Master.Bank,
		},
		History: his,
	}
	return
}

// Cancel @Title 寄卖订单取消
func (c *consignmentLogic) Cancel(manageId, consignmentId uint) error {
	orderConsignment := model.OrderConsignment{Id: consignmentId}
	if mysql.Db.Preload("CarBase.Car").First(&orderConsignment).Error != nil {
		return errors.New("订单错误")
	}
	if common.InArray(orderConsignment.Status, []int{5, 6, 7}) {
		return errors.New("订单不可取消")
	}
	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		if tx.Model(&model.Car{}).Where("id = ?", orderConsignment.CarBase.CarId).UpdateColumns(map[string]interface{}{
			"the_upper": model.CarTheUpperFalse,
			"status":    model.CarStatusCancel,
		}).RowsAffected != 1 {
			return errors.New("车辆下架失败")
		}
		if tx.Model(&model.CarBase{}).Where("id = ?", orderConsignment.CarBase.Id).UpdateColumns(map[string]interface{}{
			"the_upper": model.CarBaseTheUpperFalse,
		}).RowsAffected != 1 {
			return errors.New("寄售车辆取消失败")
		}
		if tx.Model(&model.OrderConsignment{}).Where("id = ?", consignmentId).UpdateColumns(map[string]interface{}{
			"status": model.OrderConsignmentStatusCancel,
		}).RowsAffected != 1 {
			return errors.New("寄售订单取消失败")
		}
		return nil
	})
}

// Payment @Title 打款
func (c *consignmentLogic) Payment(manageId, orderId uint, amount decimal.Decimal, proof string) error {
	orderConsignment := model.OrderConsignment{Id: orderId}
	if mysql.Db.First(&orderConsignment).Error != nil {
		return errors.New("订单错误")
	}
	if orderConsignment.Status != model.OrderConsignmentStatusSale {
		return errors.New("订单错误")
	}
	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		if tx.Model(&orderConsignment).Where(&orderConsignment).UpdateColumns(map[string]interface{}{
			"status": model.OrderConsignmentStatusFinal,
		}).RowsAffected != 1 {
			return errors.New("打款失败")
		}
		if tx.Create(&model.OrderConsignmentPayment{
			OrderId:       orderConsignment.Id,
			Amount:        amount,
			Proof:         proof,
			CreatedUserId: manageId,
		}).Error != nil {
			return errors.New("打款失败")
		}
		return nil
	})
}
