package order

import (
	"base/app/lib/bean"
	"base/app/model"
	"errors"
	"fmt"
	"git.oa00.com/go/mysql"
	"github.com/shopspring/decimal"
)

var SaleLogic = &saleLogic{}

type saleLogic struct {
}

type saleItem struct {
	Id             uint            `json:"id"`
	OrderSn        string          `json:"orderSn"`
	CarSn          string          `json:"carSn"`
	MainPhoto      string          `json:"mainPhoto"`
	ModelName      string          `json:"modelName"`
	ContractAmount decimal.Decimal `json:"contractAmount"`
	ContractSn     string          `json:"contractSn"`
	Status         uint            `json:"status"`
	StatusName     string          `json:"statusName"`
	CreatedAt      int64           `json:"createdAt"`
}
type SaleSearch struct {
	Status           uint
	OrderSn          string
	CarSn            string
	TotalAmountType  uint
	TotalAmount      decimal.Decimal
	ContractSn       string
	CreatedDateStart string
	CreatedDateEnd   string
}

// Lists @Title 车辆出售订单列表
func (s *saleLogic) Lists(search SaleSearch, page bean.Page) (lists []saleItem, total int64) {
	lists = []saleItem{}
	var orderSales []model.OrderSale
	where := mysql.Db
	// 状态
	if search.Status > 0 {
		switch search.Status {
		case 1: // 交易中
			where = where.Where("status not in ?", []int{
				model.OrderSaleStatusFinal,
				model.OrderSaleStatusCancel,
			})
		case 2: // 交易完成
			where = where.Where("status = ?", model.OrderSaleStatusFinal)
		case 3: // 交易取消
			where = where.Where("status = ?", model.OrderSaleStatusCancel)
		}

	}
	// 订单编号
	if search.OrderSn != "" {
		where = where.Where("order_sn like ?", fmt.Sprintf("%%%s%%", search.OrderSn))
	}
	// 合同筛选
	{
		contractWhere := mysql.Db.Model(&model.ContractSale{}).Select("id")
		// 订单总价
		if !search.TotalAmount.IsZero() {
			switch search.TotalAmountType {
			case 1: // 大于等于
				contractWhere = contractWhere.Where("amount >= ?", search.TotalAmount)
			case 2: // 小于等于
				contractWhere = contractWhere.Where("amount <= ?", search.TotalAmount)
			}
		}
		// 合同编号
		if search.ContractSn != "" {
			contractWhere = contractWhere.Where("contract_sn like ?", fmt.Sprintf("%%%s%%", search.ContractSn))
		}
		if _, ok := contractWhere.Statement.Clauses["WHERE"]; ok {
			where = where.Where("contract_id in (?)", contractWhere)
		}
	}
	// 车辆编号
	if search.CarSn != "" {
		where = where.Where("car_id in (?)", mysql.Db.Model(&model.Car{}).Select("id").Where("car_sn like ?", fmt.Sprintf("%%%s%%", search.CarSn)))
	}
	// 订单创建日期
	if search.CreatedDateStart != "" && search.CreatedDateEnd != "" {
		where = where.Where("created_at >= ? and created_at <= ?", search.CreatedDateStart, search.CreatedDateEnd+" 23:59:59")
	}
	mysql.Db.Model(&orderSales).Where(where).Count(&total)
	if page.HasPage(total) {
		mysql.Db.
			Preload("Car.Model").Preload("Contract").
			Where(where).Offset(page.GetStart()).Limit(page.GetLimit()).Order("id desc").Find(&orderSales)
		for _, orderSale := range orderSales {
			lists = append(lists, saleItem{
				Id:             orderSale.Id,
				OrderSn:        orderSale.OrderSn,
				CarSn:          orderSale.Car.CarSn,
				MainPhoto:      orderSale.Car.MainPhoto,
				ModelName:      orderSale.Car.Model.Name,
				ContractAmount: orderSale.Contract.Amount,
				ContractSn:     orderSale.Contract.ContractSn,
				Status:         orderSale.Status,
				StatusName:     orderSale.GetStatusName(orderSale.Status),
				CreatedAt:      orderSale.CreatedAt.Unix(),
			})
		}
	}
	return
}

type SaleOrderInfo struct {
	Id             uint               `json:"id"`
	OrderSn        string             `json:"orderSn"`
	BrokerName     string             `json:"brokerName"`
	CarModel       string             `json:"carModel"`
	CreatedAt      int64              `json:"createdAt"`
	Status         uint               `json:"status"`
	StatusName     string             `json:"statusName"`
	Price          decimal.Decimal    `json:"price"`
	CarSn          string             `json:"carSn"`
	ContractSn     string             `json:"contractSn"`
	ContractDetail saleContract       `json:"contractDetail"`
	BrokerBusiness saleBrokerBusiness `json:"brokerBusiness"`
	Customer       saleCustomer       `json:"customer"`
	History        []saleHistory      `json:"history"`
}

type saleContract struct {
	Amount                 decimal.Decimal // 成交价
	Deposit                decimal.Decimal // 定金
	DownPayment            decimal.Decimal // 首付
	BalancePayment         decimal.Decimal // 尾款
	PayTypeName            string          `json:"payTypeName"`      // 支付方式
	TransferTypeName       string          `json:"transferTypeName"` // 过户方式
	ThirdPartKind          string          `json:"thirdPartKind"`
	ThirdPartId            uint            `json:"thirdPartId"`
	ThirdPartName          string          `json:"thirdPartName"`
	SaleServiceFee         decimal.Decimal `json:"saleServiceFee"`
	SaleServiceFeeRate     decimal.Decimal // 服务费比例
	PurchaseServiceFee     decimal.Decimal `json:"purchaseSaleServiceFee"`
	PurchaseServiceFeeRate decimal.Decimal // 服务费比例
	Phone                  string          // 手机号
	Name                   string          // 姓名
	IdCard                 string          // 身份证号
	Address                string          // 地址
	OpenBank               string          // 开户行
	BankCard               string          // 银行卡号
}

type saleBrokerBusiness struct {
	Broker   saleBroker
	Business saleBusiness
}

type saleBroker struct {
	Id         uint            `json:"id"`
	Name       string          `json:"name"`
	Tel        string          `json:"tel"`
	Commission decimal.Decimal `json:"commission"`
}

type saleBusiness struct {
	Id   uint   `json:"id"`
	Name string `json:"name"`
	Tel  string `json:"tel"`
}
type saleCustomer struct {
	Id   uint   `json:"id"`
	Name string `json:"name"`
	Tel  string `json:"tel"`
}

type saleHistory struct {
	Type      uint  `json:"type"`     // 操作类型
	UserId    uint  `json:"userId"`   // 操作人员id
	UserType  uint  `json:"userType"` // 操作人员类型  1=车务 2=客户
	CreatedAt int64 `json:"createdAt"`
	UpdatedAt int64 `json:"updatedAt"`
}

// Info @Title 出售订单详情
func (s *saleLogic) Info(orderSaleId uint) (result SaleOrderInfo, err error) {
	orderSale := model.OrderSale{Id: orderSaleId}
	if mysql.Db.
		Preload("Broker.Staff.Business").
		Preload("Customer").
		Preload("Contract.BuyerMaster").
		Preload("Contract.ThirdPart").
		First(&orderSale).Error != nil {
		return result, errors.New("订单错误")
	}
	result = SaleOrderInfo{
		Id:         orderSale.Id,
		OrderSn:    orderSale.OrderSn,
		BrokerName: orderSale.Broker.Nickname,
		CarModel:   orderSale.Car.Model.Name,
		CreatedAt:  orderSale.CreatedAt.Unix(),
		Status:     orderSale.Status,
		StatusName: orderSale.GetStatusName(orderSale.Status),
		Price:      orderSale.Contract.Amount,
		CarSn:      orderSale.Car.CarSn,
		ContractSn: orderSale.Contract.ContractSn,
		ContractDetail: saleContract{
			Amount:                 orderSale.Contract.Amount,
			Deposit:                orderSale.Contract.Deposit,
			DownPayment:            orderSale.Contract.DownPayment,
			BalancePayment:         orderSale.Contract.BalancePayment,
			PayTypeName:            orderSale.Contract.GetPayTypeName(orderSale.Contract.PayType),
			TransferTypeName:       orderSale.Contract.GetTransferTypeName(orderSale.Contract.TransferType),
			ThirdPartKind:          orderSale.Contract.GetThirdPartKindName(orderSale.Contract.ThirdPartKind),
			ThirdPartId:            orderSale.Contract.ThirdPartId,
			ThirdPartName:          orderSale.Contract.ThirdPart.Name,
			SaleServiceFee:         orderSale.Contract.SaleServiceFee,
			SaleServiceFeeRate:     orderSale.Contract.SaleServiceFeeRate,
			PurchaseServiceFee:     orderSale.Contract.PurchaseServiceFee,
			PurchaseServiceFeeRate: orderSale.Contract.PurchaseServiceFeeRate,
			Phone:                  orderSale.Contract.BuyerMaster.Phone,
			Name:                   orderSale.Contract.BuyerMaster.Name,
			IdCard:                 orderSale.Contract.BuyerMaster.IdCard,
			Address:                orderSale.Contract.BuyerMaster.Address,
			OpenBank:               orderSale.Contract.BuyerMaster.Bank,
			BankCard:               orderSale.Contract.BuyerMaster.BankCard,
		},
		BrokerBusiness: saleBrokerBusiness{
			Broker: saleBroker{
				Id:         orderSale.Broker.Id,
				Name:       orderSale.Broker.Nickname,
				Tel:        orderSale.Broker.Phone,
				Commission: orderSale.Broker.Staff.Commission,
			},
			Business: saleBusiness{
				Id:   orderSale.Broker.Staff.Business.Id,
				Name: orderSale.Broker.Staff.Business.Name,
				Tel:  orderSale.Broker.Staff.Business.Telephone,
			},
		},
		Customer: saleCustomer{
			Id:   orderSale.Customer.Id,
			Name: orderSale.Customer.Nickname,
			Tel:  orderSale.Customer.Mobile,
		},
	}
	return
}
