package setting

import (
	"base/app/common"
	"base/app/model"
	"errors"
	"fmt"
	"git.oa00.com/go/acs"
	"git.oa00.com/go/mysql"
	"gorm.io/gorm"
)

var MenuLogic = &menuLogic{}

type menuLogic struct {
}

// All @Title 获取全部菜单
func (m *menuLogic) All() (result []menuItem) {
	var menus []model.SettingMenu
	mysql.Db.Order("sort,id").Find(&menus)
	for _, menu := range menus {
		result = append(result, menuItem{
			Id:        menu.Id,
			Level:     menu.Level,
			ParentId:  menu.ParentId,
			Name:      menu.Name,
			Path:      menu.Path,
			Hidden:    menu.Hidden,
			Component: menu.Component,
			Sort:      menu.Sort,
			Type:      menu.Type,
			Title:     menu.Title,
			Icon:      menu.Icon,
		})
	}
	return m.getTree(result)
}

type MenuAdd struct {
	ParentId  uint
	Name      string `binding:"required" label:"路由名称"`
	Path      string `binding:"required" label:"路由地址"`
	Hidden    uint   `binding:"required,oneof=1 2" label:"是否隐藏"` // 1=隐藏 2=显示
	Component string `binding:"required" label:"前端文件路径"`
	Sort      uint
	Type      uint   `binding:"required,oneof=1 2 3" label:"菜单类型"` //  1=目录 2=菜单 3=按钮
	Title     string `binding:"required" label:"展示名称"`
	Icon      string
	Apis      []uint
}

// Add @Title 添加菜单
func (m *menuLogic) Add(data MenuAdd) error {
	menuData := model.SettingMenu{
		ParentId:  data.ParentId,
		Name:      data.Name,
		Path:      data.Path,
		Hidden:    data.Hidden,
		Component: data.Component,
		Sort:      data.Sort,
		Type:      data.Type,
		Title:     data.Title,
		Icon:      data.Icon,
	}
	if data.ParentId > 0 {
		menu := model.SettingMenu{Id: data.ParentId}
		if mysql.Db.First(&menu).Error != nil {
			return errors.New("上级菜单错误")
		}
		menuData.Level = menu.Level + 1
	}
	if len(data.Apis) > 0 {
		var total int64
		mysql.Db.Model(&model.SettingApi{}).Where("id in ?", data.Apis).Count(&total)
		if total != int64(len(data.Apis)) {
			return errors.New("api接口选择错误")
		}
		for _, api := range data.Apis {
			menuData.Apis = append(menuData.Apis, model.SettingMenuApi{
				ApiId: api,
			})
		}
	}
	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		if tx.Create(&menuData).Error != nil {
			return errors.New("添加失败")
		}
		if len(menuData.Apis) > 0 {
			groupingName := fmt.Sprintf("menu_%d", menuData.Id)
			var policies [][]string
			for _, api := range menuData.Apis {
				policies = append(policies, []string{groupingName, fmt.Sprintf("api_%d", api)})
			}
			if _, err := acs.Enforcer.AddGroupingPolicies(policies); err != nil {
				return errors.New("添加失败")
			}
		}
		return nil
	})
}

type menuInfo struct {
	Id        uint   `json:"id"`
	ParentId  uint   `json:"parentId"`
	Name      string `json:"name"`
	Path      string `json:"path"`
	Hidden    uint   `json:"hidden"`
	Component string `json:"component"`
	Sort      uint   `json:"sort"`
	Type      uint   `json:"type"`
	Title     string `json:"title"`
	Icon      string `json:"icon"`
	Apis      []uint `json:"apis"`
}

// Info @Title 菜单详情
func (m *menuLogic) Info(menuId uint) (result menuInfo, err error) {
	menu := model.SettingMenu{Id: menuId}
	if mysql.Db.Preload("Apis").First(&menu).Error != nil {
		return result, errors.New("菜单错误")
	}
	result = menuInfo{
		Id:        menu.Id,
		ParentId:  menu.ParentId,
		Name:      menu.Name,
		Path:      menu.Path,
		Hidden:    menu.Hidden,
		Component: menu.Component,
		Sort:      menu.Sort,
		Type:      menu.Type,
		Title:     menu.Title,
		Icon:      menu.Icon,
		Apis:      []uint{},
	}
	for _, api := range menu.Apis {
		result.Apis = append(result.Apis, api.ApiId)
	}
	return
}

type MenuEdit struct {
	MenuId uint `binding:"required" label:"菜单id"`
	MenuAdd
}

// Edit @Title 编辑菜单
func (m *menuLogic) Edit(data MenuEdit) error {
	menuData := model.SettingMenu{Id: data.MenuId}
	if mysql.Db.First(&menuData).Error != nil {
		return errors.New("菜单错误")
	}
	menuData.ParentId = data.ParentId
	menuData.Name = data.Name
	menuData.Path = data.Path
	menuData.Hidden = data.Hidden
	menuData.Component = data.Component
	menuData.Sort = data.Sort
	menuData.Type = data.Type
	menuData.Title = data.Title
	menuData.Icon = data.Icon
	menuData.Level = 0
	if data.ParentId > 0 {
		if data.MenuId == data.ParentId {
			return errors.New("上级菜单错误")
		}
		menu := model.SettingMenu{Id: data.ParentId}
		if mysql.Db.First(&menu).Error != nil {
			return errors.New("上级菜单错误")
		}
		menuData.Level = menu.Level + 1
		var parents []model.SettingMenu
		common.FindParents(mysql.Db.NamingStrategy.TableName("SettingMenu"), "id", "parent_id", data.ParentId, &parents)
		for _, parent := range parents {
			if parent.Id == menuData.Id {
				return errors.New("上级菜单选择错误")
			}
		}
	}
	groupingName := fmt.Sprintf("menu_%d", menuData.Id)
	var policies [][]string
	if len(data.Apis) > 0 {
		var total int64
		mysql.Db.Model(&model.SettingApi{}).Where("id in ?", data.Apis).Count(&total)
		if total != int64(len(data.Apis)) {
			return errors.New("api接口选择错误")
		}
		for _, api := range data.Apis {
			menuData.Apis = append(menuData.Apis, model.SettingMenuApi{
				ApiId: api,
			})
			policies = append(policies, []string{groupingName, fmt.Sprintf("api_%d", api)})
		}
	}
	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		if tx.Where("menu_id = ?", menuData.Id).Delete(&model.SettingMenuApi{}).Error != nil {
			return errors.New("编辑失败")
		}
		if tx.Save(&menuData).Error != nil {
			return errors.New("编辑失败")
		}
		if _, err := acs.Enforcer.RemoveGroupingPolicies(acs.Enforcer.GetFilteredGroupingPolicy(0, groupingName)); err != nil {
			return errors.New("编辑失败")
		}
		if len(policies) > 0 {
			if _, err := acs.Enforcer.AddGroupingPolicies(policies); err != nil {
				return errors.New("编辑失败")
			}
		}
		return nil
	})
}

type menuItem struct {
	Id        uint        `json:"id"`
	Level     uint        `json:"level"`
	ParentId  uint        `json:"parentId"`
	Name      string      `json:"name"`
	Path      string      `json:"path"`
	Hidden    uint        `json:"hidden"`
	Component string      `json:"component"`
	Sort      uint        `json:"sort"`
	Type      uint        `json:"type"`
	Title     string      `json:"title"`
	Icon      string      `json:"icon"`
	Children  []*menuItem `json:"children"`
}

// @Title 获取菜单树
func (m *menuLogic) getTree(datas []menuItem) (result []menuItem) {
	result = []menuItem{}
	for i, item := range datas {
		if item.ParentId > 0 {
			for index, value := range datas {
				if item.ParentId == value.Id {
					datas[index].Children = append(datas[index].Children, &datas[i])
					break
				}
			}
		}
	}
	for _, item := range datas {
		if item.ParentId == 0 {
			result = append(result, item)
		}
	}
	return
}

// Delete @Title 删除菜单
func (m *menuLogic) Delete(menuId uint) error {
	menuData := model.SettingMenu{Id: menuId}
	if mysql.Db.First(&menuData).Error != nil {
		return errors.New("菜单错误")
	}
	sonMenu := model.SettingMenu{ParentId: menuId}
	mysql.Db.Where(&sonMenu).First(&sonMenu)
	if sonMenu.Id > 0 {
		return errors.New("请先删除子级菜单")
	}
	groupingName := fmt.Sprintf("menu_%d", menuData.Id)
	return mysql.Db.Transaction(func(tx *gorm.DB) error {
		if tx.Where("menu_id = ?", menuData.Id).Delete(&model.SettingMenuApi{}).Error != nil {
			return errors.New("删除失败")
		}
		if tx.Delete(&menuData).Error != nil {
			return errors.New("删除失败")
		}
		if _, err := acs.Enforcer.RemoveGroupingPolicies(acs.Enforcer.GetFilteredGroupingPolicy(0, groupingName)); err != nil {
			return errors.New("删除失败")
		}
		return nil
	})
}
